@@ -26,6 +26,9 @@
 #include "v6util.h"
 #include "msg.h"
 
+static HIMAGELIST (WINAPI *pImageList_Create)(int, int, UINT, int, int);
+static BOOL (WINAPI *pImageList_Destroy)(HIMAGELIST);
+
 typedef struct tagEXPECTEDNOTIFY
 {
     INT iCode;
@@ -53,7 +56,11 @@ static HWND hWndHeader;
 
 #define compare(val, exp, fmt)  ok((val) == (exp), #val " value: " fmt ", expected: " fmt "\n", (val), (exp))
 
-#define expect(expected, got) ok(expected == got, "expected %d, got %d\n", expected,got)
+#define expect(expected,got) expect_(__LINE__, expected, got)
+static inline void expect_(unsigned line, DWORD expected, DWORD got)
+{
+    ok_(__FILE__, line)(expected == got, "Expected %ld, got %ld\n", expected, got);
+}
 
 #define NUM_MSG_SEQUENCES    2
 #define PARENT_SEQ_INDEX     0
@@ -390,13 +397,13 @@ static WCHAR pszUniTestW[] = {'T','S','T',0};
 
 #define TEST_GET_ITEM(i,c)\
 {   res = getItem(hWndHeader, i, buffer);\
-    ok(res != 0, "Getting item[%d] using valid index failed unexpectedly (%d)\n", i, res);\
+    ok(res != 0, "Getting item[%d] using valid index failed unexpectedly (%ld)\n", i, res);\
     ok(strcmp(str_items[c], buffer) == 0, "Getting item[%d] returned \"%s\" expecting \"%s\"\n", i, buffer, str_items[c]);\
 }
 
 #define TEST_GET_ITEMCOUNT(i)\
 {   res = getItemCount(hWndHeader);\
-    ok(res == i, "Got Item Count as %d\n", res);\
+    ok(res == i, "Got Item Count as %ld\n", res);\
 }
 
 static LRESULT WINAPI header_subclass_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
@@ -532,58 +539,79 @@ static HWND create_custom_header_control(HWND hParent, BOOL preloadHeaderItems)
     return childHandle;
 }
 
-static void check_auto_format(void)
+static void header_item_getback(HWND hwnd, UINT mask, HDITEMA *item)
 {
-    HDITEMA hdiCreate;
-    HDITEMA hdiRead;
-    static CHAR text[] = "Test";
-    ZeroMemory(&hdiCreate, sizeof(HDITEMA));
+    int ret;
+
+    ret = SendMessageA(hwnd, HDM_INSERTITEMA, 0, (LPARAM)item);
+    ok(ret != -1, "Failed to add header item.\n");
+
+    memset(item, 0, sizeof(*item));
+    item->mask = mask;
+
+    ret = SendMessageA(hwnd, HDM_GETITEMA, 0, (LPARAM)item);
+    ok(ret != 0, "Failed to get item data.\n");
+    ret = SendMessageA(hwnd, HDM_DELETEITEM, 0, 0);
+    ok(ret != 0, "Failed to delete item.\n");
+}
+
+static void test_item_auto_format(HWND parent)
+{
+    static char text[] = "Test";
+    HDITEMA item;
+    HBITMAP hbm;
+    HWND hwnd;
+
+    hwnd = create_custom_header_control(parent, FALSE);
 
     /* Windows implicitly sets some format bits in INSERTITEM */
 
     /* HDF_STRING is automatically set and cleared for no text */
-    hdiCreate.mask = HDI_TEXT|HDI_WIDTH|HDI_FORMAT;
-    hdiCreate.pszText = text;
-    hdiCreate.cxy = 100;
-    hdiCreate.fmt=HDF_CENTER;
-    addReadDelItem(hWndHeader, &hdiCreate, HDI_FORMAT, &hdiRead);
-    ok(hdiRead.fmt == (HDF_STRING|HDF_CENTER), "HDF_STRING not set automatically (fmt=%x)\n", hdiRead.fmt);
+    item.mask = HDI_TEXT | HDI_WIDTH | HDI_FORMAT;
+    item.pszText = text;
+    item.cxy = 100;
+    item.fmt = HDF_CENTER;
+    header_item_getback(hwnd, HDI_FORMAT, &item);
+    ok(item.fmt == (HDF_STRING | HDF_CENTER), "Unexpected item format mask %#x.\n", item.fmt);
 
-    hdiCreate.mask = HDI_WIDTH|HDI_FORMAT;
-    hdiCreate.pszText = text;
-    hdiCreate.fmt = HDF_CENTER|HDF_STRING;
-    addReadDelItem(hWndHeader, &hdiCreate, HDI_FORMAT, &hdiRead);
-    ok(hdiRead.fmt == (HDF_CENTER), "HDF_STRING should be automatically cleared (fmt=%x)\n", hdiRead.fmt);
+    item.mask = HDI_WIDTH | HDI_FORMAT;
+    item.pszText = text;
+    item.fmt = HDF_CENTER | HDF_STRING;
+    header_item_getback(hwnd, HDI_FORMAT, &item);
+    ok(item.fmt == HDF_CENTER, "Unexpected item format mask %#x.\n", item.fmt);
 
     /* HDF_BITMAP is automatically set and cleared for a NULL bitmap or no bitmap */
-    hdiCreate.mask = HDI_BITMAP|HDI_WIDTH|HDI_FORMAT;
-    hdiCreate.hbm = CreateBitmap(16, 16, 1, 8, NULL);
-    hdiCreate.fmt = HDF_CENTER;
-    addReadDelItem(hWndHeader, &hdiCreate, HDI_FORMAT, &hdiRead);
-    ok(hdiRead.fmt == (HDF_BITMAP|HDF_CENTER), "HDF_BITMAP not set automatically (fmt=%x)\n", hdiRead.fmt);
-    DeleteObject(hdiCreate.hbm);
+    item.mask = HDI_BITMAP | HDI_WIDTH | HDI_FORMAT;
+    item.hbm = hbm = CreateBitmap(16, 16, 1, 8, NULL);
+    item.fmt = HDF_CENTER;
+    header_item_getback(hwnd, HDI_FORMAT, &item);
+    ok(item.fmt == (HDF_BITMAP | HDF_CENTER), "Unexpected item format mask %#x.\n", item.fmt);
+    DeleteObject(hbm);
 
-    hdiCreate.hbm = NULL;
-    hdiCreate.fmt = HDF_CENTER|HDF_BITMAP;
-    addReadDelItem(hWndHeader, &hdiCreate, HDI_FORMAT, &hdiRead);
-    ok(hdiRead.fmt == HDF_CENTER, "HDF_BITMAP not cleared automatically for NULL bitmap (fmt=%x)\n", hdiRead.fmt);
+    item.mask = HDI_BITMAP | HDI_WIDTH | HDI_FORMAT;
+    item.hbm = NULL;
+    item.fmt = HDF_CENTER | HDF_BITMAP;
+    header_item_getback(hwnd, HDI_FORMAT, &item);
+    ok(item.fmt == HDF_CENTER, "Unexpected item format mask %#x.\n", item.fmt);
 
-    hdiCreate.mask = HDI_WIDTH|HDI_FORMAT;
-    hdiCreate.fmt = HDF_CENTER|HDF_BITMAP;
-    addReadDelItem(hWndHeader, &hdiCreate, HDI_FORMAT, &hdiRead);
-    ok(hdiRead.fmt == HDF_CENTER, "HDF_BITMAP not cleared automatically for no bitmap (fmt=%x)\n", hdiRead.fmt);
+    item.mask = HDI_WIDTH | HDI_FORMAT;
+    item.fmt = HDF_CENTER | HDF_BITMAP;
+    header_item_getback(hwnd, HDI_FORMAT, &item);
+    ok(item.fmt == HDF_CENTER, "Unexpected item format mask %#x.\n", item.fmt);
 
     /* HDF_IMAGE is automatically set but not cleared */
-    hdiCreate.mask = HDI_IMAGE|HDI_WIDTH|HDI_FORMAT;
-    hdiCreate.iImage = 17;
-    addReadDelItem(hWndHeader, &hdiCreate, HDI_FORMAT, &hdiRead);
-    ok(hdiRead.fmt == (HDF_IMAGE|HDF_CENTER), "HDF_IMAGE not set automatically (fmt=%x)\n", hdiRead.fmt);
+    item.mask = HDI_IMAGE | HDI_WIDTH | HDI_FORMAT;
+    item.iImage = 17;
+    header_item_getback(hwnd, HDI_FORMAT, &item);
+    ok(item.fmt == (HDF_IMAGE | HDF_CENTER), "Unexpected item format mask %#x.\n", item.fmt);
 
-    hdiCreate.mask = HDI_WIDTH|HDI_FORMAT;
-    hdiCreate.fmt = HDF_CENTER|HDF_IMAGE;
-    hdiCreate.iImage = 0;
-    addReadDelItem(hWndHeader, &hdiCreate, HDI_FORMAT, &hdiRead);
-    ok(hdiRead.fmt == (HDF_CENTER|HDF_IMAGE), "HDF_IMAGE shouldn't be cleared automatically (fmt=%x)\n", hdiRead.fmt);
+    item.mask = HDI_WIDTH | HDI_FORMAT;
+    item.fmt = HDF_CENTER | HDF_IMAGE;
+    item.iImage = 0;
+    header_item_getback(hwnd, HDI_FORMAT, &item);
+    ok(item.fmt == (HDF_CENTER | HDF_IMAGE), "Unexpected item format mask %#x.\n", item.fmt);
+
+    DestroyWindow(hwnd);
 }
 
 static void check_auto_fields(void)
@@ -706,25 +734,25 @@ static void test_header_control (void)
     {
         TEST_GET_ITEMCOUNT(3-i);
         res = addItem(hWndHeader, 0, str_items[i]);
-        ok(res == 0, "Adding simple item failed (%d)\n", res);
+        ok(res == 0, "Adding simple item failed (%ld)\n", res);
     }
 
     TEST_GET_ITEMCOUNT(4);
     res = addItem(hWndHeader, 99, str_items[i+1]);
-    ok(res != -1, "Adding Out of Range item should fail with -1 got (%d)\n", res);
+    ok(res != -1, "Adding Out of Range item should fail with -1 got (%ld)\n", res);
     TEST_GET_ITEMCOUNT(5);
     res = addItem(hWndHeader, 5, str_items[i+1]);
-    ok(res != -1, "Adding Out of Range item should fail with -1 got (%d)\n", res);
+    ok(res != -1, "Adding Out of Range item should fail with -1 got (%ld)\n", res);
     TEST_GET_ITEMCOUNT(6);
 
     for (i = 0; i < 4; i++) { TEST_GET_ITEM(i,i); TEST_GET_ITEMCOUNT(6); }
 
     res=getItem(hWndHeader, 99, buffer);
-    ok(res == 0, "Getting Out of Range item should fail with 0 (%d), got %s\n", res,buffer);
+    ok(res == 0, "Getting Out of Range item should fail with 0 (%ld), got %s\n", res,buffer);
     res=getItem(hWndHeader, 5, buffer);
-    ok(res == 1, "Getting Out of Range item should fail with 1 (%d), got %s\n", res,buffer);
+    ok(res == 1, "Getting Out of Range item should fail with 1 (%ld), got %s\n", res,buffer);
     res=getItem(hWndHeader, -2, buffer);
-    ok(res == 0, "Getting Out of Range item should fail with 0 (%d), got %s\n", res,buffer);
+    ok(res == 0, "Getting Out of Range item should fail with 0 (%ld), got %s\n", res,buffer);
 
     if (winetest_interactive)
     {
@@ -734,17 +762,17 @@ static void test_header_control (void)
 
     TEST_GET_ITEMCOUNT(6);
     res=setItem(hWndHeader, 99, str_items[5], FALSE);
-    ok(res == 0, "Setting Out of Range item should fail with 0 (%d)\n", res);
+    ok(res == 0, "Setting Out of Range item should fail with 0 (%ld)\n", res);
     res=setItem(hWndHeader, 5, str_items[5], TRUE);
-    ok(res == 1, "Setting Out of Range item should fail with 1 (%d)\n", res);
+    ok(res == 1, "Setting Out of Range item should fail with 1 (%ld)\n", res);
     res=setItem(hWndHeader, -2, str_items[5], FALSE);
-    ok(res == 0, "Setting Out of Range item should fail with 0 (%d)\n", res);
+    ok(res == 0, "Setting Out of Range item should fail with 0 (%ld)\n", res);
     TEST_GET_ITEMCOUNT(6);
 
     for (i = 0; i < 4; i++)
     {
         res = setItem(hWndHeader, i, str_items[4], TRUE);
-        ok(res != 0, "Setting %d item failed (%d)\n", i+1, res);
+        ok(res != 0, "Setting %d item failed (%ld)\n", i+1, res);
         TEST_GET_ITEM(i, 4);
         TEST_GET_ITEMCOUNT(6);
     }
@@ -765,7 +793,6 @@ static void test_header_control (void)
     /* unexpected notifies cleared by notifies_received in setItem */
     delItem(hWndHeader, 0);
 
-    check_auto_format();
     TEST_GET_ITEMCOUNT(6);
     check_auto_fields();
     TEST_GET_ITEMCOUNT(6);
@@ -773,22 +800,22 @@ static void test_header_control (void)
     TEST_GET_ITEMCOUNT(6);
 
     res = delItem(hWndHeader, 5);
-    ok(res == 1, "Deleting Out of Range item should fail with 1 (%d)\n", res);
+    ok(res == 1, "Deleting Out of Range item should fail with 1 (%ld)\n", res);
     res = delItem(hWndHeader, -2);
-    ok(res == 0, "Deleting Out of Range item should fail with 0 (%d)\n", res);
+    ok(res == 0, "Deleting Out of Range item should fail with 0 (%ld)\n", res);
     TEST_GET_ITEMCOUNT(5);
 
     res = delItem(hWndHeader, 3);
-    ok(res != 0, "Deleting using out of range index failed (%d)\n", res);
+    ok(res != 0, "Deleting using out of range index failed (%ld)\n", res);
     TEST_GET_ITEMCOUNT(4);
     res = delItem(hWndHeader, 0);
-    ok(res != 0, "Deleting using out of range index failed (%d)\n", res);
+    ok(res != 0, "Deleting using out of range index failed (%ld)\n", res);
     TEST_GET_ITEMCOUNT(3);
     res = delItem(hWndHeader, 0);
-    ok(res != 0, "Deleting using out of range index failed (%d)\n", res);
+    ok(res != 0, "Deleting using out of range index failed (%ld)\n", res);
     TEST_GET_ITEMCOUNT(2);
     res = delItem(hWndHeader, 0);
-    ok(res != 0, "Deleting using out of range index failed (%d)\n", res);
+    ok(res != 0, "Deleting using out of range index failed (%ld)\n", res);
     TEST_GET_ITEMCOUNT(1);
 
     DestroyWindow(hWndHeader);
@@ -953,7 +980,7 @@ static void test_hdm_sethotdivider(HWND hParent)
 
 static void test_hdm_imageMessages(HWND hParent)
 {
-    HIMAGELIST hImageList = ImageList_Create (4, 4, 0, 1, 0);
+    HIMAGELIST hImageList = pImageList_Create (4, 4, 0, 1, 0);
     HIMAGELIST hIml;
     BOOL wasValid;
     HWND hChild;
@@ -973,13 +1000,13 @@ static void test_hdm_imageMessages(HWND hParent)
 
     hIml = (HIMAGELIST) SendMessageA(hChild, HDM_CREATEDRAGIMAGE, 0, 0);
     ok(hIml != NULL, "Expected non-NULL handle, got %p\n", hIml);
-    ImageList_Destroy(hIml);
+    pImageList_Destroy(hIml);
 
     ok_sequence(sequences, HEADER_SEQ_INDEX, imageMessages_seq, "imageMessages sequence testing", FALSE);
 
     DestroyWindow(hChild);
 
-    wasValid = ImageList_Destroy(hImageList);
+    wasValid = pImageList_Destroy(hImageList);
     ok(wasValid, "Header must not free image list at destruction!\n");
 }
 
@@ -1107,7 +1134,7 @@ static void test_hdm_index_messages(HWND hParent)
          ok_sequence(sequences, PARENT_SEQ_INDEX, add_header_to_parent_seq,
                                      "adder header control to parent", FALSE);
     flush_sequences(sequences, NUM_MSG_SEQUENCES);
-    for (i = 0; i < sizeof(item_texts)/sizeof(item_texts[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(item_texts); i++)
     {
         hdItem.mask = HDI_TEXT | HDI_WIDTH | HDI_FORMAT;
         hdItem.pszText = (char*)item_texts[i];
@@ -1147,7 +1174,7 @@ static void test_hdm_index_messages(HWND hParent)
 
     hdItem.mask = HDI_TEXT | HDI_WIDTH;
     hdItem.pszText = buffA;
-    hdItem.cchTextMax = sizeof(buffA)/sizeof(buffA[0]);
+    hdItem.cchTextMax = ARRAY_SIZE(buffA);
     retVal = SendMessageA(hChild, HDM_GETITEMA, 0, (LPARAM) &hdItem);
     ok(retVal == TRUE, "Getting the 1st header item should return TRUE, got %d\n", retVal);
 
@@ -1343,13 +1370,13 @@ static void test_hds_nosizing(HWND hParent)
 }
 
 #define TEST_NMCUSTOMDRAW(draw_stage, item_spec, lparam, _left, _top, _right, _bottom) \
-    ok(nm->dwDrawStage == draw_stage, "Invalid dwDrawStage %d vs %d\n", draw_stage, nm->dwDrawStage); \
+    ok(nm->dwDrawStage == draw_stage, "Invalid dwDrawStage %d vs %ld\n", draw_stage, nm->dwDrawStage); \
     if (item_spec != -1) \
-        ok(nm->dwItemSpec == item_spec, "Invalid dwItemSpec %d vs %ld\n", item_spec, nm->dwItemSpec); \
-    ok(nm->lItemlParam == lparam, "Invalid lItemlParam %d vs %ld\n", lparam, nm->lItemlParam); \
+        ok(nm->dwItemSpec == item_spec, "Invalid dwItemSpec %d vs %Id\n", item_spec, nm->dwItemSpec); \
+    ok(nm->lItemlParam == lparam, "Invalid lItemlParam %d vs %Id\n", lparam, nm->lItemlParam); \
     ok((nm->rc.top == _top && nm->rc.bottom == _bottom && nm->rc.left == _left && nm->rc.right == _right) || \
         broken(draw_stage != CDDS_ITEMPREPAINT), /* comctl32 < 5.80 */ \
-        "Invalid rect (%d,%d)-(%d,%d) vs %s\n", _left, _top, _right, _bottom, \
+        "Invalid rect (%d,%d)-(%d,%ld) vs %s\n", _left, _top, _right, _bottom, \
         wine_dbgstr_rect(&nm->rc));
 
 static LRESULT customdraw_1(int n, NMCUSTOMDRAW *nm)
@@ -1367,7 +1394,7 @@ static LRESULT customdraw_1(int n, NMCUSTOMDRAW *nm)
         return 0;
     }
 
-    ok(FALSE, "Too many custom draw messages (n=%d, nm->dwDrawStage=%d)\n", n, nm->dwDrawStage);
+    ok(FALSE, "Too many custom draw messages (n=%d, nm->dwDrawStage=%ld)\n", n, nm->dwDrawStage);
     return -1;
 }
 
@@ -1394,7 +1421,7 @@ static LRESULT customdraw_2(int n, NMCUSTOMDRAW *nm)
         return 0;
     }
 
-    ok(FALSE, "Too many custom draw messages (n=%d, nm->dwDrawStage=%d)\n", n, nm->dwDrawStage);
+    ok(FALSE, "Too many custom draw messages (n=%d, nm->dwDrawStage=%ld)\n", n, nm->dwDrawStage);
     return 0;
 }
 
@@ -1424,7 +1451,7 @@ static LRESULT customdraw_3(int n, NMCUSTOMDRAW *nm)
         return 0;
     }
 
-    ok(FALSE, "Too many custom draw messages (n=%d, nm->dwDrawStage=%d)\n", n, nm->dwDrawStage);
+    ok(FALSE, "Too many custom draw messages (n=%d, nm->dwDrawStage=%ld)\n", n, nm->dwDrawStage);
     return 0;
 }
 
@@ -1452,7 +1479,7 @@ static LRESULT customdraw_4(int n, NMCUSTOMDRAW *nm)
         return 0;
     }
 
-    ok(FALSE, "Too many custom draw messages (n=%d, nm->dwDrawStage=%d)\n", n, nm->dwDrawStage);
+    ok(FALSE, "Too many custom draw messages (n=%d, nm->dwDrawStage=%ld)\n", n, nm->dwDrawStage);
     return 0;
 }
 
@@ -1475,7 +1502,7 @@ static void test_customdraw(void)
     hWndHeader = create_header_control();
     GetClientRect(hWndHeader, &rect);
     ok(rect.right - rect.left == 670 && rect.bottom - rect.top == g_customheight,
-        "Tests will fail as header size is %dx%d instead of 670x%d\n",
+        "Tests will fail as header size is %ldx%ld instead of 670x%ld\n",
         rect.right - rect.left, rect.bottom - rect.top, g_customheight);
 
     for (i = 0; i < 3; i++)
@@ -1627,10 +1654,10 @@ static LRESULT CALLBACK HeaderTestWndProc(HWND hWnd, UINT msg, WPARAM wParam, LP
         compare(di->hwndItem,  g_DrawItem.hwndItem, "%p");
         compare(di->itemID,    g_DrawItem.itemID, "%d");
         compare(di->itemState, g_DrawItem.itemState, "%d");
-        compare(di->rcItem.left,   g_DrawItem.rcItem.left, "%d");
-        compare(di->rcItem.top,    g_DrawItem.rcItem.top, "%d");
-        compare(di->rcItem.right,  g_DrawItem.rcItem.right, "%d");
-        compare(di->rcItem.bottom, g_DrawItem.rcItem.bottom, "%d");
+        compare(di->rcItem.left,   g_DrawItem.rcItem.left, "%ld");
+        compare(di->rcItem.top,    g_DrawItem.rcItem.top, "%ld");
+        compare(di->rcItem.right,  g_DrawItem.rcItem.right, "%ld");
+        compare(di->rcItem.bottom, g_DrawItem.rcItem.bottom, "%ld");
         break;
 
     case WM_DESTROY:
@@ -1644,28 +1671,23 @@ static LRESULT CALLBACK HeaderTestWndProc(HWND hWnd, UINT msg, WPARAM wParam, LP
     return 0L;
 }
 
+static void init_functions(void)
+{
+    HMODULE hComCtl32 = LoadLibraryA("comctl32.dll");
+
+#define X(f) p##f = (void*)GetProcAddress(hComCtl32, #f);
+    X(ImageList_Create);
+    X(ImageList_Destroy);
+#undef X
+}
+
 static BOOL init(void)
 {
-    HMODULE hComctl32;
-    BOOL (WINAPI *pInitCommonControlsEx)(const INITCOMMONCONTROLSEX*);
     WNDCLASSA wc;
-    INITCOMMONCONTROLSEX iccex;
     TEXTMETRICA tm;
     HFONT hOldFont;
     HDC hdc;
 
-    hComctl32 = GetModuleHandleA("comctl32.dll");
-    pInitCommonControlsEx = (void*)GetProcAddress(hComctl32, "InitCommonControlsEx");
-    if (!pInitCommonControlsEx)
-    {
-        win_skip("InitCommonControlsEx() is missing. Skipping the tests\n");
-        return FALSE;
-    }
-
-    iccex.dwSize = sizeof(iccex);
-    iccex.dwICC  = ICC_USEREX_CLASSES;
-    pInitCommonControlsEx(&iccex);
-
     wc.style = CS_HREDRAW | CS_VREDRAW;
     wc.cbClsExtra = 0;
     wc.cbWndExtra = 0;
@@ -1685,7 +1707,7 @@ static BOOL init(void)
     GetTextMetricsA(hdc, &tm);
     /* 2 dot extra space are needed for the border */
     g_customheight = tm.tmHeight + 2;
-    trace("customdraw height: %d (dpi: %d)\n", g_customheight, GetDeviceCaps(hdc, LOGPIXELSY));
+    trace("customdraw height: %ld (dpi: %d)\n", g_customheight, GetDeviceCaps(hdc, LOGPIXELSY));
     SelectObject(hdc, hOldFont);
     ReleaseDC(0, hdc);
 
@@ -1714,22 +1736,22 @@ static void check_orderarray(HWND hwnd, DWORD start, DWORD set, DWORD expected,
         order[i-1] = start>>(4*(count-i)) & 0xf;
 
     ret = SendMessageA(hwnd, HDM_SETORDERARRAY, count, (LPARAM)order);
-    ok_(__FILE__, line)(ret, "Expected HDM_SETORDERARRAY to succeed, got %d\n", ret);
+    ok_(__FILE__, line)(ret, "Expected HDM_SETORDERARRAY to succeed, got %ld\n", ret);
 
     /* new order */
     for(i = 1; i<=count; i++)
         order[i-1] = set>>(4*(count-i)) & 0xf;
     ret = SendMessageA(hwnd, HDM_SETORDERARRAY, count, (LPARAM)order);
-    ok_(__FILE__, line)(ret, "Expected HDM_SETORDERARRAY to succeed, got %d\n", ret);
+    ok_(__FILE__, line)(ret, "Expected HDM_SETORDERARRAY to succeed, got %ld\n", ret);
 
     /* check actual order */
     ret = SendMessageA(hwnd, HDM_GETORDERARRAY, count, (LPARAM)order);
-    ok_(__FILE__, line)(ret, "Expected HDM_GETORDERARRAY to succeed, got %d\n", ret);
+    ok_(__FILE__, line)(ret, "Expected HDM_GETORDERARRAY to succeed, got %ld\n", ret);
     for(i = 1; i<=count; i++)
         array |= order[i-1]<<(4*(count-i));
 
     todo_wine_if(todo)
-        ok_(__FILE__, line)(array == expected, "Expected %x, got %x\n", expected, array);
+        ok_(__FILE__, line)(array == expected, "Expected %lx, got %lx\n", expected, array);
 }
 
 static void test_hdm_orderarray(void)
@@ -1821,17 +1843,21 @@ START_TEST(header)
     ULONG_PTR ctx_cookie;
     HANDLE hCtx;
 
+    init_functions();
+    init_msg_sequences(sequences, NUM_MSG_SEQUENCES);
+
     if (!init())
         return;
 
     test_header_control();
+    test_item_auto_format(hHeaderParentWnd);
     test_header_order();
     test_hdm_orderarray();
     test_customdraw();
 
     DestroyWindow(hHeaderParentWnd);
 
-    init_msg_sequences(sequences, NUM_MSG_SEQUENCES);
+    flush_sequences(sequences, NUM_MSG_SEQUENCES);
     parent_hwnd = create_custom_parent_window();
     ok_sequence(sequences, PARENT_SEQ_INDEX, create_parent_wnd_seq, "create parent windows", FALSE);
 
@@ -1852,9 +1878,12 @@ START_TEST(header)
         return;
     }
 
+    init_functions();
+
     /* comctl32 version 6 tests start here */
     test_hdf_fixedwidth(parent_hwnd);
     test_hds_nosizing(parent_hwnd);
+    test_item_auto_format(parent_hwnd);
 
     unload_v6_module(ctx_cookie, hCtx);
 
