@@ -27,10 +27,9 @@
    perhaps with a different name? */
 #define D3DXERR_INVALIDDATA                      0x88760b59
 
-static HRESULT (WINAPI *pD3DAssemble)(const void *data, SIZE_T datasize, const char *filename,
-        const D3D_SHADER_MACRO *defines, ID3DInclude *include, UINT flags, ID3DBlob **shader,
-        ID3DBlob **error_messages);
-static pD3DPreprocess ppD3DPreprocess;
+HRESULT WINAPI D3DAssemble(const void *data, SIZE_T datasize, const char *filename,
+        const D3D_SHADER_MACRO *defines, ID3DInclude *include, UINT flags,
+        ID3DBlob **shader, ID3DBlob **error_messages);
 
 struct shader_test {
     const char *text;
@@ -40,7 +39,7 @@ struct shader_test {
 static void dump_shader(DWORD *shader) {
     unsigned int i = 0, j = 0;
     do {
-        trace("0x%08x ", shader[i]);
+        trace("0x%08lx ", shader[i]);
         j++;
         i++;
         if(j == 6) trace("\n");
@@ -58,10 +57,9 @@ static void exec_tests(const char *name, struct shader_test tests[], unsigned in
     for(i = 0; i < count; i++) {
         /* D3DAssemble sets messages to 0 if there aren't error messages */
         messages = NULL;
-        hr = pD3DAssemble(tests[i].text, strlen(tests[i].text), NULL,
-                          NULL, NULL, D3DCOMPILE_SKIP_VALIDATION,
-                          &shader, &messages);
-        ok(hr == S_OK, "Test %s, shader %d: D3DAssemble failed with error 0x%x - %d\n", name, i, hr, hr & 0x0000FFFF);
+        hr = D3DAssemble(tests[i].text, strlen(tests[i].text), NULL, NULL,
+                NULL, D3DCOMPILE_SKIP_VALIDATION, &shader, &messages);
+        ok(hr == S_OK, "Test %s, shader %u: D3DAssemble failed with error %#lx - %ld.\n", name, i, hr, hr & 0xffff);
         if(messages) {
             trace("D3DAssemble messages:\n%s", (char *)ID3D10Blob_GetBufferPointer(messages));
             ID3D10Blob_Release(messages);
@@ -213,7 +211,7 @@ static void vs_1_1_test(void) {
             "vs_1_1\n"
             "def c12, 0, -1, -0.5, 1024\n",
             {0xfffe0101, 0x00000051, 0xa00f000c, 0x00000000, 0xbf800000, 0xbf000000,
-             0x44800000, 0x0000ffff}
+             0x44800000, 0x0000ffff},
         },
         {   /* shader 14: writemasks, swizzles */
             "vs_1_1\n"
@@ -757,11 +755,11 @@ static void vs_2_0_test(void) {
         },
         {   /* shader 22 */
             "vs_2_0\n"
-            "defi i0, -1, 1, 10, 0\n"
+            "defi i0, - 1, 1, 10, 0\n"
             "defi i1, 0, 40, 30, 10\n",
             {0xfffe0200, 0x05000030, 0xf00f0000, 0xffffffff, 0x00000001, 0x0000000a,
              0x00000000, 0x05000030, 0xf00f0001, 0x00000000, 0x00000028, 0x0000001e,
-             0x0000000a, 0x0000ffff}
+             0x0000000a, 0x0000ffff},
         },
         {   /* shader 23 */
             "vs_2_0\n"
@@ -973,7 +971,7 @@ static void ps_2_x_test(void) {
             "defi i1, 0, 40, 30, 10\n",
             {0xffff0201, 0x05000030, 0xf00f0000, 0xffffffff, 0x00000001, 0x0000000a,
              0x00000000, 0x05000030, 0xf00f0001, 0x00000000, 0x00000028, 0x0000001e,
-             0x0000000a, 0x0000ffff}
+             0x0000000a, 0x0000ffff},
         },
         {   /* shader 2 */
             "ps_2_x\n"
@@ -1174,7 +1172,12 @@ static void vs_3_0_test(void) {
             {0xfffe0300, 0x04000002, 0x800f0000, 0x90e42014, 0xf0e40800, 0x80e40002,
              0x0000ffff}
         },
-
+        {   /* shader 15 */
+            "vs.3.0\n"
+            "add r0, v0[aL + 1 + 3], r2\n",
+            {0xfffe0300, 0x04000002, 0x800f0000, 0x90e42004, 0xf0e40800, 0x80e40002,
+             0x0000ffff}
+        },
     };
 
     exec_tests("vs_3_0", tests, ARRAY_SIZE(tests));
@@ -1405,6 +1408,20 @@ static void failure_test(void) {
         /* shader 43: */
         "ps_2_0\n"
         "texm3x3vspec t3, t0\n",
+        /* shader 44: expression in defi not allowed */
+        "vs_2_0\n"
+        "defi i0, -1 - 1, 1, 10, 0\n"
+        "defi i1, 0, 40, 30, 10\n",
+        /* shader 45: '-' not allowed inside relative addressing operands */
+        "vs.3.0\n"
+        "add r0, v0[aL - 3 + 5], r2\n",
+        /* shader 46: float constants in defi */
+        "vs_2_0\n"
+        "defi i0, 1.0, 1.1, 10.2, 0.3\n"
+        "defi i1, 0, 40, 30, 10\n",
+        /* shader 47: double '-' sign */
+        "vs.3.0\n"
+        "defi c0, -1, --1, 0, 0\n",
     };
     HRESULT hr;
     unsigned int i;
@@ -1414,13 +1431,10 @@ static void failure_test(void) {
     {
         shader = NULL;
         messages = NULL;
-        hr = pD3DAssemble(tests[i], strlen(tests[i]), NULL,
-                          NULL, NULL, D3DCOMPILE_SKIP_VALIDATION,
-                          &shader, &messages);
-        ok(hr == D3DXERR_INVALIDDATA, "Failure test, shader %d: "
-           "expected D3DAssemble failure with D3DXERR_INVALIDDATA, "
-           "got 0x%x - %d\n", i, hr, hr & 0x0000FFFF);
-        if(messages) {
+        hr = D3DAssemble(tests[i], strlen(tests[i]), NULL, NULL, NULL, D3DCOMPILE_SKIP_VALIDATION, &shader, &messages);
+        ok(hr == D3DXERR_INVALIDDATA, "Test %u: Got unexpected hr %#lx.\n", i, hr);
+        if (messages)
+        {
             trace("D3DAssemble messages:\n%s", (char *)ID3D10Blob_GetBufferPointer(messages));
             ID3D10Blob_Release(messages);
         }
@@ -1542,11 +1556,10 @@ static void assembleshader_test(void) {
     /* defines test */
     shader = NULL;
     messages = NULL;
-    hr = pD3DAssemble(test1, strlen(test1), NULL,
-                      defines, NULL, D3DCOMPILE_SKIP_VALIDATION,
-                      &shader, &messages);
-    ok(hr == S_OK, "defines test failed with error 0x%x - %d\n", hr, hr & 0x0000FFFF);
-    if(messages) {
+    hr = D3DAssemble(test1, strlen(test1), NULL, defines, NULL, D3DCOMPILE_SKIP_VALIDATION, &shader, &messages);
+    ok(hr == S_OK, "Got unexpected hr %#lx.\n", hr);
+    if (messages)
+    {
         trace("D3DAssemble messages:\n%s", (char *)ID3D10Blob_GetBufferPointer(messages));
         ID3D10Blob_Release(messages);
     }
@@ -1554,19 +1567,17 @@ static void assembleshader_test(void) {
 
     /* NULL messages test */
     shader = NULL;
-    hr = pD3DAssemble(test1, strlen(test1), NULL,
-                      defines, NULL, D3DCOMPILE_SKIP_VALIDATION,
-                      &shader, NULL);
-    ok(hr == S_OK, "NULL messages test failed with error 0x%x - %d\n", hr, hr & 0x0000FFFF);
-    if(shader) ID3D10Blob_Release(shader);
+    hr = D3DAssemble(test1, strlen(test1), NULL, defines, NULL, D3DCOMPILE_SKIP_VALIDATION, &shader, NULL);
+    ok(hr == S_OK, "Got unexpected hr %#lx.\n", hr);
+    if (shader)
+        ID3D10Blob_Release(shader);
 
     /* NULL shader test */
     messages = NULL;
-    hr = pD3DAssemble(test1, strlen(test1), NULL,
-                      defines, NULL, D3DCOMPILE_SKIP_VALIDATION,
-                      NULL, &messages);
-    ok(hr == S_OK, "NULL shader test failed with error 0x%x - %d\n", hr, hr & 0x0000FFFF);
-    if(messages) {
+    hr = D3DAssemble(test1, strlen(test1), NULL, defines, NULL, D3DCOMPILE_SKIP_VALIDATION, NULL, &messages);
+    ok(hr == S_OK, "Got unexpected hr %#lx.\n", hr);
+    if (messages)
+    {
         trace("D3DAssemble messages:\n%s", (char *)ID3D10Blob_GetBufferPointer(messages));
         ID3D10Blob_Release(messages);
     }
@@ -1575,10 +1586,11 @@ static void assembleshader_test(void) {
     shader = NULL;
     messages = NULL;
     include.ID3DInclude_iface.lpVtbl = &D3DInclude_Vtbl;
-    hr = pD3DAssemble(testshader, strlen(testshader), NULL, NULL, &include.ID3DInclude_iface,
-                      D3DCOMPILE_SKIP_VALIDATION, &shader, &messages);
-    ok(hr == S_OK, "D3DInclude test failed with error 0x%x - %d\n", hr, hr & 0x0000FFFF);
-    if(messages) {
+    hr = D3DAssemble(testshader, strlen(testshader), NULL, NULL,
+            &include.ID3DInclude_iface, D3DCOMPILE_SKIP_VALIDATION, &shader, &messages);
+    ok(hr == S_OK, "Got unexpected hr %#lx.\n", hr);
+    if (messages)
+    {
         trace("D3DAssemble messages:\n%s", (char *)ID3D10Blob_GetBufferPointer(messages));
         ID3D10Blob_Release(messages);
     }
@@ -1587,11 +1599,10 @@ static void assembleshader_test(void) {
     /* NULL shader tests */
     shader = NULL;
     messages = NULL;
-    hr = pD3DAssemble(NULL, 0, NULL,
-                      NULL, NULL, D3DCOMPILE_SKIP_VALIDATION,
-                      &shader, &messages);
-    ok(hr == D3DXERR_INVALIDDATA, "NULL shader test failed with error 0x%x - %d\n", hr, hr & 0x0000FFFF);
-    if(messages) {
+    hr = D3DAssemble(NULL, 0, NULL, NULL, NULL, D3DCOMPILE_SKIP_VALIDATION, &shader, &messages);
+    ok(hr == D3DXERR_INVALIDDATA, "Got unexpected hr %#lx.\n", hr);
+    if (messages)
+    {
         trace("D3DAssemble messages:\n%s", (char *)ID3D10Blob_GetBufferPointer(messages));
         ID3D10Blob_Release(messages);
     }
@@ -1649,9 +1660,8 @@ static void d3dpreprocess_test(void)
     /* pDefines test */
     shader = NULL;
     messages = NULL;
-    hr = ppD3DPreprocess(test1, strlen(test1), NULL,
-            defines, NULL, &shader, &messages);
-    ok(hr == S_OK, "pDefines test failed with error 0x%x - %d\n", hr, hr & 0x0000FFFF);
+    hr = D3DPreprocess(test1, strlen(test1), NULL, defines, NULL, &shader, &messages);
+    ok(hr == S_OK, "Got unexpected hr %#lx.\n", hr);
     if (messages)
     {
         trace("D3DPreprocess messages:\n%s", (char *)ID3D10Blob_GetBufferPointer(messages));
@@ -1661,16 +1671,15 @@ static void d3dpreprocess_test(void)
 
     /* NULL messages test */
     shader = NULL;
-    hr = ppD3DPreprocess(test1, strlen(test1), NULL,
-            defines, NULL, &shader, NULL);
-    ok(hr == S_OK, "NULL messages test failed with error 0x%x - %d\n", hr, hr & 0x0000FFFF);
-    if (shader) ID3D10Blob_Release(shader);
+    hr = D3DPreprocess(test1, strlen(test1), NULL, defines, NULL, &shader, NULL);
+    ok(hr == S_OK, "Got unexpected hr %#lx.\n", hr);
+    if (shader)
+        ID3D10Blob_Release(shader);
 
     /* NULL shader test */
     messages = NULL;
-    hr = ppD3DPreprocess(test1, strlen(test1), NULL,
-            defines, NULL, NULL, &messages);
-    ok(hr == E_INVALIDARG, "NULL shader test failed with error 0x%x - %d\n", hr, hr & 0x0000FFFF);
+    hr = D3DPreprocess(test1, strlen(test1), NULL, defines, NULL, NULL, &messages);
+    todo_wine ok(hr == E_INVALIDARG, "Got unexpected hr %#lx.\n", hr);
     if (messages)
     {
         trace("D3DPreprocess messages:\n%s", (char *)ID3D10Blob_GetBufferPointer(messages));
@@ -1680,9 +1689,8 @@ static void d3dpreprocess_test(void)
     /* quotation marks test */
     shader = NULL;
     messages = NULL;
-    hr = ppD3DPreprocess(quotation_marks_test, strlen(quotation_marks_test), NULL,
-            NULL, NULL, &shader, &messages);
-    todo_wine ok(hr == S_OK, "quotation marks test failed with error 0x%x - %d\n", hr, hr & 0x0000FFFF);
+    hr = D3DPreprocess(quotation_marks_test, strlen(quotation_marks_test), NULL, NULL, NULL, &shader, &messages);
+    ok(hr == S_OK, "Got unexpected hr %#lx.\n", hr);
     if (messages)
     {
         trace("D3DPreprocess messages:\n%s", (char *)ID3D10Blob_GetBufferPointer(messages));
@@ -1696,9 +1704,9 @@ static void d3dpreprocess_test(void)
     {
         shader = NULL;
         messages = NULL;
-        hr = ppD3DPreprocess(include_test_shaders[i], strlen(include_test_shaders[i]), NULL, NULL,
+        hr = D3DPreprocess(include_test_shaders[i], strlen(include_test_shaders[i]), NULL, NULL,
                 &include.ID3DInclude_iface, &shader, &messages);
-        ok(hr == S_OK, "pInclude test failed with error 0x%x - %d\n", hr, hr & 0x0000FFFF);
+        ok(hr == S_OK, "Test %u: Got unexpected hr %#lx.\n", i, hr);
         if (messages)
         {
             trace("D3DPreprocess messages:\n%s", (char *)ID3D10Blob_GetBufferPointer(messages));
@@ -1710,9 +1718,8 @@ static void d3dpreprocess_test(void)
     /* NULL shader tests */
     shader = NULL;
     messages = NULL;
-    hr = ppD3DPreprocess(NULL, 0, NULL,
-            NULL, NULL, &shader, &messages);
-    ok(hr == E_INVALIDARG, "NULL shader test failed with error 0x%x - %d\n", hr, hr & 0x0000FFFF);
+    hr = D3DPreprocess(NULL, 0, NULL, NULL, NULL, &shader, &messages);
+    ok(hr == E_INVALIDARG, "Got unexpected hr %#lx.\n", hr);
     if (messages)
     {
         trace("D3DPreprocess messages:\n%s", (char *)ID3D10Blob_GetBufferPointer(messages));
@@ -1721,29 +1728,40 @@ static void d3dpreprocess_test(void)
     if (shader) ID3D10Blob_Release(shader);
 }
 
-static BOOL load_d3dcompiler(void)
+static const DWORD vs_2_0[] =
 {
-    HMODULE module;
+    0xfffe0200,                         /* vs_2_0 */
+    0x0200001f, 0x80000000, 0x900f0000, /* dcl_position v0 */
+    0x0200001f, 0x80000003, 0x900f0001, /* dcl_normal v1 */
+    0x0200001f, 0x8001000a, 0x900f0002, /* dcl_color1 v2 */
+    0x0200001f, 0x80000005, 0x900f0003, /* dcl_texcoord0 v3 */
+    0x02000001, 0xc00f0000, 0x90e40000, /* mov oPos, v0 */
+    0x02000001, 0xd00f0001, 0x90e40002, /* mov oD1, v2 */
+    0x02000001, 0xe0070000, 0x90e40003, /* mov oT0.xyz, v3 */
+    0x02000001, 0xc00f0001, 0x90ff0002, /* mov oFog, v2.w */
+    0x02000001, 0xc00f0002, 0x90ff0001, /* mov oPts, v1.w */
+    0x0000ffff
+};
 
-#if D3D_COMPILER_VERSION == 47
-    if (!(module = LoadLibraryA("d3dcompiler_47.dll"))) return FALSE;
+static void test_disassemble_shader(void)
+{
+    ID3DBlob *blob;
+    HRESULT hr;
+
+    hr = D3DDisassemble(vs_2_0, 0, 0, NULL, &blob);
+#if D3D_COMPILER_VERSION >= 46
+    ok(hr == E_INVALIDARG, "Got unexpected hr %#lx.\n", hr);
 #else
-    if (!(module = LoadLibraryA("d3dcompiler_43.dll"))) return FALSE;
+    ok(hr == E_FAIL, "Got unexpected hr %#lx.\n", hr);
 #endif
 
-    pD3DAssemble = (void*)GetProcAddress(module, "D3DAssemble");
-    ppD3DPreprocess = (void*)GetProcAddress(module, "D3DPreprocess");
-    return TRUE;
+    hr = D3DDisassemble(vs_2_0, sizeof(vs_2_0), 0, NULL, &blob);
+    ok(hr == S_OK, "Got unexpected hr %#lx.\n", hr);
+    ID3D10Blob_Release(blob);
 }
 
 START_TEST(asm)
 {
-    if (!load_d3dcompiler())
-    {
-        win_skip("Could not load DLL.\n");
-        return;
-    }
-
     preproc_test();
     ps_1_1_test();
     vs_1_1_test();
@@ -1761,4 +1779,5 @@ START_TEST(asm)
     assembleshader_test();
 
     d3dpreprocess_test();
+    test_disassemble_shader();
 }
