@@ -147,7 +147,7 @@ HRESULT WINAPI D3DXFindShaderComment(const DWORD *byte_code, DWORD fourcc, const
     const DWORD *ptr = byte_code;
     DWORD version;
 
-    TRACE("byte_code %p, fourcc %x, data %p, size %p\n", byte_code, fourcc, data, size);
+    TRACE("byte_code %p, fourcc %#lx, data %p, size %p.\n", byte_code, fourcc, data, size);
 
     if (data) *data = NULL;
     if (size) *size = 0;
@@ -197,7 +197,7 @@ HRESULT WINAPI D3DXAssembleShader(const char *data, UINT data_len, const D3DXMAC
 {
     HRESULT hr;
 
-    TRACE("data %p, data_len %u, defines %p, include %p, flags %#x, shader %p, error_messages %p\n",
+    TRACE("data %p, data_len %u, defines %p, include %p, flags %#lx, shader %p, error_messages %p.\n",
           data, data_len, defines, include, flags, shader, error_messages);
 
     /* Forward to d3dcompiler: the parameter types aren't really different,
@@ -281,7 +281,7 @@ static HRESULT WINAPI d3dx_include_from_file_open(ID3DXInclude *iface, D3DXINCLU
     if(!buffer)
         goto error;
     *buffer = pathname;
-    if(!ReadFile(file, buffer + 1, size, bytes, NULL))
+    if (!ReadFile(file, buffer + 1, size, (DWORD *)bytes, NULL))
         goto error;
 
     *data = buffer + 1;
@@ -320,7 +320,7 @@ HRESULT WINAPI D3DXAssembleShaderFromFileA(const char *filename, const D3DXMACRO
     DWORD len;
     HRESULT ret;
 
-    TRACE("filename %s, defines %p, include %p, flags %#x, shader %p, error_messages %p.\n",
+    TRACE("filename %s, defines %p, include %p, flags %#lx, shader %p, error_messages %p.\n",
             debugstr_a(filename), defines, include, flags, shader, error_messages);
 
     if (!filename) return D3DXERR_INVALIDDATA;
@@ -339,13 +339,13 @@ HRESULT WINAPI D3DXAssembleShaderFromFileA(const char *filename, const D3DXMACRO
 HRESULT WINAPI D3DXAssembleShaderFromFileW(const WCHAR *filename, const D3DXMACRO *defines,
         ID3DXInclude *include, DWORD flags, ID3DXBuffer **shader, ID3DXBuffer **error_messages)
 {
-    const void *buffer;
-    DWORD len;
-    HRESULT hr;
     struct d3dx_include_from_file include_from_file;
+    const void *buffer;
+    unsigned int len;
     char *filename_a;
+    HRESULT hr;
 
-    TRACE("filename %s, defines %p, include %p, flags %#x, shader %p, error_messages %p.\n",
+    TRACE("filename %s, defines %p, include %p, flags %#lx, shader %p, error_messages %p.\n",
             debugstr_w(filename), defines, include, flags, shader, error_messages);
 
     if(!include)
@@ -384,7 +384,7 @@ HRESULT WINAPI D3DXAssembleShaderFromResourceA(HMODULE module, const char *resou
     HRSRC res;
     DWORD len;
 
-    TRACE("module %p, resource %s, defines %p, include %p, flags %#x, shader %p, error_messages %p.\n",
+    TRACE("module %p, resource %s, defines %p, include %p, flags %#lx, shader %p, error_messages %p.\n",
             module, debugstr_a(resource), defines, include, flags, shader, error_messages);
 
     if (!(res = FindResourceA(module, resource, (const char *)RT_RCDATA)))
@@ -402,7 +402,7 @@ HRESULT WINAPI D3DXAssembleShaderFromResourceW(HMODULE module, const WCHAR *reso
     HRSRC res;
     DWORD len;
 
-    TRACE("module %p, resource %s, defines %p, include %p, flags %#x, shader %p, error_messages %p.\n",
+    TRACE("module %p, resource %s, defines %p, include %p, flags %#lx, shader %p, error_messages %p.\n",
             module, debugstr_w(resource), defines, include, flags, shader, error_messages);
 
     if (!(res = FindResourceW(module, resource, (const WCHAR *)RT_RCDATA)))
@@ -420,7 +420,7 @@ HRESULT WINAPI D3DXCompileShader(const char *data, UINT length, const D3DXMACRO
     HRESULT hr;
 
     TRACE("data %s, length %u, defines %p, include %p, function %s, profile %s, "
-            "flags %#x, shader %p, error_msgs %p, constant_table %p.\n",
+            "flags %#lx, shader %p, error_msgs %p, constant_table %p.\n",
             debugstr_a(data), length, defines, include, debugstr_a(function), debugstr_a(profile),
             flags, shader, error_msgs, constant_table);
 
@@ -452,7 +452,7 @@ HRESULT WINAPI D3DXCompileShaderFromFileA(const char *filename, const D3DXMACRO
     HRESULT ret;
 
     TRACE("filename %s, defines %p, include %p, entrypoint %s, profile %s, "
-            "flags %#x, shader %p, error_messages %p, constant_table %p.\n",
+            "flags %#lx, shader %p, error_messages %p, constant_table %p.\n",
             debugstr_a(filename), defines, include, debugstr_a(entrypoint),
             debugstr_a(profile), flags, shader, error_messages, constant_table);
 
@@ -475,14 +475,15 @@ HRESULT WINAPI D3DXCompileShaderFromFileW(const WCHAR *filename, const D3DXMACRO
         ID3DXInclude *include, const char *entrypoint, const char *profile, DWORD flags,
         ID3DXBuffer **shader, ID3DXBuffer **error_messages, ID3DXConstantTable **constant_table)
 {
-    const void *buffer;
-    DWORD len, filename_len;
-    HRESULT hr;
     struct d3dx_include_from_file include_from_file;
+    unsigned int filename_len;
+    const void *buffer;
     char *filename_a;
+    HRESULT hr;
+    UINT len;
 
     TRACE("filename %s, defines %p, include %p, entrypoint %s, profile %s, "
-            "flags %#x, shader %p, error_messages %p, constant_table %p.\n",
+            "flags %#lx, shader %p, error_messages %p, constant_table %p.\n",
             debugstr_w(filename), defines, include, debugstr_a(entrypoint), debugstr_a(profile),
             flags, shader, error_messages, constant_table);
 
@@ -533,7 +534,7 @@ HRESULT WINAPI D3DXCompileShaderFromResourceA(HMODULE module, const char *resour
     DWORD len;
 
     TRACE("module %p, resource %s, defines %p, include %p, entrypoint %s, profile %s, "
-            "flags %#x, shader %p, error_messages %p, constant_table %p.\n",
+            "flags %#lx, shader %p, error_messages %p, constant_table %p.\n",
             module, debugstr_a(resource), defines, include, debugstr_a(entrypoint), debugstr_a(profile),
             flags, shader, error_messages, constant_table);
 
@@ -554,7 +555,7 @@ HRESULT WINAPI D3DXCompileShaderFromResourceW(HMODULE module, const WCHAR *resou
     DWORD len;
 
     TRACE("module %p, resource %s, defines %p, include %p, entrypoint %s, profile %s, "
-            "flags %#x, shader %p, error_messages %p, constant_table %p.\n",
+            "flags %#lx, shader %p, error_messages %p, constant_table %p.\n",
             module, debugstr_w(resource), defines, include, debugstr_a(entrypoint), debugstr_a(profile),
             flags, shader, error_messages, constant_table);
 
@@ -603,11 +604,11 @@ HRESULT WINAPI D3DXPreprocessShaderFromFileA(const char *filename, const D3DXMAC
 HRESULT WINAPI D3DXPreprocessShaderFromFileW(const WCHAR *filename, const D3DXMACRO *defines,
         ID3DXInclude *include, ID3DXBuffer **shader, ID3DXBuffer **error_messages)
 {
-    const void *buffer;
-    DWORD len;
-    HRESULT hr;
     struct d3dx_include_from_file include_from_file;
+    const void *buffer;
     char *filename_a;
+    HRESULT hr;
+    UINT len;
 
     TRACE("filename %s, defines %p, include %p, shader %p, error_messages %p.\n",
             debugstr_w(filename), defines, include, shader, error_messages);
@@ -681,11 +682,13 @@ HRESULT WINAPI D3DXPreprocessShaderFromResourceW(HMODULE module, const WCHAR *re
 
 }
 
-struct ID3DXConstantTableImpl {
+struct ID3DXConstantTableImpl
+{
     ID3DXConstantTable ID3DXConstantTable_iface;
     LONG ref;
     char *ctab;
     DWORD size;
+    DWORD flags;
     D3DXCONSTANTTABLE_DESC desc;
     struct ctab_constant *constants;
 };
@@ -860,6 +863,9 @@ static inline struct ctab_constant *get_valid_constant(struct ID3DXConstantTable
         if (c) return c;
     }
 
+    if (table->flags & D3DXCONSTTABLE_LARGEADDRESSAWARE)
+        return NULL;
+
     return get_constant_by_name(table, NULL, handle);
 }
 
@@ -884,27 +890,28 @@ static HRESULT WINAPI ID3DXConstantTableImpl_QueryInterface(ID3DXConstantTable *
 
 static ULONG WINAPI ID3DXConstantTableImpl_AddRef(ID3DXConstantTable *iface)
 {
-    struct ID3DXConstantTableImpl *This = impl_from_ID3DXConstantTable(iface);
+    struct ID3DXConstantTableImpl *table = impl_from_ID3DXConstantTable(iface);
+    ULONG refcount = InterlockedIncrement(&table->ref);
 
-    TRACE("(%p)->(): AddRef from %d\n", This, This->ref);
+    TRACE("%p increasing refcount to %lu.\n", table, refcount);
 
-    return InterlockedIncrement(&This->ref);
+    return refcount;
 }
 
 static ULONG WINAPI ID3DXConstantTableImpl_Release(ID3DXConstantTable *iface)
 {
-    struct ID3DXConstantTableImpl *This = impl_from_ID3DXConstantTable(iface);
-    ULONG ref = InterlockedDecrement(&This->ref);
+    struct ID3DXConstantTableImpl *table = impl_from_ID3DXConstantTable(iface);
+    ULONG refcount = InterlockedDecrement(&table->ref);
 
-    TRACE("(%p)->(): Release from %d\n", This, ref + 1);
+    TRACE("%p decreasing refcount to %lu.\n", table, refcount);
 
-    if (!ref)
+    if (!refcount)
     {
-        free_constant_table(This);
-        HeapFree(GetProcessHeap(), 0, This);
+        free_constant_table(table);
+        HeapFree(GetProcessHeap(), 0, table);
     }
 
-    return ref;
+    return refcount;
 }
 
 /*** ID3DXBuffer methods ***/
@@ -1112,7 +1119,6 @@ static UINT set(struct ID3DXConstantTableImpl *table, IDirect3DDevice9 *device,
             if (inclass == D3DXPC_MATRIX_ROWS && desc->RegisterSet == D3DXRS_BOOL)
             {
                 D3DXMATRIX mat, *m, min;
-                D3DXMatrixTranspose(&mat, &min);
 
                 if (is_pointer)
                     min = *(D3DXMATRIX *)(indata[index / 16]);
@@ -1960,9 +1966,10 @@ HRESULT WINAPI D3DXGetShaderConstantTableEx(const DWORD *byte_code, DWORD flags,
     const D3DXSHADER_CONSTANTINFO *constant_info;
     DWORD i;
 
-    TRACE("byte_code %p, flags %x, constant_table %p\n", byte_code, flags, constant_table);
+    TRACE("byte_code %p, flags %#lx, constant_table %p.\n", byte_code, flags, constant_table);
 
-    if (constant_table) *constant_table = NULL;
+    if (constant_table)
+        *constant_table = NULL;
 
     if (!byte_code || !constant_table)
     {
@@ -1976,7 +1983,8 @@ HRESULT WINAPI D3DXGetShaderConstantTableEx(const DWORD *byte_code, DWORD flags,
         return D3D_OK;
     }
 
-    if (flags) FIXME("Flags (%#x) are not handled, yet!\n", flags);
+    if (flags & ~D3DXCONSTTABLE_LARGEADDRESSAWARE)
+        FIXME("Flags %#lx not handled.\n", flags);
 
     hr = D3DXFindShaderComment(byte_code, MAKEFOURCC('C','T','A','B'), &data, &size);
     if (hr != D3D_OK)
@@ -2015,10 +2023,11 @@ HRESULT WINAPI D3DXGetShaderConstantTableEx(const DWORD *byte_code, DWORD flags,
     object->size = size;
     memcpy(object->ctab, data, object->size);
 
+    object->flags = flags;
     object->desc.Creator = ctab_header->Creator ? object->ctab + ctab_header->Creator : NULL;
     object->desc.Version = ctab_header->Version;
     object->desc.Constants = ctab_header->Constants;
-    TRACE("Creator %s, Version %x, Constants %u, Target %s\n",
+    TRACE("Creator %s, Version %#lx, Constants %u, Target %s.\n",
             debugstr_a(object->desc.Creator), object->desc.Version, object->desc.Constants,
             debugstr_a(ctab_header->Target ? object->ctab + ctab_header->Target : NULL));
 
@@ -2111,7 +2120,7 @@ static ULONG WINAPI d3dx9_fragment_linker_AddRef(ID3DXFragmentLinker *iface)
     struct d3dx9_fragment_linker *linker = impl_from_ID3DXFragmentLinker(iface);
     ULONG refcount = InterlockedIncrement(&linker->ref);
 
-    TRACE("%p increasing refcount to %u.\n", linker, refcount);
+    TRACE("%p increasing refcount to %lu.\n", linker, refcount);
 
     return refcount;
 }
@@ -2121,7 +2130,7 @@ static ULONG WINAPI d3dx9_fragment_linker_Release(ID3DXFragmentLinker *iface)
     struct d3dx9_fragment_linker *linker = impl_from_ID3DXFragmentLinker(iface);
     ULONG refcount = InterlockedDecrement(&linker->ref);
 
-    TRACE("%p decreasing refcount to %u.\n", linker, refcount);
+    TRACE("%p decreasing refcount to %lu.\n", linker, refcount);
 
     if (!refcount)
     {
@@ -2207,7 +2216,7 @@ static HRESULT WINAPI d3dx9_fragment_linker_LinkShader(ID3DXFragmentLinker *ifac
         DWORD flags, const D3DXHANDLE *handles, UINT fragment_count, ID3DXBuffer **buffer,
         ID3DXBuffer **errors)
 {
-    FIXME("iface %p, profile %s, flags %#x, handles %p, fragment_count %u, buffer %p, errors %p: stub.\n",
+    FIXME("iface %p, profile %s, flags %#lx, handles %p, fragment_count %u, buffer %p, errors %p: stub.\n",
             iface, debugstr_a(profile), flags, handles, fragment_count, buffer, errors);
 
     return E_NOTIMPL;
@@ -2217,7 +2226,7 @@ static HRESULT WINAPI d3dx9_fragment_linker_LinkVertexShader(ID3DXFragmentLinker
         DWORD flags, const D3DXHANDLE *handles, UINT fragment_count, IDirect3DVertexShader9 **shader,
         ID3DXBuffer **errors)
 {
-    FIXME("iface %p, profile %s, flags %#x, handles %p, fragment_count %u, shader %p, errors %p: stub.\n",
+    FIXME("iface %p, profile %s, flags %#lx, handles %p, fragment_count %u, shader %p, errors %p: stub.\n",
             iface, debugstr_a(profile), flags, handles, fragment_count, shader, errors);
 
     return E_NOTIMPL;
@@ -2227,7 +2236,7 @@ static HRESULT WINAPI d3dx9_fragment_linker_LinkPixelShader(ID3DXFragmentLinker
         DWORD flags, const D3DXHANDLE *handles, UINT fragment_count, IDirect3DPixelShader9 **shader,
         ID3DXBuffer **errors)
 {
-    FIXME("iface %p, profile %s, flags %#x, handles %p, fragment_count %u, shader %p, errors %p: stub.\n",
+    FIXME("iface %p, profile %s, flags %#lx, handles %p, fragment_count %u, shader %p, errors %p: stub.\n",
         iface, debugstr_a(profile), flags, handles, fragment_count, shader, errors);
 
     return E_NOTIMPL;
@@ -2264,7 +2273,7 @@ HRESULT WINAPI D3DXCreateFragmentLinkerEx(IDirect3DDevice9 *device, UINT size, D
 {
     struct d3dx9_fragment_linker *object;
 
-    TRACE("device %p, size %u, flags %#x, linker %p.\n", device, size, flags, linker);
+    TRACE("device %p, size %u, flags %#lx, linker %p.\n", device, size, flags, linker);
 
     object = heap_alloc(sizeof(*object));
     if (!object)
@@ -2337,16 +2346,21 @@ HRESULT WINAPI D3DXGetShaderSamplers(const DWORD *byte_code, const char **sample
     return D3D_OK;
 }
 
-HRESULT WINAPI D3DXDisassembleShader(const DWORD *shader, BOOL colorcode, const char *comments, ID3DXBuffer **disassembly)
+HRESULT WINAPI D3DXDisassembleShader(const DWORD *shader, BOOL colorcode, const char *comments,
+        ID3DXBuffer **buffer)
 {
-   FIXME("%p %d %s %p: stub\n", shader, colorcode, debugstr_a(comments), disassembly);
-   return E_OUTOFMEMORY;
+    TRACE("shader %p, colorcode %d, comments %s, buffer %p.\n", shader, colorcode, debugstr_a(comments), buffer);
+
+    return D3DDisassemble(shader, D3DXGetShaderSize(shader), colorcode ? D3D_DISASM_ENABLE_COLOR_CODE : 0,
+            comments, (ID3DBlob **)buffer);
 }
 
 struct d3dx9_texture_shader
 {
     ID3DXTextureShader ID3DXTextureShader_iface;
     LONG ref;
+
+    ID3DXBuffer *byte_code;
 };
 
 static inline struct d3dx9_texture_shader *impl_from_ID3DXTextureShader(ID3DXTextureShader *iface)
@@ -2376,7 +2390,7 @@ static ULONG WINAPI d3dx9_texture_shader_AddRef(ID3DXTextureShader *iface)
     struct d3dx9_texture_shader *texture_shader = impl_from_ID3DXTextureShader(iface);
     ULONG refcount = InterlockedIncrement(&texture_shader->ref);
 
-    TRACE("%p increasing refcount to %u.\n", texture_shader, refcount);
+    TRACE("%p increasing refcount to %lu.\n", texture_shader, refcount);
 
     return refcount;
 }
@@ -2386,10 +2400,12 @@ static ULONG WINAPI d3dx9_texture_shader_Release(ID3DXTextureShader *iface)
     struct d3dx9_texture_shader *texture_shader = impl_from_ID3DXTextureShader(iface);
     ULONG refcount = InterlockedDecrement(&texture_shader->ref);
 
-    TRACE("%p decreasing refcount to %u.\n", texture_shader, refcount);
+    TRACE("%p decreasing refcount to %lu.\n", texture_shader, refcount);
 
     if (!refcount)
     {
+        if (texture_shader->byte_code)
+            ID3DXBuffer_Release(texture_shader->byte_code);
         HeapFree(GetProcessHeap(), 0, texture_shader);
     }
 
@@ -2398,9 +2414,14 @@ static ULONG WINAPI d3dx9_texture_shader_Release(ID3DXTextureShader *iface)
 
 static HRESULT WINAPI d3dx9_texture_shader_GetFunction(ID3DXTextureShader *iface, struct ID3DXBuffer **function)
 {
-    FIXME("iface %p, function %p stub.\n", iface, function);
+    struct d3dx9_texture_shader *texture_shader = impl_from_ID3DXTextureShader(iface);
 
-    return E_NOTIMPL;
+    TRACE("iface %p, function %p.\n", iface, function);
+
+    *function = texture_shader->byte_code;
+    ID3DXBuffer_AddRef(*function);
+
+    return S_OK;
 }
 
 static HRESULT WINAPI d3dx9_texture_shader_GetConstantBuffer(ID3DXTextureShader *iface, struct ID3DXBuffer **constant_buffer)
@@ -2592,19 +2613,30 @@ static const struct ID3DXTextureShaderVtbl d3dx9_texture_shader_vtbl =
 HRESULT WINAPI D3DXCreateTextureShader(const DWORD *function, ID3DXTextureShader **texture_shader)
 {
     struct d3dx9_texture_shader *object;
+    unsigned int size;
+    HRESULT hr;
 
     TRACE("function %p, texture_shader %p.\n", function, texture_shader);
 
     if (!function || !texture_shader)
         return D3DERR_INVALIDCALL;
 
-    object = HeapAlloc(GetProcessHeap(), 0, sizeof(*object));
-    if (!object)
+    if (!(size = D3DXGetShaderSize(function)))
+        return D3DXERR_INVALIDDATA;
+
+    if (!(object = heap_alloc_zero(sizeof(*object))))
         return E_OUTOFMEMORY;
 
     object->ID3DXTextureShader_iface.lpVtbl = &d3dx9_texture_shader_vtbl;
     object->ref = 1;
 
+    if (FAILED(hr = D3DXCreateBuffer(size, &object->byte_code)))
+    {
+        IUnknown_Release(&object->ID3DXTextureShader_iface);
+        return hr;
+    }
+    memcpy(ID3DXBuffer_GetBufferPointer(object->byte_code), function, size);
+
     *texture_shader = &object->ID3DXTextureShader_iface;
 
     return D3D_OK;
@@ -2664,7 +2696,7 @@ static HRESULT get_shader_semantics(const DWORD *byte_code, D3DXSEMANTIC *semant
         D3DDECLUSAGE_FOG,
         D3DDECLUSAGE_PSIZE
     };
-    DWORD reg_type, usage, index, version_token = *byte_code;
+    uint32_t reg_type, usage, index, version_token = *byte_code;
     BOOL is_ps = version_token >> 16 == 0xffff;
     unsigned int major, minor, i = 0, j;
     BYTE colors = 0, rastout = 0;
@@ -2686,8 +2718,8 @@ static HRESULT get_shader_semantics(const DWORD *byte_code, D3DXSEMANTIC *semant
     {
         if (has_dcl && (*byte_code & 0xffff) == D3DSIO_DCL)
         {
-            DWORD usage_token = byte_code[1];
-            DWORD reg = byte_code[2];
+            uint32_t usage_token = byte_code[1];
+            uint32_t reg = byte_code[2];
 
             reg_type = ((reg & D3DSP_REGTYPE_MASK) >> D3DSP_REGTYPE_SHIFT)
                     | ((reg & D3DSP_REGTYPE_MASK2) >> D3DSP_REGTYPE_SHIFT2);
