@@ -66,7 +66,7 @@ static ULONG WINAPI d3d_vertex_buffer7_AddRef(IDirect3DVertexBuffer7 *iface)
     struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer7(iface);
     ULONG ref = InterlockedIncrement(&buffer->ref);
 
-    TRACE("%p increasing refcount to %u.\n", buffer, ref);
+    TRACE("%p increasing refcount to %lu.\n", buffer, ref);
 
     return ref;
 }
@@ -76,21 +76,17 @@ static ULONG WINAPI d3d_vertex_buffer7_Release(IDirect3DVertexBuffer7 *iface)
     struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer7(iface);
     ULONG ref = InterlockedDecrement(&buffer->ref);
 
-    TRACE("%p decreasing refcount to %u.\n", buffer, ref);
+    TRACE("%p decreasing refcount to %lu.\n", buffer, ref);
 
     if (!ref)
     {
-        struct wined3d_buffer *vb = NULL;
-        UINT offset, stride;
-
         /* D3D7 vertex buffers don't stay bound in the device, they are passed
          * as a parameter to DrawPrimitiveVB. DrawPrimitiveVB sets them as the
          * stream source in wined3d and they should get unset there before
          * they are destroyed. */
         wined3d_mutex_lock();
-        wined3d_device_get_stream_source(buffer->ddraw->wined3d_device, 0, &vb, &offset, &stride);
-        if (vb == buffer->wined3d_buffer)
-            wined3d_device_set_stream_source(buffer->ddraw->wined3d_device, 0, NULL, 0, 0);
+        if (buffer->ddraw->stateblock_state->streams[0].buffer == buffer->wined3d_buffer)
+            wined3d_stateblock_set_stream_source(buffer->ddraw->state, 0, NULL, 0, 0);
 
         wined3d_vertex_declaration_decref(buffer->wined3d_declaration);
         wined3d_buffer_decref(buffer->wined3d_buffer);
@@ -158,15 +154,19 @@ static HRESULT WINAPI d3d_vertex_buffer7_Lock(IDirect3DVertexBuffer7 *iface,
     struct wined3d_map_desc wined3d_map_desc;
     HRESULT hr;
 
-    TRACE("iface %p, flags %#x, data %p, data_size %p.\n", iface, flags, data, data_size);
+    TRACE("iface %p, flags %#lx, data %p, data_size %p.\n", iface, flags, data, data_size);
 
     if (buffer->version != 7)
         flags &= ~(DDLOCK_NOOVERWRITE | DDLOCK_DISCARDCONTENTS);
 
+    if (buffer->discarded)
+        flags &= ~DDLOCK_DISCARDCONTENTS;
+
     if (!(flags & DDLOCK_WAIT))
         flags |= DDLOCK_DONOTWAIT;
     if (flags & DDLOCK_DISCARDCONTENTS)
     {
+        buffer->discarded = true;
         if (!buffer->dynamic)
         {
             struct wined3d_buffer *new_buffer;
@@ -218,11 +218,7 @@ static HRESULT WINAPI d3d_vertex_buffer7_Unlock(IDirect3DVertexBuffer7 *iface)
     struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer7(iface);
 
     TRACE("iface %p.\n", iface);
-
-    wined3d_mutex_lock();
     wined3d_resource_unmap(wined3d_buffer_get_resource(buffer->wined3d_buffer), 0);
-    wined3d_mutex_unlock();
-
     return D3D_OK;
 }
 
@@ -258,9 +254,10 @@ static HRESULT WINAPI d3d_vertex_buffer7_ProcessVertices(IDirect3DVertexBuffer7
             ? unsafe_impl_from_IDirect3DDevice7(device)
             : unsafe_impl_from_IDirect3DDevice3((IDirect3DDevice3 *)device);
     BOOL old_clip, do_clip, old_lighting, do_lighting;
+    const struct wined3d_stateblock_state *state;
     HRESULT hr;
 
-    TRACE("iface %p, vertex_op %#x, dst_idx %u, count %u, src_buffer %p, src_idx %u, device %p, flags %#x.\n",
+    TRACE("iface %p, vertex_op %#lx, dst_idx %lu, count %lu, src_buffer %p, src_idx %lu, device %p, flags %#lx.\n",
             iface, vertex_op, dst_idx, count, src_buffer, src_idx, device, flags);
 
     /* Vertex operations:
@@ -278,16 +275,18 @@ static HRESULT WINAPI d3d_vertex_buffer7_ProcessVertices(IDirect3DVertexBuffer7
 
     wined3d_mutex_lock();
 
+    state = device_impl->stateblock_state;
+
     /* WineD3D doesn't know d3d7 vertex operation, it uses
      * render states instead. Set the render states according to
      * the vertex ops
      */
     do_clip = !!(vertex_op & D3DVOP_CLIP);
-    old_clip = !!wined3d_device_get_render_state(device_impl->wined3d_device, WINED3D_RS_CLIPPING);
+    old_clip = !!state->rs[WINED3D_RS_CLIPPING];
     if (do_clip != old_clip)
-        wined3d_device_set_render_state(device_impl->wined3d_device, WINED3D_RS_CLIPPING, do_clip);
+        wined3d_stateblock_set_render_state(device_impl->state, WINED3D_RS_CLIPPING, do_clip);
 
-    old_lighting = !!wined3d_device_get_render_state(device_impl->wined3d_device, WINED3D_RS_LIGHTING);
+    old_lighting = !!state->rs[WINED3D_RS_LIGHTING];
     if (dst_buffer_impl->version == 3)
         do_lighting = device_impl->material && (src_buffer_impl->fvf & D3DFVF_NORMAL)
                 && (vertex_op & D3DVOP_LIGHT);
@@ -295,19 +294,20 @@ static HRESULT WINAPI d3d_vertex_buffer7_ProcessVertices(IDirect3DVertexBuffer7
         do_lighting = old_lighting && (vertex_op & D3DVOP_LIGHT);
 
     if (do_lighting != old_lighting)
-        wined3d_device_set_render_state(device_impl->wined3d_device, WINED3D_RS_LIGHTING, do_lighting);
+        wined3d_stateblock_set_render_state(device_impl->state, WINED3D_RS_LIGHTING, do_lighting);
 
-    wined3d_device_set_stream_source(device_impl->wined3d_device,
+    wined3d_stateblock_set_stream_source(device_impl->state,
             0, src_buffer_impl->wined3d_buffer, 0, get_flexible_vertex_size(src_buffer_impl->fvf));
-    wined3d_device_set_vertex_declaration(device_impl->wined3d_device, src_buffer_impl->wined3d_declaration);
+    wined3d_stateblock_set_vertex_declaration(device_impl->state, src_buffer_impl->wined3d_declaration);
+    wined3d_device_apply_stateblock(device_impl->wined3d_device, device_impl->state);
     hr = wined3d_device_process_vertices(device_impl->wined3d_device, src_idx, dst_idx,
             count, dst_buffer_impl->wined3d_buffer, NULL, flags, dst_buffer_impl->fvf);
 
     /* Restore the states if needed */
     if (do_clip != old_clip)
-        wined3d_device_set_render_state(device_impl->wined3d_device, WINED3D_RS_CLIPPING, old_clip);
+        wined3d_stateblock_set_render_state(device_impl->state, WINED3D_RS_CLIPPING, old_clip);
     if (do_lighting != old_lighting)
-        wined3d_device_set_render_state(device_impl->wined3d_device, WINED3D_RS_LIGHTING, old_lighting);
+        wined3d_stateblock_set_render_state(device_impl->state, WINED3D_RS_LIGHTING, old_lighting);
 
     wined3d_mutex_unlock();
 
@@ -369,11 +369,11 @@ static HRESULT WINAPI d3d_vertex_buffer7_Optimize(IDirect3DVertexBuffer7 *iface,
     struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer7(iface);
     static BOOL hide = FALSE;
 
-    TRACE("iface %p, device %p, flags %#x.\n", iface, device, flags);
+    TRACE("iface %p, device %p, flags %#lx.\n", iface, device, flags);
 
     if (!hide)
     {
-        FIXME("iface %p, device %p, flags %#x stub!\n", iface, device, flags);
+        FIXME("iface %p, device %p, flags %#lx stub!\n", iface, device, flags);
         hide = TRUE;
     }
 
@@ -413,7 +413,7 @@ static HRESULT WINAPI d3d_vertex_buffer7_ProcessVerticesStrided(IDirect3DVertexB
         DWORD vertex_op, DWORD dst_idx, DWORD count, D3DDRAWPRIMITIVESTRIDEDDATA *data,
         DWORD fvf, IDirect3DDevice7 *device, DWORD flags)
 {
-    FIXME("iface %p, vertex_op %#x, dst_idx %u, count %u, data %p, fvf %#x, device %p, flags %#x stub!\n",
+    FIXME("iface %p, vertex_op %#lx, dst_idx %lu, count %lu, data %p, fvf %#lx, device %p, flags %#lx stub!\n",
             iface, vertex_op, dst_idx, count, data, fvf, device, flags);
 
     return DD_OK;
@@ -443,10 +443,10 @@ HRESULT d3d_vertex_buffer_create(struct d3d_vertex_buffer **vertex_buf,
     HRESULT hr = D3D_OK;
 
     TRACE("Vertex buffer description:\n");
-    TRACE("    dwSize %u\n", desc->dwSize);
-    TRACE("    dwCaps %#x\n", desc->dwCaps);
-    TRACE("    FVF %#x\n", desc->dwFVF);
-    TRACE("    dwNumVertices %u\n", desc->dwNumVertices);
+    TRACE("    dwSize %lu\n", desc->dwSize);
+    TRACE("    dwCaps %#lx\n", desc->dwCaps);
+    TRACE("    FVF %#lx\n", desc->dwFVF);
+    TRACE("    dwNumVertices %lu\n", desc->dwNumVertices);
 
     if (!(buffer = heap_alloc_zero(sizeof(*buffer))))
         return DDERR_OUTOFMEMORY;
@@ -465,7 +465,7 @@ HRESULT d3d_vertex_buffer_create(struct d3d_vertex_buffer **vertex_buf,
 
     if (FAILED(hr = d3d_vertex_buffer_create_wined3d_buffer(buffer, FALSE, &buffer->wined3d_buffer)))
     {
-        WARN("Failed to create wined3d vertex buffer, hr %#x.\n", hr);
+        WARN("Failed to create wined3d vertex buffer, hr %#lx.\n", hr);
         if (hr == WINED3DERR_INVALIDCALL)
             hr = DDERR_INVALIDPARAMS;
         goto end;
@@ -473,7 +473,7 @@ HRESULT d3d_vertex_buffer_create(struct d3d_vertex_buffer **vertex_buf,
 
     if (!(buffer->wined3d_declaration = ddraw_find_decl(ddraw, desc->dwFVF)))
     {
-        ERR("Failed to find vertex declaration for fvf %#x.\n", desc->dwFVF);
+        ERR("Failed to find vertex declaration for fvf %#lx.\n", desc->dwFVF);
         wined3d_buffer_decref(buffer->wined3d_buffer);
         hr = DDERR_INVALIDPARAMS;
         goto end;
