@@ -33,6 +33,8 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(gdiplus);
 
+#define SQRT3     1.73205080757
+
 typedef struct path_list_node_t path_list_node_t;
 struct path_list_node_t {
     GpPointF pt;
@@ -166,6 +168,52 @@ static BOOL flatten_bezier(path_list_node_t *start, REAL x2, REAL y2, REAL x3, R
     return TRUE;
 }
 
+/* GdipAddPath* helper
+ *
+ * Several GdipAddPath functions are expected to add onto an open figure.
+ * So if the first point being added is an exact match to the last point
+ * of the existing line, that point should not be added.
+ *
+ * Parameters:
+ *  path   : path to which points should be added
+ *  points : array of points to add
+ *  count  : number of points to add (at least 1)
+ *  type   : type of the points being added
+ *
+ * Return value:
+ *  OutOfMemory : out of memory, could not lengthen path
+ *  Ok : success
+ */
+static GpStatus extend_current_figure(GpPath *path, GDIPCONST PointF *points, INT count, BYTE type)
+{
+    INT insert_index = path->pathdata.Count;
+    BYTE first_point_type = (path->newfigure ? PathPointTypeStart : PathPointTypeLine);
+
+    if(!path->newfigure &&
+            path->pathdata.Points[insert_index-1].X == points[0].X &&
+            path->pathdata.Points[insert_index-1].Y == points[0].Y)
+    {
+        points++;
+        count--;
+        first_point_type = type;
+    }
+
+    if(!count)
+        return Ok;
+
+    if(!lengthen_path(path, count))
+        return OutOfMemory;
+
+    memcpy(path->pathdata.Points + insert_index, points, sizeof(GpPointF)*count);
+    path->pathdata.Types[insert_index] = first_point_type;
+    memset(path->pathdata.Types + insert_index + 1, type, count - 1);
+
+    path->newfigure = FALSE;
+    path->pathdata.Count += count;
+
+    return Ok;
+}
+
 /*******************************************************************************
  * GdipAddPathArc   [GDIPLUS.1]
  *
@@ -195,7 +243,9 @@ static BOOL flatten_bezier(path_list_node_t *start, REAL x2, REAL y2, REAL x3, R
 GpStatus WINGDIPAPI GdipAddPathArc(GpPath *path, REAL x1, REAL y1, REAL x2,
     REAL y2, REAL startAngle, REAL sweepAngle)
 {
-    INT count, old_count, i;
+    GpPointF *points;
+    GpStatus status;
+    INT count;
 
     TRACE("(%p, %.2f, %.2f, %.2f, %.2f, %.2f, %.2f)\n",
           path, x1, y1, x2, y2, startAngle, sweepAngle);
@@ -204,26 +254,19 @@ GpStatus WINGDIPAPI GdipAddPathArc(GpPath *path, REAL x1, REAL y1, REAL x2,
         return InvalidParameter;
 
     count = arc2polybezier(NULL, x1, y1, x2, y2, startAngle, sweepAngle);
-
     if(count == 0)
         return Ok;
-    if(!lengthen_path(path, count))
+
+    points = heap_alloc_zero(sizeof(GpPointF)*count);
+    if(!points)
         return OutOfMemory;
 
-    old_count = path->pathdata.Count;
-    arc2polybezier(&path->pathdata.Points[old_count], x1, y1, x2, y2,
-                   startAngle, sweepAngle);
+    arc2polybezier(points, x1, y1, x2, y2, startAngle, sweepAngle);
 
-    for(i = 0; i < count; i++){
-        path->pathdata.Types[old_count + i] = PathPointTypeBezier;
-    }
+    status = extend_current_figure(path, points, count, PathPointTypeBezier);
 
-    path->pathdata.Types[old_count] =
-        (path->newfigure ? PathPointTypeStart : PathPointTypeLine);
-    path->newfigure = FALSE;
-    path->pathdata.Count += count;
-
-    return Ok;
+    heap_free(points);
+    return status;
 }
 
 /*******************************************************************************
@@ -243,7 +286,7 @@ GpStatus WINGDIPAPI GdipAddPathArcI(GpPath *path, INT x1, INT y1, INT x2,
 GpStatus WINGDIPAPI GdipAddPathBezier(GpPath *path, REAL x1, REAL y1, REAL x2,
     REAL y2, REAL x3, REAL y3, REAL x4, REAL y4)
 {
-    INT old_count;
+    PointF points[4];
 
     TRACE("(%p, %.2f, %.2f, %.2f, %.2f, %.2f, %.2f, %.2f, %.2f)\n",
           path, x1, y1, x2, y2, x3, y3, x4, y4);
@@ -251,30 +294,16 @@ GpStatus WINGDIPAPI GdipAddPathBezier(GpPath *path, REAL x1, REAL y1, REAL x2,
     if(!path)
         return InvalidParameter;
 
-    if(!lengthen_path(path, 4))
-        return OutOfMemory;
+    points[0].X = x1;
+    points[0].Y = y1;
+    points[1].X = x2;
+    points[1].Y = y2;
+    points[2].X = x3;
+    points[2].Y = y3;
+    points[3].X = x4;
+    points[3].Y = y4;
 
-    old_count = path->pathdata.Count;
-
-    path->pathdata.Points[old_count].X = x1;
-    path->pathdata.Points[old_count].Y = y1;
-    path->pathdata.Points[old_count + 1].X = x2;
-    path->pathdata.Points[old_count + 1].Y = y2;
-    path->pathdata.Points[old_count + 2].X = x3;
-    path->pathdata.Points[old_count + 2].Y = y3;
-    path->pathdata.Points[old_count + 3].X = x4;
-    path->pathdata.Points[old_count + 3].Y = y4;
-
-    path->pathdata.Types[old_count] =
-        (path->newfigure ? PathPointTypeStart : PathPointTypeLine);
-    path->pathdata.Types[old_count + 1] = PathPointTypeBezier;
-    path->pathdata.Types[old_count + 2] = PathPointTypeBezier;
-    path->pathdata.Types[old_count + 3] = PathPointTypeBezier;
-
-    path->newfigure = FALSE;
-    path->pathdata.Count += 4;
-
-    return Ok;
+    return extend_current_figure(path, points, 4, PathPointTypeBezier);
 }
 
 GpStatus WINGDIPAPI GdipAddPathBezierI(GpPath *path, INT x1, INT y1, INT x2,
@@ -290,30 +319,12 @@ GpStatus WINGDIPAPI GdipAddPathBezierI(GpPath *path, INT x1, INT y1, INT x2,
 GpStatus WINGDIPAPI GdipAddPathBeziers(GpPath *path, GDIPCONST GpPointF *points,
     INT count)
 {
-    INT i, old_count;
-
     TRACE("(%p, %p, %d)\n", path, points, count);
 
     if(!path || !points || ((count - 1) % 3))
         return InvalidParameter;
 
-    if(!lengthen_path(path, count))
-        return OutOfMemory;
-
-    old_count = path->pathdata.Count;
-
-    for(i = 0; i < count; i++){
-        path->pathdata.Points[old_count + i].X = points[i].X;
-        path->pathdata.Points[old_count + i].Y = points[i].Y;
-        path->pathdata.Types[old_count + i] = PathPointTypeBezier;
-    }
-
-    path->pathdata.Types[old_count] =
-        (path->newfigure ? PathPointTypeStart : PathPointTypeLine);
-    path->newfigure = FALSE;
-    path->pathdata.Count += count;
-
-    return Ok;
+    return extend_current_figure(path, points, count, PathPointTypeBezier);
 }
 
 GpStatus WINGDIPAPI GdipAddPathBeziersI(GpPath *path, GDIPCONST GpPoint *points,
@@ -415,7 +426,7 @@ GpStatus WINGDIPAPI GdipAddPathClosedCurve2(GpPath *path, GDIPCONST GpPointF *po
     pt[len_pt-1].X = pt[0].X;
     pt[len_pt-1].Y = pt[0].Y;
 
-    stat = GdipAddPathBeziers(path, pt, len_pt);
+    stat = extend_current_figure(path, pt, len_pt, PathPointTypeBezier);
 
     /* close figure */
     if(stat == Ok){
@@ -523,7 +534,7 @@ GpStatus WINGDIPAPI GdipAddPathCurve2(GpPath *path, GDIPCONST GpPointF *points,
     pt[len_pt-1].X = points[count-1].X;
     pt[len_pt-1].Y = points[count-1].Y;
 
-    stat = GdipAddPathBeziers(path, pt, len_pt);
+    stat = extend_current_figure(path, pt, len_pt, PathPointTypeBezier);
 
     heap_free(pt);
 
@@ -623,32 +634,12 @@ GpStatus WINGDIPAPI GdipAddPathEllipseI(GpPath *path, INT x, INT y, INT width,
 GpStatus WINGDIPAPI GdipAddPathLine2(GpPath *path, GDIPCONST GpPointF *points,
     INT count)
 {
-    INT i, old_count;
-
     TRACE("(%p, %p, %d)\n", path, points, count);
 
-    if(!path || !points)
+    if(!path || !points || count < 1)
         return InvalidParameter;
 
-    if(!lengthen_path(path, count))
-        return OutOfMemory;
-
-    old_count = path->pathdata.Count;
-
-    for(i = 0; i < count; i++){
-        path->pathdata.Points[old_count + i].X = points[i].X;
-        path->pathdata.Points[old_count + i].Y = points[i].Y;
-        path->pathdata.Types[old_count + i] = PathPointTypeLine;
-    }
-
-    if(path->newfigure){
-        path->pathdata.Types[old_count] = PathPointTypeStart;
-        path->newfigure = FALSE;
-    }
-
-    path->pathdata.Count += count;
-
-    return Ok;
+    return extend_current_figure(path, points, count, PathPointTypeLine);
 }
 
 GpStatus WINGDIPAPI GdipAddPathLine2I(GpPath *path, GDIPCONST GpPoint *points, INT count)
@@ -703,31 +694,19 @@ GpStatus WINGDIPAPI GdipAddPathLine2I(GpPath *path, GDIPCONST GpPoint *points, I
  */
 GpStatus WINGDIPAPI GdipAddPathLine(GpPath *path, REAL x1, REAL y1, REAL x2, REAL y2)
 {
-    INT old_count;
+    PointF points[2];
 
     TRACE("(%p, %.2f, %.2f, %.2f, %.2f)\n", path, x1, y1, x2, y2);
 
     if(!path)
         return InvalidParameter;
 
-    if(!lengthen_path(path, 2))
-        return OutOfMemory;
+    points[0].X = x1;
+    points[0].Y = y1;
+    points[1].X = x2;
+    points[1].Y = y2;
 
-    old_count = path->pathdata.Count;
-
-    path->pathdata.Points[old_count].X = x1;
-    path->pathdata.Points[old_count].Y = y1;
-    path->pathdata.Points[old_count + 1].X = x2;
-    path->pathdata.Points[old_count + 1].Y = y2;
-
-    path->pathdata.Types[old_count] =
-        (path->newfigure ? PathPointTypeStart : PathPointTypeLine);
-    path->pathdata.Types[old_count + 1] = PathPointTypeLine;
-
-    path->newfigure = FALSE;
-    path->pathdata.Count += 2;
-
-    return Ok;
+    return extend_current_figure(path, points, 2, PathPointTypeLine);
 }
 
 /*************************************************************************
@@ -1103,19 +1082,16 @@ GpStatus WINGDIPAPI GdipAddPathString(GpPath* path, GDIPCONST WCHAR* string, INT
     return status;
 }
 
-GpStatus WINGDIPAPI GdipAddPathStringI(GpPath* path, GDIPCONST WCHAR* string, INT length, GDIPCONST GpFontFamily* family, INT style, REAL emSize, GDIPCONST Rect* layoutRect, GDIPCONST GpStringFormat* format)
+GpStatus WINGDIPAPI GdipAddPathStringI(GpPath* path, GDIPCONST WCHAR* string, INT length, GDIPCONST GpFontFamily* family,
+    INT style, REAL emSize, GDIPCONST Rect* layoutRect, GDIPCONST GpStringFormat* format)
 {
-    if (layoutRect)
-    {
-        RectF layoutRectF = {
-            (REAL)layoutRect->X,
-            (REAL)layoutRect->Y,
-            (REAL)layoutRect->Width,
-            (REAL)layoutRect->Height
-        };
-        return GdipAddPathString(path, string, length, family, style, emSize, &layoutRectF, format);
-    }
-    return InvalidParameter;
+    RectF rect;
+
+    if (!layoutRect)
+        return InvalidParameter;
+
+    set_rect(&rect, layoutRect->X, layoutRect->Y, layoutRect->Width, layoutRect->Height);
+    return GdipAddPathString(path, string, length, family, style, emSize, &rect, format);
 }
 
 /*************************************************************************
@@ -1213,14 +1189,37 @@ GpStatus WINGDIPAPI GdipCreatePath(GpFillMode fill, GpPath **path)
 GpStatus WINGDIPAPI GdipCreatePath2(GDIPCONST GpPointF* points,
     GDIPCONST BYTE* types, INT count, GpFillMode fill, GpPath **path)
 {
+    int i;
+
     TRACE("(%p, %p, %d, %d, %p)\n", points, types, count, fill, path);
 
-    if(!path)
+    if(!points || !types || !path)
         return InvalidParameter;
 
+    if(count <= 0) {
+        *path = NULL;
+        return OutOfMemory;
+    }
+
     *path = heap_alloc_zero(sizeof(GpPath));
     if(!*path)  return OutOfMemory;
 
+    if(count > 1 && (types[count-1] & PathPointTypePathTypeMask) == PathPointTypeStart)
+        count = 0;
+
+    for(i = 1; i < count; i++) {
+        if((types[i] & PathPointTypePathTypeMask) == PathPointTypeBezier) {
+            if(i+2 < count &&
+                    (types[i+1] & PathPointTypePathTypeMask) == PathPointTypeBezier &&
+                    (types[i+2] & PathPointTypePathTypeMask) == PathPointTypeBezier)
+                i += 2;
+            else {
+                count = 0;
+                break;
+            }
+        }
+    }
+
     (*path)->pathdata.Points = heap_alloc_zero(count * sizeof(PointF));
     (*path)->pathdata.Types = heap_alloc_zero(count);
 
@@ -1233,6 +1232,8 @@ GpStatus WINGDIPAPI GdipCreatePath2(GDIPCONST GpPointF* points,
 
     memcpy((*path)->pathdata.Points, points, count * sizeof(PointF));
     memcpy((*path)->pathdata.Types, types, count);
+    if(count > 0)
+        (*path)->pathdata.Types[0] = PathPointTypeStart;
     (*path)->pathdata.Count = count;
     (*path)->datalen = count;
 
@@ -1804,12 +1805,12 @@ GpStatus WINGDIPAPI GdipWarpPath(GpPath *path, GpMatrix* matrix,
 }
 
 static void add_bevel_point(const GpPointF *endpoint, const GpPointF *nextpoint,
-    GpPen *pen, int right_side, path_list_node_t **last_point)
+    REAL pen_width, int right_side, path_list_node_t **last_point)
 {
     REAL segment_dy = nextpoint->Y-endpoint->Y;
     REAL segment_dx = nextpoint->X-endpoint->X;
     REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
-    REAL distance = pen->width/2.0;
+    REAL distance = pen_width / 2.0;
     REAL bevel_dx, bevel_dy;
 
     if (segment_length == 0.0)
@@ -1835,7 +1836,7 @@ static void add_bevel_point(const GpPointF *endpoint, const GpPointF *nextpoint,
 }
 
 static void widen_joint(const GpPointF *p1, const GpPointF *p2, const GpPointF *p3,
-    GpPen* pen, path_list_node_t **last_point)
+    GpPen* pen, REAL pen_width, path_list_node_t **last_point)
 {
     switch (pen->join)
     {
@@ -1843,7 +1844,7 @@ static void widen_joint(const GpPointF *p1, const GpPointF *p2, const GpPointF *
     case LineJoinMiterClipped:
         if ((p2->X - p1->X) * (p3->Y - p1->Y) > (p2->Y - p1->Y) * (p3->X - p1->X))
         {
-            float distance = pen->width/2.0;
+            float distance = pen_width / 2.0;
             float length_0 = sqrtf((p2->X-p1->X)*(p2->X-p1->X)+(p2->Y-p1->Y)*(p2->Y-p1->Y));
             float length_1 = sqrtf((p3->X-p2->X)*(p3->X-p2->X)+(p3->Y-p2->Y)*(p3->Y-p2->Y));
             float dx0 = distance * (p2->X - p1->X) / length_0;
@@ -1870,14 +1871,14 @@ static void widen_joint(const GpPointF *p1, const GpPointF *p2, const GpPointF *
         /* else fall-through */
     default:
     case LineJoinBevel:
-        add_bevel_point(p2, p1, pen, 1, last_point);
-        add_bevel_point(p2, p3, pen, 0, last_point);
+        add_bevel_point(p2, p1, pen_width, 1, last_point);
+        add_bevel_point(p2, p3, pen_width, 0, last_point);
         break;
     }
 }
 
 static void widen_cap(const GpPointF *endpoint, const GpPointF *nextpoint,
-    GpPen *pen, GpLineCap cap, GpCustomLineCap *custom, int add_first_points,
+    REAL pen_width, GpLineCap cap, GpCustomLineCap* custom_cap, int add_first_points,
     int add_last_point, path_list_node_t **last_point)
 {
     switch (cap)
@@ -1885,33 +1886,45 @@ static void widen_cap(const GpPointF *endpoint, const GpPointF *nextpoint,
     default:
     case LineCapFlat:
         if (add_first_points)
-            add_bevel_point(endpoint, nextpoint, pen, 1, last_point);
+            add_bevel_point(endpoint, nextpoint, pen_width, 1, last_point);
         if (add_last_point)
-            add_bevel_point(endpoint, nextpoint, pen, 0, last_point);
+            add_bevel_point(endpoint, nextpoint, pen_width, 0, last_point);
         break;
     case LineCapSquare:
+    case LineCapCustom:
+    case LineCapArrowAnchor:
     {
         REAL segment_dy = nextpoint->Y-endpoint->Y;
         REAL segment_dx = nextpoint->X-endpoint->X;
         REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
-        REAL distance = pen->width/2.0;
+        REAL distance = pen_width / 2.0;
         REAL bevel_dx, bevel_dy;
         REAL extend_dx, extend_dy;
 
-        extend_dx = -distance * segment_dx / segment_length;
-        extend_dy = -distance * segment_dy / segment_length;
+        extend_dx = distance * segment_dx / segment_length;
+        extend_dy = distance * segment_dy / segment_length;
 
-        bevel_dx = -distance * segment_dy / segment_length;
-        bevel_dy = distance * segment_dx / segment_length;
+        bevel_dx = -extend_dy;
+        bevel_dy = extend_dx;
+
+        if (cap == LineCapCustom)
+        {
+            extend_dx = -2.0 * custom_cap->inset * extend_dx;
+            extend_dy = -2.0 * custom_cap->inset * extend_dy;
+        }
+        else if (cap == LineCapArrowAnchor)
+        {
+            extend_dx = -3.0 * extend_dx;
+            extend_dy = -3.0 * extend_dy;
+        }
 
         if (add_first_points)
-            *last_point = add_path_list_node(*last_point, endpoint->X + extend_dx + bevel_dx,
-                endpoint->Y + extend_dy + bevel_dy, PathPointTypeLine);
+            *last_point = add_path_list_node(*last_point, endpoint->X - extend_dx + bevel_dx,
+                endpoint->Y - extend_dy + bevel_dy, PathPointTypeLine);
 
         if (add_last_point)
-            *last_point = add_path_list_node(*last_point, endpoint->X + extend_dx - bevel_dx,
-                endpoint->Y + extend_dy - bevel_dy, PathPointTypeLine);
-
+            *last_point = add_path_list_node(*last_point, endpoint->X - extend_dx - bevel_dx,
+                endpoint->Y - extend_dy - bevel_dy, PathPointTypeLine);
         break;
     }
     case LineCapRound:
@@ -1919,7 +1932,7 @@ static void widen_cap(const GpPointF *endpoint, const GpPointF *nextpoint,
         REAL segment_dy = nextpoint->Y-endpoint->Y;
         REAL segment_dx = nextpoint->X-endpoint->X;
         REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
-        REAL distance = pen->width/2.0;
+        REAL distance = pen_width / 2.0;
         REAL dx, dy, dx2, dy2;
         const REAL control_point_distance = 0.5522847498307935; /* 4/3 * (sqrt(2) - 1) */
 
@@ -1955,6 +1968,8 @@ static void widen_cap(const GpPointF *endpoint, const GpPointF *nextpoint,
             *last_point = add_path_list_node(*last_point, endpoint->X - dy,
                 endpoint->Y + dx, PathPointTypeBezier);
         }
+        else if (add_last_point)
+            add_bevel_point(endpoint, nextpoint, pen_width, 0, last_point);
         break;
     }
     case LineCapTriangle:
@@ -1962,79 +1977,281 @@ static void widen_cap(const GpPointF *endpoint, const GpPointF *nextpoint,
         REAL segment_dy = nextpoint->Y-endpoint->Y;
         REAL segment_dx = nextpoint->X-endpoint->X;
         REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
-        REAL distance = pen->width/2.0;
+        REAL distance = pen_width / 2.0;
         REAL dx, dy;
 
         dx = distance * segment_dx / segment_length;
         dy = distance * segment_dy / segment_length;
 
         if (add_first_points) {
-            add_bevel_point(endpoint, nextpoint, pen, 1, last_point);
+            add_bevel_point(endpoint, nextpoint, pen_width, 1, last_point);
 
             *last_point = add_path_list_node(*last_point, endpoint->X - dx,
                 endpoint->Y - dy, PathPointTypeLine);
         }
-        if (add_last_point)
-            add_bevel_point(endpoint, nextpoint, pen, 0, last_point);
+        if (add_first_points || add_last_point)
+            add_bevel_point(endpoint, nextpoint, pen_width, 0, last_point);
         break;
     }
     }
 }
 
-static void widen_open_figure(const GpPointF *points, GpPen *pen, int start, int end,
-    GpLineCap start_cap, GpCustomLineCap *start_custom, GpLineCap end_cap,
-    GpCustomLineCap *end_custom, path_list_node_t **last_point)
+static void widen_open_figure(const GpPointF *points, int start, int end,
+    GpPen *pen, REAL pen_width, GpLineCap start_cap, GpCustomLineCap* custom_start,
+    GpLineCap end_cap, GpCustomLineCap* custom_end, path_list_node_t **last_point);
+
+static void widen_closed_figure(const GpPointF *points, int start, int end,
+    GpPen *pen, REAL pen_width, path_list_node_t **last_point);
+
+static void add_anchor(const GpPointF *endpoint, const GpPointF *nextpoint,
+    GpPen *pen, GpLineCap cap, GpCustomLineCap *custom, path_list_node_t **last_point)
+{
+    REAL pen_width = max(pen->width, 2.0);
+    switch (cap)
+    {
+    default:
+    case LineCapNoAnchor:
+        return;
+    case LineCapSquareAnchor:
+    {
+        REAL segment_dy = nextpoint->Y-endpoint->Y;
+        REAL segment_dx = nextpoint->X-endpoint->X;
+        REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
+        REAL distance = pen_width / sqrtf(2.0);
+        REAL par_dx, par_dy;
+        REAL perp_dx, perp_dy;
+
+        par_dx = -distance * segment_dx / segment_length;
+        par_dy = -distance * segment_dy / segment_length;
+
+        perp_dx = -distance * segment_dy / segment_length;
+        perp_dy = distance * segment_dx / segment_length;
+
+        *last_point = add_path_list_node(*last_point, endpoint->X - par_dx - perp_dx,
+            endpoint->Y - par_dy - perp_dy, PathPointTypeStart);
+        *last_point = add_path_list_node(*last_point, endpoint->X - par_dx + perp_dx,
+            endpoint->Y - par_dy + perp_dy, PathPointTypeLine);
+        *last_point = add_path_list_node(*last_point, endpoint->X + par_dx + perp_dx,
+            endpoint->Y + par_dy + perp_dy, PathPointTypeLine);
+        *last_point = add_path_list_node(*last_point, endpoint->X + par_dx - perp_dx,
+            endpoint->Y + par_dy - perp_dy, PathPointTypeLine);
+        break;
+    }
+    case LineCapRoundAnchor:
+    {
+        REAL segment_dy = nextpoint->Y-endpoint->Y;
+        REAL segment_dx = nextpoint->X-endpoint->X;
+        REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
+        REAL dx, dy, dx2, dy2;
+        const REAL control_point_distance = 0.55228475; /* 4/3 * (sqrt(2) - 1) */
+
+        dx = -pen_width * segment_dx / segment_length;
+        dy = -pen_width * segment_dy / segment_length;
+
+        dx2 = dx * control_point_distance;
+        dy2 = dy * control_point_distance;
+
+        /* starting point */
+        *last_point = add_path_list_node(*last_point, endpoint->X + dy,
+            endpoint->Y - dx, PathPointTypeStart);
+
+        /* first 90-degree arc */
+        *last_point = add_path_list_node(*last_point, endpoint->X + dy + dx2,
+            endpoint->Y - dx + dy2, PathPointTypeBezier);
+        *last_point = add_path_list_node(*last_point, endpoint->X + dx + dy2,
+            endpoint->Y + dy - dx2, PathPointTypeBezier);
+        *last_point = add_path_list_node(*last_point, endpoint->X + dx,
+            endpoint->Y + dy, PathPointTypeBezier);
+
+        /* second 90-degree arc */
+        *last_point = add_path_list_node(*last_point, endpoint->X + dx - dy2,
+            endpoint->Y + dy + dx2, PathPointTypeBezier);
+        *last_point = add_path_list_node(*last_point, endpoint->X - dy + dx2,
+            endpoint->Y + dx + dy2, PathPointTypeBezier);
+        *last_point = add_path_list_node(*last_point, endpoint->X - dy,
+            endpoint->Y + dx, PathPointTypeBezier);
+
+        /* third 90-degree arc */
+        *last_point = add_path_list_node(*last_point, endpoint->X - dy - dx2,
+            endpoint->Y + dx - dy2, PathPointTypeBezier);
+        *last_point = add_path_list_node(*last_point, endpoint->X - dx - dy2,
+            endpoint->Y - dy + dx2, PathPointTypeBezier);
+        *last_point = add_path_list_node(*last_point, endpoint->X - dx,
+            endpoint->Y - dy, PathPointTypeBezier);
+
+        /* fourth 90-degree arc */
+        *last_point = add_path_list_node(*last_point, endpoint->X - dx + dy2,
+            endpoint->Y - dy - dx2, PathPointTypeBezier);
+        *last_point = add_path_list_node(*last_point, endpoint->X + dy - dx2,
+            endpoint->Y - dx - dy2, PathPointTypeBezier);
+        *last_point = add_path_list_node(*last_point, endpoint->X + dy,
+            endpoint->Y - dx, PathPointTypeBezier);
+
+        break;
+    }
+    case LineCapDiamondAnchor:
+    {
+        REAL segment_dy = nextpoint->Y-endpoint->Y;
+        REAL segment_dx = nextpoint->X-endpoint->X;
+        REAL segment_length = sqrtf(segment_dy*segment_dy + segment_dx*segment_dx);
+        REAL par_dx, par_dy;
+        REAL perp_dx, perp_dy;
+
+        par_dx = -pen_width * segment_dx / segment_length;
+        par_dy = -pen_width * segment_dy / segment_length;
+
+        perp_dx = -pen_width * segment_dy / segment_length;
+        perp_dy = pen_width * segment_dx / segment_length;
+
+        *last_point = add_path_list_node(*last_point, endpoint->X + par_dx,
+            endpoint->Y + par_dy, PathPointTypeStart);
+        *last_point = add_path_list_node(*last_point, endpoint->X - perp_dx,
+            endpoint->Y - perp_dy, PathPointTypeLine);
+        *last_point = add_path_list_node(*last_point, endpoint->X - par_dx,
+            endpoint->Y - par_dy, PathPointTypeLine);
+        *last_point = add_path_list_node(*last_point, endpoint->X + perp_dx,
+            endpoint->Y + perp_dy, PathPointTypeLine);
+        break;
+    }
+    case LineCapArrowAnchor:
+    {
+        REAL segment_dy = nextpoint->Y - endpoint->Y;
+        REAL segment_dx = nextpoint->X - endpoint->X;
+        REAL segment_length = sqrtf(segment_dy * segment_dy + segment_dx * segment_dx);
+        REAL par_dx = pen_width * segment_dx / segment_length;
+        REAL par_dy = pen_width * segment_dy / segment_length;
+        REAL perp_dx = -par_dy;
+        REAL perp_dy = par_dx;
+
+        *last_point = add_path_list_node(*last_point, endpoint->X,
+            endpoint->Y, PathPointTypeStart);
+        *last_point = add_path_list_node(*last_point, endpoint->X + SQRT3 * par_dx - perp_dx,
+            endpoint->Y + SQRT3 * par_dy - perp_dy, PathPointTypeLine);
+        *last_point = add_path_list_node(*last_point, endpoint->X + SQRT3 * par_dx + perp_dx,
+            endpoint->Y + SQRT3 * par_dy + perp_dy, PathPointTypeLine);
+        break;
+    }
+    case LineCapCustom:
+    {
+        REAL segment_dy = nextpoint->Y - endpoint->Y;
+        REAL segment_dx = nextpoint->X - endpoint->X;
+        REAL segment_length = sqrtf(segment_dy * segment_dy + segment_dx * segment_dx);
+        REAL posx, posy;
+        REAL perp_dx, perp_dy;
+        REAL sina, cosa;
+        GpPointF *tmp_points;
+
+        if(!custom)
+            break;
+
+        if (custom->type == CustomLineCapTypeAdjustableArrow)
+        {
+            GpAdjustableArrowCap *arrow = (GpAdjustableArrowCap *)custom;
+            TRACE("GpAdjustableArrowCap middle_inset: %f height: %f width: %f\n",
+                arrow->middle_inset, arrow->height, arrow->width);
+        }
+        else
+            TRACE("GpCustomLineCap fill: %d basecap: %d inset: %f join: %d scale: %f pen_width:%f\n",
+                custom->fill, custom->basecap, custom->inset, custom->join, custom->scale, pen_width);
+
+        sina = -pen_width * custom->scale * segment_dx / segment_length;
+        cosa = pen_width * custom->scale * segment_dy / segment_length;
+
+        /* Coordination where cap needs to be drawn */
+        posx = endpoint->X + sina;
+        posy = endpoint->Y - cosa;
+
+        if (!custom->fill)
+        {
+            tmp_points = heap_alloc_zero(custom->pathdata.Count * sizeof(GpPoint));
+            if (!tmp_points) {
+                ERR("Out of memory\n");
+                return;
+            }
+
+            for (INT i = 0; i < custom->pathdata.Count; i++)
+            {
+                tmp_points[i].X = posx + custom->pathdata.Points[i].X * cosa + (custom->pathdata.Points[i].Y - 1.0) * sina;
+                tmp_points[i].Y = posy + custom->pathdata.Points[i].X * sina - (custom->pathdata.Points[i].Y - 1.0) * cosa;
+            }
+            if ((custom->pathdata.Types[custom->pathdata.Count - 1] & PathPointTypeCloseSubpath) == PathPointTypeCloseSubpath)
+                widen_closed_figure(tmp_points, 0, custom->pathdata.Count - 1, pen, pen_width, last_point);
+            else
+                widen_open_figure(tmp_points, 0, custom->pathdata.Count - 1, pen, pen_width, custom->strokeEndCap, NULL, custom->strokeStartCap, NULL, last_point);
+        }
+        else
+        {
+            for (INT i = 0; i < custom->pathdata.Count; i++)
+            {
+                /* rotation of CustomCap according to line */
+                perp_dx = custom->pathdata.Points[i].X * cosa + (custom->pathdata.Points[i].Y - 1.0) * sina;
+                perp_dy = custom->pathdata.Points[i].X * sina - (custom->pathdata.Points[i].Y - 1.0) * cosa;
+                *last_point = add_path_list_node(*last_point, posx + perp_dx,
+                    posy + perp_dy, custom->pathdata.Types[i]);
+            }
+        }
+        /* FIXME: The line should be adjusted by the inset value of the custom cap. */
+        break;
+    }
+    }
+
+    (*last_point)->type |= PathPointTypeCloseSubpath;
+}
+
+static void widen_open_figure(const GpPointF *points, int start, int end,
+    GpPen *pen, REAL pen_width, GpLineCap start_cap, GpCustomLineCap* custom_start,
+    GpLineCap end_cap, GpCustomLineCap* custom_end, path_list_node_t **last_point)
 {
     int i;
     path_list_node_t *prev_point;
 
-    if (end <= start)
+    if (end <= start || pen_width == 0.0)
         return;
 
     prev_point = *last_point;
 
     widen_cap(&points[start], &points[start+1],
-        pen, start_cap, start_custom, FALSE, TRUE, last_point);
+        pen_width, start_cap, custom_start, FALSE, TRUE, last_point);
 
     for (i=start+1; i<end; i++)
-        widen_joint(&points[i-1], &points[i],
-            &points[i+1], pen, last_point);
+        widen_joint(&points[i-1], &points[i], &points[i+1],
+            pen, pen_width, last_point);
 
     widen_cap(&points[end], &points[end-1],
-        pen, end_cap, end_custom, TRUE, TRUE, last_point);
+        pen_width, end_cap, custom_end, TRUE, TRUE, last_point);
 
     for (i=end-1; i>start; i--)
-        widen_joint(&points[i+1], &points[i],
-            &points[i-1], pen, last_point);
+        widen_joint(&points[i+1], &points[i], &points[i-1],
+            pen, pen_width, last_point);
 
     widen_cap(&points[start], &points[start+1],
-        pen, start_cap, start_custom, TRUE, FALSE, last_point);
+        pen_width, start_cap, custom_start, TRUE, FALSE, last_point);
 
     prev_point->next->type = PathPointTypeStart;
     (*last_point)->type |= PathPointTypeCloseSubpath;
 }
 
-static void widen_closed_figure(GpPath *path, GpPen *pen, int start, int end,
-    path_list_node_t **last_point)
+static void widen_closed_figure(const GpPointF *points, int start, int end,
+    GpPen *pen, REAL pen_width, path_list_node_t **last_point)
 {
     int i;
     path_list_node_t *prev_point;
 
-    if (end <= start)
+    if (end <= start || pen_width == 0.0)
         return;
 
     /* left outline */
     prev_point = *last_point;
 
-    widen_joint(&path->pathdata.Points[end], &path->pathdata.Points[start],
-        &path->pathdata.Points[start+1], pen, last_point);
+    widen_joint(&points[end], &points[start],
+        &points[start+1], pen, pen_width, last_point);
 
     for (i=start+1; i<end; i++)
-        widen_joint(&path->pathdata.Points[i-1], &path->pathdata.Points[i],
-            &path->pathdata.Points[i+1], pen, last_point);
+        widen_joint(&points[i-1], &points[i],
+            &points[i+1], pen, pen_width, last_point);
 
-    widen_joint(&path->pathdata.Points[end-1], &path->pathdata.Points[end],
-        &path->pathdata.Points[start], pen, last_point);
+    widen_joint(&points[end-1], &points[end],
+        &points[start], pen, pen_width, last_point);
 
     prev_point->next->type = PathPointTypeStart;
     (*last_point)->type |= PathPointTypeCloseSubpath;
@@ -2042,22 +2259,22 @@ static void widen_closed_figure(GpPath *path, GpPen *pen, int start, int end,
     /* right outline */
     prev_point = *last_point;
 
-    widen_joint(&path->pathdata.Points[start], &path->pathdata.Points[end],
-        &path->pathdata.Points[end-1], pen, last_point);
+    widen_joint(&points[start], &points[end],
+        &points[end-1], pen, pen_width, last_point);
 
     for (i=end-1; i>start; i--)
-        widen_joint(&path->pathdata.Points[i+1], &path->pathdata.Points[i],
-            &path->pathdata.Points[i-1], pen, last_point);
+        widen_joint(&points[i+1], &points[i],
+            &points[i-1], pen, pen_width, last_point);
 
-    widen_joint(&path->pathdata.Points[start+1], &path->pathdata.Points[start],
-        &path->pathdata.Points[end], pen, last_point);
+    widen_joint(&points[start+1], &points[start],
+        &points[end], pen, pen_width, last_point);
 
     prev_point->next->type = PathPointTypeStart;
     (*last_point)->type |= PathPointTypeCloseSubpath;
 }
 
-static void widen_dashed_figure(GpPath *path, GpPen *pen, int start, int end,
-    int closed, path_list_node_t **last_point)
+static void widen_dashed_figure(GpPath *path, int start, int end, int closed,
+    GpPen *pen, REAL pen_width, path_list_node_t **last_point)
 {
     int i, j;
     REAL dash_pos=0.0;
@@ -2074,7 +2291,7 @@ static void widen_dashed_figure(GpPath *path, GpPen *pen, int start, int end,
     int draw_start_cap=0;
     static const REAL dash_dot_dot[6] = { 3.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
 
-    if (end <= start)
+    if (end <= start || pen_width == 0.0)
         return;
 
     switch (pen->dash)
@@ -2151,7 +2368,7 @@ static void widen_dashed_figure(GpPath *path, GpPen *pen, int start, int end,
                     tmp_points[num_tmp_points].X = path->pathdata.Points[i].X + segment_dx * segment_pos / segment_length;
                     tmp_points[num_tmp_points].Y = path->pathdata.Points[i].Y + segment_dy * segment_pos / segment_length;
 
-                    widen_open_figure(tmp_points, pen, 0, num_tmp_points,
+                    widen_open_figure(tmp_points, 0, num_tmp_points, pen, pen_width,
                         draw_start_cap ? pen->startcap : LineCapFlat, pen->customstart,
                         LineCapFlat, NULL, last_point);
                     draw_start_cap = 0;
@@ -2185,7 +2402,7 @@ static void widen_dashed_figure(GpPath *path, GpPen *pen, int start, int end,
     if (dash_index % 2 == 0 && num_tmp_points != 0)
     {
         /* last dash overflows last segment */
-        widen_open_figure(tmp_points, pen, 0, num_tmp_points-1,
+        widen_open_figure(tmp_points, 0, num_tmp_points-1, pen, pen_width,
             draw_start_cap ? pen->startcap : LineCapFlat, pen->customstart,
             closed ? LineCapFlat : pen->endcap, pen->customend, last_point);
     }
@@ -2201,7 +2418,6 @@ GpStatus WINGDIPAPI GdipWidenPath(GpPath *path, GpPen *pen, GpMatrix *matrix,
     GpStatus status;
     path_list_node_t *points=NULL, *last_point=NULL;
     int i, subpath_start=0, new_length;
-    BYTE type;
 
     TRACE("(%p,%p,%p,%0.2f)\n", path, pen, matrix, flatness);
 
@@ -2221,14 +2437,11 @@ GpStatus WINGDIPAPI GdipWidenPath(GpPath *path, GpPen *pen, GpMatrix *matrix,
 
     if (status == Ok)
     {
+        REAL pen_width = (pen->unit == UnitWorld) ? max(pen->width, 1.0) : pen->width;
+        BYTE *types = flat_path->pathdata.Types;
+
         last_point = points;
 
-        if (pen->endcap > LineCapTriangle)
-            FIXME("unimplemented end cap %x\n", pen->endcap);
-
-        if (pen->startcap > LineCapTriangle)
-            FIXME("unimplemented start cap %x\n", pen->startcap);
-
         if (pen->dashcap != DashCapFlat)
             FIXME("unimplemented dash cap %d\n", pen->dashcap);
 
@@ -2238,27 +2451,52 @@ GpStatus WINGDIPAPI GdipWidenPath(GpPath *path, GpPen *pen, GpMatrix *matrix,
         if (pen->align != PenAlignmentCenter)
             FIXME("unimplemented pen alignment %d\n", pen->align);
 
+        if (pen->compound_array_size != 0)
+            FIXME("unimplemented pen compoundline. Solid line will be drawn instead: %d\n", pen->compound_array_size);
+
         for (i=0; i < flat_path->pathdata.Count; i++)
         {
-            type = flat_path->pathdata.Types[i];
-
-            if ((type&PathPointTypePathTypeMask) == PathPointTypeStart)
+            if ((types[i]&PathPointTypePathTypeMask) == PathPointTypeStart)
                 subpath_start = i;
 
-            if ((type&PathPointTypeCloseSubpath) == PathPointTypeCloseSubpath)
+            if ((types[i]&PathPointTypeCloseSubpath) == PathPointTypeCloseSubpath)
             {
                 if (pen->dash != DashStyleSolid)
-                    widen_dashed_figure(flat_path, pen, subpath_start, i, 1, &last_point);
+                    widen_dashed_figure(flat_path, subpath_start, i, 1, pen, pen_width, &last_point);
                 else
-                    widen_closed_figure(flat_path, pen, subpath_start, i, &last_point);
+                    widen_closed_figure(flat_path->pathdata.Points, subpath_start, i, pen, pen_width, &last_point);
             }
             else if (i == flat_path->pathdata.Count-1 ||
-                (flat_path->pathdata.Types[i+1]&PathPointTypePathTypeMask) == PathPointTypeStart)
+                (types[i+1]&PathPointTypePathTypeMask) == PathPointTypeStart)
             {
                 if (pen->dash != DashStyleSolid)
-                    widen_dashed_figure(flat_path, pen, subpath_start, i, 0, &last_point);
+                    widen_dashed_figure(flat_path, subpath_start, i, 0, pen, pen_width, &last_point);
                 else
-                    widen_open_figure(flat_path->pathdata.Points, pen, subpath_start, i, pen->startcap, pen->customstart, pen->endcap, pen->customend, &last_point);
+                    widen_open_figure(flat_path->pathdata.Points, subpath_start, i, pen, pen_width,
+                        pen->startcap, pen->customstart, pen->endcap, pen->customend, &last_point);
+            }
+        }
+
+        for (i=0; i < flat_path->pathdata.Count; i++)
+        {
+            if ((types[i]&PathPointTypeCloseSubpath) == PathPointTypeCloseSubpath)
+                continue;
+
+            if ((types[i]&PathPointTypePathTypeMask) == PathPointTypeStart)
+                subpath_start = i;
+
+            if (i == flat_path->pathdata.Count-1 ||
+                (types[i+1]&PathPointTypePathTypeMask) == PathPointTypeStart)
+            {
+                if (pen->startcap & LineCapAnchorMask)
+                    add_anchor(&flat_path->pathdata.Points[subpath_start],
+                        &flat_path->pathdata.Points[subpath_start+1],
+                        pen, pen->startcap, pen->customstart, &last_point);
+
+                if (pen->endcap & LineCapAnchorMask)
+                    add_anchor(&flat_path->pathdata.Points[i],
+                        &flat_path->pathdata.Points[i-1],
+                        pen, pen->endcap, pen->customend, &last_point);
             }
         }
 
@@ -2405,12 +2643,8 @@ GpStatus WINGDIPAPI GdipAddPathRectanglesI(GpPath *path, GDIPCONST GpRect *rects
 
     rectsF = heap_alloc_zero(sizeof(GpRectF)*count);
 
-    for(i = 0;i < count;i++){
-        rectsF[i].X      = (REAL)rects[i].X;
-        rectsF[i].Y      = (REAL)rects[i].Y;
-        rectsF[i].Width  = (REAL)rects[i].Width;
-        rectsF[i].Height = (REAL)rects[i].Height;
-    }
+    for(i = 0;i < count;i++)
+        set_rect(&rectsF[i], rects[i].X, rects[i].Y, rects[i].Width, rects[i].Height);
 
     retstat = GdipAddPathRectangles(path, rectsF, count);
     heap_free(rectsF);
