@@ -332,11 +332,7 @@ GpStatus WINGDIPAPI GdipCombineRegionRectI(GpRegion *region,
     if (!rect)
         return InvalidParameter;
 
-    rectf.X = (REAL)rect->X;
-    rectf.Y = (REAL)rect->Y;
-    rectf.Height = (REAL)rect->Height;
-    rectf.Width = (REAL)rect->Width;
-
+    set_rect(&rectf, rect->X, rect->Y, rect->Width, rect->Height);
     return GdipCombineRegionRect(region, &rectf, mode);
 }
 
@@ -461,7 +457,7 @@ GpStatus WINGDIPAPI GdipCreateRegionRect(GDIPCONST GpRectF *rect,
 {
     GpStatus stat;
 
-    TRACE("%p, %p\n", rect, region);
+    TRACE("%s, %p\n", debugstr_rectf(rect), region);
 
     if (!(rect && region))
         return InvalidParameter;
@@ -492,11 +488,7 @@ GpStatus WINGDIPAPI GdipCreateRegionRectI(GDIPCONST GpRect *rect,
 
     TRACE("%p, %p\n", rect, region);
 
-    rectf.X = (REAL)rect->X;
-    rectf.Y = (REAL)rect->Y;
-    rectf.Width = (REAL)rect->Width;
-    rectf.Height = (REAL)rect->Height;
-
+    set_rect(&rectf, rect->X, rect->Y, rect->Width, rect->Height);
     return GdipCreateRegionRect(&rectf, region);
 }
 
@@ -780,7 +772,7 @@ static GpStatus read_element(struct memory_buffer *mbuf, GpRegion *region, regio
     type = buffer_read(mbuf, sizeof(*type));
     if (!type) return Ok;
 
-    TRACE("type %#x\n", *type);
+    TRACE("type %#lx\n", *type);
 
     node->type = *type;
 
@@ -852,7 +844,7 @@ static GpStatus read_element(struct memory_buffer *mbuf, GpRegion *region, regio
         }
         if (!VALID_MAGIC(path_header->magic))
         {
-            ERR("invalid path header magic %#x\n", path_header->magic);
+            ERR("invalid path header magic %#lx\n", path_header->magic);
             return InvalidParameter;
         }
 
@@ -874,7 +866,7 @@ static GpStatus read_element(struct memory_buffer *mbuf, GpRegion *region, regio
         path->pathdata.Count = path_header->count;
 
         if (path_header->flags & ~FLAGS_INTPATH)
-            FIXME("unhandled path flags %#x\n", path_header->flags);
+            FIXME("unhandled path flags %#lx\n", path_header->flags);
 
         if (path_header->flags & FLAGS_INTPATH)
         {
@@ -884,7 +876,7 @@ static GpStatus read_element(struct memory_buffer *mbuf, GpRegion *region, regio
             pt = buffer_read(mbuf, sizeof(*pt) * path_header->count);
             if (!pt)
             {
-                ERR("failed to read packed %u path points\n", path_header->count);
+                ERR("failed to read packed %lu path points\n", path_header->count);
                 return InvalidParameter;
             }
 
@@ -901,7 +893,7 @@ static GpStatus read_element(struct memory_buffer *mbuf, GpRegion *region, regio
             ptf = buffer_read(mbuf, sizeof(*ptf) * path_header->count);
             if (!ptf)
             {
-                ERR("failed to read %u path points\n", path_header->count);
+                ERR("failed to read %lu path points\n", path_header->count);
                 return InvalidParameter;
             }
             memcpy(path->pathdata.Points, ptf, sizeof(*ptf) * path_header->count);
@@ -910,7 +902,7 @@ static GpStatus read_element(struct memory_buffer *mbuf, GpRegion *region, regio
         types = buffer_read(mbuf, path_header->count);
         if (!types)
         {
-            ERR("failed to read %u path types\n", path_header->count);
+            ERR("failed to read %lu path types\n", path_header->count);
             return InvalidParameter;
         }
         memcpy(path->pathdata.Types, types, path_header->count);
@@ -918,7 +910,7 @@ static GpStatus read_element(struct memory_buffer *mbuf, GpRegion *region, regio
         {
             if (!buffer_read(mbuf, 4 - (path_header->count & 3)))
             {
-                ERR("failed to read rounding %u bytes\n", 4 - (path_header->count & 3));
+                ERR("failed to read rounding %lu bytes\n", 4 - (path_header->count & 3));
                 return InvalidParameter;
             }
         }
@@ -933,7 +925,7 @@ static GpStatus read_element(struct memory_buffer *mbuf, GpRegion *region, regio
         return Ok;
 
     default:
-        FIXME("element type %#x is not supported\n", *type);
+        FIXME("element type %#lx is not supported\n", *type);
         break;
     }
 
@@ -1182,7 +1174,7 @@ static GpStatus get_region_hrgn(struct region_element *element, GpGraphics *grap
             return Ok;
         }
         default:
-            FIXME("GdipGetRegionHRgn unimplemented for region type=%x\n", element->type);
+            FIXME("GdipGetRegionHRgn unimplemented for region type=%lx\n", element->type);
             *hrgn = NULL;
             return NotImplemented;
     }
@@ -1398,10 +1390,46 @@ static GpStatus transform_region_element(region_element* element, GpMatrix *matr
             return Ok;
         case RegionDataRect:
         {
-            /* We can't transform a rectangle, so convert it to a path. */
             GpRegion *new_region;
             GpPath *path;
 
+            if (matrix->matrix[1] == 0.0 && matrix->matrix[2] == 0.0)
+            {
+                GpPointF points[2];
+
+                points[0].X = element->elementdata.rect.X;
+                points[0].Y = element->elementdata.rect.Y;
+                points[1].X = element->elementdata.rect.X + element->elementdata.rect.Width;
+                points[1].Y = element->elementdata.rect.Y + element->elementdata.rect.Height;
+
+                stat = GdipTransformMatrixPoints(matrix, points, 2);
+                if (stat != Ok)
+                    return stat;
+
+                if (points[0].X > points[1].X)
+                {
+                    REAL temp;
+                    temp = points[0].X;
+                    points[0].X = points[1].X;
+                    points[1].X = temp;
+                }
+
+                if (points[0].Y > points[1].Y)
+                {
+                    REAL temp;
+                    temp = points[0].Y;
+                    points[0].Y = points[1].Y;
+                    points[1].Y = temp;
+                }
+
+                element->elementdata.rect.X = points[0].X;
+                element->elementdata.rect.Y = points[0].Y;
+                element->elementdata.rect.Width = points[1].X - points[0].X;
+                element->elementdata.rect.Height = points[1].Y - points[0].Y;
+                return Ok;
+            }
+
+            /* We can't rotate/shear a rectangle, so convert it to a path. */
             stat = GdipCreatePath(FillModeAlternate, &path);
             if (stat == Ok)
             {
