@@ -24,7 +24,11 @@
 #include "gdiplus.h"
 #include "wine/test.h"
 
-#define expect(expected, got) ok(got == expected, "Expected %.8x, got %.8x\n", expected, got)
+#define expect(expected,got) expect_(__LINE__, expected, got)
+static inline void expect_(unsigned line, DWORD expected, DWORD got)
+{
+    ok_(__FILE__, line)(expected == got, "Expected %.8ld, got %.8ld\n", expected, got);
+}
 #define expectf(expected, got) ok(fabs(got - expected) < 0.1, "Expected %.2f, got %.2f\n", expected, got)
 
 static void test_startup(void)
@@ -175,7 +179,7 @@ static void test_dasharray(void)
     dashes[4] = 14.0;
     dashes[5] = -100.0;
     dashes[6] = -100.0;
-    dashes[7] = dashes[8] = dashes[9] = dashes[10] = dashes[11] = 0.0;
+    dashes[7] = dashes[8] = dashes[9] = dashes[10] = dashes[11] = 1.0;
 
     /* setting the array sets the type to custom */
     GdipGetPenDashStyle(pen, &style);
@@ -218,10 +222,13 @@ static void test_dasharray(void)
 
     /* Some invalid array values. */
     status = GdipSetPenDashArray(pen, &dashes[7], 5);
-    expect(InvalidParameter, status);
+    expect(Ok, status);
     dashes[9] = -1.0;
     status = GdipSetPenDashArray(pen, &dashes[7], 5);
     expect(InvalidParameter, status);
+    dashes[9] = 0.0;
+    status = GdipSetPenDashArray(pen, &dashes[7], 5);
+    expect(InvalidParameter, status);
 
     /* Try to set with count = 0. */
     GdipSetPenDashStyle(pen, DashStyleDot);
@@ -343,7 +350,11 @@ static void test_compoundarray(void)
 {
     GpStatus status;
     GpPen *pen;
+    REAL *returnvalues;
     static const REAL testvalues[] = {0.2, 0.4, 0.6, 0.8};
+    static const REAL notSortedValues[] = {0.2, 0.6, 0.4, 0.8};
+    static const REAL negativeValues[] = {-1.2, 0.4, 0.6, 0.8};
+    static const REAL tooLargeValues[] = {0.2, 0.4, 0.6, 2.8};
     INT count;
 
     status = GdipSetPenCompoundArray(NULL, testvalues, 4);
@@ -360,10 +371,9 @@ static void test_compoundarray(void)
 
     count = 10;
     status = GdipGetPenCompoundCount(pen, &count);
-todo_wine {
     expect(Ok, status);
     ok(count == 0, "Unexpected compound count %d\n", count);
-}
+
     status = GdipSetPenCompoundArray(pen, NULL, 0);
     expect(InvalidParameter, status);
     status = GdipSetPenCompoundArray(pen, NULL, 4);
@@ -375,20 +385,62 @@ todo_wine {
     status = GdipSetPenCompoundArray(pen, testvalues, -2);
     expect(InvalidParameter, status);
 
+    status = GdipSetPenCompoundArray(pen, notSortedValues, 4);
+    expect(InvalidParameter, status);
+    status = GdipSetPenCompoundArray(pen, negativeValues, 4);
+    expect(InvalidParameter, status);
+    status = GdipSetPenCompoundArray(pen, tooLargeValues, 4);
+    expect(InvalidParameter, status);
+
     status = GdipSetPenCompoundArray(pen, testvalues, 4);
-    todo_wine expect(Ok, status);
+    expect(Ok, status);
     status = GdipSetPenCompoundArray(pen, NULL, 0);
     expect(InvalidParameter, status);
 
     count = 0;
     status = GdipGetPenCompoundCount(pen, &count);
-todo_wine {
     expect(Ok, status);
     ok(count == 4, "Unexpected compound count %d\n", count);
-}
+
+    returnvalues = calloc(5, sizeof(REAL));
+    /* When count larger than stored array return error */
+    status = GdipGetPenCompoundArray(pen, returnvalues, 40);
+    expect(InvalidParameter, status);
+    status = GdipGetPenCompoundArray(NULL, returnvalues, 4);
+    expect(InvalidParameter, status);
+    /* When count is zero, it should do nothing */
+    status = GdipGetPenCompoundArray(pen, returnvalues, 0);
+    expect(Ok, status);
+    ok(returnvalues[0] == 0.0, "Unexpected compound array %f\n", returnvalues[0]);
+
+    status = GdipGetPenCompoundArray(pen, returnvalues, 4);
+    expect(Ok, status);
+    ok(memcmp(returnvalues, testvalues, 4 * sizeof(REAL)) == 0, "Unexpected compound array\n");
+
+    status = GdipGetPenCompoundArray(pen, returnvalues, -10);
+    expect(Ok, status);
+    ok(memcmp(returnvalues, testvalues, 4 * sizeof(REAL)) == 0, "Unexpected compound array\n");
+
+    free(returnvalues);
     GdipDeletePen(pen);
 }
 
+static void get_pen_transform(GpPen *pen, REAL *values)
+{
+    GpMatrix *matrix;
+    GpStatus status;
+
+    status = GdipCreateMatrix(&matrix);
+    expect(Ok, status);
+
+    status = GdipGetPenTransform(pen, matrix);
+    expect(Ok, status);
+    status = GdipGetMatrixElements(matrix, values);
+    expect(Ok, status);
+
+    GdipDeleteMatrix(matrix);
+}
+
 static void test_transform(void)
 {
     GpStatus status;
@@ -431,6 +483,38 @@ static void test_transform(void)
     expectf(6.0, values[4]);
     expectf(3.0,  values[5]);
 
+    /* Translate */
+    status = GdipTranslatePenTransform(NULL, 1.0, -2.0, MatrixOrderAppend);
+    expect(InvalidParameter, status);
+
+    status = GdipTranslatePenTransform(pen, 1.0, -2.0, MatrixOrderAppend);
+    expect(Ok, status);
+
+    status = GdipGetPenTransform(pen, matrix);
+    expect(Ok, status);
+    status = GdipGetMatrixElements(matrix, values);
+    expect(Ok, status);
+    expectf(3.0,  values[0]);
+    expectf(-2.0,  values[1]);
+    expectf(5.0,  values[2]);
+    expectf(2.0, values[3]);
+    expectf(7.0, values[4]);
+    expectf(1.0,  values[5]);
+
+    status = GdipTranslatePenTransform(pen, -3.0, 5.0, MatrixOrderPrepend);
+    expect(Ok, status);
+
+    status = GdipGetPenTransform(pen, matrix);
+    expect(Ok, status);
+    status = GdipGetMatrixElements(matrix, values);
+    expect(Ok, status);
+    expectf(3.0,  values[0]);
+    expectf(-2.0,  values[1]);
+    expectf(5.0,  values[2]);
+    expectf(2.0, values[3]);
+    expectf(23.0, values[4]);
+    expectf(17.0,  values[5]);
+
     status = GdipResetPenTransform(pen);
     expect(Ok, status);
 
@@ -446,6 +530,138 @@ static void test_transform(void)
     expectf(0.0, values[4]);
     expectf(0.0, values[5]);
 
+    /* Scale */
+    status = GdipScalePenTransform(NULL, 1.0, 1.0, MatrixOrderPrepend);
+    expect(InvalidParameter, status);
+
+    status = GdipScalePenTransform(pen, 1.0, 1.0, MatrixOrderPrepend);
+    expect(Ok, status);
+
+    get_pen_transform(pen, values);
+    expectf(1.0, values[0]);
+    expectf(0.0, values[1]);
+    expectf(0.0, values[2]);
+    expectf(1.0, values[3]);
+    expectf(0.0, values[4]);
+    expectf(0.0, values[5]);
+
+    status = GdipScalePenTransform(pen, 2.0, -10.0, MatrixOrderPrepend);
+    expect(Ok, status);
+
+    get_pen_transform(pen, values);
+    expectf(2.0, values[0]);
+    expectf(0.0, values[1]);
+    expectf(0.0, values[2]);
+    expectf(-10.0, values[3]);
+    expectf(0.0, values[4]);
+    expectf(0.0, values[5]);
+
+    status = GdipScalePenTransform(pen, 2.0, -10.0, MatrixOrderAppend);
+    expect(Ok, status);
+
+    get_pen_transform(pen, values);
+    expectf(4.0, values[0]);
+    expectf(0.0, values[1]);
+    expectf(0.0, values[2]);
+    expectf(100.0, values[3]);
+    expectf(0.0, values[4]);
+    expectf(0.0, values[5]);
+
+    status = GdipTranslatePenTransform(pen, 1.0, -2.0, MatrixOrderAppend);
+    expect(Ok, status);
+
+    get_pen_transform(pen, values);
+    expectf(4.0, values[0]);
+    expectf(0.0, values[1]);
+    expectf(0.0, values[2]);
+    expectf(100.0, values[3]);
+    expectf(1.0, values[4]);
+    expectf(-2.0, values[5]);
+
+    status = GdipScalePenTransform(pen, 2.0, -10.0, MatrixOrderPrepend);
+    expect(Ok, status);
+
+    get_pen_transform(pen, values);
+    expectf(8.0, values[0]);
+    expectf(0.0, values[1]);
+    expectf(0.0, values[2]);
+    expectf(-1000.0, values[3]);
+    expectf(1.0, values[4]);
+    expectf(-2.0, values[5]);
+
+    /* Multiply */
+    status = GdipResetPenTransform(pen);
+    expect(Ok, status);
+
+    status = GdipSetMatrixElements(matrix, 2.0, 1.0, 1.0, 4.0, 1.0, 2.0);
+    expect(Ok, status);
+
+    status = GdipMultiplyPenTransform(NULL, matrix, MatrixOrderPrepend);
+    expect(InvalidParameter, status);
+
+    status = GdipMultiplyPenTransform(pen, matrix, MatrixOrderPrepend);
+    expect(Ok, status);
+
+    get_pen_transform(pen, values);
+    expectf(2.0, values[0]);
+    expectf(1.0, values[1]);
+    expectf(1.0, values[2]);
+    expectf(4.0, values[3]);
+    expectf(1.0, values[4]);
+    expectf(2.0, values[5]);
+
+    status = GdipScalePenTransform(pen, 2.0, -10.0, MatrixOrderAppend);
+    expect(Ok, status);
+
+    status = GdipMultiplyPenTransform(pen, matrix, MatrixOrderAppend);
+    expect(Ok, status);
+
+    get_pen_transform(pen, values);
+    expectf(-2.0, values[0]);
+    expectf(-36.0, values[1]);
+    expectf(-36.0, values[2]);
+    expectf(-158.0, values[3]);
+    expectf(-15.0, values[4]);
+    expectf(-76.0, values[5]);
+
+    /* Rotate */
+    status = GdipResetPenTransform(pen);
+    expect(Ok, status);
+
+    status = GdipSetMatrixElements(matrix, 2.0, 1.0, 1.0, 4.0, 1.0, 2.0);
+    expect(Ok, status);
+
+    status = GdipSetPenTransform(pen, matrix);
+    expect(Ok, status);
+
+    status = GdipRotatePenTransform(NULL, 10.0, MatrixOrderPrepend);
+    expect(InvalidParameter, status);
+
+    status = GdipRotatePenTransform(pen, 45.0, MatrixOrderPrepend);
+    expect(Ok, status);
+
+    get_pen_transform(pen, values);
+    expectf(2.12, values[0]);
+    expectf(3.54, values[1]);
+    expectf(-0.71, values[2]);
+    expectf(2.12, values[3]);
+    expectf(1.0, values[4]);
+    expectf(2.0, values[5]);
+
+    status = GdipScalePenTransform(pen, 2.0, -10.0, MatrixOrderAppend);
+    expect(Ok, status);
+
+    status = GdipRotatePenTransform(pen, 180.0, MatrixOrderAppend);
+    expect(Ok, status);
+
+    get_pen_transform(pen, values);
+    expectf(-4.24, values[0]);
+    expectf(35.36, values[1]);
+    expectf(1.41, values[2]);
+    expectf(21.21, values[3]);
+    expectf(-2.0, values[4]);
+    expectf(20.0, values[5]);
+
     GdipDeletePen(pen);
 
     GdipDeleteMatrix(matrix);
