@@ -24,15 +24,35 @@
 #include <winbase.h>
 #include <winver.h>
 #include <winnt.h>
+#include <winuser.h>
 #include <imagehlp.h>
 
 #include "wine/test.h"
 
-static HMODULE hImageHlp;
+static char *load_resource(const char *name)
+{
+    static char path[MAX_PATH];
+    DWORD written;
+    HANDLE file;
+    HRSRC res;
+    void *ptr;
 
-static BOOL (WINAPI *pImageGetDigestStream)(HANDLE, DWORD, DIGEST_FUNCTION, DIGEST_HANDLE);
-static BOOL (WINAPI *pBindImageEx)(DWORD Flags, const char *ImageName, const char *DllPath,
-                                   const char *SymbolPath, PIMAGEHLP_STATUS_ROUTINE StatusRoutine);
+    GetTempPathA(ARRAY_SIZE(path), path);
+    strcat(path, name);
+
+    file = CreateFileA(path, GENERIC_READ|GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, 0);
+    ok(file != INVALID_HANDLE_VALUE, "Failed to create file %s, error %lu.\n",
+            debugstr_a(path), GetLastError());
+
+    res = FindResourceA(NULL, name, "TESTDLL");
+    ok(!!res, "Failed to load resource, error %lu.\n", GetLastError());
+    ptr = LockResource(LoadResource(GetModuleHandleA(NULL), res));
+    WriteFile(file, ptr, SizeofResource( GetModuleHandleA(NULL), res), &written, NULL);
+    ok(written == SizeofResource(GetModuleHandleA(NULL), res), "Failed to write resource.\n");
+    CloseHandle(file);
+
+    return path;
+}
 
 /* minimal PE file image */
 #define VA_START 0x400000
@@ -151,9 +171,6 @@ struct expected_update_accum
     BOOL  todo;
 };
 
-static int status_routine_called[BindSymbolsNotUpdated+1];
-
-
 static BOOL WINAPI accumulating_stream_output(DIGEST_HANDLE handle, BYTE *pb,
  DWORD cb)
 {
@@ -187,15 +204,15 @@ static void check_updates(LPCSTR header, const struct expected_update_accum *exp
     DWORD i;
 
     todo_wine_if (expected->todo)
-        ok(expected->cUpdates == got->cUpdates, "%s: expected %d updates, got %d\n",
+        ok(expected->cUpdates == got->cUpdates, "%s: expected %ld updates, got %ld\n",
             header, expected->cUpdates, got->cUpdates);
     for (i = 0; i < min(expected->cUpdates, got->cUpdates); i++)
     {
-        ok(expected->updates[i].cb == got->updates[i].cb, "%s, update %d: expected %d bytes, got %d\n",
+        ok(expected->updates[i].cb == got->updates[i].cb, "%s, update %ld: expected %ld bytes, got %ld\n",
                 header, i, expected->updates[i].cb, got->updates[i].cb);
         if (expected->updates[i].cb && expected->updates[i].cb == got->updates[i].cb)
             ok(!memcmp(expected->updates[i].pb, got->updates[i].pb, got->updates[i].cb),
-                    "%s, update %d: unexpected value\n", header, i);
+                    "%s, update %ld: unexpected value\n", header, i);
     }
 }
 
@@ -276,40 +293,6 @@ static void update_checksum(void)
     bin.nt_headers.OptionalHeader.CheckSum = sum;
 }
 
-static BOOL CALLBACK testing_status_routine(IMAGEHLP_STATUS_REASON reason, const char *ImageName,
-                                            const char *DllName, ULONG_PTR Va, ULONG_PTR Parameter)
-{
-    char kernel32_path[MAX_PATH];
-
-    if (0 <= (int)reason && reason <= BindSymbolsNotUpdated)
-      status_routine_called[reason]++;
-    else
-      ok(0, "expected reason between 0 and %d, got %d\n", BindSymbolsNotUpdated+1, reason);
-
-    switch(reason)
-    {
-        case BindImportModule:
-            ok(!strcmp(DllName, "KERNEL32.DLL"), "expected DllName to be KERNEL32.DLL, got %s\n",
-               DllName);
-            break;
-
-        case BindImportProcedure:
-        case BindForwarderNOT:
-            GetSystemDirectoryA(kernel32_path, MAX_PATH);
-            strcat(kernel32_path, "\\KERNEL32.DLL");
-            ok(!lstrcmpiA(DllName, kernel32_path), "expected DllName to be %s, got %s\n",
-               kernel32_path, DllName);
-            ok(!strcmp((char *)Parameter, "ExitProcess"),
-               "expected Parameter to be ExitProcess, got %s\n", (char *)Parameter);
-            break;
-
-        default:
-            ok(0, "got unexpected reason %d\n", reason);
-            break;
-    }
-    return TRUE;
-}
-
 static void test_get_digest_stream(void)
 {
     BOOL ret;
@@ -318,15 +301,10 @@ static void test_get_digest_stream(void)
     DWORD count;
     struct update_accum accum = { 0, NULL };
 
-    if (!pImageGetDigestStream)
-    {
-        win_skip("ImageGetDigestStream function is not available\n");
-        return;
-    }
     SetLastError(0xdeadbeef);
-    ret = pImageGetDigestStream(NULL, 0, NULL, NULL);
+    ret = ImageGetDigestStream(NULL, 0, NULL, NULL);
     ok(!ret && GetLastError() == ERROR_INVALID_PARAMETER,
-     "expected ERROR_INVALID_PARAMETER, got %d\n", GetLastError());
+     "expected ERROR_INVALID_PARAMETER, got %ld\n", GetLastError());
     file = create_temp_file(temp_file);
     if (file == INVALID_HANDLE_VALUE)
     {
@@ -334,18 +312,18 @@ static void test_get_digest_stream(void)
         return;
     }
     SetLastError(0xdeadbeef);
-    ret = pImageGetDigestStream(file, 0, NULL, NULL);
+    ret = ImageGetDigestStream(file, 0, NULL, NULL);
     ok(!ret && GetLastError() == ERROR_INVALID_PARAMETER,
-     "expected ERROR_INVALID_PARAMETER, got %d\n", GetLastError());
+     "expected ERROR_INVALID_PARAMETER, got %ld\n", GetLastError());
     SetLastError(0xdeadbeef);
-    ret = pImageGetDigestStream(NULL, 0, accumulating_stream_output, &accum);
+    ret = ImageGetDigestStream(NULL, 0, accumulating_stream_output, &accum);
     ok(!ret && GetLastError() == ERROR_INVALID_PARAMETER,
-     "expected ERROR_INVALID_PARAMETER, got %d\n", GetLastError());
+     "expected ERROR_INVALID_PARAMETER, got %ld\n", GetLastError());
     /* Even with "valid" parameters, it fails with an empty file */
     SetLastError(0xdeadbeef);
-    ret = pImageGetDigestStream(file, 0, accumulating_stream_output, &accum);
+    ret = ImageGetDigestStream(file, 0, accumulating_stream_output, &accum);
     ok(!ret && GetLastError() == ERROR_INVALID_PARAMETER,
-     "expected ERROR_INVALID_PARAMETER, got %d\n", GetLastError());
+     "expected ERROR_INVALID_PARAMETER, got %ld\n", GetLastError());
     /* Finally, with a valid executable in the file, it succeeds.  Note that
      * the file pointer need not be positioned at the beginning.
      */
@@ -358,40 +336,99 @@ static void test_get_digest_stream(void)
     bin.nt_headers.OptionalHeader.SizeOfInitializedData = 0;
     bin.nt_headers.OptionalHeader.SizeOfImage = 0;
 
-    ret = pImageGetDigestStream(file, 0, accumulating_stream_output, &accum);
-    ok(ret, "ImageGetDigestStream failed: %d\n", GetLastError());
+    ret = ImageGetDigestStream(file, 0, accumulating_stream_output, &accum);
+    ok(ret, "ImageGetDigestStream failed: %ld\n", GetLastError());
     check_updates("flags = 0", &a1, &accum);
     free_updates(&accum);
-    ret = pImageGetDigestStream(file, CERT_PE_IMAGE_DIGEST_ALL_IMPORT_INFO,
+    ret = ImageGetDigestStream(file, CERT_PE_IMAGE_DIGEST_ALL_IMPORT_INFO,
      accumulating_stream_output, &accum);
-    ok(ret, "ImageGetDigestStream failed: %d\n", GetLastError());
+    ok(ret, "ImageGetDigestStream failed: %ld\n", GetLastError());
     check_updates("flags = CERT_PE_IMAGE_DIGEST_ALL_IMPORT_INFO", &a2, &accum);
     free_updates(&accum);
     CloseHandle(file);
     DeleteFileA(temp_file);
 }
 
+static unsigned int got_SysAllocString, got_GetOpenFileNameA, got_SHRegGetIntW;
+
+static BOOL WINAPI bind_image_cb(IMAGEHLP_STATUS_REASON reason, const char *file,
+        const char *module, ULONG_PTR va, ULONG_PTR param)
+{
+    static char last_module[MAX_PATH];
+
+    if (winetest_debug > 1)
+        trace("reason %u, file %s, module %s, va %#Ix, param %#Ix\n",
+                reason, debugstr_a(file), debugstr_a(module), va, param);
+
+    if (reason == BindImportModule)
+    {
+        ok(!strchr(module, '\\'), "got module name %s\n", debugstr_a(module));
+        strcpy(last_module, module);
+        ok(!va, "got VA %#Ix\n", va);
+        ok(!param, "got param %#Ix\n", param);
+    }
+    else if (reason == BindImportProcedure)
+    {
+        char full_path[MAX_PATH];
+        BOOL ret;
+
+        todo_wine ok(!!va, "expected nonzero VA\n");
+        ret = SearchPathA(NULL, last_module, ".dll", sizeof(full_path), full_path, NULL);
+        ok(ret, "got error %lu\n", GetLastError());
+        ok(!strcmp(module, full_path), "expected %s, got %s\n", debugstr_a(full_path), debugstr_a(module));
+
+        if (!strcmp((const char *)param, "SysAllocString"))
+        {
+            ok(!strcmp(last_module, "oleaut32.dll"), "got wrong module %s\n", debugstr_a(module));
+            ++got_SysAllocString;
+        }
+        else if (!strcmp((const char *)param, "GetOpenFileNameA"))
+        {
+            ok(!strcmp(last_module, "comdlg32.dll"), "got wrong module %s\n", debugstr_a(module));
+            ++got_GetOpenFileNameA;
+        }
+        else if (!strcmp((const char *)param, "Ordinal117"))
+        {
+            ok(!strcmp(last_module, "shlwapi.dll"), "got wrong module %s\n", debugstr_a(module));
+            ++got_SHRegGetIntW;
+        }
+    }
+    else
+    {
+        ok(0, "got unexpected reason %#x\n", reason);
+    }
+    return TRUE;
+}
+
 static void test_bind_image_ex(void)
 {
+    const char *filename = load_resource("testdll.dll");
     BOOL ret;
-    HANDLE file;
-    char temp_file[MAX_PATH];
-    DWORD count;
 
-    if (!pBindImageEx)
-    {
-        win_skip("BindImageEx function is not available\n");
-        return;
-    }
-
-    /* call with a non-existent file */
     SetLastError(0xdeadbeef);
-    ret = pBindImageEx(BIND_NO_BOUND_IMPORTS | BIND_NO_UPDATE | BIND_ALL_IMAGES, "nonexistent.dll", 0, 0,
-                       testing_status_routine);
-    todo_wine ok(!ret && ((GetLastError() == ERROR_FILE_NOT_FOUND) ||
-                 (GetLastError() == ERROR_INVALID_PARAMETER)),
-                 "expected ERROR_FILE_NOT_FOUND or ERROR_INVALID_PARAMETER, got %d\n",
-                 GetLastError());
+    ret = BindImageEx(BIND_ALL_IMAGES | BIND_NO_BOUND_IMPORTS | BIND_NO_UPDATE,
+            "nonexistent.dll", 0, 0, bind_image_cb);
+    ok(!ret, "expected failure\n");
+    ok(GetLastError() == ERROR_FILE_NOT_FOUND || GetLastError() == ERROR_INVALID_PARAMETER,
+            "got error %lu\n", GetLastError());
+
+    ret = BindImageEx(BIND_ALL_IMAGES | BIND_NO_BOUND_IMPORTS | BIND_NO_UPDATE,
+            filename, NULL, NULL, bind_image_cb);
+    ok(ret, "got error %lu\n", GetLastError());
+    ok(got_SysAllocString == 1, "got %u imports of SysAllocString\n", got_SysAllocString);
+    ok(got_GetOpenFileNameA == 1, "got %u imports of GetOpenFileNameA\n", got_GetOpenFileNameA);
+    todo_wine ok(got_SHRegGetIntW == 1, "got %u imports of SHRegGetIntW\n", got_SHRegGetIntW);
+
+    ret = DeleteFileA(filename);
+    ok(ret, "got error %lu\n", GetLastError());
+}
+
+static void test_image_load(void)
+{
+    char temp_file[MAX_PATH];
+    PLOADED_IMAGE img;
+    DWORD ret, count;
+    HANDLE file;
 
     file = create_temp_file(temp_file);
     if (file == INVALID_HANDLE_VALUE)
@@ -403,43 +440,68 @@ static void test_bind_image_ex(void)
     WriteFile(file, &bin, sizeof(bin), &count, NULL);
     CloseHandle(file);
 
-    /* call with a proper PE file, but with StatusRoutine set to NULL */
-    ret = pBindImageEx(BIND_NO_BOUND_IMPORTS | BIND_NO_UPDATE | BIND_ALL_IMAGES, temp_file, 0, 0,
-                       NULL);
-    ok(ret, "BindImageEx failed: %d\n", GetLastError());
+    img = ImageLoad(temp_file, NULL);
+    ok(img != NULL, "ImageLoad unexpectedly failed\n");
 
-    /* call with a proper PE file and StatusRoutine */
-    ret = pBindImageEx(BIND_NO_BOUND_IMPORTS | BIND_NO_UPDATE | BIND_ALL_IMAGES, temp_file, 0, 0,
-                       testing_status_routine);
-    ok(ret, "BindImageEx failed: %d\n", GetLastError());
+    if (img)
+    {
+        ok(!strcmp(img->ModuleName, temp_file),
+           "unexpected ModuleName, got %s instead of %s\n", img->ModuleName, temp_file);
+        ok(img->MappedAddress != NULL, "MappedAddress != NULL\n");
+        if (img->MappedAddress)
+        {
+            ok(!memcmp(img->MappedAddress, &bin.dos_header, sizeof(bin.dos_header)),
+               "MappedAddress doesn't point to IMAGE_DOS_HEADER\n");
+        }
+        ok(img->FileHeader != NULL, "FileHeader != NULL\n");
+        if (img->FileHeader)
+        {
+            ok(!memcmp(img->FileHeader, &bin.nt_headers, sizeof(bin.nt_headers)),
+                "FileHeader doesn't point to IMAGE_NT_HEADERS32\n");
+        }
+        ok(img->NumberOfSections == 3,
+           "unexpected NumberOfSections, got %ld instead of 3\n", img->NumberOfSections);
+        if (img->NumberOfSections >= 3)
+        {
+            ok(!strcmp((const char *)img->Sections[0].Name, ".text"),
+               "unexpected name for section 0, expected .text, got %s\n",
+               (const char *)img->Sections[0].Name);
+            ok(!strcmp((const char *)img->Sections[1].Name, ".bss"),
+               "unexpected name for section 1, expected .bss, got %s\n",
+               (const char *)img->Sections[1].Name);
+            ok(!strcmp((const char *)img->Sections[2].Name, ".idata"),
+               "unexpected name for section 2, expected .idata, got %s\n",
+               (const char *)img->Sections[2].Name);
+        }
+        ok(img->Characteristics == 0x102,
+           "unexpected Characteristics, got 0x%lx instead of 0x102\n", img->Characteristics);
+        ok(img->fSystemImage == 0,
+           "unexpected fSystemImage, got %d instead of 0\n", img->fSystemImage);
+        ok(img->fDOSImage == 0,
+           "unexpected fDOSImage, got %d instead of 0\n", img->fDOSImage);
+        todo_wine ok(img->fReadOnly == 1 || broken(!img->fReadOnly) /* <= WinXP */,
+           "unexpected fReadOnly, got %d instead of 1\n", img->fReadOnly);
+        todo_wine ok(img->Version == 1 || broken(!img->Version) /* <= WinXP */,
+           "unexpected Version, got %d instead of 1\n", img->Version);
+        ok(img->SizeOfImage == 0x600,
+           "unexpected SizeOfImage, got 0x%lx instead of 0x600\n", img->SizeOfImage);
 
-    todo_wine ok(status_routine_called[BindImportModule] == 1,
-                 "StatusRoutine was called %d times\n", status_routine_called[BindImportModule]);
+        count = 0xdeadbeef;
+        ret = GetImageUnusedHeaderBytes(img, &count);
+        todo_wine
+        ok(ret == 448, "GetImageUnusedHeaderBytes returned %lu instead of 448\n", ret);
+        todo_wine
+        ok(count == 64, "unexpected size for unused header bytes, got %lu instead of 64\n", count);
 
-    todo_wine ok((status_routine_called[BindImportProcedure] == 1)
-#if defined(_WIN64)
-                 || broken(status_routine_called[BindImportProcedure] == 0) /* < Win8 */
-#endif
-                 , "StatusRoutine was called %d times\n", status_routine_called[BindImportProcedure]);
+        ImageUnload(img);
+    }
 
     DeleteFileA(temp_file);
 }
 
 START_TEST(image)
 {
-    hImageHlp = LoadLibraryA("imagehlp.dll");
-
-    if (!hImageHlp)
-    {
-        win_skip("ImageHlp unavailable\n");
-        return;
-    }
-
-    pImageGetDigestStream = (void *) GetProcAddress(hImageHlp, "ImageGetDigestStream");
-    pBindImageEx = (void *) GetProcAddress(hImageHlp, "BindImageEx");
-
     test_get_digest_stream();
     test_bind_image_ex();
-
-    FreeLibrary(hImageHlp);
+    test_image_load();
 }
