@@ -26,23 +26,15 @@
 #include "mfidl.h"
 #include "rpcproxy.h"
 
-#include "initguid.h"
-#include "mf.h"
-
-#undef INITGUID
-#include <guiddef.h>
-#include "mfapi.h"
-#include "mferror.h"
-
 #include "mf_private.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 #include "wine/list.h"
+#include "wine/mfinternal.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mfplat);
 
-static HINSTANCE mf_instance;
+extern const GUID CLSID_FileSchemePlugin;
 
 struct activate_object
 {
@@ -82,7 +74,7 @@ static ULONG WINAPI activate_object_AddRef(IMFActivate *iface)
     struct activate_object *activate = impl_from_IMFActivate(iface);
     ULONG refcount = InterlockedIncrement(&activate->refcount);
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -92,15 +84,16 @@ static ULONG WINAPI activate_object_Release(IMFActivate *iface)
     struct activate_object *activate = impl_from_IMFActivate(iface);
     ULONG refcount = InterlockedDecrement(&activate->refcount);
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
-        activate->funcs->free_private(activate->context);
+        if (activate->funcs->free_private)
+            activate->funcs->free_private(activate->context);
         if (activate->object)
             IUnknown_Release(activate->object);
         IMFAttributes_Release(activate->attributes);
-        heap_free(activate);
+        free(activate);
     }
 
     return refcount;
@@ -402,14 +395,23 @@ static HRESULT WINAPI activate_object_ActivateObject(IMFActivate *iface, REFIID
 
 static HRESULT WINAPI activate_object_ShutdownObject(IMFActivate *iface)
 {
-    FIXME("%p.\n", iface);
+    struct activate_object *activate = impl_from_IMFActivate(iface);
+    IUnknown *object;
 
-    return E_NOTIMPL;
+    TRACE("%p.\n", iface);
+
+    if ((object = InterlockedCompareExchangePointer((void **)&activate->object, NULL, activate->object)))
+    {
+        activate->funcs->shutdown_object(activate->context, object);
+        IUnknown_Release(object);
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI activate_object_DetachObject(IMFActivate *iface)
 {
-    FIXME("%p.\n", iface);
+    TRACE("%p.\n", iface);
 
     return E_NOTIMPL;
 }
@@ -459,15 +461,14 @@ HRESULT create_activation_object(void *context, const struct activate_funcs *fun
     struct activate_object *object;
     HRESULT hr;
 
-    object = heap_alloc_zero(sizeof(*object));
-    if (!object)
+    if (!(object = calloc(1, sizeof(*object))))
         return E_OUTOFMEMORY;
 
     object->IMFActivate_iface.lpVtbl = &activate_object_vtbl;
     object->refcount = 1;
     if (FAILED(hr = MFCreateAttributes(&object->attributes, 0)))
     {
-        heap_free(object);
+        free(object);
         return hr;
     }
     object->funcs = funcs;
@@ -597,7 +598,7 @@ static ULONG WINAPI file_scheme_handler_AddRef(IMFSchemeHandler *iface)
     struct file_scheme_handler *handler = impl_from_IMFSchemeHandler(iface);
     ULONG refcount = InterlockedIncrement(&handler->refcount);
 
-    TRACE("%p, refcount %u.\n", handler, refcount);
+    TRACE("%p, refcount %lu.\n", handler, refcount);
 
     return refcount;
 }
@@ -608,7 +609,7 @@ static ULONG WINAPI file_scheme_handler_Release(IMFSchemeHandler *iface)
     ULONG refcount = InterlockedDecrement(&handler->refcount);
     struct file_scheme_handler_result *result, *next;
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -618,12 +619,12 @@ static ULONG WINAPI file_scheme_handler_Release(IMFSchemeHandler *iface)
             IMFAsyncResult_Release(result->result);
             if (result->object)
                 IUnknown_Release(result->object);
-            heap_free(result);
+            free(result);
         }
         DeleteCriticalSection(&handler->cs);
         if (handler->resolver)
             IMFSourceResolver_Release(handler->resolver);
-        heap_free(handler);
+        free(handler);
     }
 
     return refcount;
@@ -665,7 +666,7 @@ static ULONG WINAPI create_object_context_AddRef(IUnknown *iface)
     struct create_object_context *context = impl_from_IUnknown(iface);
     ULONG refcount = InterlockedIncrement(&context->refcount);
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -675,14 +676,14 @@ static ULONG WINAPI create_object_context_Release(IUnknown *iface)
     struct create_object_context *context = impl_from_IUnknown(iface);
     ULONG refcount = InterlockedDecrement(&context->refcount);
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
         if (context->props)
             IPropertyStore_Release(context->props);
-        heap_free(context->url);
-        heap_free(context);
+        free(context->url);
+        free(context);
     }
 
     return refcount;
@@ -695,23 +696,6 @@ static const IUnknownVtbl create_object_context_vtbl =
     create_object_context_Release,
 };
 
-static WCHAR *heap_strdupW(const WCHAR *str)
-{
-    WCHAR *ret = NULL;
-
-    if (str)
-    {
-        unsigned int size;
-
-        size = (lstrlenW(str) + 1) * sizeof(WCHAR);
-        ret = heap_alloc(size);
-        if (ret)
-            memcpy(ret, str, size);
-    }
-
-    return ret;
-}
-
 static HRESULT WINAPI file_scheme_handler_BeginCreateObject(IMFSchemeHandler *iface, const WCHAR *url, DWORD flags,
         IPropertyStore *props, IUnknown **cancel_cookie, IMFAsyncCallback *callback, IUnknown *state)
 {
@@ -720,7 +704,7 @@ static HRESULT WINAPI file_scheme_handler_BeginCreateObject(IMFSchemeHandler *if
     IMFAsyncResult *caller, *item;
     HRESULT hr;
 
-    TRACE("%p, %s, %#x, %p, %p, %p, %p.\n", iface, debugstr_w(url), flags, props, cancel_cookie, callback, state);
+    TRACE("%p, %s, %#lx, %p, %p, %p, %p.\n", iface, debugstr_w(url), flags, props, cancel_cookie, callback, state);
 
     if (cancel_cookie)
         *cancel_cookie = NULL;
@@ -728,8 +712,7 @@ static HRESULT WINAPI file_scheme_handler_BeginCreateObject(IMFSchemeHandler *if
     if (FAILED(hr = MFCreateAsyncResult(NULL, callback, state, &caller)))
         return hr;
 
-    context = heap_alloc(sizeof(*context));
-    if (!context)
+    if (!(context = malloc(sizeof(*context))))
     {
         IMFAsyncResult_Release(caller);
         return E_OUTOFMEMORY;
@@ -741,7 +724,7 @@ static HRESULT WINAPI file_scheme_handler_BeginCreateObject(IMFSchemeHandler *if
     if (context->props)
         IPropertyStore_AddRef(context->props);
     context->flags = flags;
-    context->url = heap_strdupW(url);
+    context->url = wcsdup(url);
     if (!context->url)
     {
         IMFAsyncResult_Release(caller);
@@ -751,17 +734,20 @@ static HRESULT WINAPI file_scheme_handler_BeginCreateObject(IMFSchemeHandler *if
 
     hr = MFCreateAsyncResult(&context->IUnknown_iface, &handler->IMFAsyncCallback_iface, (IUnknown *)caller, &item);
     IUnknown_Release(&context->IUnknown_iface);
-    IMFAsyncResult_Release(caller);
     if (SUCCEEDED(hr))
     {
         if (SUCCEEDED(hr = MFPutWorkItemEx(MFASYNC_CALLBACK_QUEUE_IO, item)))
         {
             if (cancel_cookie)
-                IMFAsyncResult_GetState(item, cancel_cookie);
+            {
+                *cancel_cookie = (IUnknown *)caller;
+                IUnknown_AddRef(*cancel_cookie);
+            }
         }
 
         IMFAsyncResult_Release(item);
     }
+    IMFAsyncResult_Release(caller);
 
     return hr;
 }
@@ -795,7 +781,7 @@ static HRESULT WINAPI file_scheme_handler_EndCreateObject(IMFSchemeHandler *ifac
         *object = found->object;
         hr = IMFAsyncResult_GetStatus(found->result);
         IMFAsyncResult_Release(found->result);
-        heap_free(found);
+        free(found);
     }
     else
     {
@@ -833,7 +819,7 @@ static HRESULT WINAPI file_scheme_handler_CancelObjectCreation(IMFSchemeHandler
         IMFAsyncResult_Release(found->result);
         if (found->object)
             IUnknown_Release(found->object);
-        heap_free(found);
+        free(found);
     }
 
     return found ? S_OK : MF_E_UNEXPECTED;
@@ -952,7 +938,7 @@ static HRESULT WINAPI file_scheme_handler_callback_Invoke(IMFAsyncCallback *ifac
         }
     }
 
-    handler_result = heap_alloc(sizeof(*handler_result));
+    handler_result = malloc(sizeof(*handler_result));
     if (handler_result)
     {
         handler_result->result = caller;
@@ -995,8 +981,7 @@ static HRESULT file_scheme_handler_construct(REFIID riid, void **obj)
 
     TRACE("%s, %p.\n", debugstr_guid(riid), obj);
 
-    handler = heap_alloc_zero(sizeof(*handler));
-    if (!handler)
+    if (!(handler = calloc(1, sizeof(*handler))))
         return E_OUTOFMEMORY;
 
     handler->IMFSchemeHandler_iface.lpVtbl = &file_scheme_handler_vtbl;
@@ -1020,7 +1005,7 @@ static const struct class_object
 }
 class_objects[] =
 {
-    { &CLSID_FileSchemeHandler, &file_scheme_handler_factory.IClassFactory_iface },
+    { &CLSID_FileSchemePlugin, &file_scheme_handler_factory.IClassFactory_iface },
 };
 
 /*******************************************************************************
@@ -1042,53 +1027,128 @@ HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void **obj)
     return CLASS_E_CLASSNOTAVAILABLE;
 }
 
-/******************************************************************
- *              DllCanUnloadNow (mf.@)
- */
-HRESULT WINAPI DllCanUnloadNow(void)
+static HRESULT prop_string_vector_append(PROPVARIANT *vector, unsigned int *capacity, BOOL unique, const WCHAR *str)
 {
-    return S_FALSE;
-}
+    WCHAR *ptrW;
+    int len, i;
 
-/***********************************************************************
- *          DllRegisterServer (mf.@)
- */
-HRESULT WINAPI DllRegisterServer(void)
-{
-    return __wine_register_resources( mf_instance );
-}
-
-/***********************************************************************
- *          DllUnregisterServer (mf.@)
- */
-HRESULT WINAPI DllUnregisterServer(void)
-{
-    return __wine_unregister_resources( mf_instance );
-}
-
-BOOL WINAPI DllMain(HINSTANCE instance, DWORD reason, LPVOID reserved)
-{
-    switch (reason)
+    if (unique)
     {
-        case DLL_WINE_PREATTACH:
-            return FALSE;    /* prefer native version */
-        case DLL_PROCESS_ATTACH:
-            mf_instance = instance;
-            DisableThreadLibraryCalls(instance);
-            break;
+        for (i = 0; i < vector->calpwstr.cElems; ++i)
+        {
+            if (!lstrcmpW(vector->calpwstr.pElems[i], str))
+                return S_OK;
+        }
     }
 
-    return TRUE;
+    if (!*capacity || *capacity - 1 < vector->calpwstr.cElems)
+    {
+        unsigned int new_count;
+        WCHAR **ptr;
+
+        new_count = *capacity ? *capacity * 2 : 10;
+        ptr = CoTaskMemRealloc(vector->calpwstr.pElems, new_count * sizeof(*vector->calpwstr.pElems));
+        if (!ptr)
+            return E_OUTOFMEMORY;
+        vector->calpwstr.pElems = ptr;
+        *capacity = new_count;
+    }
+
+    len = lstrlenW(str);
+    if (!(vector->calpwstr.pElems[vector->calpwstr.cElems] = ptrW = CoTaskMemAlloc((len + 1) * sizeof(WCHAR))))
+        return E_OUTOFMEMORY;
+
+    lstrcpyW(ptrW, str);
+    vector->calpwstr.cElems++;
+
+    return S_OK;
+}
+
+static int __cdecl qsort_string_compare(const void *a, const void *b)
+{
+    const WCHAR *left = *(const WCHAR **)a, *right = *(const WCHAR **)b;
+    return lstrcmpW(left, right);
+}
+
+static HRESULT mf_get_handler_strings(const WCHAR *path, WCHAR filter, unsigned int maxlen, PROPVARIANT *dst)
+{
+    static const HKEY hkey_roots[2] = { HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE };
+    unsigned int capacity = 0, count;
+    HRESULT hr = S_OK;
+    int i, index;
+    WCHAR *buffW;
+    DWORD size;
+
+    if (!(buffW = calloc(maxlen, sizeof(*buffW))))
+        return E_OUTOFMEMORY;
+
+    memset(dst, 0, sizeof(*dst));
+    dst->vt = VT_VECTOR | VT_LPWSTR;
+
+    for (i = 0; i < ARRAY_SIZE(hkey_roots); ++i)
+    {
+        HKEY hkey;
+
+        if (RegOpenKeyW(hkey_roots[i], path, &hkey))
+            continue;
+
+        index = 0;
+        size = maxlen;
+        count = dst->calpwstr.cElems;
+        while (!RegEnumKeyExW(hkey, index++, buffW, &size, NULL, NULL, NULL, NULL))
+        {
+            if (filter && !wcschr(buffW, filter))
+                continue;
+
+            if (FAILED(hr = prop_string_vector_append(dst, &capacity, i > 0, buffW)))
+                break;
+            size = maxlen;
+        }
+
+        /* Sort last pass results. */
+        qsort(&dst->calpwstr.pElems[count], dst->calpwstr.cElems - count, sizeof(*dst->calpwstr.pElems),
+                qsort_string_compare);
+
+        RegCloseKey(hkey);
+    }
+
+    if (FAILED(hr))
+        PropVariantClear(dst);
+
+    free(buffW);
+
+    return hr;
 }
 
 /***********************************************************************
  *      MFGetSupportedMimeTypes (mf.@)
  */
-HRESULT WINAPI MFGetSupportedMimeTypes(PROPVARIANT *array)
+HRESULT WINAPI MFGetSupportedMimeTypes(PROPVARIANT *dst)
 {
-    FIXME("(%p) stub\n", array);
+    unsigned int maxlen;
 
-    return E_NOTIMPL;
+    TRACE("%p.\n", dst);
+
+    if (!dst)
+        return E_POINTER;
+
+    /* According to RFC4288 it's 127/127 characters. */
+    maxlen = 127 /* type */ + 1 /* / */ + 127 /* subtype */ + 1;
+    return mf_get_handler_strings(L"Software\\Microsoft\\Windows Media Foundation\\ByteStreamHandlers", '/',
+            maxlen,  dst);
+}
+
+/***********************************************************************
+ *      MFGetSupportedSchemes (mf.@)
+ */
+HRESULT WINAPI MFGetSupportedSchemes(PROPVARIANT *dst)
+{
+    TRACE("%p.\n", dst);
+
+    if (!dst)
+        return E_POINTER;
+
+    return mf_get_handler_strings(L"Software\\Microsoft\\Windows Media Foundation\\SchemeHandlers", 0, 64, dst);
 }
 
 /***********************************************************************
@@ -1145,26 +1205,302 @@ HRESULT WINAPI MFEnumDeviceSources(IMFAttributes *attributes, IMFActivate ***sou
     return S_OK;
 }
 
-static HRESULT evr_create_object(IMFAttributes *attributes, void *user_context, IUnknown **obj)
+struct simple_type_handler
 {
-    FIXME("%p, %p, %p.\n", attributes, user_context, obj);
-
-    return E_NOTIMPL;
-}
-
-static void evr_free_private(void *user_context)
-{
-}
-
-static const struct activate_funcs evr_activate_funcs =
-{
-    evr_create_object,
-    evr_free_private,
+    IMFMediaTypeHandler IMFMediaTypeHandler_iface;
+    LONG refcount;
+    IMFMediaType *media_type;
+    CRITICAL_SECTION cs;
 };
 
-HRESULT WINAPI MFCreateVideoRendererActivate(HWND hwnd, IMFActivate **activate)
+static struct simple_type_handler *impl_from_IMFMediaTypeHandler(IMFMediaTypeHandler *iface)
 {
-    TRACE("%p, %p.\n", hwnd, activate);
+    return CONTAINING_RECORD(iface, struct simple_type_handler, IMFMediaTypeHandler_iface);
+}
 
-    return create_activation_object(hwnd, &evr_activate_funcs, activate);
+static HRESULT WINAPI simple_type_handler_QueryInterface(IMFMediaTypeHandler *iface, REFIID riid, void **obj)
+{
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(riid), obj);
+
+    if (IsEqualIID(riid, &IID_IMFMediaTypeHandler) ||
+            IsEqualIID(riid, &IID_IUnknown))
+    {
+        *obj = iface;
+        IMFMediaTypeHandler_AddRef(iface);
+        return S_OK;
+    }
+
+    *obj = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI simple_type_handler_AddRef(IMFMediaTypeHandler *iface)
+{
+    struct simple_type_handler *handler = impl_from_IMFMediaTypeHandler(iface);
+    ULONG refcount = InterlockedIncrement(&handler->refcount);
+
+    TRACE("%p, refcount %lu.\n", iface, refcount);
+
+    return refcount;
+}
+
+static ULONG WINAPI simple_type_handler_Release(IMFMediaTypeHandler *iface)
+{
+    struct simple_type_handler *handler = impl_from_IMFMediaTypeHandler(iface);
+    ULONG refcount = InterlockedDecrement(&handler->refcount);
+
+    TRACE("%p, refcount %lu.\n", iface, refcount);
+
+    if (!refcount)
+    {
+        if (handler->media_type)
+            IMFMediaType_Release(handler->media_type);
+        DeleteCriticalSection(&handler->cs);
+        free(handler);
+    }
+
+    return refcount;
+}
+
+static HRESULT WINAPI simple_type_handler_IsMediaTypeSupported(IMFMediaTypeHandler *iface, IMFMediaType *in_type,
+        IMFMediaType **out_type)
+{
+    struct simple_type_handler *handler = impl_from_IMFMediaTypeHandler(iface);
+    DWORD flags = 0;
+    HRESULT hr;
+
+    TRACE("%p, %p, %p.\n", iface, in_type, out_type);
+
+    if (out_type)
+        *out_type = NULL;
+
+    EnterCriticalSection(&handler->cs);
+    if (!handler->media_type)
+        hr = MF_E_UNEXPECTED;
+    else
+    {
+        if (SUCCEEDED(hr = IMFMediaType_IsEqual(handler->media_type, in_type, &flags)))
+            hr = (flags & (MF_MEDIATYPE_EQUAL_MAJOR_TYPES | MF_MEDIATYPE_EQUAL_FORMAT_TYPES)) ==
+                    (MF_MEDIATYPE_EQUAL_MAJOR_TYPES | MF_MEDIATYPE_EQUAL_FORMAT_TYPES) ? S_OK : E_FAIL;
+    }
+    LeaveCriticalSection(&handler->cs);
+
+    return hr;
+}
+
+static HRESULT WINAPI simple_type_handler_GetMediaTypeCount(IMFMediaTypeHandler *iface, DWORD *count)
+{
+    TRACE("%p, %p.\n", iface, count);
+
+    if (!count)
+        return E_POINTER;
+
+    *count = 1;
+
+    return S_OK;
+}
+
+static HRESULT WINAPI simple_type_handler_GetMediaTypeByIndex(IMFMediaTypeHandler *iface, DWORD index,
+        IMFMediaType **type)
+{
+    struct simple_type_handler *handler = impl_from_IMFMediaTypeHandler(iface);
+
+    TRACE("%p, %lu, %p.\n", iface, index, type);
+
+    if (index > 0)
+        return MF_E_NO_MORE_TYPES;
+
+    EnterCriticalSection(&handler->cs);
+    *type = handler->media_type;
+    if (*type)
+        IMFMediaType_AddRef(*type);
+    LeaveCriticalSection(&handler->cs);
+
+    return S_OK;
+}
+
+static HRESULT WINAPI simple_type_handler_SetCurrentMediaType(IMFMediaTypeHandler *iface, IMFMediaType *media_type)
+{
+    struct simple_type_handler *handler = impl_from_IMFMediaTypeHandler(iface);
+
+    TRACE("%p, %p.\n", iface, media_type);
+
+    EnterCriticalSection(&handler->cs);
+    if (handler->media_type)
+        IMFMediaType_Release(handler->media_type);
+    handler->media_type = media_type;
+    if (handler->media_type)
+        IMFMediaType_AddRef(handler->media_type);
+    LeaveCriticalSection(&handler->cs);
+
+    return S_OK;
+}
+
+static HRESULT WINAPI simple_type_handler_GetCurrentMediaType(IMFMediaTypeHandler *iface, IMFMediaType **media_type)
+{
+    struct simple_type_handler *handler = impl_from_IMFMediaTypeHandler(iface);
+
+    TRACE("%p, %p.\n", iface, media_type);
+
+    if (!media_type)
+        return E_POINTER;
+
+    EnterCriticalSection(&handler->cs);
+    *media_type = handler->media_type;
+    if (*media_type)
+        IMFMediaType_AddRef(*media_type);
+    LeaveCriticalSection(&handler->cs);
+
+    return S_OK;
+}
+
+static HRESULT WINAPI simple_type_handler_GetMajorType(IMFMediaTypeHandler *iface, GUID *type)
+{
+    struct simple_type_handler *handler = impl_from_IMFMediaTypeHandler(iface);
+    HRESULT hr;
+
+    TRACE("%p, %p.\n", iface, type);
+
+    EnterCriticalSection(&handler->cs);
+    if (handler->media_type)
+        hr = IMFMediaType_GetGUID(handler->media_type, &MF_MT_MAJOR_TYPE, type);
+    else
+        hr = MF_E_NOT_INITIALIZED;
+    LeaveCriticalSection(&handler->cs);
+
+    return hr;
+}
+
+static const IMFMediaTypeHandlerVtbl simple_type_handler_vtbl =
+{
+    simple_type_handler_QueryInterface,
+    simple_type_handler_AddRef,
+    simple_type_handler_Release,
+    simple_type_handler_IsMediaTypeSupported,
+    simple_type_handler_GetMediaTypeCount,
+    simple_type_handler_GetMediaTypeByIndex,
+    simple_type_handler_SetCurrentMediaType,
+    simple_type_handler_GetCurrentMediaType,
+    simple_type_handler_GetMajorType,
+};
+
+/*******************************************************************************
+ *      MFCreateSimpleTypeHandler (mf.@)
+ */
+HRESULT WINAPI MFCreateSimpleTypeHandler(IMFMediaTypeHandler **handler)
+{
+    struct simple_type_handler *object;
+
+    TRACE("%p.\n", handler);
+
+    if (!(object = calloc(1, sizeof(*object))))
+        return E_OUTOFMEMORY;
+
+    object->IMFMediaTypeHandler_iface.lpVtbl = &simple_type_handler_vtbl;
+    object->refcount = 1;
+    InitializeCriticalSection(&object->cs);
+
+    *handler = &object->IMFMediaTypeHandler_iface;
+
+    return S_OK;
+}
+
+/*******************************************************************************
+ *      MFRequireProtectedEnvironment (mf.@)
+ */
+HRESULT WINAPI MFRequireProtectedEnvironment(IMFPresentationDescriptor *pd)
+{
+    BOOL selected, protected = FALSE;
+    unsigned int i = 0, value;
+    IMFStreamDescriptor *sd;
+
+    TRACE("%p.\n", pd);
+
+    while (SUCCEEDED(IMFPresentationDescriptor_GetStreamDescriptorByIndex(pd, i++, &selected, &sd)))
+    {
+        value = 0;
+        protected = SUCCEEDED(IMFStreamDescriptor_GetUINT32(sd, &MF_SD_PROTECTED, &value)) && value;
+        IMFStreamDescriptor_Release(sd);
+        if (protected) break;
+    }
+
+    return protected ? S_OK : S_FALSE;
+}
+
+static HRESULT create_media_sink(const CLSID *clsid, IMFByteStream *stream, IMFMediaType *video_type,
+        IMFMediaType *audio_type, IMFMediaSink **sink)
+{
+    IMFSinkClassFactory *factory;
+    HRESULT hr;
+
+    if (FAILED(hr = CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, &IID_IMFSinkClassFactory, (void **)&factory)))
+        return hr;
+
+    hr = IMFSinkClassFactory_CreateMediaSink(factory, stream, video_type, audio_type, sink);
+    IMFSinkClassFactory_Release(factory);
+
+    return hr;
+}
+
+/*******************************************************************************
+ *      MFCreate3GPMediaSink (mf.@)
+ */
+HRESULT WINAPI MFCreate3GPMediaSink(IMFByteStream *stream, IMFMediaType *video_type,
+         IMFMediaType *audio_type, IMFMediaSink **sink)
+{
+    TRACE("%p, %p, %p, %p.\n", stream, video_type, audio_type, sink);
+
+    return create_media_sink(&CLSID_MF3GPSinkClassFactory, stream, video_type, audio_type, sink);
+}
+
+/*******************************************************************************
+ *      MFCreateAC3MediaSink (mf.@)
+ */
+HRESULT WINAPI MFCreateAC3MediaSink(IMFByteStream *stream, IMFMediaType *audio_type, IMFMediaSink **sink)
+{
+    TRACE("%p, %p, %p.\n", stream, audio_type, sink);
+
+    return create_media_sink(&CLSID_MFAC3SinkClassFactory, stream, NULL, audio_type, sink);
+}
+
+/*******************************************************************************
+ *      MFCreateADTSMediaSink (mf.@)
+ */
+HRESULT WINAPI MFCreateADTSMediaSink(IMFByteStream *stream, IMFMediaType *audio_type, IMFMediaSink **sink)
+{
+    TRACE("%p, %p, %p.\n", stream, audio_type, sink);
+
+    return create_media_sink(&CLSID_MFADTSSinkClassFactory, stream, NULL, audio_type, sink);
+}
+
+/*******************************************************************************
+ *      MFCreateMP3MediaSink (mf.@)
+ */
+HRESULT WINAPI MFCreateMP3MediaSink(IMFByteStream *stream, IMFMediaSink **sink)
+{
+    TRACE("%p, %p.\n", stream, sink);
+
+    return create_media_sink(&CLSID_MFMP3SinkClassFactory, stream, NULL, NULL, sink);
+}
+
+/*******************************************************************************
+ *      MFCreateMPEG4MediaSink (mf.@)
+ */
+HRESULT WINAPI MFCreateMPEG4MediaSink(IMFByteStream *stream, IMFMediaType *video_type,
+         IMFMediaType *audio_type, IMFMediaSink **sink)
+{
+    TRACE("%p, %p, %p, %p.\n", stream, video_type, audio_type, sink);
+
+    return create_media_sink(&CLSID_MFMPEG4SinkClassFactory, stream, video_type, audio_type, sink);
+}
+
+/*******************************************************************************
+ *      MFCreateFMPEG4MediaSink (mf.@)
+ */
+HRESULT WINAPI MFCreateFMPEG4MediaSink(IMFByteStream *stream, IMFMediaType *video_type,
+         IMFMediaType *audio_type, IMFMediaSink **sink)
+{
+    TRACE("%p, %p, %p, %p.\n", stream, video_type, audio_type, sink);
+
+    return create_media_sink(&CLSID_MFFMPEG4SinkClassFactory, stream, video_type, audio_type, sink);
 }
