@@ -33,14 +33,7 @@
 static BOOL (WINAPI *pIsWow64Process)(HANDLE, BOOL *);
 
 static const WCHAR software_renderW[] =
-    { 'S','o','f','t','w','a','r','e','\\',
-      'M','i','c','r','o','s','o','f','t','\\',
-      'W','i','n','d','o','w','s','\\',
-      'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\',
-      'M','M','D','e','v','i','c','e','s','\\',
-      'A','u','d','i','o','\\',
-      'R','e','n','d','e','r',0 };
-static const WCHAR propertiesW[] = {'P','r','o','p','e','r','t','i','e','s',0};
+    L"Software\\Microsoft\\Windows\\CurrentVersion\\MMDevices\\Audio\\Render";
 
 
 static void test_propertystore(IPropertyStore *store)
@@ -52,30 +45,30 @@ static void test_propertystore(IPropertyStore *store)
 
     pv.vt = VT_EMPTY;
     hr = IPropertyStore_GetValue(store, &PKEY_AudioEndpoint_GUID, &pv);
-    ok(hr == S_OK, "Failed with %08x\n", hr);
+    ok(hr == S_OK, "Failed with %08lx\n", hr);
     ok(pv.vt == VT_LPWSTR, "Value should be %i, is %i\n", VT_LPWSTR, pv.vt);
     if (hr == S_OK && pv.vt == VT_LPWSTR)
     {
-        WideCharToMultiByte(CP_ACP, 0, pv.u.pwszVal, -1, temp, sizeof(temp)-1, NULL, NULL);
+        WideCharToMultiByte(CP_ACP, 0, pv.pwszVal, -1, temp, sizeof(temp)-1, NULL, NULL);
         trace("guid: %s\n", temp);
         PropVariantClear(&pv);
     }
 
     pv.vt = VT_EMPTY;
     hr = IPropertyStore_GetValue(store, (const PROPERTYKEY*)&DEVPKEY_DeviceInterface_FriendlyName, &pv);
-    ok(hr == S_OK, "Failed with %08x\n", hr);
-    ok(pv.vt == VT_LPWSTR && pv.u.pwszVal, "FriendlyName value had wrong type: 0x%x or was NULL\n", pv.vt);
+    ok(hr == S_OK, "Failed with %08lx\n", hr);
+    ok(pv.vt == VT_LPWSTR && pv.pwszVal, "FriendlyName value had wrong type: 0x%x or was NULL\n", pv.vt);
     PropVariantClear(&pv);
 
     pv.vt = VT_EMPTY;
     hr = IPropertyStore_GetValue(store, (const PROPERTYKEY*)&DEVPKEY_DeviceInterface_Enabled, &pv);
-    ok(hr == S_OK, "Failed with %08x\n", hr);
+    ok(hr == S_OK, "Failed with %08lx\n", hr);
     ok(pv.vt == VT_EMPTY, "Key should not be found\n");
     PropVariantClear(&pv);
 
     pv.vt = VT_EMPTY;
     hr = IPropertyStore_GetValue(store, (const PROPERTYKEY*)&DEVPKEY_DeviceInterface_ClassGuid, &pv);
-    ok(hr == S_OK, "Failed with %08x\n", hr);
+    ok(hr == S_OK, "Failed with %08lx\n", hr);
     ok(pv.vt == VT_EMPTY, "Key should not be found\n");
     PropVariantClear(&pv);
 }
@@ -91,9 +84,9 @@ static void test_deviceinterface(IPropertyStore *store)
 
     pv.vt = VT_EMPTY;
     hr = IPropertyStore_GetValue(store, &deviceinterface_key, &pv);
-    ok(hr == S_OK, "GetValue failed: %08x\n", hr);
+    ok(hr == S_OK, "GetValue failed: %08lx\n", hr);
     ok(pv.vt == VT_LPWSTR, "Got wrong variant type: 0x%x\n", pv.vt);
-    trace("device interface: %s\n", wine_dbgstr_w(pv.u.pwszVal));
+    trace("device interface: %s\n", wine_dbgstr_w(pv.pwszVal));
     PropVariantClear(&pv);
 }
 
@@ -110,12 +103,12 @@ static void test_getat(IPropertyStore *store)
 
     hr = IPropertyStore_GetCount(store, &propcount);
 
-    ok(hr == S_OK, "Failed with %08x\n", hr);
-    ok(propcount > 0, "Propcount %d should be greater than zero\n", propcount);
+    ok(hr == S_OK, "Failed with %08lx\n", hr);
+    ok(propcount > 0, "Propcount %ld should be greater than zero\n", propcount);
 
     for (prop = 0; prop < propcount; prop++) {
 	hr = IPropertyStore_GetAt(store, prop, &pkey);
-	ok(hr == S_OK, "Failed with %08x\n", hr);
+	ok(hr == S_OK, "Failed with %08lx\n", hr);
 	if (IsEqualPropertyKey(pkey, DEVPKEY_Device_FriendlyName))
 	    found_name = TRUE;
 	if (IsEqualPropertyKey(pkey, DEVPKEY_Device_DeviceDesc))
@@ -138,49 +131,49 @@ static void test_setvalue_on_wow64(IPropertyStore *store)
     static const PROPERTYKEY PKEY_Bogus = {
         {0x1da5d803, 0xd492, 0x4edd, {0x8c, 0x23, 0xe0, 0xc0, 0xff, 0xee, 0x7f, 0x00}}, 0x7f
     };
-    static const WCHAR bogusW[] = {'{','1','D','A','5','D','8','0','3','-','D','4','9','2','-','4','E','D','D','-','8','C','2','3','-','E','0','C','0','F','F','E','E','7','F','0','0','}',',','1','2','7',0};
+    static const WCHAR bogusW[] = L"{1DA5D803-D492-4EDD-8C23-E0C0FFEE7F00},127";
 
     PropVariantInit(&pv);
 
     pv.vt = VT_EMPTY;
     hr = IPropertyStore_GetValue(store, &PKEY_AudioEndpoint_GUID, &pv);
-    ok(hr == S_OK, "Failed to get Endpoint GUID: %08x\n", hr);
+    ok(hr == S_OK, "Failed to get Endpoint GUID: %08lx\n", hr);
 
-    guidW = pv.u.pwszVal;
+    guidW = pv.pwszVal;
 
     pv.vt = VT_UI4;
-    pv.u.ulVal = 0xAB;
+    pv.ulVal = 0xAB;
 
     hr = IPropertyStore_SetValue(store, &PKEY_Bogus, &pv);
-    ok(hr == S_OK || hr == E_ACCESSDENIED, "SetValue failed: %08x\n", hr);
+    ok(hr == S_OK || hr == E_ACCESSDENIED, "SetValue failed: %08lx\n", hr);
     if (hr != S_OK)
     {
         win_skip("Missing permission to write to registry\n");
         return;
     }
 
-    pv.u.ulVal = 0x00;
+    pv.ulVal = 0x00;
 
     hr = IPropertyStore_GetValue(store, &PKEY_Bogus, &pv);
-    ok(hr == S_OK, "GetValue failed: %08x\n", hr);
-    ok(pv.u.ulVal == 0xAB, "Got wrong value: 0x%x\n", pv.u.ulVal);
+    ok(hr == S_OK, "GetValue failed: %08lx\n", hr);
+    ok(pv.ulVal == 0xAB, "Got wrong value: 0x%lx\n", pv.ulVal);
 
     /* should find the key in 64-bit view */
     ret = RegOpenKeyExW(HKEY_LOCAL_MACHINE, software_renderW, 0, KEY_READ|KEY_WOW64_64KEY, &root);
-    ok(ret == ERROR_SUCCESS, "Couldn't open mmdevices Render key: %u\n", ret);
+    ok(ret == ERROR_SUCCESS, "Couldn't open mmdevices Render key: %lu\n", ret);
 
     ret = RegOpenKeyExW(root, guidW, 0, KEY_READ|KEY_WOW64_64KEY, &devkey);
-    ok(ret == ERROR_SUCCESS, "Couldn't open mmdevice guid key: %u\n", ret);
+    ok(ret == ERROR_SUCCESS, "Couldn't open mmdevice guid key: %lu\n", ret);
 
-    ret = RegOpenKeyExW(devkey, propertiesW, 0, KEY_READ|KEY_WOW64_64KEY, &props);
-    ok(ret == ERROR_SUCCESS, "Couldn't open mmdevice property key: %u\n", ret);
+    ret = RegOpenKeyExW(devkey, L"Properties", 0, KEY_READ|KEY_WOW64_64KEY, &props);
+    ok(ret == ERROR_SUCCESS, "Couldn't open mmdevice property key: %lu\n", ret);
 
     /* Note: the registry key exists even without calling IPropStore::Commit */
     size = sizeof(regval);
     ret = RegQueryValueExW(props, bogusW, NULL, &type, (LPBYTE)&regval, &size);
-    ok(ret == ERROR_SUCCESS, "Couldn't get bogus propertykey value: %u\n", ret);
-    ok(type == REG_DWORD, "Got wrong value type: %u\n", type);
-    ok(regval == 0xAB, "Got wrong value: 0x%x\n", regval);
+    ok(ret == ERROR_SUCCESS, "Couldn't get bogus propertykey value: %lu\n", ret);
+    ok(type == REG_DWORD, "Got wrong value type: %lu\n", type);
+    ok(regval == 0xAB, "Got wrong value: 0x%lx\n", regval);
 
     RegCloseKey(props);
     RegCloseKey(devkey);
@@ -190,7 +183,7 @@ static void test_setvalue_on_wow64(IPropertyStore *store)
 
     /* should NOT find the key in 32-bit view */
     ret = RegOpenKeyExW(HKEY_LOCAL_MACHINE, software_renderW, 0, KEY_READ, &root);
-    ok(ret == ERROR_FILE_NOT_FOUND, "Wrong error when opening mmdevices Render key: %u\n", ret);
+    ok(ret == ERROR_FILE_NOT_FOUND, "Wrong error when opening mmdevices Render key: %lu\n", ret);
 }
 
 START_TEST(propstore)
@@ -211,39 +204,39 @@ START_TEST(propstore)
     hr = CoCreateInstance(&CLSID_MMDeviceEnumerator, NULL, CLSCTX_INPROC_SERVER, &IID_IMMDeviceEnumerator, (void**)&mme);
     if (FAILED(hr))
     {
-        skip("mmdevapi not available: 0x%08x\n", hr);
+        skip("mmdevapi not available: 0x%08lx\n", hr);
         goto cleanup;
     }
 
     hr = IMMDeviceEnumerator_GetDefaultAudioEndpoint(mme, eRender, eMultimedia, &dev);
-    ok(hr == S_OK || hr == E_NOTFOUND, "GetDefaultAudioEndpoint failed: 0x%08x\n", hr);
+    ok(hr == S_OK || hr == E_NOTFOUND, "GetDefaultAudioEndpoint failed: 0x%08lx\n", hr);
     if (hr != S_OK)
     {
         if (hr == E_NOTFOUND)
             skip("No sound card available\n");
         else
-            skip("GetDefaultAudioEndpoint returns 0x%08x\n", hr);
+            skip("GetDefaultAudioEndpoint returns 0x%08lx\n", hr);
         goto cleanup;
     }
     store = NULL;
     hr = IMMDevice_OpenPropertyStore(dev, 3, &store);
-    ok(hr == E_INVALIDARG, "Wrong hr returned: %08x\n", hr);
+    ok(hr == E_INVALIDARG, "Wrong hr returned: %08lx\n", hr);
     if (hr != S_OK)
         /* It seems on windows returning with E_INVALIDARG doesn't
          * set store to NULL, so just don't set store to non-null
          * before calling this function
          */
-        ok(!store, "Store set to non-NULL on failure: %p/%08x\n", store, hr);
+        ok(!store, "Store set to non-NULL on failure: %p/%08lx\n", store, hr);
     else if (store)
         IPropertyStore_Release(store);
     hr = IMMDevice_OpenPropertyStore(dev, STGM_READ, NULL);
-    ok(hr == E_POINTER, "Wrong hr returned: %08x\n", hr);
+    ok(hr == E_POINTER, "Wrong hr returned: %08lx\n", hr);
 
     store = NULL;
     hr = IMMDevice_OpenPropertyStore(dev, STGM_READWRITE, &store);
     if(hr == E_ACCESSDENIED)
         hr = IMMDevice_OpenPropertyStore(dev, STGM_READ, &store);
-    ok(hr == S_OK, "Opening valid store returned %08x\n", hr);
+    ok(hr == S_OK, "Opening valid store returned %08lx\n", hr);
     if (store)
     {
         test_propertystore(store);
