@@ -17,7 +17,6 @@
  */
 
 #include <stdarg.h>
-#include <assert.h>
 
 #define COBJMACROS
 
@@ -130,15 +129,43 @@ static HRESULT WINAPI HTMLAreaElement_get_coords(IHTMLAreaElement *iface, BSTR *
 static HRESULT WINAPI HTMLAreaElement_put_href(IHTMLAreaElement *iface, BSTR v)
 {
     HTMLAreaElement *This = impl_from_IHTMLAreaElement(iface);
-    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
-    return E_NOTIMPL;
+    nsAString nsstr;
+    nsresult nsres;
+
+    TRACE("(%p)->(%s)\n", This, debugstr_w(v));
+
+    nsAString_InitDepend(&nsstr, v);
+    nsres = nsIDOMHTMLAreaElement_SetHref(This->nsarea, &nsstr);
+    nsAString_Finish(&nsstr);
+    if(NS_FAILED(nsres))
+        return E_FAIL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLAreaElement_get_href(IHTMLAreaElement *iface, BSTR *p)
 {
     HTMLAreaElement *This = impl_from_IHTMLAreaElement(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    nsAString href_str;
+    nsresult nsres;
+    HRESULT hres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsAString_Init(&href_str, NULL);
+    nsres = nsIDOMHTMLAreaElement_GetHref(This->nsarea, &href_str);
+    if(NS_SUCCEEDED(nsres)) {
+        const PRUnichar *href;
+
+        nsAString_GetData(&href_str, &href);
+        hres = nsuri_to_url(href, TRUE, p);
+    }else {
+        ERR("GetHref failed: %08lx\n", nsres);
+        hres = E_FAIL;
+    }
+
+    nsAString_Finish(&href_str);
+    return hres;
 }
 
 static HRESULT WINAPI HTMLAreaElement_put_target(IHTMLAreaElement *iface, BSTR v)
@@ -413,14 +440,14 @@ static HRESULT HTMLAreaElement_handle_event(HTMLDOMNode *iface, DWORD eid, nsIDO
         nsAString_Init(&href_str, NULL);
         nsres = nsIDOMHTMLAreaElement_GetHref(This->nsarea, &href_str);
         if (NS_FAILED(nsres)) {
-            ERR("Could not get area href: %08x\n", nsres);
+            ERR("Could not get area href: %08lx\n", nsres);
             goto fallback;
         }
 
         nsAString_Init(&target_str, NULL);
         nsres = nsIDOMHTMLAreaElement_GetTarget(This->nsarea, &target_str);
         if (NS_FAILED(nsres)) {
-            ERR("Could not get area target: %08x\n", nsres);
+            ERR("Could not get area target: %08lx\n", nsres);
             goto fallback;
         }
 
@@ -440,6 +467,7 @@ static const NodeImplVtbl HTMLAreaElementImplVtbl = {
     HTMLElement_destructor,
     HTMLElement_cpc,
     HTMLElement_clone,
+    HTMLElement_dispatch_nsevent_hook,
     HTMLAreaElement_handle_event,
     HTMLElement_get_attr_col
 };
@@ -450,6 +478,7 @@ static const tid_t HTMLAreaElement_iface_tids[] = {
     0
 };
 static dispex_static_data_t HTMLAreaElement_dispex = {
+    L"HTMLAreaElement",
     NULL,
     DispHTMLAreaElement_tid,
     HTMLAreaElement_iface_tids,
@@ -461,7 +490,7 @@ HRESULT HTMLAreaElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTM
     HTMLAreaElement *ret;
     nsresult nsres;
 
-    ret = heap_alloc_zero(sizeof(HTMLAreaElement));
+    ret = calloc(1, sizeof(HTMLAreaElement));
     if(!ret)
         return E_OUTOFMEMORY;
 
