@@ -17,7 +17,6 @@
  */
 
 #include <stdarg.h>
-#include <assert.h>
 
 #define COBJMACROS
 
@@ -25,6 +24,7 @@
 #include "winbase.h"
 #include "winuser.h"
 #include "ole2.h"
+#include "mshtmdid.h"
 
 #include "wine/debug.h"
 
@@ -39,6 +39,7 @@ struct HTMLImg {
     IHTMLImgElement IHTMLImgElement_iface;
 
     nsIDOMHTMLImageElement *nsimg;
+    eventid_t skip_event;
 };
 
 static inline HTMLImg *impl_from_IHTMLImgElement(IHTMLImgElement *iface)
@@ -108,7 +109,7 @@ static HRESULT WINAPI HTMLImgElement_put_isMap(IHTMLImgElement *iface, VARIANT_B
 
     nsres = nsIDOMHTMLImageElement_SetIsMap(This->nsimg, v != VARIANT_FALSE);
     if (NS_FAILED(nsres)) {
-        ERR("Set IsMap failed: %08x\n", nsres);
+        ERR("Set IsMap failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -128,7 +129,7 @@ static HRESULT WINAPI HTMLImgElement_get_isMap(IHTMLImgElement *iface, VARIANT_B
 
     nsres = nsIDOMHTMLImageElement_GetIsMap(This->nsimg, &b);
     if (NS_FAILED(nsres)) {
-        ERR("Get IsMap failed: %08x\n", nsres);
+        ERR("Get IsMap failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -223,7 +224,7 @@ static HRESULT WINAPI HTMLImgElement_get_border(IHTMLImgElement *iface, VARIANT
 static HRESULT WINAPI HTMLImgElement_put_vspace(IHTMLImgElement *iface, LONG v)
 {
     HTMLImg *This = impl_from_IHTMLImgElement(iface);
-    FIXME("(%p)->(%d)\n", This, v);
+    FIXME("(%p)->(%ld)\n", This, v);
     return E_NOTIMPL;
 }
 
@@ -237,7 +238,7 @@ static HRESULT WINAPI HTMLImgElement_get_vspace(IHTMLImgElement *iface, LONG *p)
 static HRESULT WINAPI HTMLImgElement_put_hspace(IHTMLImgElement *iface, LONG v)
 {
     HTMLImg *This = impl_from_IHTMLImgElement(iface);
-    FIXME("(%p)->(%d)\n", This, v);
+    FIXME("(%p)->(%ld)\n", This, v);
     return E_NOTIMPL;
 }
 
@@ -260,7 +261,7 @@ static HRESULT WINAPI HTMLImgElement_put_alt(IHTMLImgElement *iface, BSTR v)
     nsres = nsIDOMHTMLImageElement_SetAlt(This->nsimg, &alt_str);
     nsAString_Finish(&alt_str);
     if(NS_FAILED(nsres))
-        ERR("SetAlt failed: %08x\n", nsres);
+        ERR("SetAlt failed: %08lx\n", nsres);
 
     return S_OK;
 }
@@ -281,6 +282,7 @@ static HRESULT WINAPI HTMLImgElement_get_alt(IHTMLImgElement *iface, BSTR *p)
 static HRESULT WINAPI HTMLImgElement_put_src(IHTMLImgElement *iface, BSTR v)
 {
     HTMLImg *This = impl_from_IHTMLImgElement(iface);
+    HRESULT hres = S_OK;
     nsAString src_str;
     nsresult nsres;
 
@@ -290,9 +292,31 @@ static HRESULT WINAPI HTMLImgElement_put_src(IHTMLImgElement *iface, BSTR v)
     nsres = nsIDOMHTMLImageElement_SetSrc(This->nsimg, &src_str);
     nsAString_Finish(&src_str);
     if(NS_FAILED(nsres))
-        ERR("SetSrc failed: %08x\n", nsres);
+        ERR("SetSrc failed: %08lx\n", nsres);
 
-    return S_OK;
+    if(dispex_compat_mode(&This->element.node.event_target.dispex) < COMPAT_MODE_IE9) {
+        eventid_t eventid;
+        cpp_bool complete;
+        UINT32 height = 0;
+        DOMEvent *event;
+
+        /* Synchronously send load event if the image was completed immediately (such as from cache) */
+        This->skip_event = EVENTID_INVALID_ID;
+
+        nsres = nsIDOMHTMLImageElement_GetComplete(This->nsimg, &complete);
+        if(NS_SUCCEEDED(nsres) && complete) {
+            nsIDOMHTMLImageElement_GetNaturalHeight(This->nsimg, &height);
+            eventid = height ? EVENTID_LOAD : EVENTID_ERROR;
+
+            hres = create_document_event(This->element.node.doc, eventid, &event);
+            if(SUCCEEDED(hres)) {
+                This->skip_event = eventid;
+                dispatch_event(&This->element.node.event_target, event);
+                IDOMEvent_Release(&event->IDOMEvent_iface);
+            }
+        }
+    }
+    return hres;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_src(IHTMLImgElement *iface, BSTR *p)
@@ -303,8 +327,6 @@ static HRESULT WINAPI HTMLImgElement_get_src(IHTMLImgElement *iface, BSTR *p)
     nsresult nsres;
     HRESULT hres = S_OK;
 
-    static const WCHAR blockedW[] = {'B','L','O','C','K','E','D',':',':',0};
-
     TRACE("(%p)->(%p)\n", This, p);
 
     nsAString_Init(&src_str, NULL);
@@ -312,16 +334,16 @@ static HRESULT WINAPI HTMLImgElement_get_src(IHTMLImgElement *iface, BSTR *p)
     if(NS_SUCCEEDED(nsres)) {
         nsAString_GetData(&src_str, &src);
 
-        if(!strncmpiW(src, blockedW, sizeof(blockedW)/sizeof(WCHAR)-1)) {
+        if(!wcsnicmp(src, L"BLOCKED::", ARRAY_SIZE(L"BLOCKED::")-1)) {
             TRACE("returning BLOCKED::\n");
-            *p = SysAllocString(blockedW);
+            *p = SysAllocString(L"BLOCKED::");
             if(!*p)
                 hres = E_OUTOFMEMORY;
         }else {
             hres = nsuri_to_url(src, TRUE, p);
         }
     }else {
-        ERR("GetSrc failed: %08x\n", nsres);
+        ERR("GetSrc failed: %08lx\n", nsres);
         hres = E_FAIL;
     }
 
@@ -388,7 +410,7 @@ static HRESULT WINAPI HTMLImgElement_get_complete(IHTMLImgElement *iface, VARIAN
 
     nsres = nsIDOMHTMLImageElement_GetComplete(This->nsimg, &complete);
     if(NS_FAILED(nsres)) {
-        ERR("GetComplete failed: %08x\n", nsres);
+        ERR("GetComplete failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -423,7 +445,7 @@ static HRESULT WINAPI HTMLImgElement_put_align(IHTMLImgElement *iface, BSTR v)
     nsres = nsIDOMHTMLImageElement_SetAlign(This->nsimg, &str);
     nsAString_Finish(&str);
     if (NS_FAILED(nsres)){
-        ERR("Set Align(%s) failed: %08x\n", debugstr_w(v), nsres);
+        ERR("Set Align(%s) failed: %08lx\n", debugstr_w(v), nsres);
         return E_FAIL;
     }
 
@@ -523,11 +545,11 @@ static HRESULT WINAPI HTMLImgElement_put_width(IHTMLImgElement *iface, LONG v)
     HTMLImg *This = impl_from_IHTMLImgElement(iface);
     nsresult nsres;
 
-    TRACE("(%p)->(%d)\n", This, v);
+    TRACE("(%p)->(%ld)\n", This, v);
 
     nsres = nsIDOMHTMLImageElement_SetWidth(This->nsimg, v);
     if(NS_FAILED(nsres)) {
-        ERR("SetWidth failed: %08x\n", nsres);
+        ERR("SetWidth failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -544,7 +566,7 @@ static HRESULT WINAPI HTMLImgElement_get_width(IHTMLImgElement *iface, LONG *p)
 
     nsres = nsIDOMHTMLImageElement_GetWidth(This->nsimg, &width);
     if(NS_FAILED(nsres)) {
-        ERR("GetWidth failed: %08x\n", nsres);
+        ERR("GetWidth failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -557,11 +579,11 @@ static HRESULT WINAPI HTMLImgElement_put_height(IHTMLImgElement *iface, LONG v)
     HTMLImg *This = impl_from_IHTMLImgElement(iface);
     nsresult nsres;
 
-    TRACE("(%p)->(%d)\n", This, v);
+    TRACE("(%p)->(%ld)\n", This, v);
 
     nsres = nsIDOMHTMLImageElement_SetHeight(This->nsimg, v);
     if(NS_FAILED(nsres)) {
-        ERR("SetHeight failed: %08x\n", nsres);
+        ERR("SetHeight failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -578,7 +600,7 @@ static HRESULT WINAPI HTMLImgElement_get_height(IHTMLImgElement *iface, LONG *p)
 
     nsres = nsIDOMHTMLImageElement_GetHeight(This->nsimg, &height);
     if(NS_FAILED(nsres)) {
-        ERR("GetHeight failed: %08x\n", nsres);
+        ERR("GetHeight failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -680,6 +702,18 @@ static HRESULT HTMLImgElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
     return S_OK;
 }
 
+static HRESULT HTMLImgElement_dispatch_nsevent_hook(HTMLDOMNode *iface, DOMEvent *event)
+{
+    HTMLImg *This = impl_from_HTMLDOMNode(iface);
+
+    if(event->event_id == This->skip_event) {
+        This->skip_event = EVENTID_INVALID_ID;
+        return S_OK;
+    }
+
+    return S_FALSE;
+}
+
 static HRESULT HTMLImgElement_get_readystate(HTMLDOMNode *iface, BSTR *p)
 {
     HTMLImg *This = impl_from_HTMLDOMNode(iface);
@@ -713,6 +747,7 @@ static const NodeImplVtbl HTMLImgElementImplVtbl = {
     HTMLElement_destructor,
     HTMLElement_cpc,
     HTMLElement_clone,
+    HTMLImgElement_dispatch_nsevent_hook,
     HTMLElement_handle_event,
     HTMLElement_get_attr_col,
     NULL,
@@ -723,37 +758,52 @@ static const NodeImplVtbl HTMLImgElementImplVtbl = {
     NULL,
     NULL,
     NULL,
+    NULL,
     HTMLImgElement_traverse,
     HTMLImgElement_unlink
 };
 
 static const tid_t HTMLImgElement_iface_tids[] = {
     HTMLELEMENT_TIDS,
-    IHTMLImgElement_tid,
     0
 };
+
+static void HTMLImgElement_init_dispex_info(dispex_data_t *info, compat_mode_t mode)
+{
+    static const dispex_hook_t img_ie11_hooks[] = {
+        {DISPID_IHTMLIMGELEMENT_FILESIZE, NULL},
+        {DISPID_UNKNOWN}
+    };
+
+    HTMLElement_init_dispex_info(info, mode);
+
+    dispex_info_add_interface(info, IHTMLImgElement_tid, mode >= COMPAT_MODE_IE11 ? img_ie11_hooks : NULL);
+}
+
 static dispex_static_data_t HTMLImgElement_dispex = {
+    L"HTMLImageElement",
     NULL,
     DispHTMLImg_tid,
     HTMLImgElement_iface_tids,
-    HTMLElement_init_dispex_info
+    HTMLImgElement_init_dispex_info
 };
 
-HRESULT HTMLImgElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nselem, HTMLElement **elem)
+HRESULT HTMLImgElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
 {
     HTMLImg *ret;
     nsresult nsres;
 
-    ret = heap_alloc_zero(sizeof(HTMLImg));
+    ret = calloc(1, sizeof(HTMLImg));
     if(!ret)
         return E_OUTOFMEMORY;
 
     ret->IHTMLImgElement_iface.lpVtbl = &HTMLImgElementVtbl;
     ret->element.node.vtbl = &HTMLImgElementImplVtbl;
+    ret->skip_event = EVENTID_INVALID_ID;
 
     HTMLElement_Init(&ret->element, doc, nselem, &HTMLImgElement_dispex);
 
-    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLImageElement, (void**)&ret->nsimg);
+    nsres = nsIDOMElement_QueryInterface(nselem, &IID_nsIDOMHTMLImageElement, (void**)&ret->nsimg);
     assert(nsres == NS_OK);
 
     *elem = &ret->element;
@@ -793,7 +843,7 @@ static ULONG WINAPI HTMLImageElementFactory_AddRef(IHTMLImageElementFactory *ifa
     HTMLImageElementFactory *This = impl_from_IHTMLImageElementFactory(iface);
     LONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     return ref;
 }
@@ -803,10 +853,10 @@ static ULONG WINAPI HTMLImageElementFactory_Release(IHTMLImageElementFactory *if
     HTMLImageElementFactory *This = impl_from_IHTMLImageElementFactory(iface);
     LONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     if(!ref)
-        heap_free(This);
+        free(This);
 
     return ref;
 }
@@ -856,7 +906,7 @@ static LONG var_to_size(const VARIANT *v)
 
         hres = VarI4FromStr(V_BSTR(v), 0, 0, &ret);
         if(FAILED(hres)) {
-            FIXME("VarI4FromStr failed: %08x\n", hres);
+            FIXME("VarI4FromStr failed: %08lx\n", hres);
             return 0;
         }
         return ret;
@@ -874,12 +924,10 @@ static HRESULT WINAPI HTMLImageElementFactory_create(IHTMLImageElementFactory *i
     HTMLDocumentNode *doc;
     IHTMLImgElement *img;
     HTMLElement *elem;
-    nsIDOMHTMLElement *nselem;
+    nsIDOMElement *nselem;
     LONG l;
     HRESULT hres;
 
-    static const PRUnichar imgW[] = {'I','M','G',0};
-
     TRACE("(%p)->(%s %s %p)\n", This, debugstr_variant(&width),
             debugstr_variant(&height), img_elem);
 
@@ -892,12 +940,12 @@ static HRESULT WINAPI HTMLImageElementFactory_create(IHTMLImageElementFactory *i
 
     *img_elem = NULL;
 
-    hres = create_nselem(doc, imgW, &nselem);
+    hres = create_nselem(doc, L"IMG", &nselem);
     if(FAILED(hres))
         return hres;
 
     hres = HTMLElement_Create(doc, (nsIDOMNode*)nselem, FALSE, &elem);
-    nsIDOMHTMLElement_Release(nselem);
+    nsIDOMElement_Release(nselem);
     if(FAILED(hres)) {
         ERR("HTMLElement_Create failed\n");
         return hres;
@@ -907,7 +955,7 @@ static HRESULT WINAPI HTMLImageElementFactory_create(IHTMLImageElementFactory *i
             (void**)&img);
     IHTMLElement_Release(&elem->IHTMLElement_iface);
     if(FAILED(hres)) {
-        ERR("IHTMLElement_QueryInterface failed: 0x%08x\n", hres);
+        ERR("IHTMLElement_QueryInterface failed: 0x%08lx\n", hres);
         return hres;
     }
 
@@ -979,6 +1027,7 @@ static const dispex_static_data_vtbl_t HTMLImageElementFactory_dispex_vtbl = {
 };
 
 static dispex_static_data_t HTMLImageElementFactory_dispex = {
+    L"Function",
     &HTMLImageElementFactory_dispex_vtbl,
     IHTMLImageElementFactory_tid,
     HTMLImageElementFactory_iface_tids
@@ -988,7 +1037,7 @@ HRESULT HTMLImageElementFactory_Create(HTMLInnerWindow *window, HTMLImageElement
 {
     HTMLImageElementFactory *ret;
 
-    ret = heap_alloc(sizeof(HTMLImageElementFactory));
+    ret = malloc(sizeof(HTMLImageElementFactory));
     if(!ret)
         return E_OUTOFMEMORY;
 
@@ -996,8 +1045,8 @@ HRESULT HTMLImageElementFactory_Create(HTMLInnerWindow *window, HTMLImageElement
     ret->ref = 1;
     ret->window = window;
 
-    init_dispex(&ret->dispex, (IUnknown*)&ret->IHTMLImageElementFactory_iface,
-            &HTMLImageElementFactory_dispex);
+    init_dispatch(&ret->dispex, (IUnknown*)&ret->IHTMLImageElementFactory_iface,
+                  &HTMLImageElementFactory_dispex, dispex_compat_mode(&window->event_target.dispex));
 
     *ret_val = ret;
     return S_OK;
