@@ -157,7 +157,7 @@ static nsrefcnt NSAPI nsSingletonEnumerator_Release(nsISimpleEnumerator *iface)
     if(!ref) {
         if(This->value)
             nsISupports_Release(This->value);
-        heap_free(This);
+        free(This);
     }
 
     return ref;
@@ -199,7 +199,7 @@ static nsISimpleEnumerator *create_singleton_enumerator(nsISupports *value)
 {
     nsSingletonEnumerator *ret;
 
-    ret = heap_alloc(sizeof(*ret));
+    ret = malloc(sizeof(*ret));
     if(!ret)
         return NULL;
 
@@ -415,7 +415,7 @@ static BOOL install_wine_gecko(void)
     len = GetSystemDirectoryW(app, MAX_PATH-ARRAY_SIZE(controlW));
     memcpy(app+len, controlW, sizeof(controlW));
 
-    args = heap_alloc(len*sizeof(WCHAR) + sizeof(controlW) + sizeof(argsW));
+    args = malloc(len * sizeof(WCHAR) + sizeof(controlW) + sizeof(argsW));
     if(!args)
         return FALSE;
 
@@ -427,7 +427,7 @@ static BOOL install_wine_gecko(void)
     memset(&si, 0, sizeof(si));
     si.cb = sizeof(si);
     ret = CreateProcessW(app, args, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi);
-    heap_free(args);
+    free(args);
     if (ret) {
         CloseHandle(pi.hThread);
         WaitForSingleObject(pi.hProcess, INFINITE);
@@ -460,7 +460,7 @@ static void set_environment(LPCWSTR gre_path)
 
     len = GetEnvironmentVariableW(L"PATH", NULL, 0);
     gre_path_len = lstrlenW(gre_path);
-    path = heap_alloc((len+gre_path_len+1)*sizeof(WCHAR));
+    path = malloc((len + gre_path_len + 1) * sizeof(WCHAR));
     if(!path)
         return;
     GetEnvironmentVariableW(L"PATH", path, len);
@@ -474,7 +474,7 @@ static void set_environment(LPCWSTR gre_path)
         lstrcpyW(path+len, gre_path);
         SetEnvironmentVariableW(L"PATH", path);
     }
-    heap_free(path);
+    free(path);
 }
 
 static void set_bool_pref(nsIPrefBranch *pref, const char *pref_name, BOOL val)
@@ -560,6 +560,7 @@ static BOOL init_xpcom(const PRUnichar *gre_path)
     }
 
     nsres = NS_InitXPCOM2(&pServMgr, gre_dir, (nsIDirectoryServiceProvider*)&nsDirectoryServiceProvider2);
+    nsIFile_Release(gre_dir);
     if(NS_FAILED(nsres)) {
         ERR("NS_InitXPCOM2 failed: %08lx\n", nsres);
         FreeLibrary(xul_handle);
@@ -664,7 +665,7 @@ static WCHAR *check_version(const WCHAR *path)
     }
 
     len = wcslen(path);
-    file_name = heap_alloc((len + 12) * sizeof(WCHAR));
+    file_name = malloc((len + 12) * sizeof(WCHAR));
     if(!file_name)
         return NULL;
 
@@ -677,7 +678,7 @@ static WCHAR *check_version(const WCHAR *path)
     file_name[len] = 0;
     if(hfile == INVALID_HANDLE_VALUE) {
         TRACE("%s not found\n", debugstr_w(file_name));
-        heap_free(file_name);
+        free(file_name);
         return NULL;
     }
 
@@ -690,7 +691,7 @@ static WCHAR *check_version(const WCHAR *path)
     if(strcmp(version, GECKO_VERSION_STRING)) {
         ERR("Unexpected version %s, expected \"%s\"\n", debugstr_a(version),
             GECKO_VERSION_STRING);
-        heap_free(file_name);
+        free(file_name);
         return NULL;
     }
 
@@ -717,11 +718,11 @@ static WCHAR *find_wine_gecko_reg(void)
     return check_version(buffer);
 }
 
-static WCHAR *heap_strcat(const WCHAR *str1, const WCHAR *str2)
+static WCHAR *strdupWW(const WCHAR *str1, const WCHAR *str2)
 {
     size_t len1 = lstrlenW(str1);
     size_t len2 = lstrlenW(str2);
-    WCHAR *ret = heap_alloc((len1 + len2 + 1) * sizeof(WCHAR));
+    WCHAR *ret = malloc((len1 + len2 + 1) * sizeof(WCHAR));
     if(!ret) return NULL;
     memcpy(ret, str1, len1 * sizeof(WCHAR));
     memcpy(ret + len1, str2, len2 * sizeof(WCHAR));
@@ -735,14 +736,14 @@ static WCHAR *find_wine_gecko_datadir(void)
     WCHAR *path = NULL, *ret;
 
     if((data_dir = _wgetenv(L"WINEDATADIR")))
-        path = heap_strcat(data_dir, L"\\gecko\\" GECKO_DIR_NAME);
+        path = strdupWW(data_dir, L"\\gecko\\" GECKO_DIR_NAME);
     else if((data_dir = _wgetenv(L"WINEBUILDDIR")))
-        path = heap_strcat(data_dir, L"\\..\\gecko\\" GECKO_DIR_NAME);
+        path = strdupWW(data_dir, L"\\..\\gecko\\" GECKO_DIR_NAME);
     if(!path)
         return NULL;
 
     ret = check_version(path);
-    heap_free(path);
+    free(path);
     return ret;
 }
 
@@ -763,7 +764,7 @@ static WCHAR *find_wine_gecko_unix(const char *unix_path)
 
     ret = check_version(dos_dir);
 
-    heap_free(dos_dir);
+    HeapFree(GetProcessHeap(), 0, dos_dir);
     return ret;
 }
 
@@ -811,7 +812,7 @@ BOOL load_gecko(void)
 
         if(gecko_path) {
             ret = load_xul(gecko_path);
-            heap_free(gecko_path);
+            free(gecko_path);
         }else {
            MESSAGE("Could not find Wine Gecko. HTML rendering will be disabled.\n");
         }
@@ -911,6 +912,10 @@ HRESULT map_nsresult(nsresult nsres)
         return E_UNEXPECTED;
     case NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR:
         return 0x80700007; /* according to tests */
+    case NS_ERROR_DOM_SYNTAX_ERR:
+        return E_INVALIDARG; /* FIXME: Throw SyntaxError for IE9+ modes */
+    case NS_BINDING_ABORTED:
+        return E_ABORT;
     }
     return E_FAIL;
 }
@@ -1228,7 +1233,7 @@ void setup_editor_controller(GeckoBrowser *This)
     }
 
     nsres = nsIEditingSession_GetEditorForWindow(editing_session,
-            This->doc->basedoc.window->window_proxy, &This->editor);
+            This->doc->window->window_proxy, &This->editor);
     nsIEditingSession_Release(editing_session);
     if(NS_FAILED(nsres)) {
         ERR("Could not get editor: %08lx\n", nsres);
@@ -1288,7 +1293,7 @@ BOOL is_gecko_path(const char *path)
     WCHAR *buf, *ptr;
     BOOL ret;
 
-    buf = heap_strdupUtoW(path);
+    buf = strdupUtoW(path);
     if(!buf || lstrlenW(buf) < gecko_path_len)
         return FALSE;
 
@@ -1301,7 +1306,7 @@ BOOL is_gecko_path(const char *path)
     buf[gecko_path_len] = 0;
 
     ret = !wcsicmp(buf, gecko_path);
-    heap_free(buf);
+    free(buf);
     return ret;
 }
 
@@ -1403,7 +1408,7 @@ static nsrefcnt NSAPI nsWeakReference_Release(nsIWeakReference *iface)
 
     if(!ref) {
         assert(!This->browser);
-        heap_free(This);
+        free(This);
     }
 
     return ref;
@@ -1501,7 +1506,7 @@ static nsrefcnt NSAPI nsWebBrowserChrome_Release(nsIWebBrowserChrome *iface)
             This->weak_reference->browser = NULL;
             nsIWeakReference_Release(&This->weak_reference->nsIWeakReference_iface);
         }
-        heap_free(This);
+        free(This);
     }
 
     return ref;
@@ -1681,7 +1686,12 @@ static nsresult NSAPI nsContextMenuListener_OnShowContextMenu(nsIContextMenuList
     case CONTEXT_TEXT: {
         nsISelection *selection;
 
-        nsres = nsIDOMHTMLDocument_GetSelection(This->doc->basedoc.doc_node->nsdoc, &selection);
+        if(!This->doc->doc_node->html_document) {
+            FIXME("Not implemented for XML document\n");
+            break;
+        }
+
+        nsres = nsIDOMHTMLDocument_GetSelection(This->doc->doc_node->html_document, &selection);
         if(NS_SUCCEEDED(nsres) && selection) {
             cpp_bool is_collapsed;
 
@@ -2145,7 +2155,7 @@ static nsresult NSAPI nsSupportsWeakReference_GetWeakReference(nsISupportsWeakRe
     TRACE("(%p)->(%p)\n", This, _retval);
 
     if(!This->weak_reference) {
-        This->weak_reference = heap_alloc(sizeof(nsWeakReference));
+        This->weak_reference = malloc(sizeof(nsWeakReference));
         if(!This->weak_reference)
             return NS_ERROR_OUT_OF_MEMORY;
 
@@ -2292,7 +2302,7 @@ HRESULT create_gecko_browser(HTMLDocumentObj *doc, GeckoBrowser **_ret)
     if(!load_gecko())
         return CLASS_E_CLASSNOTAVAILABLE;
 
-    ret = heap_alloc_zero(sizeof(GeckoBrowser));
+    ret = calloc(1, sizeof(GeckoBrowser));
     if(!ret)
         return E_OUTOFMEMORY;
 
