@@ -17,7 +17,6 @@
  */
 
 #include <stdarg.h>
-#include <assert.h>
 
 #define COBJMACROS
 
@@ -66,11 +65,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 /* See jscript.h in jscript.dll. */
 #define SCRIPTLANGUAGEVERSION_HTML 0x400
 #define SCRIPTLANGUAGEVERSION_ES5  0x102
-
-static const WCHAR documentW[] = {'d','o','c','u','m','e','n','t',0};
-static const WCHAR windowW[] = {'w','i','n','d','o','w',0};
-static const WCHAR script_endW[] = {'<','/','S','C','R','I','P','T','>',0};
-static const WCHAR emptyW[] = {0};
+#define SCRIPTLANGUAGEVERSION_ES6  0x103
 
 struct ScriptHost {
     IActiveScriptSite              IActiveScriptSite_iface;
@@ -104,14 +99,14 @@ static BOOL set_script_prop(ScriptHost *script_host, DWORD property, VARIANT *va
     hres = IActiveScript_QueryInterface(script_host->script, &IID_IActiveScriptProperty,
             (void**)&script_prop);
     if(FAILED(hres)) {
-        WARN("Could not get IActiveScriptProperty iface: %08x\n", hres);
+        WARN("Could not get IActiveScriptProperty iface: %08lx\n", hres);
         return FALSE;
     }
 
     hres = IActiveScriptProperty_SetProperty(script_prop, property, NULL, val);
     IActiveScriptProperty_Release(script_prop);
     if(FAILED(hres)) {
-        WARN("SetProperty(%x) failed: %08x\n", property, hres);
+        WARN("SetProperty(%lx) failed: %08lx\n", property, hres);
         return FALSE;
     }
 
@@ -129,19 +124,19 @@ static BOOL init_script_engine(ScriptHost *script_host)
 
     hres = IActiveScript_QueryInterface(script_host->script, &IID_IActiveScriptParse, (void**)&script_host->parse);
     if(FAILED(hres)) {
-        WARN("Could not get IActiveScriptParse: %08x\n", hres);
+        WARN("Could not get IActiveScriptParse: %08lx\n", hres);
         return FALSE;
     }
 
     hres = IActiveScript_QueryInterface(script_host->script, &IID_IObjectSafety, (void**)&safety);
     if(FAILED(hres)) {
-        FIXME("Could not get IObjectSafety: %08x\n", hres);
+        FIXME("Could not get IObjectSafety: %08lx\n", hres);
         return FALSE;
     }
 
     hres = IObjectSafety_GetInterfaceSafetyOptions(safety, &IID_IActiveScriptParse, &supported_opts, &enabled_opts);
     if(FAILED(hres)) {
-        FIXME("GetInterfaceSafetyOptions failed: %08x\n", hres);
+        FIXME("GetInterfaceSafetyOptions failed: %08lx\n", hres);
     }else if(!(supported_opts & INTERFACE_USES_DISPEX)) {
         FIXME("INTERFACE_USES_DISPEX is not supported\n");
     }else {
@@ -149,7 +144,7 @@ static BOOL init_script_engine(ScriptHost *script_host)
                 INTERFACESAFE_FOR_UNTRUSTED_DATA|INTERFACE_USES_DISPEX|INTERFACE_USES_SECURITY_MANAGER,
                 INTERFACESAFE_FOR_UNTRUSTED_DATA|INTERFACE_USES_DISPEX|INTERFACE_USES_SECURITY_MANAGER);
         if(FAILED(hres))
-            FIXME("SetInterfaceSafetyOptions failed: %08x\n", hres);
+            FIXME("SetInterfaceSafetyOptions failed: %08lx\n", hres);
     }
 
     IObjectSafety_Release(safety);
@@ -159,7 +154,9 @@ static BOOL init_script_engine(ScriptHost *script_host)
     compat_mode = lock_document_mode(script_host->window->doc);
     script_mode = compat_mode < COMPAT_MODE_IE8 ? SCRIPTLANGUAGEVERSION_5_7 : SCRIPTLANGUAGEVERSION_5_8;
     if(IsEqualGUID(&script_host->guid, &CLSID_JScript)) {
-        if(compat_mode >= COMPAT_MODE_IE9)
+        if(compat_mode >= COMPAT_MODE_IE11)
+            script_mode = SCRIPTLANGUAGEVERSION_ES6;
+        else if(compat_mode >= COMPAT_MODE_IE9)
             script_mode = SCRIPTLANGUAGEVERSION_ES5;
         script_mode |= SCRIPTLANGUAGEVERSION_HTML;
     }
@@ -178,44 +175,44 @@ static BOOL init_script_engine(ScriptHost *script_host)
 
     hres = IActiveScriptParse_InitNew(script_host->parse);
     if(FAILED(hres)) {
-        WARN("InitNew failed: %08x\n", hres);
+        WARN("InitNew failed: %08lx\n", hres);
         return FALSE;
     }
 
     hres = IActiveScript_SetScriptSite(script_host->script, &script_host->IActiveScriptSite_iface);
     if(FAILED(hres)) {
-        WARN("SetScriptSite failed: %08x\n", hres);
+        WARN("SetScriptSite failed: %08lx\n", hres);
         IActiveScript_Close(script_host->script);
         return FALSE;
     }
 
     hres = IActiveScript_GetScriptState(script_host->script, &state);
     if(FAILED(hres))
-        WARN("GetScriptState failed: %08x\n", hres);
+        WARN("GetScriptState failed: %08lx\n", hres);
     else if(state != SCRIPTSTATE_INITIALIZED)
         FIXME("state = %x\n", state);
 
     hres = IActiveScript_SetScriptState(script_host->script, SCRIPTSTATE_STARTED);
     if(FAILED(hres)) {
-        WARN("Starting script failed: %08x\n", hres);
+        WARN("Starting script failed: %08lx\n", hres);
         return FALSE;
     }
 
-    hres = IActiveScript_AddNamedItem(script_host->script, windowW,
+    hres = IActiveScript_AddNamedItem(script_host->script, L"window",
             SCRIPTITEM_ISVISIBLE|SCRIPTITEM_ISSOURCE|SCRIPTITEM_GLOBALMEMBERS);
     if(SUCCEEDED(hres)) {
         V_VT(&var) = VT_BOOL;
         V_BOOL(&var) = VARIANT_TRUE;
         set_script_prop(script_host, SCRIPTPROP_ABBREVIATE_GLOBALNAME_RESOLUTION, &var);
     }else {
-       WARN("AddNamedItem failed: %08x\n", hres);
+       WARN("AddNamedItem failed: %08lx\n", hres);
     }
 
     hres = IActiveScript_QueryInterface(script_host->script, &IID_IActiveScriptParseProcedure2,
                                         (void**)&script_host->parse_proc);
     if(FAILED(hres)) {
         /* FIXME: QI for IActiveScriptParseProcedure */
-        WARN("Could not get IActiveScriptParseProcedure iface: %08x\n", hres);
+        WARN("Could not get IActiveScriptParseProcedure iface: %08lx\n", hres);
     }
 
     return TRUE;
@@ -311,7 +308,7 @@ static ULONG WINAPI ActiveScriptSite_AddRef(IActiveScriptSite *iface)
     ScriptHost *This = impl_from_IActiveScriptSite(iface);
     LONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     return ref;
 }
@@ -321,13 +318,13 @@ static ULONG WINAPI ActiveScriptSite_Release(IActiveScriptSite *iface)
     ScriptHost *This = impl_from_IActiveScriptSite(iface);
     LONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     if(!ref) {
         release_script_engine(This);
         if(This->window)
             list_remove(&This->entry);
-        heap_free(This);
+        free(This);
     }
 
     return ref;
@@ -348,16 +345,16 @@ static HRESULT WINAPI ActiveScriptSite_GetItemInfo(IActiveScriptSite *iface, LPC
 {
     ScriptHost *This = impl_from_IActiveScriptSite(iface);
 
-    TRACE("(%p)->(%s %x %p %p)\n", This, debugstr_w(pstrName), dwReturnMask, ppiunkItem, ppti);
+    TRACE("(%p)->(%s %lx %p %p)\n", This, debugstr_w(pstrName), dwReturnMask, ppiunkItem, ppti);
 
     if(dwReturnMask != SCRIPTINFO_IUNKNOWN) {
-        FIXME("Unsupported mask %x\n", dwReturnMask);
+        FIXME("Unsupported mask %lx\n", dwReturnMask);
         return E_NOTIMPL;
     }
 
     *ppiunkItem = NULL;
 
-    if(wcscmp(pstrName, windowW))
+    if(wcscmp(pstrName, L"window"))
         return DISP_E_MEMBERNOTFOUND;
 
     if(!This->window)
@@ -595,7 +592,7 @@ static HRESULT WINAPI ActiveScriptSiteDebug_GetDocumentContextFromPosition(IActi
             CTXARG_T dwSourceContext, ULONG uCharacterOffset, ULONG uNumChars, IDebugDocumentContext **ppsc)
 {
     ScriptHost *This = impl_from_IActiveScriptSiteDebug(iface);
-    FIXME("(%p)->(%s %u %u %p)\n", This, wine_dbgstr_longlong(dwSourceContext), uCharacterOffset,
+    FIXME("(%p)->(%s %lu %lu %p)\n", This, wine_dbgstr_longlong(dwSourceContext), uCharacterOffset,
           uNumChars, ppsc);
     return E_NOTIMPL;
 }
@@ -677,7 +674,7 @@ static HRESULT WINAPI ASServiceProvider_QueryService(IServiceProvider *iface, RE
         if(!This->window || !This->window->doc)
             return E_NOINTERFACE;
 
-        return IHTMLDocument2_QueryInterface(&This->window->doc->basedoc.IHTMLDocument2_iface, riid, ppv);
+        return IHTMLDocument2_QueryInterface(&This->window->doc->IHTMLDocument2_iface, riid, ppv);
     }
 
     FIXME("(%p)->(%s %s %p)\n", This, debugstr_guid(guidService), debugstr_guid(riid), ppv);
@@ -696,7 +693,7 @@ static ScriptHost *create_script_host(HTMLInnerWindow *window, const GUID *guid)
     ScriptHost *ret;
     HRESULT hres;
 
-    ret = heap_alloc_zero(sizeof(*ret));
+    ret = calloc(1, sizeof(*ret));
     if(!ret)
         return NULL;
 
@@ -716,7 +713,7 @@ static ScriptHost *create_script_host(HTMLInnerWindow *window, const GUID *guid)
     hres = CoCreateInstance(&ret->guid, NULL, CLSCTX_INPROC_SERVER|CLSCTX_INPROC_HANDLER,
             &IID_IActiveScript, (void**)&ret->script);
     if(FAILED(hres))
-        WARN("Could not load script engine: %08x\n", hres);
+        WARN("Could not load script engine: %08lx\n", hres);
     else if(!init_script_engine(ret))
         release_script_engine(ret);
 
@@ -728,12 +725,22 @@ static void dispatch_script_readystatechange_event(HTMLScriptElement *script)
     DOMEvent *event;
     HRESULT hres;
 
-    hres = create_document_event(script->element.node.doc, EVENTID_READYSTATECHANGE, &event);
-    if(FAILED(hres))
-        return;
+    if(script->readystate != READYSTATE_LOADED ||
+       dispex_compat_mode(&script->element.node.event_target.dispex) < COMPAT_MODE_IE10) {
+        hres = create_document_event(script->element.node.doc, EVENTID_READYSTATECHANGE, &event);
+        if(SUCCEEDED(hres)) {
+            dispatch_event(&script->element.node.event_target, event);
+            IDOMEvent_Release(&event->IDOMEvent_iface);
+        }
+    }
 
-    dispatch_event(&script->element.node.event_target, event);
-    IDOMEvent_Release(&event->IDOMEvent_iface);
+    if(script->readystate == READYSTATE_LOADED) {
+        hres = create_document_event(script->element.node.doc, EVENTID_LOAD, &event);
+        if(SUCCEEDED(hres)) {
+            dispatch_event(&script->element.node.event_target, event);
+            IDOMEvent_Release(&event->IDOMEvent_iface);
+        }
+    }
 }
 
 typedef struct {
@@ -763,6 +770,10 @@ static void set_script_elem_readystate(HTMLScriptElement *script_elem, READYSTAT
 {
     script_elem->readystate = readystate;
 
+    if(readystate != READYSTATE_LOADED &&
+       dispex_compat_mode(&script_elem->element.node.event_target.dispex) >= COMPAT_MODE_IE11)
+        return;
+
     if(readystate != READYSTATE_INTERACTIVE) {
         if(!script_elem->element.node.doc->window->parser_callback_cnt) {
             fire_readystatechange_task_t *task;
@@ -771,7 +782,7 @@ static void set_script_elem_readystate(HTMLScriptElement *script_elem, READYSTAT
             if(script_elem->pending_readystatechange_event)
                 return;
 
-            task = heap_alloc(sizeof(*task));
+            task = malloc(sizeof(*task));
             if(!task)
                 return;
 
@@ -800,19 +811,21 @@ static void parse_elem_text(ScriptHost *script_host, HTMLScriptElement *script_e
     VariantInit(&var);
     memset(&excepinfo, 0, sizeof(excepinfo));
     TRACE(">>>\n");
-    hres = IActiveScriptParse_ParseScriptText(script_host->parse, text, windowW, NULL, script_endW,
+    hres = IActiveScriptParse_ParseScriptText(script_host->parse, text, L"window", NULL, L"</SCRIPT>",
                                               0, 0, SCRIPTTEXT_ISVISIBLE|SCRIPTTEXT_HOSTMANAGESSOURCE,
                                               &var, &excepinfo);
     if(SUCCEEDED(hres))
         TRACE("<<<\n");
     else
-        WARN("<<< %08x\n", hres);
+        WARN("<<< %08lx\n", hres);
 }
 
 typedef struct {
     BSCallback bsc;
 
     HTMLScriptElement *script_elem;
+    nsILoadGroup *load_group;
+    nsIRequest *request;
     DWORD scheme;
 
     DWORD size;
@@ -825,7 +838,7 @@ static HRESULT get_binding_text(ScriptBSC *bsc, WCHAR **ret)
     WCHAR *text;
 
     if(!bsc->bsc.read) {
-        text = heap_alloc(sizeof(WCHAR));
+        text = malloc(sizeof(WCHAR));
         if(!text)
             return E_OUTOFMEMORY;
         *text = 0;
@@ -840,7 +853,7 @@ static HRESULT get_binding_text(ScriptBSC *bsc, WCHAR **ret)
             return E_FAIL;
         }
 
-        text = heap_alloc(bsc->bsc.read+sizeof(WCHAR));
+        text = malloc(bsc->bsc.read + sizeof(WCHAR));
         if(!text)
             return E_OUTOFMEMORY;
 
@@ -856,7 +869,7 @@ static HRESULT get_binding_text(ScriptBSC *bsc, WCHAR **ret)
         DWORD len;
 
         len = MultiByteToWideChar(cp, 0, bsc->buf, bsc->bsc.read, NULL, 0);
-        text = heap_alloc((len+1)*sizeof(WCHAR));
+        text = malloc((len + 1) * sizeof(WCHAR));
         if(!text)
             return E_OUTOFMEMORY;
 
@@ -893,7 +906,7 @@ static void script_file_available(ScriptBSC *bsc)
 
         TRACE("Adding to queue\n");
 
-        queue = heap_alloc(sizeof(*queue));
+        queue = malloc(sizeof(*queue));
         if(!queue)
             return;
 
@@ -936,8 +949,15 @@ static void ScriptBSC_destroy(BSCallback *bsc)
         This->script_elem = NULL;
     }
 
-    heap_free(This->buf);
-    heap_free(This);
+    if(This->request) {
+        ERR("Unfinished request\n");
+        nsIRequest_Release(This->request);
+    }
+    if(This->load_group)
+        nsILoadGroup_Release(This->load_group);
+
+    free(This->buf);
+    free(This);
 }
 
 static HRESULT ScriptBSC_init_bindinfo(BSCallback *bsc)
@@ -948,9 +968,19 @@ static HRESULT ScriptBSC_init_bindinfo(BSCallback *bsc)
 static HRESULT ScriptBSC_start_binding(BSCallback *bsc)
 {
     ScriptBSC *This = impl_from_BSCallback(bsc);
+    nsresult nsres;
 
     This->script_elem->binding = &This->bsc;
 
+    if(This->load_group) {
+        nsres = create_onload_blocker_request(&This->request);
+        if(NS_SUCCEEDED(nsres)) {
+            nsres = nsILoadGroup_AddRequest(This->load_group, This->request, NULL);
+            if(NS_FAILED(nsres))
+                ERR("AddRequest failed: %08lx\n", nsres);
+        }
+    }
+
     /* FIXME: We should find a better to decide if 'loading' state is supposed to be used by the protocol. */
     if(This->scheme == URL_SCHEME_HTTPS || This->scheme == URL_SCHEME_HTTP)
         set_script_elem_readystate(This->script_elem, READYSTATE_LOADING);
@@ -961,6 +991,7 @@ static HRESULT ScriptBSC_start_binding(BSCallback *bsc)
 static HRESULT ScriptBSC_stop_binding(BSCallback *bsc, HRESULT result)
 {
     ScriptBSC *This = impl_from_BSCallback(bsc);
+    nsresult nsres;
 
     if(SUCCEEDED(result) && !This->script_elem)
         result = E_UNEXPECTED;
@@ -974,12 +1005,20 @@ static HRESULT ScriptBSC_stop_binding(BSCallback *bsc, HRESULT result)
     if(SUCCEEDED(result)) {
         script_file_available(This);
     }else {
-        FIXME("binding failed %08x\n", result);
-        heap_free(This->buf);
+        FIXME("binding failed %08lx\n", result);
+        free(This->buf);
         This->buf = NULL;
         This->size = 0;
     }
 
+    if(This->request) {
+        nsres = nsILoadGroup_RemoveRequest(This->load_group, This->request, NULL, NS_OK);
+        if(NS_FAILED(nsres))
+            ERR("RemoveRequest failed: %08lx\n", nsres);
+        nsIRequest_Release(This->request);
+        This->request = NULL;
+    }
+
     IHTMLScriptElement_Release(&This->script_elem->IHTMLScriptElement_iface);
     This->script_elem = NULL;
     return S_OK;
@@ -992,7 +1031,7 @@ static HRESULT ScriptBSC_read_data(BSCallback *bsc, IStream *stream)
     HRESULT hres;
 
     if(!This->buf) {
-        This->buf = heap_alloc(128);
+        This->buf = malloc(128);
         if(!This->buf)
             return E_OUTOFMEMORY;
         This->size = 128;
@@ -1001,7 +1040,7 @@ static HRESULT ScriptBSC_read_data(BSCallback *bsc, IStream *stream)
     do {
         if(This->bsc.read >= This->size) {
             void *new_buf;
-            new_buf = heap_realloc(This->buf, This->size << 1);
+            new_buf = realloc(This->buf, This->size << 1);
             if(!new_buf)
                 return E_OUTOFMEMORY;
             This->size <<= 1;
@@ -1014,7 +1053,7 @@ static HRESULT ScriptBSC_read_data(BSCallback *bsc, IStream *stream)
     return S_OK;
 }
 
-static HRESULT ScriptBSC_on_progress(BSCallback *bsc, ULONG status_code, LPCWSTR status_text)
+static HRESULT ScriptBSC_on_progress(BSCallback *bsc, ULONG progress, ULONG total, ULONG status_code, LPCWSTR status_text)
 {
     return S_OK;
 }
@@ -1072,7 +1111,7 @@ HRESULT load_script(HTMLScriptElement *script_elem, const WCHAR *src, BOOL async
         return hres;
     }
 
-    bsc = heap_alloc_zero(sizeof(*bsc));
+    bsc = calloc(1, sizeof(*bsc));
     if(!bsc) {
         IMoniker_Release(mon);
         IUri_Release(uri);
@@ -1084,12 +1123,26 @@ HRESULT load_script(HTMLScriptElement *script_elem, const WCHAR *src, BOOL async
 
     hres = IUri_GetScheme(uri, &bsc->scheme);
     IUri_Release(uri);
-    if(FAILED(hres))
+    if(hres != S_OK)
         bsc->scheme = URL_SCHEME_UNKNOWN;
 
     IHTMLScriptElement_AddRef(&script_elem->IHTMLScriptElement_iface);
     bsc->script_elem = script_elem;
 
+    if(window->bscallback && window->bscallback->nschannel &&
+       window->bscallback->nschannel->load_group) {
+        cpp_bool contains;
+        nsresult nsres;
+
+        nsres = nsIDOMNode_Contains(script_elem->element.node.doc->node.nsnode,
+                                    script_elem->element.node.nsnode, &contains);
+        if(NS_SUCCEEDED(nsres) && contains) {
+            TRACE("script %p will block load event\n", script_elem);
+            bsc->load_group = window->bscallback->nschannel->load_group;
+            nsILoadGroup_AddRef(bsc->load_group);
+        }
+    }
+
     hres = start_binding(window, &bsc->bsc, NULL);
 
     IBindStatusCallback_Release(&bsc->bsc.IBindStatusCallback_iface);
@@ -1109,7 +1162,7 @@ static void parse_inline_script(ScriptHost *script_host, HTMLScriptElement *scri
     set_script_elem_readystate(script_elem, READYSTATE_INTERACTIVE);
 
     if(NS_FAILED(nsres)) {
-        ERR("GetText failed: %08x\n", nsres);
+        ERR("GetText failed: %08lx\n", nsres);
     }else if(*text) {
         parse_elem_text(script_host, script_elem, text);
     }
@@ -1136,7 +1189,7 @@ static BOOL parse_script_elem(ScriptHost *script_host, HTMLScriptElement *script
             return FALSE;
         }
     }else {
-        ERR("GetEvent failed: %08x\n", nsres);
+        ERR("GetEvent failed: %08lx\n", nsres);
     }
     nsAString_Finish(&event_str);
 
@@ -1145,7 +1198,7 @@ static BOOL parse_script_elem(ScriptHost *script_host, HTMLScriptElement *script
     nsAString_GetData(&src_str, &src);
 
     if(NS_FAILED(nsres)) {
-        ERR("GetSrc failed: %08x\n", nsres);
+        ERR("GetSrc failed: %08lx\n", nsres);
     }else if(*src) {
         load_script(script_elem, src, FALSE);
         is_complete = script_elem->parsed;
@@ -1169,17 +1222,10 @@ static GUID get_default_script_guid(HTMLInnerWindow *window)
 
 static BOOL get_guid_from_type(LPCWSTR type, GUID *guid)
 {
-    static const WCHAR text_javascriptW[] =
-        {'t','e','x','t','/','j','a','v','a','s','c','r','i','p','t',0};
-    static const WCHAR text_jscriptW[] =
-        {'t','e','x','t','/','j','s','c','r','i','p','t',0};
-    static const WCHAR text_vbscriptW[] =
-        {'t','e','x','t','/','v','b','s','c','r','i','p','t',0};
-
     /* FIXME: Handle more types */
-    if(!wcsicmp(type, text_javascriptW) || !wcsicmp(type, text_jscriptW)) {
+    if(!wcsicmp(type, L"text/javascript") || !wcsicmp(type, L"text/jscript")) {
         *guid = CLSID_JScript;
-    }else if(!wcsicmp(type, text_vbscriptW)) {
+    }else if(!wcsicmp(type, L"text/vbscript")) {
         *guid = CLSID_VBScript;
     }else {
         FIXME("Unknown type %s\n", debugstr_w(type));
@@ -1210,8 +1256,6 @@ static BOOL get_script_guid(HTMLInnerWindow *window, nsIDOMHTMLScriptElement *ns
     BOOL ret = FALSE;
     nsresult nsres;
 
-    static const PRUnichar languageW[] = {'l','a','n','g','u','a','g','e',0};
-
     nsAString_Init(&val_str, NULL);
 
     nsres = nsIDOMHTMLScriptElement_GetType(nsscript, &val_str);
@@ -1225,13 +1269,13 @@ static BOOL get_script_guid(HTMLInnerWindow *window, nsIDOMHTMLScriptElement *ns
             return ret;
         }
     }else {
-        ERR("GetType failed: %08x\n", nsres);
+        ERR("GetType failed: %08lx\n", nsres);
     }
 
     nsres = nsIDOMHTMLScriptElement_QueryInterface(nsscript, &IID_nsIDOMElement, (void**)&nselem);
     assert(nsres == NS_OK);
 
-    nsres = get_elem_attr_value(nselem, languageW, &val_str, &language);
+    nsres = get_elem_attr_value(nselem, L"language", &val_str, &language);
     nsIDOMElement_Release(nselem);
     if(NS_SUCCEEDED(nsres)) {
         if(*language) {
@@ -1308,8 +1352,6 @@ IDispatch *script_parse_event(HTMLInnerWindow *window, LPCWSTR text)
     IDispatch *disp;
     HRESULT hres;
 
-    static const WCHAR delimiterW[] = {'\"',0};
-
     TRACE("%s\n", debugstr_w(text));
 
     for(ptr = text; iswalnum(*ptr); ptr++);
@@ -1317,7 +1359,7 @@ IDispatch *script_parse_event(HTMLInnerWindow *window, LPCWSTR text)
         LPWSTR language;
         BOOL b;
 
-        language = heap_alloc((ptr-text+1)*sizeof(WCHAR));
+        language = malloc((ptr - text + 1) * sizeof(WCHAR));
         if(!language)
             return NULL;
 
@@ -1326,7 +1368,7 @@ IDispatch *script_parse_event(HTMLInnerWindow *window, LPCWSTR text)
 
         b = get_guid_from_language(language, &guid);
 
-        heap_free(language);
+        free(language);
 
         if(!b) {
             WARN("Could not find language\n");
@@ -1349,11 +1391,11 @@ IDispatch *script_parse_event(HTMLInnerWindow *window, LPCWSTR text)
     if(!script_host || !script_host->parse_proc)
         return NULL;
 
-    hres = IActiveScriptParseProcedure2_ParseProcedureText(script_host->parse_proc, ptr, NULL, emptyW,
-            NULL, NULL, delimiterW, 0 /* FIXME */, 0,
+    hres = IActiveScriptParseProcedure2_ParseProcedureText(script_host->parse_proc, ptr, NULL, L"",
+            NULL, NULL, L"\"", 0 /* FIXME */, 0,
             SCRIPTPROC_HOSTMANAGESSOURCE|SCRIPTPROC_IMPLICIT_THIS|SCRIPTPROC_IMPLICIT_PARENTS, &disp);
     if(FAILED(hres)) {
-        WARN("ParseProcedureText failed: %08x\n", hres);
+        WARN("ParseProcedureText failed: %08lx\n", hres);
         return NULL;
     }
 
@@ -1368,8 +1410,6 @@ HRESULT exec_script(HTMLInnerWindow *window, const WCHAR *code, const WCHAR *lan
     GUID guid;
     HRESULT hres;
 
-    static const WCHAR delimW[] = {'"',0};
-
     if(!get_guid_from_language(lang, &guid)) {
         WARN("Could not find script GUID\n");
         return CO_E_CLASSSTRING;
@@ -1388,11 +1428,11 @@ HRESULT exec_script(HTMLInnerWindow *window, const WCHAR *code, const WCHAR *lan
 
     memset(&ei, 0, sizeof(ei));
     TRACE(">>>\n");
-    hres = IActiveScriptParse_ParseScriptText(script_host->parse, code, NULL, NULL, delimW, 0, 0, SCRIPTTEXT_ISVISIBLE, ret, &ei);
+    hres = IActiveScriptParse_ParseScriptText(script_host->parse, code, NULL, NULL, L"\"", 0, 0, SCRIPTTEXT_ISVISIBLE, ret, &ei);
     if(SUCCEEDED(hres))
         TRACE("<<<\n");
     else
-        WARN("<<< %08x\n", hres);
+        WARN("<<< %08lx\n", hres);
 
     return hres;
 }
@@ -1405,7 +1445,7 @@ IDispatch *get_script_disp(ScriptHost *script_host)
     if(!script_host->script)
         return NULL;
 
-    hres = IActiveScript_GetScriptDispatch(script_host->script, windowW, &disp);
+    hres = IActiveScript_GetScriptDispatch(script_host->script, L"window", &disp);
     if(FAILED(hres))
         return NULL;
 
@@ -1423,7 +1463,7 @@ static EventTarget *find_event_target(HTMLDocumentNode *doc, HTMLScriptElement *
     nsAString_Init(&target_id_str, NULL);
     nsres = nsIDOMHTMLScriptElement_GetHtmlFor(script_elem->nsscript, &target_id_str);
     if(NS_FAILED(nsres)) {
-        ERR("GetScriptFor failed: %08x\n", nsres);
+        ERR("GetScriptFor failed: %08lx\n", nsres);
         nsAString_Finish(&target_id_str);
         return NULL;
     }
@@ -1431,10 +1471,10 @@ static EventTarget *find_event_target(HTMLDocumentNode *doc, HTMLScriptElement *
     nsAString_GetData(&target_id_str, &target_id);
     if(!*target_id) {
         FIXME("Empty for attribute\n");
-    }else if(!wcscmp(target_id, documentW)) {
+    }else if(!wcscmp(target_id, L"document")) {
         event_target = &doc->node.event_target;
-        htmldoc_addref(&doc->basedoc);
-    }else if(!wcscmp(target_id, windowW)) {
+        IHTMLDOMNode_AddRef(&doc->node.IHTMLDOMNode_iface);
+    }else if(!wcscmp(target_id, L"window")) {
         if(doc->window) {
             event_target = &doc->window->event_target;
             IDispatchEx_AddRef(&event_target->dispex.IDispatchEx_iface);
@@ -1502,7 +1542,7 @@ static IDispatch *parse_event_elem(HTMLDocumentNode *doc, HTMLScriptElement *scr
         const PRUnichar *event_val;
 
         nsAString_GetData(&nsstr, &event_val);
-        event = heap_strdupW(event_val);
+        event = wcsdup(event_val);
     }
     nsAString_Finish(&nsstr);
     if(!event)
@@ -1510,7 +1550,7 @@ static IDispatch *parse_event_elem(HTMLDocumentNode *doc, HTMLScriptElement *scr
 
     if(!parse_event_str(event, &args)) {
         WARN("parsing %s failed\n", debugstr_w(event));
-        heap_free(event);
+        free(event);
         return NULL;
     }
 
@@ -1521,17 +1561,17 @@ static IDispatch *parse_event_elem(HTMLDocumentNode *doc, HTMLScriptElement *scr
 
         nsAString_GetData(&nsstr, &text);
         hres = IActiveScriptParseProcedure2_ParseProcedureText(script_host->parse_proc, text, args,
-                emptyW, NULL, NULL, script_endW, 0, 0,
+                L"", NULL, NULL, L"</SCRIPT>", 0, 0,
                 SCRIPTPROC_HOSTMANAGESSOURCE|SCRIPTPROC_IMPLICIT_THIS|SCRIPTPROC_IMPLICIT_PARENTS, &disp);
         if(FAILED(hres))
             disp = NULL;
     }else {
-        ERR("GetText failed: %08x\n", nsres);
+        ERR("GetText failed: %08lx\n", nsres);
         disp = NULL;
     }
     nsAString_Finish(&nsstr);
     if(!disp) {
-        heap_free(event);
+        free(event);
         return NULL;
     }
 
@@ -1554,18 +1594,16 @@ void bind_event_scripts(HTMLDocumentNode *doc)
     nsresult nsres;
     HRESULT hres;
 
-    static const PRUnichar selectorW[] = {'s','c','r','i','p','t','[','e','v','e','n','t',']',0};
-
     TRACE("%p\n", doc);
 
-    if(!doc->nsdoc)
+    if(!doc->dom_document)
         return;
 
-    nsAString_InitDepend(&selector_str, selectorW);
-    nsres = nsIDOMHTMLDocument_QuerySelectorAll(doc->nsdoc, &selector_str, &node_list);
+    nsAString_InitDepend(&selector_str, L"script[event]");
+    nsres = nsIDOMDocument_QuerySelectorAll(doc->dom_document, &selector_str, &node_list);
     nsAString_Finish(&selector_str);
     if(NS_FAILED(nsres)) {
-        ERR("QuerySelectorAll failed: %08x\n", nsres);
+        ERR("QuerySelectorAll failed: %08lx\n", nsres);
         return;
     }
 
@@ -1578,7 +1616,7 @@ void bind_event_scripts(HTMLDocumentNode *doc)
     for(i=0; i < length; i++) {
         nsres = nsIDOMNodeList_Item(node_list, i, &script_node);
         if(NS_FAILED(nsres) || !script_node) {
-            ERR("Item(%d) failed: %08x\n", i, nsres);
+            ERR("Item(%d) failed: %08lx\n", i, nsres);
             continue;
         }
 
@@ -1587,6 +1625,7 @@ void bind_event_scripts(HTMLDocumentNode *doc)
         nsIDOMNode_Release(script_node);
 
         hres = script_elem_from_nsscript(nsscript, &script_elem);
+        nsIDOMHTMLScriptElement_Release(nsscript);
         if(FAILED(hres))
             continue;
 
@@ -1606,7 +1645,7 @@ void bind_event_scripts(HTMLDocumentNode *doc)
                     node_release(&plugin_container->element.node);
             }
 
-            heap_free(event);
+            free(event);
             IDispatch_Release(event_disp);
         }
 
@@ -1678,13 +1717,14 @@ static BOOL use_gecko_script(IUri *uri)
         return FALSE;
 
     hres = IInternetSecurityManager_MapUrlToZone(get_security_manager(), display_uri, &zone, 0);
-    SysFreeString(display_uri);
     if(FAILED(hres)) {
-        WARN("Could not map %s to zone: %08x\n", debugstr_w(display_uri), hres);
+        WARN("Could not map %s to zone: %08lx\n", debugstr_w(display_uri), hres);
+        SysFreeString(display_uri);
         return TRUE;
     }
 
-    TRACE("zone %d\n", zone);
+    SysFreeString(display_uri);
+    TRACE("zone %ld\n", zone);
     return zone == URLZONE_UNTRUSTED;
 }
 
@@ -1713,7 +1753,7 @@ void update_browser_script_mode(GeckoBrowser *browser, IUri *uri)
     }
 
     if(NS_FAILED(nsres))
-        ERR("JavaScript setup failed: %08x\n", nsres);
+        ERR("JavaScript setup failed: %08lx\n", nsres);
 }
 
 void release_script_hosts(HTMLInnerWindow *window)
@@ -1726,7 +1766,7 @@ void release_script_hosts(HTMLInnerWindow *window)
 
         list_remove(&queue_iter->entry);
         IHTMLScriptElement_Release(&queue_iter->script->IHTMLScriptElement_iface);
-        heap_free(queue_iter);
+        free(queue_iter);
     }
 
     while(!list_empty(&window->script_hosts)) {
