@@ -29,7 +29,6 @@
 #include "msidefs.h"
 #include "winver.h"
 #include "shlwapi.h"
-#include "wine/unicode.h"
 #include "wine/debug.h"
 #include "msipriv.h"
 
@@ -55,20 +54,20 @@ void msi_parse_version_string(LPCWSTR verStr, PDWORD ms, PDWORD ls)
     const WCHAR *ptr;
     int x1 = 0, x2 = 0, x3 = 0, x4 = 0;
 
-    x1 = atoiW(verStr);
-    ptr = strchrW(verStr, '.');
+    x1 = wcstol(verStr, NULL, 10);
+    ptr = wcschr(verStr, '.');
     if (ptr)
     {
-        x2 = atoiW(ptr + 1);
-        ptr = strchrW(ptr + 1, '.');
+        x2 = wcstol(ptr + 1, NULL, 10);
+        ptr = wcschr(ptr + 1, '.');
     }
     if (ptr)
     {
-        x3 = atoiW(ptr + 1);
-        ptr = strchrW(ptr + 1, '.');
+        x3 = wcstol(ptr + 1, NULL, 10);
+        ptr = wcschr(ptr + 1, '.');
     }
     if (ptr)
-        x4 = atoiW(ptr + 1);
+        x4 = wcstol(ptr + 1, NULL, 10);
     /* FIXME: byte-order dependent? */
     *ms = x1 << 16 | x2;
     if (ls) *ls = x3 << 16 | x4;
@@ -83,13 +82,7 @@ void msi_parse_version_string(LPCWSTR verStr, PDWORD ms, PDWORD ls)
  */
 static UINT get_signature( MSIPACKAGE *package, MSISIGNATURE *sig, const WCHAR *name )
 {
-    static const WCHAR query[] = {
-        's','e','l','e','c','t',' ','*',' ',
-        'f','r','o','m',' ',
-        'S','i','g','n','a','t','u','r','e',' ',
-        'w','h','e','r','e',' ','S','i','g','n','a','t','u','r','e',' ','=',' ',
-        '\'','%','s','\'',0};
-    LPWSTR minVersion, maxVersion, p;
+    WCHAR *minVersion, *maxVersion, *p;
     MSIRECORD *row;
     DWORD time;
 
@@ -97,7 +90,7 @@ static UINT get_signature( MSIPACKAGE *package, MSISIGNATURE *sig, const WCHAR *
 
     memset(sig, 0, sizeof(*sig));
     sig->Name = name;
-    row = MSI_QueryGetRecord( package->db, query, name );
+    row = MSI_QueryGetRecord( package->db, L"SELECT * FROM `Signature` WHERE `Signature` = '%s'", name );
     if (!row)
     {
         TRACE("failed to query signature for %s\n", debugstr_w(name));
@@ -106,23 +99,23 @@ static UINT get_signature( MSIPACKAGE *package, MSISIGNATURE *sig, const WCHAR *
 
     /* get properties */
     sig->File = msi_dup_record_field(row,2);
-    if ((p = strchrW(sig->File, '|')))
+    if ((p = wcschr(sig->File, '|')))
     {
         p++;
-        memmove(sig->File, p, (strlenW(p) + 1) * sizeof(WCHAR));
+        memmove(sig->File, p, (lstrlenW(p) + 1) * sizeof(WCHAR));
     }
 
     minVersion = msi_dup_record_field(row,3);
     if (minVersion)
     {
         msi_parse_version_string( minVersion, &sig->MinVersionMS, &sig->MinVersionLS );
-        msi_free( minVersion );
+        free( minVersion );
     }
     maxVersion = msi_dup_record_field(row,4);
     if (maxVersion)
     {
         msi_parse_version_string( maxVersion, &sig->MaxVersionMS, &sig->MaxVersionLS );
-        msi_free( maxVersion );
+        free( maxVersion );
     }
     sig->MinSize = MSI_RecordGetInteger(row,5);
     if (sig->MinSize == MSI_NULL_INTEGER)
@@ -146,7 +139,7 @@ static UINT get_signature( MSIPACKAGE *package, MSISIGNATURE *sig, const WCHAR *
     TRACE("MaxVersion is %d.%d.%d.%d\n", HIWORD(sig->MaxVersionMS),
           LOWORD(sig->MaxVersionMS), HIWORD(sig->MaxVersionLS),
           LOWORD(sig->MaxVersionLS));
-    TRACE("MinSize is %d, MaxSize is %d;\n", sig->MinSize, sig->MaxSize);
+    TRACE("MinSize is %lu, MaxSize is %lu\n", sig->MinSize, sig->MaxSize);
     TRACE("Languages is %s\n", debugstr_w(sig->Languages));
 
     msiobj_release( &row->hdr );
@@ -157,14 +150,15 @@ static UINT get_signature( MSIPACKAGE *package, MSISIGNATURE *sig, const WCHAR *
 /* Frees any memory allocated in sig */
 static void free_signature( MSISIGNATURE *sig )
 {
-    msi_free(sig->File);
-    msi_free(sig->Languages);
+    free(sig->File);
+    free(sig->Languages);
 }
 
 static WCHAR *search_file( MSIPACKAGE *package, WCHAR *path, MSISIGNATURE *sig )
 {
     VS_FIXEDFILEINFO *info;
-    DWORD attr, size;
+    DWORD attr;
+    UINT size;
     LPWSTR val = NULL;
     LPBYTE buffer;
 
@@ -175,7 +169,7 @@ static WCHAR *search_file( MSIPACKAGE *package, WCHAR *path, MSISIGNATURE *sig )
 
         attr = msi_get_file_attributes( package, path );
         if (attr != INVALID_FILE_ATTRIBUTES && (attr & FILE_ATTRIBUTE_DIRECTORY))
-            return strdupW(path);
+            return wcsdup(path);
 
         return NULL;
     }
@@ -186,9 +180,9 @@ static WCHAR *search_file( MSIPACKAGE *package, WCHAR *path, MSISIGNATURE *sig )
 
     size = msi_get_file_version_info( package, path, 0, NULL );
     if (!size)
-        return strdupW(path);
+        return wcsdup(path);
 
-    buffer = msi_alloc(size);
+    buffer = malloc(size);
     if (!buffer)
         return NULL;
 
@@ -196,7 +190,7 @@ static WCHAR *search_file( MSIPACKAGE *package, WCHAR *path, MSISIGNATURE *sig )
     if (!size)
         goto done;
 
-    if (!VerQueryValueW(buffer, szBackSlash, (LPVOID)&info, &size) || !info)
+    if (!VerQueryValueW(buffer, L"\\", (LPVOID)&info, &size) || !info)
         goto done;
 
     if (sig->MinVersionLS || sig->MinVersionMS)
@@ -219,26 +213,15 @@ static WCHAR *search_file( MSIPACKAGE *package, WCHAR *path, MSISIGNATURE *sig )
             goto done;
     }
 
-    val = strdupW(path);
+    val = wcsdup(path);
 
 done:
-    msi_free(buffer);
+    free(buffer);
     return val;
 }
 
 static UINT search_components( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig )
 {
-    static const WCHAR query[] =  {
-        'S','E','L','E','C','T',' ','*',' ',
-        'F','R','O','M',' ',
-        '`','C','o','m','p','L','o','c','a','t','o','r','`',' ',
-        'W','H','E','R','E',' ','`','S','i','g','n','a','t','u','r','e','_','`',' ','=',' ',
-        '\'','%','s','\'',0};
-    static const WCHAR sigquery[] = {
-        'S','E','L','E','C','T',' ','*',' ','F','R','O','M',' ',
-        '`','S','i','g','n','a','t','u','r','e','`',' ',
-        'W','H','E','R','E',' ','`','S','i','g','n','a','t','u','r','e','`',' ','=',' ',
-        '\'','%','s','\'',0};
     MSIRECORD *row, *rec;
     LPCWSTR signature, guid;
     BOOL sigpresent = TRUE;
@@ -253,7 +236,7 @@ static UINT search_components( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATU
 
     *appValue = NULL;
 
-    row = MSI_QueryGetRecord(package->db, query, sig->Name);
+    row = MSI_QueryGetRecord(package->db, L"SELECT * FROM `CompLocator` WHERE `Signature_` = '%s'", sig->Name);
     if (!row)
     {
         TRACE("failed to query CompLocator for %s\n", debugstr_w(sig->Name));
@@ -264,7 +247,7 @@ static UINT search_components( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATU
     guid = MSI_RecordGetString(row, 2);
     type = MSI_RecordGetInteger(row, 3);
 
-    rec = MSI_QueryGetRecord(package->db, sigquery, signature);
+    rec = MSI_QueryGetRecord(package->db, L"SELECT * FROM `Signature` WHERE `Signature` = '%s'", signature);
     if (!rec)
         sigpresent = FALSE;
 
@@ -287,13 +270,13 @@ static UINT search_components( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATU
     {
         if (type == msidbLocatorTypeFileName)
         {
-            ptr = strrchrW(path, '\\');
+            ptr = wcsrchr(path, '\\');
             *(ptr + 1) = '\0';
         }
         else
             PathAddBackslashW(path);
 
-        *appValue = strdupW(path);
+        *appValue = wcsdup(path);
     }
     else if (sigpresent)
     {
@@ -302,7 +285,7 @@ static UINT search_components( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATU
 
         attr = msi_get_file_attributes( package, path );
         if (attr != INVALID_FILE_ATTRIBUTES && !(attr & FILE_ATTRIBUTE_DIRECTORY))
-            *appValue = strdupW(path);
+            *appValue = wcsdup(path);
     }
 
 done:
@@ -313,9 +296,6 @@ done:
 
 static void convert_reg_value( DWORD regType, const BYTE *value, DWORD sz, WCHAR **appValue )
 {
-    static const WCHAR dwordFmt[] = { '#','%','d','\0' };
-    static const WCHAR binPre[] = { '#','x','\0' };
-    static const WCHAR binFmt[] = { '%','0','2','X','\0' };
     LPWSTR ptr;
     DWORD i;
 
@@ -325,38 +305,38 @@ static void convert_reg_value( DWORD regType, const BYTE *value, DWORD sz, WCHAR
             if (*(LPCWSTR)value == '#')
             {
                 /* escape leading pound with another */
-                *appValue = msi_alloc(sz + sizeof(WCHAR));
+                *appValue = malloc(sz + sizeof(WCHAR));
                 (*appValue)[0] = '#';
-                strcpyW(*appValue + 1, (LPCWSTR)value);
+                lstrcpyW(*appValue + 1, (LPCWSTR)value);
             }
             else
             {
-                *appValue = msi_alloc(sz);
-                strcpyW(*appValue, (LPCWSTR)value);
+                *appValue = malloc(sz);
+                lstrcpyW(*appValue, (LPCWSTR)value);
             }
             break;
         case REG_DWORD:
             /* 7 chars for digits, 1 for NULL, 1 for #, and 1 for sign
              * char if needed
              */
-            *appValue = msi_alloc(10 * sizeof(WCHAR));
-            sprintfW(*appValue, dwordFmt, *(const DWORD *)value);
+            *appValue = malloc(10 * sizeof(WCHAR));
+            swprintf(*appValue, 10, L"#%d", *(const DWORD *)value);
             break;
         case REG_EXPAND_SZ:
             sz = ExpandEnvironmentStringsW((LPCWSTR)value, NULL, 0);
-            *appValue = msi_alloc(sz * sizeof(WCHAR));
+            *appValue = malloc(sz * sizeof(WCHAR));
             ExpandEnvironmentStringsW((LPCWSTR)value, *appValue, sz);
             break;
         case REG_BINARY:
             /* #x<nibbles>\0 */
-            *appValue = msi_alloc((sz * 2 + 3) * sizeof(WCHAR));
-            lstrcpyW(*appValue, binPre);
-            ptr = *appValue + lstrlenW(binPre);
+            *appValue = malloc((sz * 2 + 3) * sizeof(WCHAR));
+            lstrcpyW(*appValue, L"#x");
+            ptr = *appValue + lstrlenW(L"#x");
             for (i = 0; i < sz; i++, ptr += 2)
-                sprintfW(ptr, binFmt, value[i]);
+                swprintf(ptr, 3, L"%02X", value[i]);
             break;
         default:
-            WARN("unimplemented for values of type %d\n", regType);
+            WARN( "unimplemented for values of type %lu\n", regType );
             *appValue = NULL;
     }
 }
@@ -365,10 +345,6 @@ static UINT search_directory( MSIPACKAGE *, MSISIGNATURE *, const WCHAR *, int,
 
 static UINT search_reg( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig )
 {
-    static const WCHAR query[] =  {
-        'S','E','L','E','C','T',' ','*',' ','F','R','O','M',' ',
-        'R','e','g','L','o','c','a','t','o','r',' ','W','H','E','R','E',' ',
-        'S','i','g','n','a','t','u','r','e','_',' ','=',' ', '\'','%','s','\'',0};
     const WCHAR *keyPath, *valueName;
     WCHAR *deformatted = NULL, *ptr = NULL, *end;
     int root, type;
@@ -383,7 +359,7 @@ static UINT search_reg( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig
 
     *appValue = NULL;
 
-    row = MSI_QueryGetRecord( package->db, query, sig->Name );
+    row = MSI_QueryGetRecord( package->db, L"SELECT * FROM `RegLocator` WHERE `Signature_` = '%s'", sig->Name );
     if (!row)
     {
         TRACE("failed to query RegLocator for %s\n", debugstr_w(sig->Name));
@@ -425,7 +401,7 @@ static UINT search_reg( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig
         goto end;
     }
 
-    msi_free(deformatted);
+    free(deformatted);
     deformat_string(package, valueName, &deformatted);
 
     rc = RegQueryValueExW(key, deformatted, NULL, NULL, NULL, &sz);
@@ -437,7 +413,7 @@ static UINT search_reg( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig
     /* FIXME: sanity-check sz before allocating (is there an upper-limit
      * on the value of a property?)
      */
-    value = msi_alloc( sz );
+    value = malloc(sz);
     rc = RegQueryValueExW(key, deformatted, NULL, &regType, value, &sz);
     if (rc)
     {
@@ -455,15 +431,15 @@ static UINT search_reg( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig
         sz = ExpandEnvironmentStringsW((LPCWSTR)value, NULL, 0);
         if (sz)
         {
-            LPWSTR buf = msi_alloc(sz * sizeof(WCHAR));
+            WCHAR *buf = malloc(sz * sizeof(WCHAR));
             ExpandEnvironmentStringsW((LPCWSTR)value, buf, sz);
-            msi_free(value);
+            free(value);
             value = (LPBYTE)buf;
         }
     }
 
     if ((regType == REG_SZ || regType == REG_EXPAND_SZ) &&
-        (ptr = strchrW((LPWSTR)value, '"')) && (end = strchrW(++ptr, '"')))
+        (ptr = wcschr((LPWSTR)value, '"')) && (end = wcschr(++ptr, '"')))
         *end = '\0';
     else
         ptr = (LPWSTR)value;
@@ -484,9 +460,9 @@ static UINT search_reg( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig
               type, debugstr_w(keyPath), debugstr_w(valueName));
     }
 end:
-    msi_free( value );
+    free( value );
     RegCloseKey( key );
-    msi_free( deformatted );
+    free( deformatted );
 
     msiobj_release(&row->hdr);
     return ERROR_SUCCESS;
@@ -498,10 +474,10 @@ static LPWSTR get_ini_field(LPWSTR buf, int field)
     int i = 1;
 
     if (field == 0)
-        return strdupW(buf);
+        return wcsdup(buf);
 
     beg = buf;
-    while ((end = strchrW(beg, ',')) && i < field)
+    while ((end = wcschr(beg, ',')) && i < field)
     {
         beg = end + 1;
         while (*beg == ' ')
@@ -510,22 +486,16 @@ static LPWSTR get_ini_field(LPWSTR buf, int field)
         i++;
     }
 
-    end = strchrW(beg, ',');
+    end = wcschr(beg, ',');
     if (!end)
         end = beg + lstrlenW(beg);
 
     *end = '\0';
-    return strdupW(beg);
+    return wcsdup(beg);
 }
 
 static UINT search_ini( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig )
 {
-    static const WCHAR query[] =  {
-        's','e','l','e','c','t',' ','*',' ',
-        'f','r','o','m',' ',
-        'I','n','i','L','o','c','a','t','o','r',' ',
-        'w','h','e','r','e',' ',
-        'S','i','g','n','a','t','u','r','e','_',' ','=',' ','\'','%','s','\'',0};
     MSIRECORD *row;
     LPWSTR fileName, section, key;
     int field, type;
@@ -535,7 +505,7 @@ static UINT search_ini( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig
 
     *appValue = NULL;
 
-    row = MSI_QueryGetRecord( package->db, query, sig->Name );
+    row = MSI_QueryGetRecord( package->db, L"SELECT * FROM `IniLocator` WHERE `Signature_` = '%s'", sig->Name );
     if (!row)
     {
         TRACE("failed to query IniLocator for %s\n", debugstr_w(sig->Name));
@@ -569,9 +539,9 @@ static UINT search_ini( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig
         }
     }
 
-    msi_free(fileName);
-    msi_free(section);
-    msi_free(key);
+    free(fileName);
+    free(section);
+    free(key);
 
     msiobj_release(&row->hdr);
 
@@ -600,48 +570,48 @@ static void expand_any_path( MSIPACKAGE *package, WCHAR *src, WCHAR *dst, size_t
     dst[0] = '\0';
 
     /* Ignore the short portion of the path */
-    if ((ptr = strchrW(src, '|')))
+    if ((ptr = wcschr(src, '|')))
         ptr++;
     else
         ptr = src;
 
     deformat_string(package, ptr, &deformatted);
-    if (!deformatted || strlenW(deformatted) > len - 1)
+    if (!deformatted || lstrlenW(deformatted) > len - 1)
     {
-        msi_free(deformatted);
+        free(deformatted);
         return;
     }
 
     lstrcpyW(dst, deformatted);
     dst[lstrlenW(deformatted)] = '\0';
-    msi_free(deformatted);
+    free(deformatted);
 }
 
 static LANGID *parse_languages( const WCHAR *languages, DWORD *num_ids )
 {
     UINT i, count = 1;
-    WCHAR *str = strdupW( languages ), *p, *q;
+    WCHAR *str = wcsdup( languages ), *p, *q;
     LANGID *ret;
 
     if (!str) return NULL;
-    for (p = q = str; (q = strchrW( q, ',' )); q++) count++;
+    for (p = q = str; (q = wcschr( q, ',' )); q++) count++;
 
-    if (!(ret = msi_alloc( count * sizeof(LANGID) )))
+    if (!(ret = malloc( count * sizeof(LANGID) )))
     {
-        msi_free( str );
+        free( str );
         return NULL;
     }
     i = 0;
     while (*p)
     {
-        q = strchrW( p, ',' );
+        q = wcschr( p, ',' );
         if (q) *q = 0;
-        ret[i] = atoiW( p );
+        ret[i] = wcstol( p, NULL, 10 );
         if (!q) break;
         p = q + 1;
         i++;
     }
-    msi_free( str );
+    free( str );
     *num_ids = count;
     return ret;
 }
@@ -653,12 +623,13 @@ static BOOL match_languages( const void *version, const WCHAR *languages )
         USHORT id;
         USHORT codepage;
     } *lang;
-    DWORD len, num_ids, i, j;
+    UINT len, j;
+    DWORD num_ids, i;
     BOOL found = FALSE;
     LANGID *ids;
 
     if (!languages || !languages[0]) return TRUE;
-    if (!VerQueryValueW( version, szLangResource, (void **)&lang, &len )) return FALSE;
+    if (!VerQueryValueW( version, L"\\VarFileInfo\\Translation", (void **)&lang, &len )) return FALSE;
     if (!(ids = parse_languages( languages, &num_ids ))) return FALSE;
 
     for (i = 0; i < num_ids; i++)
@@ -672,7 +643,7 @@ static BOOL match_languages( const void *version, const WCHAR *languages )
     }
 
 done:
-    msi_free( ids );
+    free( ids );
     return found;
 }
 
@@ -692,10 +663,10 @@ static UINT file_version_matches( MSIPACKAGE *package, const MSISIGNATURE *sig,
     *matches = FALSE;
 
     if (!size) return ERROR_SUCCESS;
-    if (!(version = msi_alloc( size ))) return ERROR_OUTOFMEMORY;
+    if (!(version = malloc( size ))) return ERROR_OUTOFMEMORY;
 
     if (msi_get_file_version_info( package, filePath, size, version ))
-        VerQueryValueW( version, szBackSlash, (void **)&info, &len );
+        VerQueryValueW( version, L"\\", (void **)&info, &len );
 
     if (info)
     {
@@ -731,7 +702,7 @@ static UINT file_version_matches( MSIPACKAGE *package, const MSISIGNATURE *sig,
         }
         else *matches = TRUE;
     }
-    msi_free( version );
+    free( version );
     return ERROR_SUCCESS;
 }
 
@@ -789,7 +760,6 @@ static UINT file_matches_sig( MSIPACKAGE *package, const MSISIGNATURE *sig, cons
 static UINT recurse_search_directory( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig, const WCHAR *dir,
                                       int depth )
 {
-    static const WCHAR starDotStarW[] = { '*','.','*',0 };
     HANDLE hFind;
     WIN32_FIND_DATAW findData;
     UINT rc = ERROR_SUCCESS;
@@ -808,8 +778,8 @@ static UINT recurse_search_directory( MSIPACKAGE *package, WCHAR **appValue, MSI
      * here.  Add two because we might need to add a backslash if the dir name
      * isn't backslash-terminated.
      */
-    len = dirLen + max(fileLen, strlenW(starDotStarW)) + 2;
-    buf = msi_alloc(len * sizeof(WCHAR));
+    len = dirLen + max(fileLen, lstrlenW(L"*.*")) + 2;
+    buf = malloc(len * sizeof(WCHAR));
     if (!buf)
         return ERROR_OUTOFMEMORY;
 
@@ -838,14 +808,14 @@ static UINT recurse_search_directory( MSIPACKAGE *package, WCHAR **appValue, MSI
     {
         lstrcpyW(buf, dir);
         PathAddBackslashW(buf);
-        lstrcatW(buf, starDotStarW);
+        lstrcatW(buf, L"*.*");
 
         hFind = msi_find_first_file( package, buf, &findData );
         if (hFind != INVALID_HANDLE_VALUE)
         {
             if (findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY &&
-                strcmpW( findData.cFileName, szDot ) &&
-                strcmpW( findData.cFileName, szDotDot ))
+                wcscmp( findData.cFileName, L"." ) &&
+                wcscmp( findData.cFileName, L".." ))
             {
                 lstrcpyW(subpath, dir);
                 PathAppendW(subpath, findData.cFileName);
@@ -854,8 +824,8 @@ static UINT recurse_search_directory( MSIPACKAGE *package, WCHAR **appValue, MSI
 
             while (rc == ERROR_SUCCESS && !*appValue && msi_find_next_file( package, hFind, &findData ))
             {
-                if (!strcmpW( findData.cFileName, szDot ) ||
-                    !strcmpW( findData.cFileName, szDotDot ))
+                if (!wcscmp( findData.cFileName, L"." ) ||
+                    !wcscmp( findData.cFileName, L".." ))
                     continue;
 
                 lstrcpyW(subpath, dir);
@@ -869,7 +839,7 @@ static UINT recurse_search_directory( MSIPACKAGE *package, WCHAR **appValue, MSI
     }
 
     if (*appValue != buf)
-        msi_free(buf);
+        free(buf);
 
     return rc;
 }
@@ -881,7 +851,7 @@ static UINT check_directory( MSIPACKAGE *package, const WCHAR *dir, WCHAR **appV
     if (attr != INVALID_FILE_ATTRIBUTES && (attr & FILE_ATTRIBUTE_DIRECTORY))
     {
         TRACE("directory exists, returning %s\n", debugstr_w(dir));
-        *appValue = strdupW(dir);
+        *appValue = wcsdup(dir);
     }
 
     return ERROR_SUCCESS;
@@ -889,7 +859,7 @@ static UINT check_directory( MSIPACKAGE *package, const WCHAR *dir, WCHAR **appV
 
 static BOOL is_full_path( const WCHAR *path )
 {
-    WCHAR first = toupperW(path[0]);
+    WCHAR first = towupper(path[0]);
     BOOL ret;
 
     if (first >= 'A' && first <= 'Z' && path[1] == ':')
@@ -950,7 +920,7 @@ static UINT search_directory( MSIPACKAGE *package, MSISIGNATURE *sig, const WCHA
     if (attr != INVALID_FILE_ATTRIBUTES && (attr & FILE_ATTRIBUTE_DIRECTORY) &&
         val && val[lstrlenW(val) - 1] != '\\')
     {
-        val = msi_realloc(val, (lstrlenW(val) + 2) * sizeof(WCHAR));
+        val = realloc(val, (wcslen(val) + 2) * sizeof(WCHAR));
         if (!val)
             rc = ERROR_OUTOFMEMORY;
         else
@@ -967,12 +937,6 @@ static UINT search_sig_name( MSIPACKAGE *, const WCHAR *, MSISIGNATURE *, WCHAR
 
 static UINT search_dr( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig )
 {
-    static const WCHAR query[] =  {
-        's','e','l','e','c','t',' ','*',' ',
-        'f','r','o','m',' ',
-        'D','r','L','o','c','a','t','o','r',' ',
-        'w','h','e','r','e',' ',
-        'S','i','g','n','a','t','u','r','e','_',' ','=',' ', '\'','%','s','\'',0};
     LPWSTR parent = NULL;
     LPCWSTR parentName;
     WCHAR path[MAX_PATH];
@@ -986,7 +950,7 @@ static UINT search_dr( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig
 
     *appValue = NULL;
 
-    row = MSI_QueryGetRecord( package->db, query, sig->Name );
+    row = MSI_QueryGetRecord( package->db, L"SELECT * FROM `DrLocator` WHERE `Signature_` = '%s'", sig->Name );
     if (!row)
     {
         TRACE("failed to query DrLocator for %s\n", debugstr_w(sig->Name));
@@ -1019,7 +983,7 @@ static UINT search_dr( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig
     if (sz)
         expand_any_path( package, path, expanded, MAX_PATH );
     else
-        strcpyW(expanded, path);
+        lstrcpyW(expanded, path);
 
     if (parent)
     {
@@ -1031,16 +995,16 @@ static UINT search_dr( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig
             PathAddBackslashW(parent);
         }
 
-        strcpyW(path, parent);
-        strcatW(path, expanded);
+        lstrcpyW(path, parent);
+        lstrcatW(path, expanded);
     }
-    else if (sz) strcpyW(path, expanded);
+    else if (sz) lstrcpyW(path, expanded);
 
     PathAddBackslashW(path);
 
     rc = search_directory( package, sig, path, depth, appValue );
 
-    msi_free(parent);
+    free(parent);
     msiobj_release(&row->hdr);
     TRACE("returning %d\n", rc);
     return rc;
@@ -1088,10 +1052,10 @@ static UINT ITERATE_AppSearch(MSIRECORD *row, LPVOID param)
     if (value)
     {
         r = msi_set_property( package->db, propName, value, -1 );
-        if (r == ERROR_SUCCESS && !strcmpW( propName, szSourceDir ))
+        if (r == ERROR_SUCCESS && !wcscmp( propName, L"SourceDir" ))
             msi_reset_source_folders( package );
 
-        msi_free(value);
+        free(value);
     }
     free_signature( &sig );
 
@@ -1106,21 +1070,18 @@ static UINT ITERATE_AppSearch(MSIRECORD *row, LPVOID param)
 
 UINT ACTION_AppSearch(MSIPACKAGE *package)
 {
-    static const WCHAR query[] =  {
-        'S','E','L','E','C','T',' ','*',' ','F','R','O','M',' ',
-        'A','p','p','S','e','a','r','c','h',0};
     MSIQUERY *view;
     UINT r;
 
-    if (msi_action_is_unique(package, szAppSearch))
+    if (msi_action_is_unique(package, L"AppSearch"))
     {
         TRACE("Skipping AppSearch action: already done in UI sequence\n");
         return ERROR_SUCCESS;
     }
     else
-        msi_register_unique_action(package, szAppSearch);
+        msi_register_unique_action(package, L"AppSearch");
 
-    r = MSI_OpenQuery( package->db, &view, query );
+    r = MSI_OpenQuery( package->db, &view, L"SELECT * FROM `AppSearch`" );
     if (r != ERROR_SUCCESS)
         return ERROR_SUCCESS;
 
@@ -1137,8 +1098,6 @@ static UINT ITERATE_CCPSearch(MSIRECORD *row, LPVOID param)
     MSISIGNATURE sig;
     UINT r = ERROR_SUCCESS;
 
-    static const WCHAR success[] = {'C','C','P','_','S','u','c','c','e','s','s',0};
-
     signature = MSI_RecordGetString(row, 1);
 
     TRACE("%s\n", debugstr_w(signature));
@@ -1147,8 +1106,8 @@ static UINT ITERATE_CCPSearch(MSIRECORD *row, LPVOID param)
     if (value)
     {
         TRACE("Found signature %s\n", debugstr_w(signature));
-        msi_set_property( package->db, success, szOne, -1 );
-        msi_free(value);
+        msi_set_property( package->db, L"CCP_Success", L"1", -1 );
+        free(value);
         r = ERROR_NO_MORE_ITEMS;
     }
 
@@ -1158,21 +1117,18 @@ static UINT ITERATE_CCPSearch(MSIRECORD *row, LPVOID param)
 
 UINT ACTION_CCPSearch(MSIPACKAGE *package)
 {
-    static const WCHAR query[] =  {
-        'S','E','L','E','C','T',' ','*',' ','F','R','O','M',' ',
-        'C','C','P','S','e','a','r','c','h',0};
     MSIQUERY *view;
     UINT r;
 
-    if (msi_action_is_unique(package, szCCPSearch))
+    if (msi_action_is_unique(package, L"CCPSearch"))
     {
         TRACE("Skipping AppSearch action: already done in UI sequence\n");
         return ERROR_SUCCESS;
     }
     else
-        msi_register_unique_action(package, szCCPSearch);
+        msi_register_unique_action(package, L"CCPSearch");
 
-    r = MSI_OpenQuery(package->db, &view, query);
+    r = MSI_OpenQuery(package->db, &view, L"SELECT * FROM `CCPSearch`");
     if (r != ERROR_SUCCESS)
         return ERROR_SUCCESS;
 
