@@ -24,7 +24,6 @@
 #include "winbase.h"
 #include "winerror.h"
 #include "wine/debug.h"
-#include "wine/unicode.h"
 #include "msi.h"
 #include "msiquery.h"
 #include "objbase.h"
@@ -69,7 +68,7 @@ static UINT CREATE_execute( struct tagMSIVIEW *view, MSIRECORD *record )
     if (cv->bIsTemp && !cv->hold)
         return ERROR_SUCCESS;
 
-    return msi_create_table( cv->db, cv->name, cv->col_info, persist );
+    return msi_create_table( cv->db, cv->name, cv->col_info, persist, cv->hold );
 }
 
 static UINT CREATE_close( struct tagMSIVIEW *view )
@@ -117,7 +116,7 @@ static UINT CREATE_delete( struct tagMSIVIEW *view )
     TRACE("%p\n", cv );
 
     msiobj_release( &cv->db->hdr );
-    msi_free( cv );
+    free( cv );
 
     return ERROR_SUCCESS;
 }
@@ -152,7 +151,7 @@ static UINT check_columns( const column_info *col_info )
     /* check for two columns with the same name */
     for( c1 = col_info; c1; c1 = c1->next )
         for( c2 = c1->next; c2; c2 = c2->next )
-            if (!strcmpW( c1->column, c2->column ))
+            if (!wcscmp( c1->column, c2->column ))
                 return ERROR_BAD_QUERY_SYNTAX;
 
     return ERROR_SUCCESS;
@@ -173,7 +172,7 @@ UINT CREATE_CreateView( MSIDATABASE *db, MSIVIEW **view, LPCWSTR table,
     if( r != ERROR_SUCCESS )
         return r;
 
-    cv = msi_alloc_zero( sizeof *cv );
+    cv = calloc( 1, sizeof *cv );
     if( !cv )
         return ERROR_FUNCTION_FAILED;
 
@@ -182,7 +181,7 @@ UINT CREATE_CreateView( MSIDATABASE *db, MSIVIEW **view, LPCWSTR table,
         if (!col->table)
             col->table = table;
 
-        if( !col->temporary )
+        if( !(col->type & MSITYPE_TEMPORARY) )
             temp = FALSE;
         else if ( col->type & MSITYPE_KEY )
             tempprim = TRUE;
@@ -190,7 +189,7 @@ UINT CREATE_CreateView( MSIDATABASE *db, MSIVIEW **view, LPCWSTR table,
 
     if ( !temp && tempprim )
     {
-        msi_free( cv );
+        free( cv );
         return ERROR_FUNCTION_FAILED;
     }
 
