@@ -48,6 +48,15 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(msi);
 
+BOOL msi_get_temp_file_name( MSIPACKAGE *package, const WCHAR *tmp_path, const WCHAR *prefix, WCHAR *tmp_filename )
+{
+    BOOL ret;
+    msi_disable_fs_redirection( package );
+    ret = GetTempFileNameW( tmp_path, prefix, 0, tmp_filename );
+    msi_revert_fs_redirection( package );
+    return ret;
+}
+
 HANDLE msi_create_file( MSIPACKAGE *package, const WCHAR *filename, DWORD access, DWORD sharing, DWORD creation,
                         DWORD flags )
 {
@@ -160,30 +169,30 @@ DWORD msi_get_file_version_info( MSIPACKAGE *package, const WCHAR *path, DWORD b
 
 VS_FIXEDFILEINFO *msi_get_disk_file_version( MSIPACKAGE *package, const WCHAR *filename )
 {
-    static const WCHAR name[] = {'\\',0};
     VS_FIXEDFILEINFO *ptr, *ret;
-    DWORD version_size, size;
+    DWORD version_size;
+    UINT size;
     void *version;
 
     if (!(version_size = msi_get_file_version_info( package, filename, 0, NULL ))) return NULL;
-    if (!(version = msi_alloc( version_size ))) return NULL;
+    if (!(version = malloc( version_size ))) return NULL;
 
     msi_get_file_version_info( package, filename, version_size, version );
 
-    if (!VerQueryValueW( version, name, (void **)&ptr, &size ))
+    if (!VerQueryValueW( version, L"\\", (void **)&ptr, &size ))
     {
-        msi_free( version );
+        free( version );
         return NULL;
     }
 
-    if (!(ret = msi_alloc( size )))
+    if (!(ret = malloc( size )))
     {
-        msi_free( version );
+        free( version );
         return NULL;
     }
 
     memcpy( ret, ptr, size );
-    msi_free( version );
+    free( version );
     return ret;
 }
 
@@ -205,7 +214,7 @@ BOOL msi_create_full_path( MSIPACKAGE *package, const WCHAR *path )
     WCHAR *new_path;
     int len;
 
-    if (!(new_path = msi_alloc( (lstrlenW( path ) + 1) * sizeof(WCHAR) ))) return FALSE;
+    if (!(new_path = malloc( (wcslen( path ) + 1) * sizeof(WCHAR) ))) return FALSE;
     lstrcpyW( new_path, path );
 
     while ((len = lstrlenW( new_path )) && new_path[len - 1] == '\\')
@@ -235,7 +244,7 @@ BOOL msi_create_full_path( MSIPACKAGE *package, const WCHAR *path )
         }
         new_path[len] = '\\';
     }
-    msi_free( new_path );
+    free( new_path );
     return ret;
 }
 
@@ -267,7 +276,7 @@ static BOOL is_obsoleted_by_patch( MSIPACKAGE *package, MSIFILE *file )
 {
     if (!list_empty( &package->patches ) && file->disk_id < MSI_INITIAL_MEDIA_TRANSFORM_DISKID)
     {
-        if (!msi_get_property_int( package->db, szInstalled, 0 )) return FALSE;
+        if (!msi_get_property_int( package->db, L"Installed", 0 )) return FALSE;
         return TRUE;
     }
     if (is_registered_patch_media( package, file->disk_id )) return TRUE;
@@ -296,7 +305,7 @@ static msi_file_state calculate_install_state( MSIPACKAGE *package, MSIFILE *fil
     DWORD size;
 
     comp->Action = msi_get_component_action( package, comp );
-    if (!comp->Enabled || comp->Action != INSTALLSTATE_LOCAL || (comp->assembly && comp->assembly->installed))
+    if (!comp->Enabled || comp->Action != INSTALLSTATE_LOCAL)
     {
         TRACE("skipping %s (not scheduled for install)\n", debugstr_w(file->File));
         return msifs_skipped;
@@ -306,8 +315,7 @@ static msi_file_state calculate_install_state( MSIPACKAGE *package, MSIFILE *fil
         TRACE("skipping %s (obsoleted by patch)\n", debugstr_w(file->File));
         return msifs_skipped;
     }
-    if ((msi_is_global_assembly( comp ) && !comp->assembly->installed) ||
-        msi_get_file_attributes( package, file->TargetPath ) == INVALID_FILE_ATTRIBUTES)
+    if (msi_get_file_attributes( package, file->TargetPath ) == INVALID_FILE_ATTRIBUTES)
     {
         TRACE("installing %s (missing)\n", debugstr_w(file->File));
         return msifs_missing;
@@ -332,7 +340,7 @@ static msi_file_state calculate_install_state( MSIPACKAGE *package, MSIFILE *fil
                       HIWORD(file_version->dwFileVersionLS), LOWORD(file_version->dwFileVersionLS));
                 state = msifs_present;
             }
-            msi_free( file_version );
+            free( file_version );
             return state;
         }
         else if ((font_version = msi_get_font_file_version( package, file->TargetPath )))
@@ -349,13 +357,13 @@ static msi_file_state calculate_install_state( MSIPACKAGE *package, MSIFILE *fil
                       debugstr_w(file->File), debugstr_w(file->Version), debugstr_w(font_version));
                 state = msifs_present;
             }
-            msi_free( font_version );
+            free( font_version );
             return state;
         }
     }
     if ((size = msi_get_disk_file_size( package, file->TargetPath )) != file->FileSize)
     {
-        TRACE("overwriting %s (old size %u new size %u)\n", debugstr_w(file->File), size, file->FileSize);
+        TRACE("overwriting %s (old size %lu new size %d)\n", debugstr_w(file->File), size, file->FileSize);
         return msifs_overwrite;
     }
     if (file->hash.dwFileHashInfoSize)
@@ -434,16 +442,16 @@ static UINT copy_install_file(MSIPACKAGE *package, MSIFILE *file, LPWSTR source)
 
         TRACE("file in use, scheduling rename operation\n");
 
-        if (!(pathW = strdupW( file->TargetPath ))) return ERROR_OUTOFMEMORY;
+        if (!(pathW = wcsdup( file->TargetPath ))) return ERROR_OUTOFMEMORY;
         if ((p = wcsrchr(pathW, '\\'))) *p = 0;
         len = lstrlenW( pathW ) + 16;
-        if (!(tmpfileW = msi_alloc(len * sizeof(WCHAR))))
+        if (!(tmpfileW = malloc(len * sizeof(WCHAR))))
         {
-            msi_free( pathW );
+            free( pathW );
             return ERROR_OUTOFMEMORY;
         }
-        if (!GetTempFileNameW( pathW, szMsi, 0, tmpfileW )) tmpfileW[0] = 0;
-        msi_free( pathW );
+        if (!GetTempFileNameW( pathW, L"msi", 0, tmpfileW )) tmpfileW[0] = 0;
+        free( pathW );
 
         if (msi_copy_file( package, source, tmpfileW, FALSE ) &&
             msi_move_file( package, file->TargetPath, NULL, MOVEFILE_DELAY_UNTIL_REBOOT ) &&
@@ -458,7 +466,7 @@ static UINT copy_install_file(MSIPACKAGE *package, MSIFILE *file, LPWSTR source)
             WARN("failed to schedule rename operation: %d)\n", gle);
             DeleteFileW( tmpfileW );
         }
-        msi_free(tmpfileW);
+        free(tmpfileW);
     }
 
     return gle;
@@ -513,7 +521,7 @@ static BOOL installfiles_cb(MSIPACKAGE *package, LPCWSTR filename, DWORD action,
         {
             create_directory( package, file->Component->Directory );
         }
-        *path = strdupW( file->TargetPath );
+        *path = wcsdup( file->TargetPath );
         *attrs = file->Attributes;
         *(MSIFILE **)user = file;
     }
@@ -538,17 +546,17 @@ WCHAR *msi_resolve_file_source( MSIPACKAGE *package, MSIFILE *file )
 
     if (file->LongName && msi_get_file_attributes( package, path ) == INVALID_FILE_ATTRIBUTES)
     {
-        msi_free( path );
+        free( path );
         path = msi_build_directory_name( 2, p, file->LongName );
     }
-    msi_free( p );
+    free( p );
     TRACE("file %s source resolves to %s\n", debugstr_w(file->File), debugstr_w(path));
     return path;
 }
 
 /*
  * ACTION_InstallFiles()
- * 
+ *
  * For efficiency, this is done in two passes:
  * 1) Correct all the TargetPaths and determine what files are to be installed.
  * 2) Extract Cabinets and copy files.
@@ -562,16 +570,16 @@ UINT ACTION_InstallFiles(MSIPACKAGE *package)
     msi_set_sourcedir_props(package, FALSE);
 
     if (package->script == SCRIPT_NONE)
-        return msi_schedule_action(package, SCRIPT_INSTALL, szInstallFiles);
+        return msi_schedule_action(package, SCRIPT_INSTALL, L"InstallFiles");
 
     schedule_install_files(package);
-    mi = msi_alloc_zero( sizeof(MSIMEDIAINFO) );
+    mi = calloc(1, sizeof(MSIMEDIAINFO));
 
     LIST_FOR_EACH_ENTRY( file, &package->files, MSIFILE, entry )
     {
         BOOL is_global_assembly = msi_is_global_assembly( file->Component );
 
-        msi_file_update_ui( package, file, szInstallFiles );
+        msi_file_update_ui( package, file, L"InstallFiles" );
 
         rc = msi_load_media_info( package, file->Sequence, mi );
         if (rc != ERROR_SUCCESS)
@@ -583,7 +591,7 @@ UINT ACTION_InstallFiles(MSIPACKAGE *package)
 
         if (file->state != msifs_hashmatch &&
             file->state != msifs_skipped &&
-            (file->state != msifs_present || !msi_get_property_int( package->db, szInstalled, 0 )) &&
+            (file->state != msifs_present || !msi_get_property_int( package->db, L"Installed", 0 )) &&
             (rc = ready_media( package, file->IsCompressed, mi )))
         {
             ERR("Failed to ready media for %s\n", debugstr_w(file->File));
@@ -627,11 +635,11 @@ UINT ACTION_InstallFiles(MSIPACKAGE *package)
             {
                 ERR("Failed to copy %s to %s (%u)\n", debugstr_w(source), debugstr_w(file->TargetPath), rc);
                 rc = ERROR_INSTALL_FAILURE;
-                msi_free(source);
+                free(source);
                 goto done;
             }
             if (!is_global_assembly) file->state = msifs_installed;
-            msi_free(source);
+            free(source);
         }
         else if (!is_global_assembly && file->state != msifs_installed &&
                  !(file->Attributes & msidbFileAttributesPatchAdded))
@@ -641,22 +649,6 @@ UINT ACTION_InstallFiles(MSIPACKAGE *package)
             goto done;
         }
     }
-    LIST_FOR_EACH_ENTRY( file, &package->files, MSIFILE, entry )
-    {
-        MSICOMPONENT *comp = file->Component;
-
-        if (!msi_is_global_assembly( comp ) || comp->assembly->installed ||
-            (file->state != msifs_missing && file->state != msifs_overwrite)) continue;
-
-        rc = msi_install_assembly( package, comp );
-        if (rc != ERROR_SUCCESS)
-        {
-            ERR("Failed to install assembly\n");
-            rc = ERROR_INSTALL_FAILURE;
-            break;
-        }
-        file->state = msifs_installed;
-    }
 
 done:
     msi_free_media_info(mi);
@@ -697,7 +689,7 @@ static BOOL patchfiles_cb(MSIPACKAGE *package, LPCWSTR file, DWORD action,
         }
 
         patch->path = msi_create_temp_file( package->db );
-        *path = strdupW( patch->path );
+        *path = wcsdup( patch->path );
         *attrs = patch->File->Attributes;
         *(MSIFILEPATCH **)user = patch;
     }
@@ -722,16 +714,16 @@ static UINT patch_file( MSIPACKAGE *package, MSIFILEPATCH *patch )
     }
     else
     {
-        WARN("failed to patch %s: %08x\n", debugstr_w(patch->File->TargetPath), GetLastError());
+        WARN( "failed to patch %s: %#lx\n", debugstr_w(patch->File->TargetPath), GetLastError() );
         r = ERROR_INSTALL_FAILURE;
     }
     DeleteFileW( patch->path );
     DeleteFileW( tmpfile );
-    msi_free( tmpfile );
+    free( tmpfile );
     return r;
 }
 
-static UINT patch_assembly( MSIPACKAGE *package, MSIASSEMBLY *assembly, MSIFILEPATCH *patch )
+UINT msi_patch_assembly( MSIPACKAGE *package, MSIASSEMBLY *assembly, MSIFILEPATCH *patch )
 {
     UINT r = ERROR_FUNCTION_FAILED;
     IAssemblyName *name;
@@ -743,17 +735,17 @@ static UINT patch_assembly( MSIPACKAGE *package, MSIASSEMBLY *assembly, MSIFILEP
     while ((IAssemblyEnum_GetNextAssembly( iter, NULL, &name, 0 ) == S_OK))
     {
         WCHAR *displayname, *path;
-        UINT len = 0;
+        DWORD len = 0;
         HRESULT hr;
 
         hr = IAssemblyName_GetDisplayName( name, NULL, &len, 0 );
-        if (hr != E_NOT_SUFFICIENT_BUFFER || !(displayname = msi_alloc( len * sizeof(WCHAR) )))
+        if (hr != E_NOT_SUFFICIENT_BUFFER || !(displayname = malloc( len * sizeof(WCHAR) )))
             break;
 
         hr = IAssemblyName_GetDisplayName( name, displayname, &len, 0 );
         if (FAILED( hr ))
         {
-            msi_free( displayname );
+            free( displayname );
             break;
         }
 
@@ -761,18 +753,18 @@ static UINT patch_assembly( MSIPACKAGE *package, MSIASSEMBLY *assembly, MSIFILEP
         {
             if (!msi_copy_file( package, path, patch->File->TargetPath, FALSE ))
             {
-                ERR("Failed to copy file %s -> %s (%u)\n", debugstr_w(path),
-                    debugstr_w(patch->File->TargetPath), GetLastError() );
-                msi_free( path );
-                msi_free( displayname );
+                ERR( "failed to copy file %s -> %s (%lu)\n", debugstr_w(path),
+                     debugstr_w(patch->File->TargetPath), GetLastError() );
+                free( path );
+                free( displayname );
                 IAssemblyName_Release( name );
                 break;
             }
             r = patch_file( package, patch );
-            msi_free( path );
+            free( path );
         }
 
-        msi_free( displayname );
+        free( displayname );
         IAssemblyName_Release( name );
         if (r == ERROR_SUCCESS) break;
     }
@@ -790,9 +782,9 @@ UINT ACTION_PatchFiles( MSIPACKAGE *package )
     TRACE("%p\n", package);
 
     if (package->script == SCRIPT_NONE)
-        return msi_schedule_action(package, SCRIPT_INSTALL, szPatchFiles);
+        return msi_schedule_action(package, SCRIPT_INSTALL, L"PatchFiles");
 
-    mi = msi_alloc_zero( sizeof(MSIMEDIAINFO) );
+    mi = calloc( 1, sizeof(MSIMEDIAINFO) );
 
     TRACE("extracting files\n");
 
@@ -842,27 +834,14 @@ UINT ACTION_PatchFiles( MSIPACKAGE *package )
     {
         MSICOMPONENT *comp = patch->File->Component;
 
-        if (!patch->path) continue;
-
-        if (msi_is_global_assembly( comp ))
-            rc = patch_assembly( package, comp->assembly, patch );
-        else
-            rc = patch_file( package, patch );
+        if (msi_is_global_assembly( comp ) || !patch->path) continue;
 
+        rc = patch_file( package, patch );
         if (rc && !(patch->Attributes & msidbPatchAttributesNonVital))
         {
             ERR("Failed to apply patch to file: %s\n", debugstr_w(patch->File->File));
             break;
         }
-
-        if (msi_is_global_assembly( comp ))
-        {
-            if ((rc = msi_install_assembly( package, comp )))
-            {
-                ERR("Failed to install patched assembly\n");
-                break;
-            }
-        }
     }
 
 done:
@@ -898,7 +877,7 @@ static BOOL move_file( MSIPACKAGE *package, const WCHAR *source, const WCHAR *de
         ret = msi_move_file( package, source, dest, MOVEFILE_REPLACE_EXISTING );
         if (!ret)
         {
-            WARN("msi_move_file failed: %u\n", GetLastError());
+            WARN( "msi_move_file failed: %lu\n", GetLastError() );
             return FALSE;
         }
     }
@@ -908,7 +887,7 @@ static BOOL move_file( MSIPACKAGE *package, const WCHAR *source, const WCHAR *de
         ret = msi_copy_file( package, source, dest, FALSE );
         if (!ret)
         {
-            WARN("msi_copy_file failed: %u\n", GetLastError());
+            WARN( "msi_copy_file failed: %lu\n", GetLastError() );
             return FALSE;
         }
     }
@@ -926,7 +905,7 @@ static WCHAR *wildcard_to_file( const WCHAR *wildcard, const WCHAR *filename )
     dirlen = ptr - wildcard + 1;
 
     pathlen = dirlen + lstrlenW(filename) + 1;
-    if (!(path = msi_alloc(pathlen * sizeof(WCHAR)))) return NULL;
+    if (!(path = malloc(pathlen * sizeof(WCHAR)))) return NULL;
 
     lstrcpynW(path, wildcard, dirlen + 1);
     lstrcatW(path, filename);
@@ -936,9 +915,9 @@ static WCHAR *wildcard_to_file( const WCHAR *wildcard, const WCHAR *filename )
 
 static void free_file_entry(FILE_LIST *file)
 {
-    msi_free(file->source);
-    msi_free(file->dest);
-    msi_free(file);
+    free(file->source);
+    free(file->dest);
+    free(file);
 }
 
 static void free_list(FILE_LIST *list)
@@ -958,11 +937,11 @@ static BOOL add_wildcard( FILE_LIST *files, const WCHAR *source, WCHAR *dest )
     WCHAR *ptr, *filename;
     DWORD size;
 
-    new = msi_alloc_zero(sizeof(FILE_LIST));
+    new = calloc(1, sizeof(FILE_LIST));
     if (!new)
         return FALSE;
 
-    new->source = strdupW(source);
+    new->source = wcsdup(source);
     ptr = wcsrchr(dest, '\\') + 1;
     filename = wcsrchr(new->source, '\\') + 1;
 
@@ -974,7 +953,7 @@ static BOOL add_wildcard( FILE_LIST *files, const WCHAR *source, WCHAR *dest )
         new->destname = new->sourcename;
 
     size = (ptr - dest) + lstrlenW(filename) + 1;
-    new->dest = msi_alloc(size * sizeof(WCHAR));
+    new->dest = malloc(size * sizeof(WCHAR));
     if (!new->dest)
     {
         free_file_entry(new);
@@ -1029,7 +1008,7 @@ static BOOL move_files_wildcard( MSIPACKAGE *package, const WCHAR *source, WCHAR
         }
 
         add_wildcard(&files, path, dest);
-        msi_free(path);
+        free(path);
     }
 
     /* no files match the wildcard */
@@ -1039,7 +1018,7 @@ static BOOL move_files_wildcard( MSIPACKAGE *package, const WCHAR *source, WCHAR
     /* only the first wildcard match gets renamed to dest */
     file = LIST_ENTRY(list_head(&files.entry), FILE_LIST, entry);
     size = (wcsrchr(file->dest, '\\') - file->dest) + lstrlenW(file->destname) + 2;
-    file->dest = msi_realloc(file->dest, size * sizeof(WCHAR));
+    file->dest = realloc(file->dest, size * sizeof(WCHAR));
     if (!file->dest)
     {
         res = FALSE;
@@ -1116,20 +1095,20 @@ static UINT ITERATE_MoveFiles( MSIRECORD *rec, LPVOID param )
         if (msi_get_file_attributes( package, sourcedir ) == INVALID_FILE_ATTRIBUTES)
             goto done;
 
-        source = strdupW(sourcedir);
+        source = wcsdup(sourcedir);
         if (!source)
             goto done;
     }
     else
     {
         size = lstrlenW(sourcedir) + lstrlenW(sourcename) + 2;
-        source = msi_alloc(size * sizeof(WCHAR));
+        source = malloc(size * sizeof(WCHAR));
         if (!source)
             goto done;
 
         lstrcpyW(source, sourcedir);
         if (source[lstrlenW(source) - 1] != '\\')
-            lstrcatW(source, szBackSlash);
+            lstrcatW(source, L"\\");
         lstrcatW(source, sourcename);
     }
 
@@ -1141,18 +1120,18 @@ static UINT ITERATE_MoveFiles( MSIRECORD *rec, LPVOID param )
         {
             WCHAR *p;
             if (sourcename)
-                destname = strdupW(sourcename);
+                destname = wcsdup(sourcename);
             else if ((p = wcsrchr(sourcedir, '\\')))
-                destname = strdupW(p + 1);
+                destname = wcsdup(p + 1);
             else
-                destname = strdupW(sourcedir);
+                destname = wcsdup(sourcedir);
             if (!destname)
                 goto done;
         }
     }
     else
     {
-        destname = strdupW(MSI_RecordGetString(rec, 4));
+        destname = wcsdup(MSI_RecordGetString(rec, 4));
         if (destname) msi_reduce_to_long_filename(destname);
     }
 
@@ -1161,13 +1140,13 @@ static UINT ITERATE_MoveFiles( MSIRECORD *rec, LPVOID param )
         size = lstrlenW(destname);
 
     size += lstrlenW(destdir) + 2;
-    dest = msi_alloc(size * sizeof(WCHAR));
+    dest = malloc(size * sizeof(WCHAR));
     if (!dest)
         goto done;
 
     lstrcpyW(dest, destdir);
     if (dest[lstrlenW(dest) - 1] != '\\')
-        lstrcatW(dest, szBackSlash);
+        lstrcatW(dest, L"\\");
 
     if (destname)
         lstrcatW(dest, destname);
@@ -1176,7 +1155,7 @@ static UINT ITERATE_MoveFiles( MSIRECORD *rec, LPVOID param )
     {
         if (!msi_create_full_path( package, destdir ))
         {
-            WARN("failed to create directory %u\n", GetLastError());
+            WARN( "failed to create directory %lu\n", GetLastError() );
             goto done;
         }
     }
@@ -1194,27 +1173,24 @@ done:
     MSI_ProcessMessage(package, INSTALLMESSAGE_ACTIONDATA, uirow);
     msiobj_release( &uirow->hdr );
 
-    msi_free(sourcedir);
-    msi_free(destdir);
-    msi_free(destname);
-    msi_free(source);
-    msi_free(dest);
+    free(sourcedir);
+    free(destdir);
+    free(destname);
+    free(source);
+    free(dest);
 
     return ERROR_SUCCESS;
 }
 
 UINT ACTION_MoveFiles( MSIPACKAGE *package )
 {
-    static const WCHAR query[] = {
-        'S','E','L','E','C','T',' ','*',' ','F','R','O','M',' ',
-        '`','M','o','v','e','F','i','l','e','`',0};
     MSIQUERY *view;
     UINT rc;
 
     if (package->script == SCRIPT_NONE)
-        return msi_schedule_action(package, SCRIPT_INSTALL, szMoveFiles);
+        return msi_schedule_action(package, SCRIPT_INSTALL, L"MoveFiles");
 
-    rc = MSI_DatabaseOpenViewW(package->db, query, &view);
+    rc = MSI_DatabaseOpenViewW(package->db, L"SELECT * FROM `MoveFile`", &view);
     if (rc != ERROR_SUCCESS)
         return ERROR_SUCCESS;
 
@@ -1231,13 +1207,13 @@ static WCHAR *get_duplicate_filename( MSIPACKAGE *package, MSIRECORD *row, const
     if (MSI_RecordIsNull( row, 4 ))
     {
         len = lstrlenW( src ) + 1;
-        if (!(dst_name = msi_alloc( len * sizeof(WCHAR)))) return NULL;
+        if (!(dst_name = malloc( len * sizeof(WCHAR)))) return NULL;
         lstrcpyW( dst_name, wcsrchr( src, '\\' ) + 1 );
     }
     else
     {
         MSI_RecordGetStringW( row, 4, NULL, &len );
-        if (!(dst_name = msi_alloc( ++len * sizeof(WCHAR) ))) return NULL;
+        if (!(dst_name = malloc( ++len * sizeof(WCHAR) ))) return NULL;
         MSI_RecordGetStringW( row, 4, dst_name, &len );
         msi_reduce_to_long_filename( dst_name );
     }
@@ -1245,7 +1221,7 @@ static WCHAR *get_duplicate_filename( MSIPACKAGE *package, MSIRECORD *row, const
     if (MSI_RecordIsNull( row, 5 ))
     {
         WCHAR *p;
-        dst_path = strdupW( src );
+        dst_path = wcsdup( src );
         p = wcsrchr( dst_path, '\\' );
         if (p) *p = 0;
     }
@@ -1253,7 +1229,7 @@ static WCHAR *get_duplicate_filename( MSIPACKAGE *package, MSIRECORD *row, const
     {
         const WCHAR *dst_key = MSI_RecordGetString( row, 5 );
 
-        dst_path = strdupW( msi_get_target_folder( package, dst_key ) );
+        dst_path = wcsdup( msi_get_target_folder( package, dst_key ) );
         if (!dst_path)
         {
             /* try a property */
@@ -1261,7 +1237,7 @@ static WCHAR *get_duplicate_filename( MSIPACKAGE *package, MSIRECORD *row, const
             if (!dst_path)
             {
                 FIXME("Unable to get destination folder, try AppSearch properties\n");
-                msi_free( dst_name );
+                free( dst_name );
                 return NULL;
             }
         }
@@ -1270,8 +1246,8 @@ static WCHAR *get_duplicate_filename( MSIPACKAGE *package, MSIRECORD *row, const
     dst = msi_build_directory_name( 2, dst_path, dst_name );
     msi_create_full_path( package, dst_path );
 
-    msi_free( dst_name );
-    msi_free( dst_path );
+    free( dst_name );
+    free( dst_path );
     return dst;
 }
 
@@ -1320,8 +1296,8 @@ static UINT ITERATE_DuplicateFiles(MSIRECORD *row, LPVOID param)
     TRACE("Duplicating file %s to %s\n", debugstr_w(file->TargetPath), debugstr_w(dest));
     if (!msi_copy_file( package, file->TargetPath, dest, TRUE ))
     {
-        WARN("Failed to copy file %s -> %s (%u)\n",
-             debugstr_w(file->TargetPath), debugstr_w(dest), GetLastError());
+        WARN( "failed to copy file %s -> %s (%lu)\n",
+              debugstr_w(file->TargetPath), debugstr_w(dest), GetLastError() );
     }
     FIXME("We should track these duplicate files as well\n");
 
@@ -1332,22 +1308,19 @@ static UINT ITERATE_DuplicateFiles(MSIRECORD *row, LPVOID param)
     MSI_ProcessMessage(package, INSTALLMESSAGE_ACTIONDATA, uirow);
     msiobj_release( &uirow->hdr );
 
-    msi_free(dest);
+    free(dest);
     return ERROR_SUCCESS;
 }
 
 UINT ACTION_DuplicateFiles(MSIPACKAGE *package)
 {
-    static const WCHAR query[] = {
-        'S','E','L','E','C','T',' ','*',' ','F','R','O','M',' ',
-        '`','D','u','p','l','i','c','a','t','e','F','i','l','e','`',0};
     MSIQUERY *view;
     UINT rc;
 
     if (package->script == SCRIPT_NONE)
-        return msi_schedule_action(package, SCRIPT_INSTALL, szDuplicateFiles);
+        return msi_schedule_action(package, SCRIPT_INSTALL, L"DuplicateFiles");
 
-    rc = MSI_DatabaseOpenViewW(package->db, query, &view);
+    rc = MSI_DatabaseOpenViewW(package->db, L"SELECT * FROM `DuplicateFile`", &view);
     if (rc != ERROR_SUCCESS)
         return ERROR_SUCCESS;
 
@@ -1401,7 +1374,7 @@ static UINT ITERATE_RemoveDuplicateFiles( MSIRECORD *row, LPVOID param )
     TRACE("Removing duplicate %s of %s\n", debugstr_w(dest), debugstr_w(file->TargetPath));
     if (!msi_delete_file( package, dest ))
     {
-        WARN("Failed to delete duplicate file %s (%u)\n", debugstr_w(dest), GetLastError());
+        WARN( "failed to delete duplicate file %s (%lu)\n", debugstr_w(dest), GetLastError() );
     }
 
     uirow = MSI_CreateRecord( 9 );
@@ -1410,22 +1383,19 @@ static UINT ITERATE_RemoveDuplicateFiles( MSIRECORD *row, LPVOID param )
     MSI_ProcessMessage(package, INSTALLMESSAGE_ACTIONDATA, uirow);
     msiobj_release( &uirow->hdr );
 
-    msi_free(dest);
+    free(dest);
     return ERROR_SUCCESS;
 }
 
 UINT ACTION_RemoveDuplicateFiles( MSIPACKAGE *package )
 {
-    static const WCHAR query[] = {
-        'S','E','L','E','C','T',' ','*',' ','F','R','O','M',' ',
-        '`','D','u','p','l','i','c','a','t','e','F','i','l','e','`',0};
     MSIQUERY *view;
     UINT rc;
 
     if (package->script == SCRIPT_NONE)
-        return msi_schedule_action(package, SCRIPT_INSTALL, szRemoveDuplicateFiles);
+        return msi_schedule_action(package, SCRIPT_INSTALL, L"RemoveDuplicateFiles");
 
-    rc = MSI_DatabaseOpenViewW( package->db, query, &view );
+    rc = MSI_DatabaseOpenViewW( package->db, L"SELECT * FROM `DuplicateFile`", &view );
     if (rc != ERROR_SUCCESS)
         return ERROR_SUCCESS;
 
@@ -1500,13 +1470,13 @@ static UINT ITERATE_RemoveFiles(MSIRECORD *row, LPVOID param)
         return ERROR_SUCCESS;
     }
     size = 0;
-    if ((filename = strdupW( MSI_RecordGetString(row, 3) )))
+    if ((filename = wcsdup( MSI_RecordGetString(row, 3) )))
     {
         msi_reduce_to_long_filename( filename );
         size = lstrlenW( filename );
     }
     size += lstrlenW(dir) + 2;
-    path = msi_alloc(size * sizeof(WCHAR));
+    path = malloc(size * sizeof(WCHAR));
     if (!path)
     {
         ret = ERROR_OUTOFMEMORY;
@@ -1535,9 +1505,9 @@ done:
     MSI_ProcessMessage(package, INSTALLMESSAGE_ACTIONDATA, uirow);
     msiobj_release( &uirow->hdr );
 
-    msi_free(filename);
-    msi_free(path);
-    msi_free(dir);
+    free(filename);
+    free(path);
+    free(dir);
     return ret;
 }
 
@@ -1557,18 +1527,15 @@ static void remove_folder( MSIFOLDER *folder )
 
 UINT ACTION_RemoveFiles( MSIPACKAGE *package )
 {
-    static const WCHAR query[] = {
-        'S','E','L','E','C','T',' ','*',' ','F','R','O','M',' ',
-        '`','R','e','m','o','v','e','F','i','l','e','`',0};
     MSIQUERY *view;
     MSICOMPONENT *comp;
     MSIFILE *file;
     UINT r;
 
     if (package->script == SCRIPT_NONE)
-        return msi_schedule_action(package, SCRIPT_INSTALL, szRemoveFiles);
+        return msi_schedule_action(package, SCRIPT_INSTALL, L"RemoveFiles");
 
-    r = MSI_DatabaseOpenViewW(package->db, query, &view);
+    r = MSI_DatabaseOpenViewW(package->db, L"SELECT * FROM `RemoveFile`", &view);
     if (r == ERROR_SUCCESS)
     {
         r = MSI_IterateRecords(view, NULL, ITERATE_RemoveFiles, package);
@@ -1583,7 +1550,7 @@ UINT ACTION_RemoveFiles( MSIPACKAGE *package )
         VS_FIXEDFILEINFO *ver;
 
         comp = file->Component;
-        msi_file_update_ui( package, file, szRemoveFiles );
+        msi_file_update_ui( package, file, L"RemoveFiles" );
 
         comp->Action = msi_get_component_action( package, comp );
         if (comp->Action != INSTALLSTATE_ABSENT || comp->Installed == INSTALLSTATE_SOURCE)
@@ -1604,10 +1571,10 @@ UINT ACTION_RemoveFiles( MSIPACKAGE *package )
             if (ver && msi_compare_file_versions( ver, file->Version ) > 0)
             {
                 TRACE("newer version detected, not removing file\n");
-                msi_free( ver );
+                free( ver );
                 continue;
             }
-            msi_free( ver );
+            free( ver );
         }
 
         if (file->state == msifs_installed)
@@ -1618,7 +1585,7 @@ UINT ACTION_RemoveFiles( MSIPACKAGE *package )
         msi_set_file_attributes( package, file->TargetPath, FILE_ATTRIBUTE_NORMAL );
         if (!msi_delete_file( package, file->TargetPath ))
         {
-            WARN("failed to delete %s (%u)\n",  debugstr_w(file->TargetPath), GetLastError());
+            WARN( "failed to delete %s (%lu)\n",  debugstr_w(file->TargetPath), GetLastError() );
         }
         file->state = msifs_missing;
 
