@@ -26,15 +26,19 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(msvcp);
 
+#define CXX_FRAME_MAGIC_VC6 0x19930520
+
+CREATE_TYPE_INFO_VTABLE
+
 #define CLASS_IS_SIMPLE_TYPE          1
 #define CLASS_HAS_VIRTUAL_BASE_CLASS  4
 
-void WINAPI _CxxThrowException(exception*,const cxx_exception_type*);
+int* __cdecl __processing_throw(void);
 
 #if _MSVCP_VER >= 70 || defined(_MSVCIRT)
 typedef const char **exception_name;
 #define EXCEPTION_STR(name) (*name)
-#define EXCEPTION_NAME(str) ((exception_name)&str)
+#define EXCEPTION_NAME(str) (&str)
 #else
 typedef const char *exception_name;
 #define EXCEPTION_STR(name) (name)
@@ -44,53 +48,25 @@ typedef const char *exception_name;
 void (CDECL *_Raise_handler)(const exception*) = NULL;
 
 /* vtables */
-extern const vtable_ptr MSVCP_exception_vtable;
+extern const vtable_ptr exception_vtable;
 /* ??_7bad_alloc@std@@6B@ */
-extern const vtable_ptr MSVCP_bad_alloc_vtable;
+extern const vtable_ptr bad_alloc_vtable;
 /* ??_7logic_error@std@@6B@ */
-extern const vtable_ptr MSVCP_logic_error_vtable;
+extern const vtable_ptr logic_error_vtable;
 /* ??_7length_error@std@@6B@ */
-extern const vtable_ptr MSVCP_length_error_vtable;
+extern const vtable_ptr length_error_vtable;
 /* ??_7out_of_range@std@@6B@ */
-extern const vtable_ptr MSVCP_out_of_range_vtable;
-extern const vtable_ptr MSVCP_invalid_argument_vtable;
+extern const vtable_ptr out_of_range_vtable;
+extern const vtable_ptr invalid_argument_vtable;
 /* ??_7runtime_error@std@@6B@ */
-extern const vtable_ptr MSVCP_runtime_error_vtable;
-extern const vtable_ptr MSVCP__System_error_vtable;
-extern const vtable_ptr MSVCP_system_error_vtable;
-extern const vtable_ptr MSVCP_failure_vtable;
+extern const vtable_ptr runtime_error_vtable;
+extern const vtable_ptr _System_error_vtable;
+extern const vtable_ptr system_error_vtable;
+extern const vtable_ptr failure_vtable;
 /* ??_7bad_cast@std@@6B@ */
-extern const vtable_ptr MSVCP_bad_cast_vtable;
+extern const vtable_ptr bad_cast_vtable;
 /* ??_7range_error@std@@6B@ */
-extern const vtable_ptr MSVCP_range_error_vtable;
-
-static void MSVCP_type_info_dtor(type_info * _this)
-{
-    free(_this->name);
-}
-
-/* Unexported */
-DEFINE_THISCALL_WRAPPER(MSVCP_type_info_vector_dtor,8)
-void * __thiscall MSVCP_type_info_vector_dtor(type_info * _this, unsigned int flags)
-{
-    TRACE("(%p %x)\n", _this, flags);
-    if (flags & 2)
-    {
-        /* we have an array, with the number of elements stored before the first object */
-        INT_PTR i, *ptr = (INT_PTR *)_this - 1;
-
-        for (i = *ptr - 1; i >= 0; i--) MSVCP_type_info_dtor(_this + i);
-        MSVCRT_operator_delete(ptr);
-    }
-    else
-    {
-        MSVCP_type_info_dtor(_this);
-        if (flags & 1) MSVCRT_operator_delete(_this);
-    }
-    return _this;
-}
-
-DEFINE_RTTI_DATA0( type_info, 0, ".?AVtype_info@@" )
+extern const vtable_ptr range_error_vtable;
 
 /* ??0exception@@QAE@ABQBD@Z */
 /* ??0exception@@QEAA@AEBQEBD@Z */
@@ -99,7 +75,7 @@ exception* __thiscall MSVCP_exception_ctor(exception *this, exception_name name)
 {
     TRACE("(%p %s)\n", this, EXCEPTION_STR(name));
 
-    this->vtable = &MSVCP_exception_vtable;
+    this->vtable = &exception_vtable;
     if(EXCEPTION_STR(name)) {
         unsigned int name_len = strlen(EXCEPTION_STR(name)) + 1;
         this->name = malloc(name_len);
@@ -112,17 +88,17 @@ exception* __thiscall MSVCP_exception_ctor(exception *this, exception_name name)
     return this;
 }
 
-DEFINE_THISCALL_WRAPPER(MSVCP_exception_copy_ctor,8)
-exception* __thiscall MSVCP_exception_copy_ctor(exception *this, const exception *rhs)
+DEFINE_THISCALL_WRAPPER(exception_copy_ctor,8)
+exception* __thiscall exception_copy_ctor(exception *this, const exception *rhs)
 {
     TRACE("(%p,%p)\n", this, rhs);
 
     if(!rhs->do_free) {
-        this->vtable = &MSVCP_exception_vtable;
+        this->vtable = &exception_vtable;
         this->name = rhs->name;
         this->do_free = FALSE;
     } else
-        MSVCP_exception_ctor(this, EXCEPTION_NAME(rhs->name));
+        MSVCP_exception_ctor(this, (exception_name)EXCEPTION_NAME(rhs->name));
     TRACE("name = %s\n", this->name);
     return this;
 }
@@ -133,7 +109,7 @@ DEFINE_THISCALL_WRAPPER(MSVCP_exception_default_ctor,4)
 exception* __thiscall MSVCP_exception_default_ctor(exception *this)
 {
     TRACE("(%p)\n", this);
-    this->vtable = &MSVCP_exception_vtable;
+    this->vtable = &exception_vtable;
     this->name = NULL;
     this->do_free = FALSE;
     return this;
@@ -143,7 +119,7 @@ DEFINE_THISCALL_WRAPPER(MSVCP_exception_dtor,4)
 void __thiscall MSVCP_exception_dtor(exception *this)
 {
     TRACE("(%p)\n", this);
-    this->vtable = &MSVCP_exception_vtable;
+    this->vtable = &exception_vtable;
     if(this->do_free)
         free(this->name);
 }
@@ -158,11 +134,11 @@ void * __thiscall MSVCP_exception_vector_dtor(exception *this, unsigned int flag
 
         for(i=*ptr-1; i>=0; i--)
             MSVCP_exception_dtor(this+i);
-        MSVCRT_operator_delete(ptr);
+        operator_delete(ptr);
     } else {
         MSVCP_exception_dtor(this);
         if(flags & 1)
-            MSVCRT_operator_delete(this);
+            operator_delete(this);
     }
 
     return this;
@@ -174,7 +150,7 @@ void * __thiscall MSVCP_exception_scalar_dtor(exception *this, unsigned int flag
 {
     TRACE("(%p %x)\n", this, flags);
     MSVCP_exception_dtor(this);
-    if (flags & 1) MSVCRT_operator_delete(this);
+    if (flags & 1) operator_delete(this);
     return this;
 }
 
@@ -184,7 +160,7 @@ DEFINE_THISCALL_WRAPPER(MSVCP_exception_assign, 8)
 exception* __thiscall MSVCP_exception_assign(exception *this, const exception *assign)
 {
     MSVCP_exception_dtor(this);
-    return MSVCP_exception_copy_ctor(this, assign);
+    return exception_copy_ctor(this, assign);
 }
 
 /* ?_Doraise@bad_alloc@std@@MBEXXZ */
@@ -231,7 +207,7 @@ bad_alloc* __thiscall MSVCP_bad_alloc_ctor(bad_alloc *this, exception_name name)
 {
     TRACE("%p %s\n", this, EXCEPTION_STR(name));
     MSVCP_exception_ctor(this, name);
-    this->vtable = &MSVCP_bad_alloc_vtable;
+    this->vtable = &bad_alloc_vtable;
     return this;
 }
 
@@ -240,18 +216,18 @@ bad_alloc* __thiscall MSVCP_bad_alloc_ctor(bad_alloc *this, exception_name name)
 DEFINE_THISCALL_WRAPPER(MSVCP_bad_alloc_default_ctor, 4)
 bad_alloc* __thiscall MSVCP_bad_alloc_default_ctor(bad_alloc *this)
 {
-    static const char name[] = "bad allocation";
+    static const char *name = "bad allocation";
     return MSVCP_bad_alloc_ctor(this, EXCEPTION_NAME(name));
 }
 
 /* ??0bad_alloc@std@@QAE@ABV01@@Z */
 /* ??0bad_alloc@std@@QEAA@AEBV01@@Z */
-DEFINE_THISCALL_WRAPPER(MSVCP_bad_alloc_copy_ctor, 8)
-bad_alloc* __thiscall MSVCP_bad_alloc_copy_ctor(bad_alloc *this, const bad_alloc *rhs)
+DEFINE_THISCALL_WRAPPER(bad_alloc_copy_ctor, 8)
+bad_alloc* __thiscall bad_alloc_copy_ctor(bad_alloc *this, const bad_alloc *rhs)
 {
     TRACE("%p %p\n", this, rhs);
-    MSVCP_exception_copy_ctor(this, rhs);
-    this->vtable = &MSVCP_bad_alloc_vtable;
+    exception_copy_ctor(this, rhs);
+    this->vtable = &bad_alloc_vtable;
     return this;
 }
 
@@ -274,11 +250,11 @@ void * __thiscall MSVCP_bad_alloc_vector_dtor(bad_alloc *this, unsigned int flag
 
         for(i=*ptr-1; i>=0; i--)
             MSVCP_bad_alloc_dtor(this+i);
-        MSVCRT_operator_delete(ptr);
+        operator_delete(ptr);
     } else {
         MSVCP_bad_alloc_dtor(this);
         if(flags & 1)
-            MSVCRT_operator_delete(this);
+            operator_delete(this);
     }
 
     return this;
@@ -290,7 +266,7 @@ DEFINE_THISCALL_WRAPPER(MSVCP_bad_alloc_assign, 8)
 bad_alloc* __thiscall MSVCP_bad_alloc_assign(bad_alloc *this, const bad_alloc *assign)
 {
     MSVCP_bad_alloc_dtor(this);
-    return MSVCP_bad_alloc_copy_ctor(this, assign);
+    return bad_alloc_copy_ctor(this, assign);
 }
 
 DEFINE_RTTI_DATA1(bad_alloc, 0, &exception_rtti_base_descriptor, ".?AVbad_alloc@std@@")
@@ -320,22 +296,22 @@ logic_error* __thiscall MSVCP_logic_error_ctor( logic_error *this, exception_nam
 #else
     MSVCP_exception_ctor(&this->e, name);
 #endif
-    this->e.vtable = &MSVCP_logic_error_vtable;
+    this->e.vtable = &logic_error_vtable;
     return this;
 }
 
 /* ??0logic_error@std@@QAE@ABV01@@Z */
 /* ??0logic_error@std@@QEAA@AEBV01@@Z */
-DEFINE_THISCALL_WRAPPER(MSVCP_logic_error_copy_ctor, 8)
-logic_error* __thiscall MSVCP_logic_error_copy_ctor(
+DEFINE_THISCALL_WRAPPER(logic_error_copy_ctor, 8)
+logic_error* __thiscall logic_error_copy_ctor(
         logic_error *this, const logic_error *rhs)
 {
     TRACE("%p %p\n", this, rhs);
-    MSVCP_exception_copy_ctor(&this->e, &rhs->e);
+    exception_copy_ctor(&this->e, &rhs->e);
 #if _MSVCP_VER <= 90 && !defined _MSVCIRT
     MSVCP_basic_string_char_copy_ctor(&this->str, &rhs->str);
 #endif
-    this->e.vtable = &MSVCP_logic_error_vtable;
+    this->e.vtable = &logic_error_vtable;
     return this;
 }
 
@@ -378,11 +354,11 @@ void* __thiscall MSVCP_logic_error_vector_dtor(
 
         for(i=*ptr-1; i>=0; i--)
             MSVCP_logic_error_dtor(this+i);
-        MSVCRT_operator_delete(ptr);
+        operator_delete(ptr);
     } else {
         MSVCP_logic_error_dtor(this);
         if(flags & 1)
-            MSVCRT_operator_delete(this);
+            operator_delete(this);
     }
 
     return this;
@@ -394,7 +370,7 @@ void * __thiscall MSVCP_logic_error_scalar_dtor(logic_error *this, unsigned int
 {
     TRACE("(%p %x)\n", this, flags);
     MSVCP_logic_error_dtor(this);
-    if (flags & 1) MSVCRT_operator_delete(this);
+    if (flags & 1) operator_delete(this);
     return this;
 }
 
@@ -404,7 +380,7 @@ DEFINE_THISCALL_WRAPPER(MSVCP_logic_error_assign, 8)
 logic_error* __thiscall MSVCP_logic_error_assign(logic_error *this, const logic_error *assign)
 {
     MSVCP_logic_error_dtor(this);
-    return MSVCP_logic_error_copy_ctor(this, assign);
+    return logic_error_copy_ctor(this, assign);
 }
 
 /* ?what@logic_error@std@@UBEPBDXZ */
@@ -425,7 +401,7 @@ DEFINE_RTTI_DATA1(logic_error, 0, &exception_rtti_base_descriptor, ".?AVlogic_er
 #else
 DEFINE_RTTI_DATA1(logic_error, 0, &exception_rtti_base_descriptor, ".?AVlogic_error@@")
 #endif
-DEFINE_CXX_DATA1(logic_error, &exception_cxx_type_info, MSVCP_logic_error_dtor)
+DEFINE_CXX_TYPE_INFO(logic_error)
 
 /* length_error class data */
 typedef logic_error length_error;
@@ -434,19 +410,19 @@ static length_error* MSVCP_length_error_ctor( length_error *this, exception_name
 {
     TRACE("%p %s\n", this, EXCEPTION_STR(name));
     MSVCP_logic_error_ctor(this, name);
-    this->e.vtable = &MSVCP_length_error_vtable;
+    this->e.vtable = &length_error_vtable;
     return this;
 }
 
 /* ??0length_error@std@@QAE@ABV01@@Z */
 /* ??0length_error@std@@QEAA@AEBV01@@Z */
-DEFINE_THISCALL_WRAPPER(MSVCP_length_error_copy_ctor, 8)
-length_error* __thiscall MSVCP_length_error_copy_ctor(
+DEFINE_THISCALL_WRAPPER(length_error_copy_ctor, 8)
+length_error* __thiscall length_error_copy_ctor(
         length_error *this, const length_error *rhs)
 {
     TRACE("%p %p\n", this, rhs);
-    MSVCP_logic_error_copy_ctor(this, rhs);
-    this->e.vtable = &MSVCP_length_error_vtable;
+    logic_error_copy_ctor(this, rhs);
+    this->e.vtable = &length_error_vtable;
     return this;
 }
 
@@ -468,7 +444,7 @@ DEFINE_THISCALL_WRAPPER(MSVCP_length_error_assign, 8)
 length_error* __thiscall MSVCP_length_error_assign(length_error *this, const length_error *assign)
 {
     MSVCP_logic_error_dtor(this);
-    return MSVCP_length_error_copy_ctor(this, assign);
+    return length_error_copy_ctor(this, assign);
 }
 
 DEFINE_RTTI_DATA2(length_error, 0, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AVlength_error@std@@")
@@ -481,19 +457,19 @@ static out_of_range* MSVCP_out_of_range_ctor( out_of_range *this, exception_name
 {
     TRACE("%p %s\n", this, EXCEPTION_STR(name));
     MSVCP_logic_error_ctor(this, name);
-    this->e.vtable = &MSVCP_out_of_range_vtable;
+    this->e.vtable = &out_of_range_vtable;
     return this;
 }
 
 /* ??0out_of_range@std@@QAE@ABV01@@Z */
 /* ??0out_of_range@std@@QEAA@AEBV01@@Z */
-DEFINE_THISCALL_WRAPPER(MSVCP_out_of_range_copy_ctor, 8)
-out_of_range* __thiscall MSVCP_out_of_range_copy_ctor(
+DEFINE_THISCALL_WRAPPER(out_of_range_copy_ctor, 8)
+out_of_range* __thiscall out_of_range_copy_ctor(
         out_of_range *this, const out_of_range *rhs)
 {
     TRACE("%p %p\n", this, rhs);
-    MSVCP_logic_error_copy_ctor(this, rhs);
-    this->e.vtable = &MSVCP_out_of_range_vtable;
+    logic_error_copy_ctor(this, rhs);
+    this->e.vtable = &out_of_range_vtable;
     return this;
 }
 
@@ -515,7 +491,7 @@ DEFINE_THISCALL_WRAPPER(MSVCP_out_of_range_assign, 8)
 out_of_range* __thiscall MSVCP_out_of_range_assign(out_of_range *this, const out_of_range *assign)
 {
     MSVCP_logic_error_dtor(this);
-    return MSVCP_out_of_range_copy_ctor(this, assign);
+    return out_of_range_copy_ctor(this, assign);
 }
 
 DEFINE_RTTI_DATA2(out_of_range, 0, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AVout_of_range@std@@")
@@ -528,17 +504,17 @@ static invalid_argument* MSVCP_invalid_argument_ctor( invalid_argument *this, ex
 {
     TRACE("%p %s\n", this, EXCEPTION_STR(name));
     MSVCP_logic_error_ctor(this, name);
-    this->e.vtable = &MSVCP_invalid_argument_vtable;
+    this->e.vtable = &invalid_argument_vtable;
     return this;
 }
 
-DEFINE_THISCALL_WRAPPER(MSVCP_invalid_argument_copy_ctor, 8)
-invalid_argument* __thiscall MSVCP_invalid_argument_copy_ctor(
+DEFINE_THISCALL_WRAPPER(invalid_argument_copy_ctor, 8)
+invalid_argument* __thiscall invalid_argument_copy_ctor(
         invalid_argument *this, invalid_argument *rhs)
 {
     TRACE("%p %p\n", this, rhs);
-    MSVCP_logic_error_copy_ctor(this, rhs);
-    this->e.vtable = &MSVCP_invalid_argument_vtable;
+    logic_error_copy_ctor(this, rhs);
+    this->e.vtable = &invalid_argument_vtable;
     return this;
 }
 
@@ -566,22 +542,22 @@ static runtime_error* MSVCP_runtime_error_ctor( runtime_error *this, exception_n
 #else
     MSVCP_exception_ctor(&this->e, name);
 #endif
-    this->e.vtable = &MSVCP_runtime_error_vtable;
+    this->e.vtable = &runtime_error_vtable;
     return this;
 }
 
 /* ??0runtime_error@std@@QAE@ABV01@@Z */
 /* ??0runtime_error@std@@QEAA@AEBV01@@Z */
-DEFINE_THISCALL_WRAPPER(MSVCP_runtime_error_copy_ctor, 8)
-runtime_error* __thiscall MSVCP_runtime_error_copy_ctor(
+DEFINE_THISCALL_WRAPPER(runtime_error_copy_ctor, 8)
+runtime_error* __thiscall runtime_error_copy_ctor(
         runtime_error *this, const runtime_error *rhs)
 {
     TRACE("%p %p\n", this, rhs);
-    MSVCP_exception_copy_ctor(&this->e, &rhs->e);
+    exception_copy_ctor(&this->e, &rhs->e);
 #if _MSVCP_VER <= 90 && !defined _MSVCIRT
     MSVCP_basic_string_char_copy_ctor(&this->str, &rhs->str);
 #endif
-    this->e.vtable = &MSVCP_runtime_error_vtable;
+    this->e.vtable = &runtime_error_vtable;
     return this;
 }
 
@@ -622,11 +598,11 @@ void* __thiscall MSVCP_runtime_error_vector_dtor(
 
         for(i=*ptr-1; i>=0; i--)
             MSVCP_runtime_error_dtor(this+i);
-        MSVCRT_operator_delete(ptr);
+        operator_delete(ptr);
     } else {
         MSVCP_runtime_error_dtor(this);
         if(flags & 1)
-            MSVCRT_operator_delete(this);
+            operator_delete(this);
     }
 
     return this;
@@ -638,7 +614,7 @@ DEFINE_THISCALL_WRAPPER(MSVCP_runtime_error_assign, 8)
 runtime_error* __thiscall MSVCP_runtime_error_assign(runtime_error *this, const runtime_error *assign)
 {
     MSVCP_runtime_error_dtor(this);
-    return MSVCP_runtime_error_copy_ctor(this, assign);
+    return runtime_error_copy_ctor(this, assign);
 }
 
 /* ?what@runtime_error@std@@UBEPBDXZ */
@@ -675,20 +651,20 @@ static failure* MSVCP_failure_ctor( failure *this, exception_name name )
     /* FIXME: set err correctly */
     this->err = 0;
 #endif
-    this->base.e.vtable = &MSVCP_failure_vtable;
+    this->base.e.vtable = &failure_vtable;
     return this;
 }
 
-DEFINE_THISCALL_WRAPPER(MSVCP_failure_copy_ctor, 8)
-failure* __thiscall MSVCP_failure_copy_ctor(
+DEFINE_THISCALL_WRAPPER(failure_copy_ctor, 8)
+failure* __thiscall failure_copy_ctor(
         failure *this, failure *rhs)
 {
     TRACE("%p %p\n", this, rhs);
-    MSVCP_runtime_error_copy_ctor(&this->base, &rhs->base);
+    runtime_error_copy_ctor(&this->base, &rhs->base);
 #if _MSVCP_VER > 90
     this->err = rhs->err;
 #endif
-    this->base.e.vtable = &MSVCP_failure_vtable;
+    this->base.e.vtable = &failure_vtable;
     return this;
 }
 
@@ -715,23 +691,23 @@ const char* __thiscall MSVCP_failure_what(failure *this)
 }
 
 #if _MSVCP_VER > 90
-DEFINE_THISCALL_WRAPPER(MSVCP_system_error_copy_ctor, 8)
-system_error* __thiscall MSVCP_system_error_copy_ctor(
+DEFINE_THISCALL_WRAPPER(system_error_copy_ctor, 8)
+system_error* __thiscall system_error_copy_ctor(
         system_error *this, system_error *rhs)
 {
-    MSVCP_failure_copy_ctor(this, rhs);
-    this->base.e.vtable = &MSVCP_system_error_vtable;
+    failure_copy_ctor(this, rhs);
+    this->base.e.vtable = &system_error_vtable;
     return this;
 }
 #endif
 
 #if _MSVCP_VER > 110
-DEFINE_THISCALL_WRAPPER(MSVCP__System_error_copy_ctor, 8)
-_System_error* __thiscall MSVCP__System_error_copy_ctor(
+DEFINE_THISCALL_WRAPPER(_System_error_copy_ctor, 8)
+_System_error* __thiscall _System_error_copy_ctor(
         _System_error *this, _System_error *rhs)
 {
-    MSVCP_failure_copy_ctor(this, rhs);
-    this->base.e.vtable = &MSVCP__System_error_vtable;
+    failure_copy_ctor(this, rhs);
+    this->base.e.vtable = &_System_error_vtable;
     return this;
 }
 #endif
@@ -776,7 +752,7 @@ bad_cast* __thiscall MSVCP_bad_cast_ctor(bad_cast *this, const char *name)
 {
     TRACE("%p %s\n", this, name);
     MSVCP_exception_ctor(this, EXCEPTION_NAME(name));
-    this->vtable = &MSVCP_bad_cast_vtable;
+    this->vtable = &bad_cast_vtable;
     return this;
 }
 
@@ -790,12 +766,12 @@ bad_cast* __thiscall MSVCP_bad_cast_default_ctor(bad_cast *this)
 
 /* ??0bad_cast@std@@QAE@ABV01@@Z */
 /* ??0bad_cast@std@@QEAA@AEBV01@@Z */
-DEFINE_THISCALL_WRAPPER(MSVCP_bad_cast_copy_ctor, 8)
-bad_cast* __thiscall MSVCP_bad_cast_copy_ctor(bad_cast *this, const bad_cast *rhs)
+DEFINE_THISCALL_WRAPPER(bad_cast_copy_ctor, 8)
+bad_cast* __thiscall bad_cast_copy_ctor(bad_cast *this, const bad_cast *rhs)
 {
     TRACE("%p %p\n", this, rhs);
-    MSVCP_exception_copy_ctor(this, rhs);
-    this->vtable = &MSVCP_bad_cast_vtable;
+    exception_copy_ctor(this, rhs);
+    this->vtable = &bad_cast_vtable;
     return this;
 }
 
@@ -818,11 +794,11 @@ void * __thiscall MSVCP_bad_cast_vector_dtor(bad_cast *this, unsigned int flags)
 
         for(i=*ptr-1; i>=0; i--)
             MSVCP_bad_cast_dtor(this+i);
-        MSVCRT_operator_delete(ptr);
+        operator_delete(ptr);
     } else {
         MSVCP_bad_cast_dtor(this);
         if(flags & 1)
-            MSVCRT_operator_delete(this);
+            operator_delete(this);
     }
 
     return this;
@@ -837,13 +813,12 @@ bad_cast* __thiscall MSVCP_bad_cast_opequals(bad_cast *this, const bad_cast *rhs
 
     if(this != rhs) {
         MSVCP_exception_dtor(this);
-        MSVCP_exception_copy_ctor(this, rhs);
+        exception_copy_ctor(this, rhs);
     }
     return this;
 }
 
 DEFINE_RTTI_DATA1(bad_cast, 0, &exception_rtti_base_descriptor, ".?AVbad_cast@std@@")
-DEFINE_CXX_DATA1(bad_cast, &exception_cxx_type_info, MSVCP_bad_cast_dtor)
 
 /* range_error class data */
 typedef runtime_error range_error;
@@ -852,19 +827,19 @@ static range_error* MSVCP_range_error_ctor( range_error *this, exception_name na
 {
     TRACE("%p %s\n", this, EXCEPTION_STR(name));
     MSVCP_runtime_error_ctor(this, name);
-    this->e.vtable = &MSVCP_range_error_vtable;
+    this->e.vtable = &range_error_vtable;
     return this;
 }
 
 /* ??0range_error@std@@QAE@ABV01@@Z */
 /* ??0range_error@std@@QEAA@AEBV01@@Z */
-DEFINE_THISCALL_WRAPPER(MSVCP_range_error_copy_ctor, 8)
-range_error* __thiscall MSVCP_range_error_copy_ctor(
+DEFINE_THISCALL_WRAPPER(range_error_copy_ctor, 8)
+range_error* __thiscall range_error_copy_ctor(
         range_error *this, const range_error *rhs)
 {
     TRACE("%p %p\n", this, rhs);
-    MSVCP_runtime_error_copy_ctor(this, rhs);
-    this->e.vtable = &MSVCP_range_error_vtable;
+    runtime_error_copy_ctor(this, rhs);
+    this->e.vtable = &range_error_vtable;
     return this;
 }
 
@@ -886,76 +861,179 @@ DEFINE_THISCALL_WRAPPER(MSVCP_range_error_assign, 8)
 range_error* __thiscall MSVCP_range_error_assign(range_error *this, const range_error *assign)
 {
     MSVCP_runtime_error_dtor(this);
-    return MSVCP_range_error_copy_ctor(this, assign);
+    return range_error_copy_ctor(this, assign);
 }
 
 DEFINE_RTTI_DATA2(range_error, 0, &runtime_error_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AVrange_error@std@@")
 DEFINE_CXX_DATA2(range_error, &runtime_error_cxx_type_info, &exception_cxx_type_info, MSVCP_runtime_error_dtor)
 
 /* ?_Nomemory@std@@YAXXZ */
-void __cdecl _Nomemory(void)
+void __cdecl DECLSPEC_NORETURN _Nomemory(void)
 {
+    bad_alloc e;
+
     TRACE("()\n");
-    throw_exception(EXCEPTION_BAD_ALLOC, NULL);
+
+    MSVCP_bad_alloc_default_ctor(&e);
+    _CxxThrowException(&e, &bad_alloc_cxx_type);
 }
 
 /* ?_Xmem@tr1@std@@YAXXZ */
-void __cdecl _Xmem(void)
+void __cdecl DECLSPEC_NORETURN _Xmem(void)
 {
+    bad_alloc e;
+
     TRACE("()\n");
-    throw_exception(EXCEPTION_BAD_ALLOC, NULL);
+
+    MSVCP_bad_alloc_default_ctor(&e);
+    _CxxThrowException(&e, &bad_alloc_cxx_type);
 }
 
 /* ?_Xinvalid_argument@std@@YAXPBD@Z */
 /* ?_Xinvalid_argument@std@@YAXPEBD@Z */
-void __cdecl _Xinvalid_argument(const char *str)
+void __cdecl DECLSPEC_NORETURN _Xinvalid_argument(const char *str)
 {
+    exception_name name = EXCEPTION_NAME(str);
+    invalid_argument e;
+
     TRACE("(%s)\n", debugstr_a(str));
-    throw_exception(EXCEPTION_INVALID_ARGUMENT, str);
+
+    MSVCP_invalid_argument_ctor(&e, name);
+    _CxxThrowException(&e, &invalid_argument_cxx_type);
 }
 
 /* ?_Xlength_error@std@@YAXPBD@Z */
 /* ?_Xlength_error@std@@YAXPEBD@Z */
-void __cdecl _Xlength_error(const char *str)
+void __cdecl DECLSPEC_NORETURN _Xlength_error(const char *str)
 {
+    exception_name name = EXCEPTION_NAME(str);
+    length_error e;
+
     TRACE("(%s)\n", debugstr_a(str));
-    throw_exception(EXCEPTION_LENGTH_ERROR, str);
+
+    MSVCP_length_error_ctor(&e, name);
+    _CxxThrowException(&e, &length_error_cxx_type);
 }
 
 /* ?_Xout_of_range@std@@YAXPBD@Z */
 /* ?_Xout_of_range@std@@YAXPEBD@Z */
-void __cdecl _Xout_of_range(const char *str)
+void __cdecl DECLSPEC_NORETURN _Xout_of_range(const char *str)
 {
+    exception_name name = EXCEPTION_NAME(str);
+    out_of_range e;
+
     TRACE("(%s)\n", debugstr_a(str));
-    throw_exception(EXCEPTION_OUT_OF_RANGE, str);
+
+    MSVCP_out_of_range_ctor(&e, name);
+    _CxxThrowException(&e, &out_of_range_cxx_type);
 }
 
 /* ?_Xruntime_error@std@@YAXPBD@Z */
 /* ?_Xruntime_error@std@@YAXPEBD@Z */
-void __cdecl _Xruntime_error(const char *str)
+void __cdecl DECLSPEC_NORETURN _Xruntime_error(const char *str)
 {
+    exception_name name = EXCEPTION_NAME(str);
+    runtime_error e;
+
     TRACE("(%s)\n", debugstr_a(str));
-    throw_exception(EXCEPTION_RUNTIME_ERROR, str);
+
+    MSVCP_runtime_error_ctor(&e, name);
+    _CxxThrowException(&e, &runtime_error_cxx_type);
 }
 
 /* ?uncaught_exception@std@@YA_NXZ */
-MSVCP_bool __cdecl MSVCP__uncaught_exception(void)
+bool __cdecl MSVCP__uncaught_exception(void)
 {
     return __uncaught_exception();
 }
 
-#if _MSVCP_VER >= 140
-int __cdecl __uncaught_exceptions(void)
-{
-    return *UCRTBASE___processing_throw();
-}
-
+#if _MSVCP_VER >= 110
 typedef struct
 {
     EXCEPTION_RECORD *rec;
-    int *ref; /* not binary compatible with native */
+    LONG *ref; /* not binary compatible with native */
 } exception_ptr;
 
+static void exception_ptr_rethrow(const exception_ptr *ep)
+{
+    TRACE("(%p)\n", ep);
+
+    if (!ep->rec)
+    {
+        static const char *exception_msg = "bad exception";
+        exception e;
+
+        MSVCP_exception_ctor(&e, &exception_msg);
+        _CxxThrowException(&e, &exception_cxx_type);
+        return;
+    }
+
+    RaiseException(ep->rec->ExceptionCode, ep->rec->ExceptionFlags & (~EH_UNWINDING),
+            ep->rec->NumberParameters, ep->rec->ExceptionInformation);
+}
+
+/* ?_Rethrow_future_exception@std@@YAXVexception_ptr@1@@Z */
+void __cdecl _Rethrow_future_exception(const exception_ptr ep)
+{
+    exception_ptr_rethrow(&ep);
+}
+#endif
+
+#if _MSVCP_VER >= 140
+void** CDECL __current_exception(void);
+
+/* compute the this pointer for a base class of a given type */
+static inline void *get_this_pointer( const this_ptr_offsets *off, void *object )
+{
+    if (!object) return NULL;
+
+    if (off->vbase_descr >= 0)
+    {
+        int *offset_ptr;
+
+        /* move this ptr to vbase descriptor */
+        object = (char *)object + off->vbase_descr;
+        /* and fetch additional offset from vbase descriptor */
+        offset_ptr = (int *)(*(char **)object + off->vbase_offset);
+        object = (char *)object + *offset_ptr;
+    }
+
+    object = (char *)object + off->this_offset;
+    return object;
+}
+
+#ifdef __i386__
+extern void call_copy_ctor( void *func, void *this, void *src, int has_vbase );
+__ASM_GLOBAL_FUNC( call_copy_ctor,
+                   "pushl %ebp\n\t"
+                   __ASM_CFI(".cfi_adjust_cfa_offset 4\n\t")
+                   __ASM_CFI(".cfi_rel_offset %ebp,0\n\t")
+                   "movl %esp, %ebp\n\t"
+                   __ASM_CFI(".cfi_def_cfa_register %ebp\n\t")
+                   "pushl $1\n\t"
+                   "movl 12(%ebp), %ecx\n\t"
+                   "pushl 16(%ebp)\n\t"
+                   "call *8(%ebp)\n\t"
+                   "leave\n"
+                   __ASM_CFI(".cfi_def_cfa %esp,4\n\t")
+                   __ASM_CFI(".cfi_same_value %ebp\n\t")
+                   "ret" );
+#else
+static inline void call_copy_ctor( void *func, void *this, void *src, int has_vbase )
+{
+    TRACE( "calling copy ctor %p object %p src %p\n", func, this, src );
+    if (has_vbase)
+        ((void (__cdecl*)(void*, void*, BOOL))func)(this, src, 1);
+    else
+        ((void (__cdecl*)(void*, void*))func)(this, src);
+}
+#endif
+
+int __cdecl __uncaught_exceptions(void)
+{
+    return *__processing_throw();
+}
+
 /*********************************************************************
  * ?__ExceptionPtrCreate@@YAXPAX@Z
  * ?__ExceptionPtrCreate@@YAXPEAX@Z
@@ -968,7 +1046,7 @@ void __cdecl __ExceptionPtrCreate(exception_ptr *ep)
     ep->ref = NULL;
 }
 
-#if defined(__i386__) && !defined(__MINGW32__)
+#ifdef __ASM_USE_THISCALL_WRAPPER
 extern void call_dtor(const cxx_exception_type *type, void *func, void *object);
 
 __ASM_GLOBAL_FUNC( call_dtor,
@@ -1012,6 +1090,246 @@ void __cdecl __ExceptionPtrDestroy(exception_ptr *ep)
         HeapFree(GetProcessHeap(), 0, ep->ref);
     }
 }
+
+/*********************************************************************
+ * ?__ExceptionPtrCopy@@YAXPAXPBX@Z
+ * ?__ExceptionPtrCopy@@YAXPEAXPEBX@Z
+ */
+void __cdecl __ExceptionPtrCopy(exception_ptr *ep, const exception_ptr *copy)
+{
+    TRACE("(%p %p)\n", ep, copy);
+
+    /* don't destroy object stored in ep */
+    *ep = *copy;
+    if (ep->ref)
+        InterlockedIncrement(copy->ref);
+}
+
+/*********************************************************************
+ * ?__ExceptionPtrAssign@@YAXPAXPBX@Z
+ * ?__ExceptionPtrAssign@@YAXPEAXPEBX@Z
+ */
+void __cdecl __ExceptionPtrAssign(exception_ptr *ep, const exception_ptr *assign)
+{
+    TRACE("(%p %p)\n", ep, assign);
+
+    /* don't destroy object stored in ep */
+    if (ep->ref)
+        InterlockedDecrement(ep->ref);
+
+    *ep = *assign;
+    if (ep->ref)
+        InterlockedIncrement(ep->ref);
+}
+
+/*********************************************************************
+ * ?__ExceptionPtrRethrow@@YAXPBX@Z
+ * ?__ExceptionPtrRethrow@@YAXPEBX@Z
+ */
+void __cdecl __ExceptionPtrRethrow(const exception_ptr *ep)
+{
+    exception_ptr_rethrow(ep);
+}
+
+/*********************************************************************
+ * ?__ExceptionPtrCurrentException@@YAXPAX@Z
+ * ?__ExceptionPtrCurrentException@@YAXPEAX@Z
+ */
+#ifndef __x86_64__
+void __cdecl __ExceptionPtrCurrentException(exception_ptr *ep)
+{
+    void **current_exception = __current_exception();
+    EXCEPTION_RECORD *rec = current_exception ? *current_exception : NULL;
+
+    TRACE("(%p)\n", ep);
+
+    if (!rec)
+    {
+        ep->rec = NULL;
+        ep->ref = NULL;
+        return;
+    }
+
+    ep->rec = HeapAlloc(GetProcessHeap(), 0, sizeof(EXCEPTION_RECORD));
+    ep->ref = HeapAlloc(GetProcessHeap(), 0, sizeof(int));
+
+    *ep->rec = *rec;
+    *ep->ref = 1;
+
+    if (ep->rec->ExceptionCode == CXX_EXCEPTION)
+    {
+        const cxx_exception_type *et = (void*)ep->rec->ExceptionInformation[2];
+        const cxx_type_info *ti;
+        void **data, *obj;
+
+        ti = et->type_info_table->info[0];
+        data = HeapAlloc(GetProcessHeap(), 0, ti->size);
+
+        obj = (void*)ep->rec->ExceptionInformation[1];
+        if (ti->flags & CLASS_IS_SIMPLE_TYPE)
+        {
+            memcpy(data, obj, ti->size);
+            if (ti->size == sizeof(void *)) *data = get_this_pointer(&ti->offsets, *data);
+        }
+        else if (ti->copy_ctor)
+        {
+            call_copy_ctor(ti->copy_ctor, data, get_this_pointer(&ti->offsets, obj),
+                    ti->flags & CLASS_HAS_VIRTUAL_BASE_CLASS);
+        }
+        else
+            memcpy(data, get_this_pointer(&ti->offsets, obj), ti->size);
+        ep->rec->ExceptionInformation[1] = (ULONG_PTR)data;
+    }
+    return;
+}
+#else
+void __cdecl __ExceptionPtrCurrentException(exception_ptr *ep)
+{
+    void **current_exception = __current_exception();
+    EXCEPTION_RECORD *rec = current_exception ? *current_exception : NULL;
+
+    TRACE("(%p)\n", ep);
+
+    if (!rec)
+    {
+        ep->rec = NULL;
+        ep->ref = NULL;
+        return;
+    }
+
+    ep->rec = HeapAlloc(GetProcessHeap(), 0, sizeof(EXCEPTION_RECORD));
+    ep->ref = HeapAlloc(GetProcessHeap(), 0, sizeof(int));
+
+    *ep->rec = *rec;
+    *ep->ref = 1;
+
+    if (ep->rec->ExceptionCode == CXX_EXCEPTION)
+    {
+        const cxx_exception_type *et = (void*)ep->rec->ExceptionInformation[2];
+        const cxx_type_info *ti;
+        void **data, *obj;
+        char *base = RtlPcToFileHeader((void*)et, (void**)&base);
+
+        ti = (const cxx_type_info*)(base + ((const cxx_type_info_table*)(base + et->type_info_table))->info[0]);
+        data = HeapAlloc(GetProcessHeap(), 0, ti->size);
+
+        obj = (void*)ep->rec->ExceptionInformation[1];
+        if (ti->flags & CLASS_IS_SIMPLE_TYPE)
+        {
+            memcpy(data, obj, ti->size);
+            if (ti->size == sizeof(void *)) *data = get_this_pointer(&ti->offsets, *data);
+        }
+        else if (ti->copy_ctor)
+        {
+            call_copy_ctor(base + ti->copy_ctor, data, get_this_pointer(&ti->offsets, obj),
+                    ti->flags & CLASS_HAS_VIRTUAL_BASE_CLASS);
+        }
+        else
+            memcpy(data, get_this_pointer(&ti->offsets, obj), ti->size);
+        ep->rec->ExceptionInformation[1] = (ULONG_PTR)data;
+    }
+    return;
+}
+#endif
+
+/*********************************************************************
+ * ?__ExceptionPtrToBool@@YA_NPBX@Z
+ * ?__ExceptionPtrToBool@@YA_NPEBX@Z
+ */
+bool __cdecl __ExceptionPtrToBool(exception_ptr *ep)
+{
+    return !!ep->rec;
+}
+
+/*********************************************************************
+ * ?__ExceptionPtrCopyException@@YAXPAXPBX1@Z
+ * ?__ExceptionPtrCopyException@@YAXPEAXPEBX1@Z
+ */
+#ifndef __x86_64__
+void __cdecl __ExceptionPtrCopyException(exception_ptr *ep,
+        exception *object, const cxx_exception_type *type)
+{
+    const cxx_type_info *ti;
+    void **data;
+
+    __ExceptionPtrDestroy(ep);
+
+    ep->rec = HeapAlloc(GetProcessHeap(), 0, sizeof(EXCEPTION_RECORD));
+    ep->ref = HeapAlloc(GetProcessHeap(), 0, sizeof(int));
+    *ep->ref = 1;
+
+    memset(ep->rec, 0, sizeof(EXCEPTION_RECORD));
+    ep->rec->ExceptionCode = CXX_EXCEPTION;
+    ep->rec->ExceptionFlags = EH_NONCONTINUABLE;
+    ep->rec->NumberParameters = 3;
+    ep->rec->ExceptionInformation[0] = CXX_FRAME_MAGIC_VC6;
+    ep->rec->ExceptionInformation[2] = (ULONG_PTR)type;
+
+    ti = type->type_info_table->info[0];
+    data = HeapAlloc(GetProcessHeap(), 0, ti->size);
+    if (ti->flags & CLASS_IS_SIMPLE_TYPE)
+    {
+        memcpy(data, object, ti->size);
+        if (ti->size == sizeof(void *)) *data = get_this_pointer(&ti->offsets, *data);
+    }
+    else if (ti->copy_ctor)
+    {
+        call_copy_ctor(ti->copy_ctor, data, get_this_pointer(&ti->offsets, object),
+                ti->flags & CLASS_HAS_VIRTUAL_BASE_CLASS);
+    }
+    else
+        memcpy(data, get_this_pointer(&ti->offsets, object), ti->size);
+    ep->rec->ExceptionInformation[1] = (ULONG_PTR)data;
+}
+#else
+void __cdecl __ExceptionPtrCopyException(exception_ptr *ep,
+        exception *object, const cxx_exception_type *type)
+{
+    const cxx_type_info *ti;
+    void **data;
+    char *base;
+
+    RtlPcToFileHeader((void*)type, (void**)&base);
+    __ExceptionPtrDestroy(ep);
+
+    ep->rec = HeapAlloc(GetProcessHeap(), 0, sizeof(EXCEPTION_RECORD));
+    ep->ref = HeapAlloc(GetProcessHeap(), 0, sizeof(int));
+    *ep->ref = 1;
+
+    memset(ep->rec, 0, sizeof(EXCEPTION_RECORD));
+    ep->rec->ExceptionCode = CXX_EXCEPTION;
+    ep->rec->ExceptionFlags = EH_NONCONTINUABLE;
+    ep->rec->NumberParameters = 4;
+    ep->rec->ExceptionInformation[0] = CXX_FRAME_MAGIC_VC6;
+    ep->rec->ExceptionInformation[2] = (ULONG_PTR)type;
+    ep->rec->ExceptionInformation[3] = (ULONG_PTR)base;
+
+    ti = (const cxx_type_info*)(base + ((const cxx_type_info_table*)(base + type->type_info_table))->info[0]);
+    data = HeapAlloc(GetProcessHeap(), 0, ti->size);
+    if (ti->flags & CLASS_IS_SIMPLE_TYPE)
+    {
+        memcpy(data, object, ti->size);
+        if (ti->size == sizeof(void *)) *data = get_this_pointer(&ti->offsets, *data);
+    }
+    else if (ti->copy_ctor)
+    {
+        call_copy_ctor(base + ti->copy_ctor, data, get_this_pointer(&ti->offsets, object),
+                ti->flags & CLASS_HAS_VIRTUAL_BASE_CLASS);
+    }
+    else
+        memcpy(data, get_this_pointer(&ti->offsets, object), ti->size);
+    ep->rec->ExceptionInformation[1] = (ULONG_PTR)data;
+}
+#endif
+
+/*********************************************************************
+ * ?__ExceptionPtrCompare@@YA_NPBX0@Z
+ * ?__ExceptionPtrCompare@@YA_NPEBX0@Z
+ */
+bool __cdecl __ExceptionPtrCompare(const exception_ptr *ep1, const exception_ptr *ep2)
+{
+    return ep1->rec == ep2->rec;
+}
 #endif
 
 #if _MSVCP_VER >= 70 || defined(_MSVCIRT)
@@ -1020,11 +1338,7 @@ void __cdecl __ExceptionPtrDestroy(exception_ptr *ep)
 #define EXCEPTION_VTABLE(name,funcs) __ASM_VTABLE(name,funcs VTABLE_ADD_FUNC(MSVCP_exception__Doraise))
 #endif
 
-#ifndef __GNUC__
-void __asm_dummy_vtables(void) {
-#endif
-    __ASM_VTABLE(type_info,
-            VTABLE_ADD_FUNC(MSVCP_type_info_vector_dtor));
+__ASM_BLOCK_BEGIN(exception_vtables)
     EXCEPTION_VTABLE(exception,
             VTABLE_ADD_FUNC(MSVCP_exception_vector_dtor)
             VTABLE_ADD_FUNC(MSVCP_exception_what));
@@ -1065,69 +1379,36 @@ void __asm_dummy_vtables(void) {
     EXCEPTION_VTABLE(range_error,
             VTABLE_ADD_FUNC(MSVCP_runtime_error_vector_dtor)
             VTABLE_ADD_FUNC(MSVCP_runtime_error_what));
-#ifndef __GNUC__
-}
-#endif
+__ASM_BLOCK_END
 
-/* Internal: throws selected exception */
-void throw_exception(exception_type et, const char *str)
+/* Internal: throws exception */
+void DECLSPEC_NORETURN throw_exception(const char *str)
 {
     exception_name name = EXCEPTION_NAME(str);
+    exception e;
 
-    switch(et) {
-    case EXCEPTION_RERAISE:
-        _CxxThrowException(NULL, NULL);
-    case EXCEPTION: {
-        exception e;
-        MSVCP_exception_ctor(&e, name);
-        _CxxThrowException(&e, &exception_cxx_type);
-    }
-    case EXCEPTION_BAD_ALLOC: {
-        bad_alloc e;
-        MSVCP_bad_alloc_ctor(&e, name);
-        _CxxThrowException(&e, &bad_alloc_cxx_type);
-    }
-    case EXCEPTION_BAD_CAST: {
-        bad_cast e;
-        MSVCP_bad_cast_ctor(&e, str);
-        _CxxThrowException(&e, &bad_cast_cxx_type);
-    }
-    case EXCEPTION_LOGIC_ERROR: {
-        logic_error e;
-        MSVCP_logic_error_ctor(&e, name);
-        _CxxThrowException((exception*)&e, &logic_error_cxx_type);
-    }
-    case EXCEPTION_LENGTH_ERROR: {
-        length_error e;
-        MSVCP_length_error_ctor(&e, name);
-        _CxxThrowException((exception*)&e, &length_error_cxx_type);
-    }
-    case EXCEPTION_OUT_OF_RANGE: {
-        out_of_range e;
-        MSVCP_out_of_range_ctor(&e, name);
-        _CxxThrowException((exception*)&e, &out_of_range_cxx_type);
-    }
-    case EXCEPTION_INVALID_ARGUMENT: {
-        invalid_argument e;
-        MSVCP_invalid_argument_ctor(&e, name);
-        _CxxThrowException((exception*)&e, &invalid_argument_cxx_type);
-    }
-    case EXCEPTION_RUNTIME_ERROR: {
-        runtime_error e;
-        MSVCP_runtime_error_ctor(&e, name);
-        _CxxThrowException((exception*)&e, &runtime_error_cxx_type);
-    }
-    case EXCEPTION_FAILURE: {
-        failure e;
-        MSVCP_failure_ctor(&e, name);
-        _CxxThrowException((exception*)&e, &failure_cxx_type);
-    }
-    case EXCEPTION_RANGE_ERROR: {
-        range_error e;
-        MSVCP_range_error_ctor(&e, name);
-        _CxxThrowException((exception*)&e, &range_error_cxx_type);
-    }
-    }
+    MSVCP_exception_ctor(&e, name);
+    _CxxThrowException(&e, &exception_cxx_type);
+}
+
+/* Internal: throws range_error exception */
+void DECLSPEC_NORETURN throw_range_error(const char *str)
+{
+    exception_name name = EXCEPTION_NAME(str);
+    range_error e;
+
+    MSVCP_range_error_ctor(&e, name);
+    _CxxThrowException(&e, &range_error_cxx_type);
+}
+
+/* Internal: throws failure exception */
+void DECLSPEC_NORETURN throw_failure(const char *str)
+{
+    exception_name name = EXCEPTION_NAME(str);
+    failure e;
+
+    MSVCP_failure_ctor(&e, name);
+    _CxxThrowException(&e, &failure_cxx_type);
 }
 
 void init_exception(void *base)
@@ -1153,7 +1434,7 @@ void init_exception(void *base)
 
     init_exception_cxx(base);
     init_bad_alloc_cxx(base);
-    init_logic_error_cxx(base);
+    init_logic_error_cxx_type_info(base);
     init_length_error_cxx(base);
     init_out_of_range_cxx(base);
     init_invalid_argument_cxx(base);
@@ -1165,7 +1446,6 @@ void init_exception(void *base)
     init_system_error_cxx_type_info(base);
 #endif
     init_failure_cxx(base);
-    init_bad_cast_cxx(base);
     init_range_error_cxx(base);
 #endif
 }
