@@ -20,25 +20,24 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
-#include "wine/unicode.h"
 #include "msvcrt.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(msvcrt);
 
-/*********************************************************************
- *		getenv (MSVCRT.@)
- */
-char * CDECL MSVCRT_getenv(const char *name)
+static char * getenv_helper(const char *name)
 {
-    char **environ;
-    unsigned int length=strlen(name);
+    char **env;
+    size_t len;
 
-    for (environ = MSVCRT__environ; *environ; environ++)
+    if (!name) return NULL;
+    len = strlen(name);
+
+    for (env = MSVCRT__environ; *env; env++)
     {
-        char *str = *environ;
+        char *str = *env;
         char *pos = strchr(str,'=');
-        if (pos && ((pos - str) == length) && !MSVCRT__strnicmp(str,name,length))
+        if (pos && ((pos - str) == len) && !_strnicmp(str, name, len))
         {
             TRACE("(%s): got %s\n", debugstr_a(name), debugstr_a(pos + 1));
             return pos + 1;
@@ -48,22 +47,32 @@ char * CDECL MSVCRT_getenv(const char *name)
 }
 
 /*********************************************************************
- *		_wgetenv (MSVCRT.@)
+ *              getenv (MSVCRT.@)
  */
-MSVCRT_wchar_t * CDECL MSVCRT__wgetenv(const MSVCRT_wchar_t *name)
+char * CDECL getenv(const char *name)
 {
-    MSVCRT_wchar_t **environ;
-    unsigned int length=strlenW(name);
+    if (!MSVCRT_CHECK_PMT(name != NULL)) return NULL;
+
+    return getenv_helper(name);
+}
+
+static wchar_t * wgetenv_helper(const wchar_t *name)
+{
+    wchar_t **env;
+    size_t len;
+
+    if (!name) return NULL;
+    len = wcslen(name);
 
     /* Initialize the _wenviron array if it's not already created. */
     if (!MSVCRT__wenviron)
         MSVCRT__wenviron = msvcrt_SnapshotOfEnvironmentW(NULL);
 
-    for (environ = MSVCRT__wenviron; *environ; environ++)
+    for (env = MSVCRT__wenviron; *env; env++)
     {
-        MSVCRT_wchar_t *str = *environ;
-        MSVCRT_wchar_t *pos = strchrW(str,'=');
-        if (pos && ((pos - str) == length) && !strncmpiW(str,name,length))
+        wchar_t *str = *env;
+        wchar_t *pos = wcschr(str,'=');
+        if (pos && ((pos - str) == len) && !_wcsnicmp(str, name, len))
         {
             TRACE("(%s): got %s\n", debugstr_w(name), debugstr_w(pos + 1));
             return pos + 1;
@@ -72,6 +81,16 @@ MSVCRT_wchar_t * CDECL MSVCRT__wgetenv(const MSVCRT_wchar_t *name)
     return NULL;
 }
 
+/*********************************************************************
+ *              _wgetenv (MSVCRT.@)
+ */
+wchar_t * CDECL _wgetenv(const wchar_t *name)
+{
+    if (!MSVCRT_CHECK_PMT(name != NULL)) return NULL;
+
+    return wgetenv_helper(name);
+}
+
 /*********************************************************************
  *		_putenv (MSVCRT.@)
  */
@@ -121,17 +140,17 @@ finish:
 /*********************************************************************
  *		_wputenv (MSVCRT.@)
  */
-int CDECL _wputenv(const MSVCRT_wchar_t *str)
+int CDECL _wputenv(const wchar_t *str)
 {
- MSVCRT_wchar_t *name, *value;
- MSVCRT_wchar_t *dst;
+ wchar_t *name, *value;
+ wchar_t *dst;
  int ret;
 
  TRACE("%s\n", debugstr_w(str));
 
  if (!str)
    return -1;
- name = HeapAlloc(GetProcessHeap(), 0, (strlenW(str) + 1) * sizeof(MSVCRT_wchar_t));
+ name = HeapAlloc(GetProcessHeap(), 0, (wcslen(str) + 1) * sizeof(wchar_t));
  if (!name)
    return -1;
  dst = name;
@@ -164,19 +183,24 @@ finish:
 /*********************************************************************
  *		_putenv_s (MSVCRT.@)
  */
-int CDECL _putenv_s(const char *name, const char *value)
+errno_t CDECL _putenv_s(const char *name, const char *value)
 {
-    int ret;
+    errno_t ret = 0;
 
     TRACE("%s %s\n", debugstr_a(name), debugstr_a(value));
 
-    if (!MSVCRT_CHECK_PMT(name != NULL)) return -1;
-    if (!MSVCRT_CHECK_PMT(value != NULL)) return -1;
+    if (!MSVCRT_CHECK_PMT(name != NULL)) return EINVAL;
+    if (!MSVCRT_CHECK_PMT(value != NULL)) return EINVAL;
 
-    ret = SetEnvironmentVariableA(name, value[0] ? value : NULL) ? 0 : -1;
-
-    /* _putenv returns success on deletion of nonexistent variable, unlike [Rtl]SetEnvironmentVariable */
-    if ((ret == -1) && (GetLastError() == ERROR_ENVVAR_NOT_FOUND)) ret = 0;
+    if (!SetEnvironmentVariableA(name, value[0] ? value : NULL))
+    {
+        /* _putenv returns success on deletion of nonexistent variable */
+        if (GetLastError() != ERROR_ENVVAR_NOT_FOUND)
+        {
+            msvcrt_set_errno(GetLastError());
+            ret = *_errno();
+        }
+    }
 
     MSVCRT__environ = msvcrt_SnapshotOfEnvironmentA(MSVCRT__environ);
     MSVCRT__wenviron = msvcrt_SnapshotOfEnvironmentW(MSVCRT__wenviron);
@@ -187,19 +211,24 @@ int CDECL _putenv_s(const char *name, const char *value)
 /*********************************************************************
  *		_wputenv_s (MSVCRT.@)
  */
-int CDECL _wputenv_s(const MSVCRT_wchar_t *name, const MSVCRT_wchar_t *value)
+errno_t CDECL _wputenv_s(const wchar_t *name, const wchar_t *value)
 {
-    int ret;
+    errno_t ret = 0;
 
     TRACE("%s %s\n", debugstr_w(name), debugstr_w(value));
 
-    if (!MSVCRT_CHECK_PMT(name != NULL)) return -1;
-    if (!MSVCRT_CHECK_PMT(value != NULL)) return -1;
+    if (!MSVCRT_CHECK_PMT(name != NULL)) return EINVAL;
+    if (!MSVCRT_CHECK_PMT(value != NULL)) return EINVAL;
 
-    ret = SetEnvironmentVariableW(name, value[0] ? value : NULL) ? 0 : -1;
-
-    /* _putenv returns success on deletion of nonexistent variable, unlike [Rtl]SetEnvironmentVariable */
-    if ((ret == -1) && (GetLastError() == ERROR_ENVVAR_NOT_FOUND)) ret = 0;
+    if (!SetEnvironmentVariableW(name, value[0] ? value : NULL))
+    {
+        /* _putenv returns success on deletion of nonexistent variable */
+        if (GetLastError() != ERROR_ENVVAR_NOT_FOUND)
+        {
+            msvcrt_set_errno(GetLastError());
+            ret = *_errno();
+        }
+    }
 
     MSVCRT__environ = msvcrt_SnapshotOfEnvironmentA(MSVCRT__environ);
     MSVCRT__wenviron = msvcrt_SnapshotOfEnvironmentW(MSVCRT__wenviron);
@@ -212,21 +241,21 @@ int CDECL _wputenv_s(const MSVCRT_wchar_t *name, const MSVCRT_wchar_t *value)
 /******************************************************************
  *		_dupenv_s (MSVCR80.@)
  */
-int CDECL _dupenv_s(char **buffer, MSVCRT_size_t *numberOfElements, const char *varname)
+int CDECL _dupenv_s(char **buffer, size_t *numberOfElements, const char *varname)
 {
-    char*               e;
-    MSVCRT_size_t       sz;
+    char *e;
+    size_t sz;
 
-    if (!MSVCRT_CHECK_PMT(buffer != NULL)) return MSVCRT_EINVAL;
-    if (!MSVCRT_CHECK_PMT(varname != NULL)) return MSVCRT_EINVAL;
+    if (!MSVCRT_CHECK_PMT(buffer != NULL)) return EINVAL;
+    if (!MSVCRT_CHECK_PMT(varname != NULL)) return EINVAL;
 
-    if (!(e = MSVCRT_getenv(varname))) return *MSVCRT__errno() = MSVCRT_EINVAL;
+    if (!(e = getenv(varname))) return *_errno() = EINVAL;
 
     sz = strlen(e) + 1;
-    if (!(*buffer = MSVCRT_malloc(sz)))
+    if (!(*buffer = malloc(sz)))
     {
         if (numberOfElements) *numberOfElements = 0;
-        return *MSVCRT__errno() = MSVCRT_ENOMEM;
+        return *_errno() = ENOMEM;
     }
     strcpy(*buffer, e);
     if (numberOfElements) *numberOfElements = sz;
@@ -236,24 +265,24 @@ int CDECL _dupenv_s(char **buffer, MSVCRT_size_t *numberOfElements, const char *
 /******************************************************************
  *		_wdupenv_s (MSVCR80.@)
  */
-int CDECL _wdupenv_s(MSVCRT_wchar_t **buffer, MSVCRT_size_t *numberOfElements,
-                     const MSVCRT_wchar_t *varname)
+int CDECL _wdupenv_s(wchar_t **buffer, size_t *numberOfElements,
+                     const wchar_t *varname)
 {
-    MSVCRT_wchar_t*     e;
-    MSVCRT_size_t       sz;
+    wchar_t *e;
+    size_t sz;
 
-    if (!MSVCRT_CHECK_PMT(buffer != NULL)) return MSVCRT_EINVAL;
-    if (!MSVCRT_CHECK_PMT(varname != NULL)) return MSVCRT_EINVAL;
+    if (!MSVCRT_CHECK_PMT(buffer != NULL)) return EINVAL;
+    if (!MSVCRT_CHECK_PMT(varname != NULL)) return EINVAL;
 
-    if (!(e = MSVCRT__wgetenv(varname))) return *MSVCRT__errno() = MSVCRT_EINVAL;
+    if (!(e = _wgetenv(varname))) return *_errno() = EINVAL;
 
-    sz = strlenW(e) + 1;
-    if (!(*buffer = MSVCRT_malloc(sz * sizeof(MSVCRT_wchar_t))))
+    sz = wcslen(e) + 1;
+    if (!(*buffer = malloc(sz * sizeof(wchar_t))))
     {
         if (numberOfElements) *numberOfElements = 0;
-        return *MSVCRT__errno() = MSVCRT_ENOMEM;
+        return *_errno() = ENOMEM;
     }
-    strcpyW(*buffer, e);
+    wcscpy(*buffer, e);
     if (numberOfElements) *numberOfElements = sz;
     return 0;
 }
@@ -263,24 +292,20 @@ int CDECL _wdupenv_s(MSVCRT_wchar_t **buffer, MSVCRT_size_t *numberOfElements,
 /******************************************************************
  *		getenv_s (MSVCRT.@)
  */
-int CDECL getenv_s(MSVCRT_size_t *pReturnValue, char* buffer, MSVCRT_size_t numberOfElements, const char *varname)
+int CDECL getenv_s(size_t *ret_len, char* buffer, size_t len, const char *varname)
 {
-    char*       e;
+    char *e;
 
-    if (!MSVCRT_CHECK_PMT(pReturnValue != NULL)) return MSVCRT_EINVAL;
-    if (!MSVCRT_CHECK_PMT(!(buffer == NULL && numberOfElements > 0))) return MSVCRT_EINVAL;
-    if (!MSVCRT_CHECK_PMT(varname != NULL)) return MSVCRT_EINVAL;
+    if (!MSVCRT_CHECK_PMT(ret_len != NULL)) return EINVAL;
+    *ret_len = 0;
+    if (!MSVCRT_CHECK_PMT((buffer && len > 0) || (!buffer && !len))) return EINVAL;
+    if (buffer) buffer[0] = 0;
+
+    if (!(e = getenv_helper(varname))) return 0;
+    *ret_len = strlen(e) + 1;
+    if (!len) return 0;
+    if (len < *ret_len) return ERANGE;
 
-    if (!(e = MSVCRT_getenv(varname)))
-    {
-        *pReturnValue = 0;
-        return *MSVCRT__errno() = MSVCRT_EINVAL;
-    }
-    *pReturnValue = strlen(e) + 1;
-    if (numberOfElements < *pReturnValue)
-    {
-        return *MSVCRT__errno() = MSVCRT_ERANGE;
-    }
     strcpy(buffer, e);
     return 0;
 }
@@ -288,33 +313,29 @@ int CDECL getenv_s(MSVCRT_size_t *pReturnValue, char* buffer, MSVCRT_size_t numb
 /******************************************************************
  *		_wgetenv_s (MSVCRT.@)
  */
-int CDECL _wgetenv_s(MSVCRT_size_t *pReturnValue, MSVCRT_wchar_t *buffer, MSVCRT_size_t numberOfElements,
-                     const MSVCRT_wchar_t *varname)
+int CDECL _wgetenv_s(size_t *ret_len, wchar_t *buffer, size_t len,
+                     const wchar_t *varname)
 {
-    MSVCRT_wchar_t*     e;
+    wchar_t *e;
 
-    if (!MSVCRT_CHECK_PMT(pReturnValue != NULL)) return MSVCRT_EINVAL;
-    if (!MSVCRT_CHECK_PMT(!(buffer == NULL && numberOfElements > 0))) return MSVCRT_EINVAL;
-    if (!MSVCRT_CHECK_PMT(varname != NULL)) return MSVCRT_EINVAL;
+    if (!MSVCRT_CHECK_PMT(ret_len != NULL)) return EINVAL;
+    *ret_len = 0;
+    if (!MSVCRT_CHECK_PMT((buffer && len > 0) || (!buffer && !len))) return EINVAL;
+    if (buffer) buffer[0] = 0;
 
-    if (!(e = MSVCRT__wgetenv(varname)))
-    {
-        *pReturnValue = 0;
-        return *MSVCRT__errno() = MSVCRT_EINVAL;
-    }
-    *pReturnValue = strlenW(e) + 1;
-    if (numberOfElements < *pReturnValue)
-    {
-        return *MSVCRT__errno() = MSVCRT_ERANGE;
-    }
-    strcpyW(buffer, e);
+    if (!(e = wgetenv_helper(varname))) return 0;
+    *ret_len = wcslen(e) + 1;
+    if (!len) return 0;
+    if (len < *ret_len) return ERANGE;
+
+    wcscpy(buffer, e);
     return 0;
 }
 
 /*********************************************************************
  *		_get_environ (MSVCRT.@)
  */
-void CDECL MSVCRT__get_environ(char ***ptr)
+void CDECL _get_environ(char ***ptr)
 {
     *ptr = MSVCRT__environ;
 }
@@ -322,7 +343,7 @@ void CDECL MSVCRT__get_environ(char ***ptr)
 /*********************************************************************
  *		_get_wenviron (MSVCRT.@)
  */
-void CDECL MSVCRT__get_wenviron(MSVCRT_wchar_t ***ptr)
+void CDECL _get_wenviron(wchar_t ***ptr)
 {
     *ptr = MSVCRT__wenviron;
 }
