@@ -20,6 +20,7 @@
  */
 
 #include <stdarg.h>
+#include <stdlib.h>
 #include <math.h>
 
 #define NONAMELESSUNION
@@ -32,7 +33,6 @@
 #include "winperf.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 #include "wine/list.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(pdh);
@@ -47,15 +47,6 @@ static CRITICAL_SECTION_DEBUG pdh_handle_cs_debug =
 };
 static CRITICAL_SECTION pdh_handle_cs = { &pdh_handle_cs_debug, -1, 0, 0, 0, 0 };
 
-static inline WCHAR *pdh_strdup( const WCHAR *src )
-{
-    WCHAR *dst;
-
-    if (!src) return NULL;
-    if ((dst = heap_alloc( (lstrlenW( src ) + 1) * sizeof(WCHAR) ))) lstrcpyW( dst, src );
-    return dst;
-}
-
 static inline WCHAR *pdh_strdup_aw( const char *src )
 {
     int len;
@@ -63,29 +54,10 @@ static inline WCHAR *pdh_strdup_aw( const char *src )
 
     if (!src) return NULL;
     len = MultiByteToWideChar( CP_ACP, 0, src, -1, NULL, 0 );
-    if ((dst = heap_alloc( len * sizeof(WCHAR) ))) MultiByteToWideChar( CP_ACP, 0, src, -1, dst, len );
+    if ((dst = malloc( len * sizeof(WCHAR) ))) MultiByteToWideChar( CP_ACP, 0, src, -1, dst, len );
     return dst;
 }
 
-BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
-{
-    TRACE("(0x%p, %d, %p)\n",hinstDLL,fdwReason,lpvReserved);
-    switch (fdwReason)
-    {
-    case DLL_WINE_PREATTACH:
-        return FALSE;    /* prefer native version */
-    case DLL_PROCESS_ATTACH:
-        DisableThreadLibraryCalls(hinstDLL);
-        break;
-    case DLL_PROCESS_DETACH:
-        if (lpvReserved) break;
-        DeleteCriticalSection(&pdh_handle_cs);
-        break;
-    }
-
-    return TRUE;
-}
-
 union value
 {
     LONG     longvalue;
@@ -117,7 +89,7 @@ static struct counter *create_counter( void )
 {
     struct counter *counter;
 
-    if ((counter = heap_alloc_zero( sizeof(struct counter) )))
+    if ((counter = calloc( 1, sizeof(struct counter) )))
     {
         counter->magic = PDH_MAGIC_COUNTER;
         return counter;
@@ -128,8 +100,8 @@ static struct counter *create_counter( void )
 static void destroy_counter( struct counter *counter )
 {
     counter->magic = 0;
-    heap_free( counter->path );
-    heap_free( counter );
+    free( counter->path );
+    free( counter );
 }
 
 #define PDH_MAGIC_QUERY     0x50444830 /* 'PDH0' */
@@ -149,7 +121,7 @@ static struct query *create_query( void )
 {
     struct query *query;
 
-    if ((query = heap_alloc_zero( sizeof(struct query) )))
+    if ((query = calloc( 1, sizeof(struct query) )))
     {
         query->magic = PDH_MAGIC_QUERY;
         list_init( &query->counters );
@@ -161,7 +133,7 @@ static struct query *create_query( void )
 static void destroy_query( struct query *query )
 {
     query->magic = 0;
-    heap_free( query );
+    free( query );
 }
 
 struct source
@@ -174,12 +146,6 @@ struct source
     LONGLONG        base;                           /* samples per second */
 };
 
-static const WCHAR path_processor_time[] =
-    {'\\','P','r','o','c','e','s','s','o','r','(','_','T','o','t','a','l',')',
-     '\\','%',' ','P','r','o','c','e','s','s','o','r',' ','T','i','m','e',0};
-static const WCHAR path_uptime[] =
-    {'\\','S','y','s','t','e','m', '\\', 'S','y','s','t','e','m',' ','U','p',' ','T','i','m','e',0};
-
 static void CALLBACK collect_processor_time( struct counter *counter )
 {
     counter->two.largevalue = 500000; /* FIXME */
@@ -202,8 +168,8 @@ static void CALLBACK collect_uptime( struct counter *counter )
 /* counter source registry */
 static const struct source counter_sources[] =
 {
-    { 6,    path_processor_time,    collect_processor_time,     TYPE_PROCESSOR_TIME,    -5,     10000000 },
-    { 674,  path_uptime,            collect_uptime,             TYPE_UPTIME,            -3,     1000 }
+    { 6,   L"\\Processor(_Total)\\% Processor Time", collect_processor_time, TYPE_PROCESSOR_TIME, -5, 10000000 },
+    { 674, L"\\System\\System Up Time",              collect_uptime,         TYPE_UPTIME,         -3, 1000 }
 };
 
 static BOOL is_local_machine( const WCHAR *name, DWORD len )
@@ -238,7 +204,7 @@ PDH_STATUS WINAPI PdhAddCounterA( PDH_HQUERY query, LPCSTR path,
     PDH_STATUS ret;
     WCHAR *pathW;
 
-    TRACE("%p %s %lx %p\n", query, debugstr_a(path), userdata, counter);
+    TRACE("%p %s %Ix %p\n", query, debugstr_a(path), userdata, counter);
 
     if (!path) return PDH_INVALID_ARGUMENT;
 
@@ -247,7 +213,7 @@ PDH_STATUS WINAPI PdhAddCounterA( PDH_HQUERY query, LPCSTR path,
 
     ret = PdhAddCounterW( query, pathW, userdata, counter );
 
-    heap_free( pathW );
+    free( pathW );
     return ret;
 }
 
@@ -261,7 +227,7 @@ PDH_STATUS WINAPI PdhAddCounterW( PDH_HQUERY hquery, LPCWSTR path,
     struct counter *counter;
     unsigned int i;
 
-    TRACE("%p %s %lx %p\n", hquery, debugstr_w(path), userdata, hcounter);
+    TRACE("%p %s %Ix %p\n", hquery, debugstr_w(path), userdata, hcounter);
 
     if (!path  || !hcounter) return PDH_INVALID_ARGUMENT;
 
@@ -279,7 +245,7 @@ PDH_STATUS WINAPI PdhAddCounterW( PDH_HQUERY hquery, LPCWSTR path,
         {
             if ((counter = create_counter()))
             {
-                counter->path         = pdh_strdup( counter_sources[i].path );
+                counter->path         = wcsdup( counter_sources[i].path );
                 counter->collect      = counter_sources[i].collect;
                 counter->type         = counter_sources[i].type;
                 counter->defaultscale = counter_sources[i].scale;
@@ -307,9 +273,10 @@ PDH_STATUS WINAPI PdhAddCounterW( PDH_HQUERY hquery, LPCWSTR path,
 PDH_STATUS WINAPI PdhAddEnglishCounterA( PDH_HQUERY query, LPCSTR path,
                                          DWORD_PTR userdata, PDH_HCOUNTER *counter )
 {
-    TRACE("%p %s %lx %p\n", query, debugstr_a(path), userdata, counter);
+    TRACE("%p %s %Ix %p\n", query, debugstr_a(path), userdata, counter);
 
-    if (!query) return PDH_INVALID_ARGUMENT;
+    if (!counter) return PDH_INVALID_ARGUMENT;
+    if (!query) return PDH_INVALID_HANDLE;
     return PdhAddCounterA( query, path, userdata, counter );
 }
 
@@ -319,9 +286,10 @@ PDH_STATUS WINAPI PdhAddEnglishCounterA( PDH_HQUERY query, LPCSTR path,
 PDH_STATUS WINAPI PdhAddEnglishCounterW( PDH_HQUERY query, LPCWSTR path,
                                          DWORD_PTR userdata, PDH_HCOUNTER *counter )
 {
-    TRACE("%p %s %lx %p\n", query, debugstr_w(path), userdata, counter);
+    TRACE("%p %s %Ix %p\n", query, debugstr_w(path), userdata, counter);
 
-    if (!query) return PDH_INVALID_ARGUMENT;
+    if (!counter) return PDH_INVALID_ARGUMENT;
+    if (!query) return PDH_INVALID_HANDLE;
     return PdhAddCounterW( query, path, userdata, counter );
 }
 
@@ -349,7 +317,7 @@ static PDH_STATUS format_value( struct counter *counter, DWORD format, union val
     }
     else
     {
-        WARN("unknown format %x\n", format);
+        WARN("unknown format %lx\n", format);
         return PDH_INVALID_ARGUMENT;
     }
     return ERROR_SUCCESS;
@@ -365,7 +333,7 @@ PDH_STATUS WINAPI PdhCalculateCounterFromRawValue( PDH_HCOUNTER handle, DWORD fo
     PDH_STATUS ret;
     struct counter *counter = handle;
 
-    TRACE("%p 0x%08x %p %p %p\n", handle, format, raw1, raw2, value);
+    TRACE("%p 0x%08lx %p %p %p\n", handle, format, raw1, raw2, value);
 
     if (!value) return PDH_INVALID_ARGUMENT;
 
@@ -515,7 +483,7 @@ PDH_STATUS WINAPI PdhCollectQueryDataEx( PDH_HQUERY handle, DWORD interval, HAND
     PDH_STATUS ret;
     struct query *query = handle;
 
-    TRACE("%p %d %p\n", handle, interval, event);
+    TRACE("%p %ld %p\n", handle, interval, event);
 
     EnterCriticalSection( &pdh_handle_cs );
     if (!query || query->magic != PDH_MAGIC_QUERY)
@@ -607,7 +575,7 @@ PDH_STATUS WINAPI PdhCollectQueryDataWithTime( PDH_HQUERY handle, LONGLONG *time
  */
 PDH_STATUS WINAPI PdhExpandWildCardPathA( LPCSTR szDataSource, LPCSTR szWildCardPath, LPSTR mszExpandedPathList, LPDWORD pcchPathListLength, DWORD dwFlags )
 {
-    FIXME("%s, %s, %p, %p, 0x%x: stub\n", debugstr_a(szDataSource), debugstr_a(szWildCardPath), mszExpandedPathList, pcchPathListLength, dwFlags);
+    FIXME("%s, %s, %p, %p, 0x%lx: stub\n", debugstr_a(szDataSource), debugstr_a(szWildCardPath), mszExpandedPathList, pcchPathListLength, dwFlags);
     return PDH_NOT_IMPLEMENTED;
 }
 
@@ -616,7 +584,7 @@ PDH_STATUS WINAPI PdhExpandWildCardPathA( LPCSTR szDataSource, LPCSTR szWildCard
  */
 PDH_STATUS WINAPI PdhExpandWildCardPathW( LPCWSTR szDataSource, LPCWSTR szWildCardPath, LPWSTR mszExpandedPathList, LPDWORD pcchPathListLength, DWORD dwFlags )
 {
-    FIXME("%s, %s, %p, %p, 0x%x: stub\n", debugstr_w(szDataSource), debugstr_w(szWildCardPath), mszExpandedPathList, pcchPathListLength, dwFlags);
+    FIXME("%s, %s, %p, %p, 0x%lx: stub\n", debugstr_w(szDataSource), debugstr_w(szWildCardPath), mszExpandedPathList, pcchPathListLength, dwFlags);
     return PDH_NOT_IMPLEMENTED;
 }
 
@@ -768,7 +736,7 @@ PDH_STATUS WINAPI PdhGetFormattedCounterValue( PDH_HCOUNTER handle, DWORD format
     PDH_STATUS ret;
     struct counter *counter = handle;
 
-    TRACE("%p %x %p %p\n", handle, format, type, value);
+    TRACE("%p %lx %p %p\n", handle, format, type, value);
 
     if (!value) return PDH_INVALID_ARGUMENT;
 
@@ -845,8 +813,8 @@ PDH_STATUS WINAPI PdhLookupPerfIndexByNameA( LPCSTR machine, LPCSTR name, LPDWOR
 
     ret = PdhLookupPerfIndexByNameW( machineW, nameW, index );
 
-    heap_free( nameW );
-    heap_free( machineW );
+    free( nameW );
+    free( machineW );
     return ret;
 }
 
@@ -887,7 +855,7 @@ PDH_STATUS WINAPI PdhLookupPerfNameByIndexA( LPCSTR machine, DWORD index, LPSTR
     WCHAR bufferW[PDH_MAX_COUNTER_NAME];
     DWORD sizeW = ARRAY_SIZE(bufferW);
 
-    TRACE("%s %d %p %p\n", debugstr_a(machine), index, buffer, size);
+    TRACE("%s %ld %p %p\n", debugstr_a(machine), index, buffer, size);
 
     if (!buffer || !size) return PDH_INVALID_ARGUMENT;
 
@@ -901,7 +869,7 @@ PDH_STATUS WINAPI PdhLookupPerfNameByIndexA( LPCSTR machine, DWORD index, LPSTR
         else WideCharToMultiByte( CP_ACP, 0, bufferW, -1, buffer, required, NULL, NULL );
         *size = required;
     }
-    heap_free( machineW );
+    free( machineW );
     return ret;
 }
 
@@ -913,7 +881,7 @@ PDH_STATUS WINAPI PdhLookupPerfNameByIndexW( LPCWSTR machine, DWORD index, LPWST
     PDH_STATUS ret;
     unsigned int i;
 
-    TRACE("%s %d %p %p\n", debugstr_w(machine), index, buffer, size);
+    TRACE("%s %ld %p %p\n", debugstr_w(machine), index, buffer, size);
 
     if (machine)
     {
@@ -952,12 +920,12 @@ PDH_STATUS WINAPI PdhOpenQueryA( LPCSTR source, DWORD_PTR userdata, PDH_HQUERY *
     PDH_STATUS ret;
     WCHAR *sourceW = NULL;
 
-    TRACE("%s %lx %p\n", debugstr_a(source), userdata, query);
+    TRACE("%s %Ix %p\n", debugstr_a(source), userdata, query);
 
     if (source && !(sourceW = pdh_strdup_aw( source ))) return PDH_MEMORY_ALLOCATION_FAILURE;
 
     ret = PdhOpenQueryW( sourceW, userdata, query );
-    heap_free( sourceW );
+    free( sourceW );
 
     return ret;
 }
@@ -969,7 +937,7 @@ PDH_STATUS WINAPI PdhOpenQueryW( LPCWSTR source, DWORD_PTR userdata, PDH_HQUERY
 {
     struct query *query;
 
-    TRACE("%s %lx %p\n", debugstr_w(source), userdata, handle);
+    TRACE("%s %Ix %p\n", debugstr_w(source), userdata, handle);
 
     if (!handle) return PDH_INVALID_ARGUMENT;
 
@@ -1053,7 +1021,7 @@ PDH_STATUS WINAPI PdhValidatePathA( LPCSTR path )
 
     ret = PdhValidatePathW( pathW );
 
-    heap_free( pathW );
+    free( pathW );
     return ret;
 }
 
@@ -1094,6 +1062,23 @@ PDH_STATUS WINAPI PdhVbAddCounter( PDH_HQUERY query, LPCSTR path, PDH_HCOUNTER *
     return PDH_NOT_IMPLEMENTED;
 }
 
+/***********************************************************************
+ *              PdhVbGetDoubleCounterValue   (PDH.@)
+ */
+double WINAPI PdhVbGetDoubleCounterValue( PDH_HCOUNTER handle, PDH_STATUS *counter_status )
+{
+    PDH_FMT_COUNTERVALUE value;
+    PDH_STATUS status;
+
+    TRACE( "%p %p\n", handle, counter_status );
+
+    memset( &value, 0, sizeof(value) );
+    status = PdhGetFormattedCounterValue( handle, PDH_FMT_DOUBLE, NULL, &value );
+
+    if (counter_status) *counter_status = status;
+    return value.u.doubleValue;
+}
+
 /***********************************************************************
  *              PdhValidatePathExA   (PDH.@)
  */
@@ -1135,7 +1120,7 @@ PDH_STATUS WINAPI PdhMakeCounterPathA( PDH_COUNTER_PATH_ELEMENTS_A *e, LPSTR buf
     WCHAR *bufferW;
     DWORD buflenW;
 
-    TRACE("%p %p %p 0x%08x\n", e, buffer, buflen, flags);
+    TRACE("%p %p %p 0x%08lx\n", e, buffer, buflen, flags);
 
     if (!e || !buflen) return PDH_INVALID_ARGUMENT;
 
@@ -1151,7 +1136,7 @@ PDH_STATUS WINAPI PdhMakeCounterPathA( PDH_COUNTER_PATH_ELEMENTS_A *e, LPSTR buf
     ret = PdhMakeCounterPathW( &eW, NULL, &buflenW, flags );
     if (ret == PDH_MORE_DATA)
     {
-        if ((bufferW = heap_alloc( buflenW * sizeof(WCHAR) )))
+        if ((bufferW = malloc( buflenW * sizeof(WCHAR) )))
         {
             if (!(ret = PdhMakeCounterPathW( &eW, bufferW, &buflenW, flags )))
             {
@@ -1160,18 +1145,18 @@ PDH_STATUS WINAPI PdhMakeCounterPathA( PDH_COUNTER_PATH_ELEMENTS_A *e, LPSTR buf
                 else ret = PDH_MORE_DATA;
                 *buflen = len;
             }
-            heap_free( bufferW );
+            free( bufferW );
         }
         else
             ret = PDH_MEMORY_ALLOCATION_FAILURE;
     }
 
 done:
-    heap_free( eW.szMachineName );
-    heap_free( eW.szObjectName );
-    heap_free( eW.szInstanceName );
-    heap_free( eW.szParentInstance );
-    heap_free( eW.szCounterName );
+    free( eW.szMachineName );
+    free( eW.szObjectName );
+    free( eW.szInstanceName );
+    free( eW.szParentInstance );
+    free( eW.szCounterName );
     return ret;
 }
 
@@ -1181,19 +1166,13 @@ done:
 PDH_STATUS WINAPI PdhMakeCounterPathW( PDH_COUNTER_PATH_ELEMENTS_W *e, LPWSTR buffer,
                                        LPDWORD buflen, DWORD flags )
 {
-    static const WCHAR bslash[] = {'\\',0};
-    static const WCHAR fslash[] = {'/',0};
-    static const WCHAR lparen[] = {'(',0};
-    static const WCHAR rparen[] = {')',0};
-    static const WCHAR fmt[]    = {'#','%','u',0};
-
     WCHAR path[PDH_MAX_COUNTER_NAME], instance[12];
     PDH_STATUS ret = ERROR_SUCCESS;
     DWORD len;
 
-    TRACE("%p %p %p 0x%08x\n", e, buffer, buflen, flags);
+    TRACE("%p %p %p 0x%08lx\n", e, buffer, buflen, flags);
 
-    if (flags) FIXME("unimplemented flags 0x%08x\n", flags);
+    if (flags) FIXME("unimplemented flags 0x%08lx\n", flags);
 
     if (!e || !e->szCounterName || !e->szObjectName || !buflen)
         return PDH_INVALID_ARGUMENT;
@@ -1201,26 +1180,25 @@ PDH_STATUS WINAPI PdhMakeCounterPathW( PDH_COUNTER_PATH_ELEMENTS_W *e, LPWSTR bu
     path[0] = 0;
     if (e->szMachineName)
     {
-        lstrcatW(path, bslash);
-        lstrcatW(path, bslash);
+        lstrcatW(path, L"\\\\");
         lstrcatW(path, e->szMachineName);
     }
-    lstrcatW(path, bslash);
+    lstrcatW(path, L"\\");
     lstrcatW(path, e->szObjectName);
     if (e->szInstanceName)
     {
-        lstrcatW(path, lparen);
+        lstrcatW(path, L"(");
         if (e->szParentInstance)
         {
             lstrcatW(path, e->szParentInstance);
-            lstrcatW(path, fslash);
+            lstrcatW(path, L"/");
         }
         lstrcatW(path, e->szInstanceName);
-        swprintf(instance, ARRAY_SIZE(instance), fmt, e->dwInstanceIndex);
+        swprintf(instance, ARRAY_SIZE(instance), L"#%u", e->dwInstanceIndex);
         lstrcatW(path, instance);
-        lstrcatW(path, rparen);
+        lstrcatW(path, L")");
     }
-    lstrcatW(path, bslash);
+    lstrcatW(path, L"\\");
     lstrcatW(path, e->szCounterName);
 
     len = lstrlenW(path) + 1;
@@ -1237,7 +1215,7 @@ PDH_STATUS WINAPI PdhEnumObjectItemsA(LPCSTR szDataSource, LPCSTR szMachineName,
                                       LPSTR mszCounterList, LPDWORD pcchCounterListLength, LPSTR mszInstanceList,
                                       LPDWORD pcchInstanceListLength, DWORD dwDetailLevel, DWORD dwFlags)
 {
-    FIXME("%s, %s, %s, %p, %p, %p, %p, %d, 0x%x: stub\n", debugstr_a(szDataSource), debugstr_a(szMachineName),
+    FIXME("%s, %s, %s, %p, %p, %p, %p, %ld, 0x%lx: stub\n", debugstr_a(szDataSource), debugstr_a(szMachineName),
          debugstr_a(szObjectName), mszCounterList, pcchCounterListLength, mszInstanceList,
          pcchInstanceListLength, dwDetailLevel, dwFlags);
 
@@ -1251,7 +1229,7 @@ PDH_STATUS WINAPI PdhEnumObjectItemsW(LPCWSTR szDataSource, LPCWSTR szMachineNam
                                       LPWSTR mszCounterList, LPDWORD pcchCounterListLength, LPWSTR mszInstanceList,
                                       LPDWORD pcchInstanceListLength, DWORD dwDetailLevel, DWORD dwFlags)
 {
-    FIXME("%s, %s, %s, %p, %p, %p, %p, %d, 0x%x: stub\n", debugstr_w(szDataSource), debugstr_w(szMachineName),
+    FIXME("%s, %s, %s, %p, %p, %p, %p, %ld, 0x%lx: stub\n", debugstr_w(szDataSource), debugstr_w(szMachineName),
          debugstr_w(szObjectName), mszCounterList, pcchCounterListLength, mszInstanceList,
          pcchInstanceListLength, dwDetailLevel, dwFlags);
 
@@ -1263,7 +1241,7 @@ PDH_STATUS WINAPI PdhEnumObjectItemsW(LPCWSTR szDataSource, LPCWSTR szMachineNam
  */
 PDH_STATUS WINAPI PdhSetDefaultRealTimeDataSource( DWORD source )
 {
-    FIXME("%u\n", source);
+    FIXME("%lu\n", source);
     return ERROR_SUCCESS;
 }
 
