@@ -18,15 +18,9 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <stdarg.h>
 #include <string.h>
 #include <sys/types.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
 
 #define COBJMACROS
 
@@ -41,7 +35,6 @@
 #include "objbase.h"
 #include "pidl.h"
 #include "shell32_main.h"
-#include "undocshell.h"
 #include "shresdef.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(shell);
@@ -61,8 +54,7 @@ typedef struct
 
 static HDPA sic_hdpa;
 static INIT_ONCE sic_init_once = INIT_ONCE_STATIC_INIT;
-static HIMAGELIST ShellSmallIconList;
-static HIMAGELIST ShellBigIconList;
+static HIMAGELIST shell_imagelists[SHIL_LAST+1];
 
 static CRITICAL_SECTION SHELL32_SicCS;
 static CRITICAL_SECTION_DEBUG critsect_debug =
@@ -74,10 +66,6 @@ static CRITICAL_SECTION_DEBUG critsect_debug =
 static CRITICAL_SECTION SHELL32_SicCS = { &critsect_debug, -1, 0, 0, 0, 0 };
 
 
-static const WCHAR WindowMetrics[] = {'C','o','n','t','r','o','l',' ','P','a','n','e','l','\\','D','e','s','k','t','o','p','\\',
-                                      'W','i','n','d','o','w','M','e','t','r','i','c','s',0};
-static const WCHAR ShellIconSize[] = {'S','h','e','l','l',' ','I','c','o','n',' ','S','i','z','e',0};
-
 #define SIC_COMPARE_LISTINDEX 1
 
 /*****************************************************************************
@@ -90,7 +78,7 @@ static INT CALLBACK SIC_CompareEntries( LPVOID p1, LPVOID p2, LPARAM lparam)
 {
         LPSIC_ENTRY e1 = p1, e2 = p2;
 
-	TRACE("%p %p %8lx\n", p1, p2, lparam);
+	TRACE("%p %p %8Ix\n", p1, p2, lparam);
 
 	/* Icons in the cache are keyed by the name of the file they are
 	 * loaded from, their resource index and the fact if they have a shortcut
@@ -104,7 +92,7 @@ static INT CALLBACK SIC_CompareEntries( LPVOID p1, LPVOID p2, LPARAM lparam)
 	    (e1->dwFlags & GIL_FORSHORTCUT) != (e2->dwFlags & GIL_FORSHORTCUT)) 
 	  return 1;
 
-	if (strcmpiW(e1->sSourceFile,e2->sSourceFile))
+	if (wcsicmp(e1->sSourceFile,e2->sSourceFile))
 	  return 1;
 
 	return 0;
@@ -130,7 +118,7 @@ HRESULT SIC_get_location( int list_idx, WCHAR *file, DWORD *size, int *res_idx )
     if (dpa_idx != -1)
     {
         found = DPA_GetPtr( sic_hdpa, dpa_idx );
-        needed = (strlenW( found->sSourceFile ) + 1) * sizeof(WCHAR);
+        needed = (lstrlenW( found->sSourceFile ) + 1) * sizeof(WCHAR);
         if (needed <= *size)
         {
             memcpy( file, found->sSourceFile, needed );
@@ -158,8 +146,9 @@ static int SIC_LoadOverlayIcon(int icon_idx);
  *  Creates a new icon as a copy of the passed-in icon, overlaid with a
  *  shortcut image. 
  */
-static HICON SIC_OverlayShortcutImage(HICON SourceIcon, BOOL large)
-{	ICONINFO SourceIconInfo, ShortcutIconInfo, TargetIconInfo;
+static HICON SIC_OverlayShortcutImage(HICON SourceIcon, int type)
+{
+    ICONINFO SourceIconInfo, ShortcutIconInfo, TargetIconInfo;
 	HICON ShortcutIcon, TargetIcon;
 	BITMAP SourceBitmapInfo, ShortcutBitmapInfo;
 	HDC SourceDC = NULL,
@@ -186,21 +175,16 @@ static HICON SIC_OverlayShortcutImage(HICON SourceIcon, BOOL large)
                               resource id, but not all icons are present yet
                               so we can't use icon indices */
 
-	if (s_imgListIdx != -1)
-	{
-	    if (large)
-	        ShortcutIcon = ImageList_GetIcon(ShellBigIconList, s_imgListIdx, ILD_TRANSPARENT);
-	    else
-	        ShortcutIcon = ImageList_GetIcon(ShellSmallIconList, s_imgListIdx, ILD_TRANSPARENT);
-	} else
-	    ShortcutIcon = NULL;
+    if (s_imgListIdx != -1)
+        ShortcutIcon = ImageList_GetIcon(shell_imagelists[type], s_imgListIdx, ILD_TRANSPARENT);
+    else
+        ShortcutIcon = NULL;
 
-	if (NULL == ShortcutIcon
-	    || ! GetIconInfo(ShortcutIcon, &ShortcutIconInfo)
-	    || 0 == GetObjectW(ShortcutIconInfo.hbmColor, sizeof(BITMAP), &ShortcutBitmapInfo))
-	{
-	  return NULL;
-	}
+    if (NULL == ShortcutIcon || ! GetIconInfo(ShortcutIcon, &ShortcutIconInfo)
+            || 0 == GetObjectW(ShortcutIconInfo.hbmColor, sizeof(BITMAP), &ShortcutBitmapInfo))
+    {
+        return NULL;
+    }
 
 	TargetIconInfo = SourceIconInfo;
 	TargetIconInfo.hbmMask = NULL;
@@ -303,111 +287,121 @@ fail:
 
 /*****************************************************************************
  * SIC_IconAppend			[internal]
- *
- * NOTES
- *  appends an icon pair to the end of the cache
  */
-static INT SIC_IconAppend (LPCWSTR sSourceFile, INT dwSourceIndex, HICON hSmallIcon, HICON hBigIcon, DWORD dwFlags)
-{	LPSIC_ENTRY lpsice;
-	INT ret, index, index1;
-	WCHAR path[MAX_PATH];
-	TRACE("%s %i %p %p\n", debugstr_w(sSourceFile), dwSourceIndex, hSmallIcon ,hBigIcon);
+static INT SIC_IconAppend (const WCHAR *sourcefile, INT src_index, HICON *hicons, DWORD flags)
+{
+    INT ret, index, index1;
+    WCHAR path[MAX_PATH];
+    SIC_ENTRY *entry;
+    unsigned int i;
 
-	lpsice = SHAlloc(sizeof(SIC_ENTRY));
+    TRACE("%s %i %p %#lx\n", debugstr_w(sourcefile), src_index, hicons, flags);
 
-	GetFullPathNameW(sSourceFile, MAX_PATH, path, NULL);
-	lpsice->sSourceFile = HeapAlloc( GetProcessHeap(), 0, (strlenW(path)+1)*sizeof(WCHAR) );
-	strcpyW( lpsice->sSourceFile, path );
+    entry = SHAlloc(sizeof(*entry));
 
-	lpsice->dwSourceIndex = dwSourceIndex;
-	lpsice->dwFlags = dwFlags;
+    GetFullPathNameW(sourcefile, MAX_PATH, path, NULL);
+    entry->sSourceFile = heap_alloc( (lstrlenW(path)+1)*sizeof(WCHAR) );
+    lstrcpyW( entry->sSourceFile, path );
 
-	EnterCriticalSection(&SHELL32_SicCS);
+    entry->dwSourceIndex = src_index;
+    entry->dwFlags = flags;
 
-	index = DPA_InsertPtr(sic_hdpa, 0x7fff, lpsice);
-	if ( INVALID_INDEX == index )
-	{
-	  HeapFree(GetProcessHeap(), 0, lpsice->sSourceFile);
-	  SHFree(lpsice);
-	  ret = INVALID_INDEX;
-	}
-	else
-	{
-	  index = ImageList_AddIcon (ShellSmallIconList, hSmallIcon);
-	  index1= ImageList_AddIcon (ShellBigIconList, hBigIcon);
+    EnterCriticalSection(&SHELL32_SicCS);
 
-	  if (index!=index1)
-	  {
-	    FIXME("iconlists out of sync 0x%x 0x%x\n", index, index1);
-	  }
-	  lpsice->dwListIndex = index;
-	  ret = lpsice->dwListIndex;
-	}
+    index = DPA_InsertPtr(sic_hdpa, 0x7fff, entry);
+    if ( INVALID_INDEX == index )
+    {
+        heap_free(entry->sSourceFile);
+        SHFree(entry);
+        ret = INVALID_INDEX;
+    }
+    else
+    {
+        index = -1;
+        for (i = 0; i < ARRAY_SIZE(shell_imagelists); i++)
+        {
+            index1 = ImageList_AddIcon(shell_imagelists[i], hicons[i]);
+            if (index != -1 && index1 != index)
+                WARN("Imagelists out of sync, list %d.\n", i);
+            index = index1;
+        }
 
-	LeaveCriticalSection(&SHELL32_SicCS);
-	return ret;
+        entry->dwListIndex = index;
+        ret = entry->dwListIndex;
+    }
+
+    LeaveCriticalSection(&SHELL32_SicCS);
+    return ret;
 }
 
 static BOOL get_imagelist_icon_size(int list, SIZE *size)
 {
-    HIMAGELIST image_list;
+    int cx, cy;
+    if (list < 0 || list >= ARRAY_SIZE(shell_imagelists)) return FALSE;
 
-    if (list < SHIL_LARGE || list > SHIL_SMALL) return FALSE;
-    image_list = (list == SHIL_LARGE) ? ShellBigIconList : ShellSmallIconList;
-
-    return ImageList_GetIconSize( image_list, &size->cx, &size->cy );
+    if (!ImageList_GetIconSize( shell_imagelists[list], &cx, &cy )) return FALSE;
+    size->cx = cx;
+    size->cy = cy;
+    return TRUE;
 }
 
 /****************************************************************************
  * SIC_LoadIcon				[internal]
  *
  * NOTES
- *  gets small/big icon by number from a file
+ *  gets icons by index from the file
  */
-static INT SIC_LoadIcon (LPCWSTR sSourceFile, INT dwSourceIndex, DWORD dwFlags)
+static INT SIC_LoadIcon (const WCHAR *sourcefile, INT index, DWORD flags)
 {
-	HICON	hiconLarge=0;
-	HICON	hiconSmall=0;
-	HICON 	hiconLargeShortcut;
-	HICON	hiconSmallShortcut;
-        int ret;
-        SIZE size;
+    HICON hicons[ARRAY_SIZE(shell_imagelists)] = { 0 };
+    HICON hshortcuts[ARRAY_SIZE(hicons)] = { 0 };
+    unsigned int i;
+    SIZE size;
+    INT ret = -1;
 
-        get_imagelist_icon_size( SHIL_LARGE, &size );
-        PrivateExtractIconsW( sSourceFile, dwSourceIndex, size.cx, size.cy, &hiconLarge, 0, 1, 0 );
-        get_imagelist_icon_size( SHIL_SMALL, &size );
-        PrivateExtractIconsW( sSourceFile, dwSourceIndex, size.cx, size.cy, &hiconSmall, 0, 1, 0 );
+    for (i = 0; i < ARRAY_SIZE(hicons); i++)
+    {
+        if (!get_imagelist_icon_size( i, &size ) ||
+            !PrivateExtractIconsW( sourcefile, index, size.cx, size.cy, &hicons[i], 0, 1, 0 ))
+            WARN("Failed to load icon %d from %s.\n", index, debugstr_w(sourcefile));
+        if (!hicons[i]) goto fail;
+    }
 
-	if ( !hiconLarge ||  !hiconSmall)
-	{
-	  WARN("failure loading icon %i from %s (%p %p)\n", dwSourceIndex, debugstr_w(sSourceFile), hiconLarge, hiconSmall);
-	  return -1;
-	}
+    if (flags & GIL_FORSHORTCUT)
+    {
+        BOOL failed = FALSE;
 
-	if (0 != (dwFlags & GIL_FORSHORTCUT))
-	{
-	  hiconLargeShortcut = SIC_OverlayShortcutImage(hiconLarge, TRUE);
-	  hiconSmallShortcut = SIC_OverlayShortcutImage(hiconSmall, FALSE);
-	  if (NULL != hiconLargeShortcut && NULL != hiconSmallShortcut)
-	  {
-            DestroyIcon( hiconLarge );
-            DestroyIcon( hiconSmall );
-	    hiconLarge = hiconLargeShortcut;
-	    hiconSmall = hiconSmallShortcut;
-	  }
-	  else
-	  {
-	    WARN("Failed to create shortcut overlaid icons\n");
-	    if (NULL != hiconLargeShortcut) DestroyIcon(hiconLargeShortcut);
-	    if (NULL != hiconSmallShortcut) DestroyIcon(hiconSmallShortcut);
-	    dwFlags &= ~ GIL_FORSHORTCUT;
-	  }
-	}
+        for (i = 0; i < ARRAY_SIZE(hshortcuts); i++)
+        {
+            if (!(hshortcuts[i] = SIC_OverlayShortcutImage(hicons[i], i)))
+            {
+                WARN("Failed to create shortcut overlaid icons.\n");
+                failed = TRUE;
+            }
+        }
 
-        ret = SIC_IconAppend( sSourceFile, dwSourceIndex, hiconSmall, hiconLarge, dwFlags );
-        DestroyIcon( hiconLarge );
-        DestroyIcon( hiconSmall );
-        return ret;
+        if (failed)
+        {
+            for (i = 0; i < ARRAY_SIZE(hshortcuts); i++)
+                DestroyIcon(hshortcuts[i]);
+            flags &= ~GIL_FORSHORTCUT;
+        }
+        else
+        {
+            for (i = 0; i < ARRAY_SIZE(hicons); i++)
+            {
+                DestroyIcon(hicons[i]);
+                hicons[i] = hshortcuts[i];
+            }
+        }
+    }
+
+    ret = SIC_IconAppend( sourcefile, index, hicons, flags );
+
+fail:
+    for (i = 0; i < ARRAY_SIZE(hicons); i++)
+        DestroyIcon(hicons[i]);
+    return ret;
 }
 
 static int get_shell_icon_size(void)
@@ -416,12 +410,12 @@ static int get_shell_icon_size(void)
     DWORD value = 32, size = sizeof(buf), type;
     HKEY key;
 
-    if (!RegOpenKeyW( HKEY_CURRENT_USER, WindowMetrics, &key ))
+    if (!RegOpenKeyW( HKEY_CURRENT_USER, L"Control Panel\\Desktop\\WindowMetrics", &key ))
     {
-        if (!RegQueryValueExW( key, ShellIconSize, NULL, &type, (BYTE *)buf, &size ) && type == REG_SZ)
+        if (!RegQueryValueExW( key, L"Shell Icon Size", NULL, &type, (BYTE *)buf, &size ) && type == REG_SZ)
         {
             if (size == sizeof(buf)) buf[size / sizeof(WCHAR) - 1] = 0;
-            value = atoiW( buf );
+            value = wcstol( buf, NULL, 10 );
         }
         RegCloseKey( key );
     }
@@ -433,58 +427,64 @@ static int get_shell_icon_size(void)
  */
 static BOOL WINAPI SIC_Initialize( INIT_ONCE *once, void *param, void **context )
 {
-	HICON		hSm, hLg;
-	int		cx_small, cy_small;
-	int		cx_large, cy_large;
+    HICON hicons[ARRAY_SIZE(shell_imagelists)];
+    SIZE sizes[ARRAY_SIZE(shell_imagelists)];
+    BOOL failed = FALSE;
+    unsigned int i;
 
-        if (!IsProcessDPIAware())
+    if (!IsProcessDPIAware())
+    {
+        sizes[SHIL_LARGE].cx = sizes[SHIL_LARGE].cy = get_shell_icon_size();
+        sizes[SHIL_SMALL].cx = GetSystemMetrics( SM_CXSMICON );
+        sizes[SHIL_SMALL].cy = GetSystemMetrics( SM_CYSMICON );
+    }
+    else
+    {
+        sizes[SHIL_LARGE].cx = GetSystemMetrics( SM_CXICON );
+        sizes[SHIL_LARGE].cy = GetSystemMetrics( SM_CYICON );
+        sizes[SHIL_SMALL].cx = sizes[SHIL_LARGE].cx / 2;
+        sizes[SHIL_SMALL].cy = sizes[SHIL_LARGE].cy / 2;
+    }
+
+    sizes[SHIL_EXTRALARGE].cx = (GetSystemMetrics( SM_CXICON ) * 3) / 2;
+    sizes[SHIL_EXTRALARGE].cy = (GetSystemMetrics( SM_CYICON ) * 3) / 2;
+    sizes[SHIL_SYSSMALL].cx = GetSystemMetrics( SM_CXSMICON );
+    sizes[SHIL_SYSSMALL].cy = GetSystemMetrics( SM_CYSMICON );
+    sizes[SHIL_JUMBO].cx = sizes[SHIL_JUMBO].cy = 256;
+
+    TRACE("large %ldx%ld small %ldx%ld\n", sizes[SHIL_LARGE].cx, sizes[SHIL_LARGE].cy, sizes[SHIL_SMALL].cx, sizes[SHIL_SMALL].cy);
+
+    sic_hdpa = DPA_Create(16);
+    if (!sic_hdpa)
+        return(FALSE);
+
+    for (i = 0; i < ARRAY_SIZE(shell_imagelists); i++)
+    {
+        shell_imagelists[i] = ImageList_Create(sizes[i].cx, sizes[i].cy, ILC_COLOR32 | ILC_MASK, 0, 0x20);
+        ImageList_SetBkColor(shell_imagelists[i], CLR_NONE);
+
+        /* Load the generic file icon, which is used as the default if an icon isn't found. */
+        if (!(hicons[i] = LoadImageA(shell32_hInstance, MAKEINTRESOURCEA(IDI_SHELL_FILE),
+            IMAGE_ICON, sizes[i].cx, sizes[i].cy, LR_SHARED)))
         {
-            cx_large = cy_large = get_shell_icon_size();
-            cx_small = GetSystemMetrics( SM_CXSMICON );
-            cy_small = GetSystemMetrics( SM_CYSMICON );
-        }
-        else
-        {
-            cx_large = GetSystemMetrics( SM_CXICON );
-            cy_large = GetSystemMetrics( SM_CYICON );
-            cx_small = cx_large / 2;
-            cy_small = cy_large / 2;
+            failed = TRUE;
         }
+    }
 
-        TRACE("large %dx%d small %dx%d\n", cx_large, cy_large, cx_small, cx_small);
+    if (failed)
+    {
+        FIXME("Failed to load IDI_SHELL_FILE icon!\n");
+        return FALSE;
+    }
 
-	sic_hdpa = DPA_Create(16);
+    SIC_IconAppend(swShell32Name, IDI_SHELL_FILE - 1, hicons, 0);
+    SIC_IconAppend(swShell32Name, -IDI_SHELL_FILE, hicons, 0);
 
-	if (!sic_hdpa)
-	{
-	  return(FALSE);
-	}
+    TRACE("small list=%p, large list=%p\n", shell_imagelists[SHIL_SMALL], shell_imagelists[SHIL_LARGE]);
 
-        ShellSmallIconList = ImageList_Create(cx_small,cy_small,ILC_COLOR32|ILC_MASK,0,0x20);
-        ShellBigIconList = ImageList_Create(cx_large,cy_large,ILC_COLOR32|ILC_MASK,0,0x20);
-
-        ImageList_SetBkColor(ShellSmallIconList, CLR_NONE);
-        ImageList_SetBkColor(ShellBigIconList, CLR_NONE);
-
-        /* Load the document icon, which is used as the default if an icon isn't found. */
-        hSm = LoadImageA(shell32_hInstance, MAKEINTRESOURCEA(IDI_SHELL_FILE),
-                                IMAGE_ICON, cx_small, cy_small, LR_SHARED);
-        hLg = LoadImageA(shell32_hInstance, MAKEINTRESOURCEA(IDI_SHELL_FILE),
-                                IMAGE_ICON, cx_large, cy_large, LR_SHARED);
-
-        if (!hSm || !hLg) 
-        {
-          FIXME("Failed to load IDI_SHELL_FILE icon!\n");
-          return FALSE;
-        }
-
-        SIC_IconAppend (swShell32Name, IDI_SHELL_FILE-1, hSm, hLg, 0);
-        SIC_IconAppend (swShell32Name, -IDI_SHELL_FILE, hSm, hLg, 0);
-   
-	TRACE("hIconSmall=%p hIconBig=%p\n",ShellSmallIconList, ShellBigIconList);
-
-	return TRUE;
+    return TRUE;
 }
+
 /*************************************************************************
  * SIC_Destroy
  *
@@ -492,26 +492,29 @@ static BOOL WINAPI SIC_Initialize( INIT_ONCE *once, void *param, void **context
  */
 static INT CALLBACK sic_free( LPVOID ptr, LPVOID lparam )
 {
-	HeapFree(GetProcessHeap(), 0, ((LPSIC_ENTRY)ptr)->sSourceFile);
+	heap_free(((LPSIC_ENTRY)ptr)->sSourceFile);
 	SHFree(ptr);
 	return TRUE;
 }
 
 void SIC_Destroy(void)
 {
-	TRACE("\n");
+    unsigned int i;
 
-	EnterCriticalSection(&SHELL32_SicCS);
+    TRACE("\n");
 
-	if (sic_hdpa) DPA_DestroyCallback(sic_hdpa, sic_free, NULL );
+    EnterCriticalSection(&SHELL32_SicCS);
 
-	if (ShellSmallIconList)
-	    ImageList_Destroy(ShellSmallIconList);
-	if (ShellBigIconList)
-	    ImageList_Destroy(ShellBigIconList);
+    if (sic_hdpa) DPA_DestroyCallback(sic_hdpa, sic_free, NULL );
 
-	LeaveCriticalSection(&SHELL32_SicCS);
-	DeleteCriticalSection(&SHELL32_SicCS);
+    for (i = 0; i < ARRAY_SIZE(shell_imagelists); i++)
+    {
+        if (shell_imagelists[i])
+            ImageList_Destroy(shell_imagelists[i]);
+    }
+
+    LeaveCriticalSection(&SHELL32_SicCS);
+    DeleteCriticalSection(&SHELL32_SicCS);
 }
 
 /*****************************************************************************
@@ -569,41 +572,35 @@ INT SIC_GetIconIndex (LPCWSTR sSourceFile, INT dwSourceIndex, DWORD dwFlags )
  */
 static int SIC_LoadOverlayIcon(int icon_idx)
 {
-	WCHAR buffer[1024], wszIdx[8];
+	WCHAR buffer[1024], wszIdx[12];
 	HKEY hKeyShellIcons;
 	LPCWSTR iconPath;
 	int iconIdx;
 
-	static const WCHAR wszShellIcons[] = {
-	    'S','o','f','t','w','a','r','e','\\','M','i','c','r','o','s','o','f','t','\\',
-	    'W','i','n','d','o','w','s','\\','C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\',
-	    'E','x','p','l','o','r','e','r','\\','S','h','e','l','l',' ','I','c','o','n','s',0
-	}; 
-	static const WCHAR wszNumFmt[] = {'%','d',0};
-
 	iconPath = swShell32Name;	/* default: load icon from shell32.dll */
 	iconIdx = icon_idx;
 
-	if (RegOpenKeyExW(HKEY_LOCAL_MACHINE, wszShellIcons, 0, KEY_READ, &hKeyShellIcons) == ERROR_SUCCESS)
+	if (RegOpenKeyExW(HKEY_LOCAL_MACHINE, L"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Icons",
+                          0, KEY_READ, &hKeyShellIcons) == ERROR_SUCCESS)
 	{
 	    DWORD count = sizeof(buffer);
 
-	    sprintfW(wszIdx, wszNumFmt, icon_idx);
+	    swprintf(wszIdx, ARRAY_SIZE(wszIdx), L"%d", icon_idx);
 
 	    /* read icon path and index */
 	    if (RegQueryValueExW(hKeyShellIcons, wszIdx, NULL, NULL, (LPBYTE)buffer, &count) == ERROR_SUCCESS)
 	    {
-		LPWSTR p = strchrW(buffer, ',');
+		LPWSTR p = wcschr(buffer, ',');
 
 		if (!p)
 		{
-		    ERR("Icon index in %s/%s corrupted, no comma.\n", debugstr_w(wszShellIcons),debugstr_w(wszIdx));
+		    ERR("Icon index in Shell Icons/%s corrupted, no comma.\n", debugstr_w(wszIdx));
 		    RegCloseKey(hKeyShellIcons);
 		    return -1;
 		}
 		*p++ = 0;
 		iconPath = buffer;
-		iconIdx = atoiW(p);
+		iconIdx = wcstol(p, NULL, 10);
 	    }
 
 	    RegCloseKey(hKeyShellIcons);
@@ -621,14 +618,16 @@ static int SIC_LoadOverlayIcon(int icon_idx)
  *  imglist[1|2] [OUT] pointer which receives imagelist handles
  *
  */
-BOOL WINAPI Shell_GetImageLists(HIMAGELIST * lpBigList, HIMAGELIST * lpSmallList)
+BOOL WINAPI Shell_GetImageLists(HIMAGELIST *large_list, HIMAGELIST *small_list)
 {
-    TRACE("(%p,%p)\n",lpBigList,lpSmallList);
+    TRACE("(%p, %p)\n", large_list, small_list);
+
     InitOnceExecuteOnce( &sic_init_once, SIC_Initialize, NULL, NULL );
-    if (lpBigList) *lpBigList = ShellBigIconList;
-    if (lpSmallList) *lpSmallList = ShellSmallIconList;
+    if (large_list) *large_list = shell_imagelists[SHIL_LARGE];
+    if (small_list) *small_list = shell_imagelists[SHIL_SMALL];
     return TRUE;
 }
+
 /*************************************************************************
  * PidlToSicIndex			[INTERNAL]
  *
@@ -738,7 +737,7 @@ HRESULT WINAPI SHMapIDListToImageListIndexAsync(IUnknown *pts, IShellFolder *psf
  * Shell_GetCachedImageIndex		[SHELL32.72]
  *
  */
-static INT Shell_GetCachedImageIndexA(LPCSTR szPath, INT nIndex, BOOL bSimulateDoc)
+INT WINAPI Shell_GetCachedImageIndexA(LPCSTR szPath, INT nIndex, BOOL bSimulateDoc)
 {
 	INT ret, len;
 	LPWSTR szTemp;
@@ -746,17 +745,17 @@ static INT Shell_GetCachedImageIndexA(LPCSTR szPath, INT nIndex, BOOL bSimulateD
 	WARN("(%s,%08x,%08x) semi-stub.\n",debugstr_a(szPath), nIndex, bSimulateDoc);
 
 	len = MultiByteToWideChar( CP_ACP, 0, szPath, -1, NULL, 0 );
-	szTemp = HeapAlloc( GetProcessHeap(), 0, len * sizeof(WCHAR) );
+	szTemp = heap_alloc( len * sizeof(WCHAR) );
 	MultiByteToWideChar( CP_ACP, 0, szPath, -1, szTemp, len );
 
 	ret = SIC_GetIconIndex( szTemp, nIndex, 0 );
 
-	HeapFree( GetProcessHeap(), 0, szTemp );
+	heap_free( szTemp );
 
 	return ret;
 }
 
-static INT Shell_GetCachedImageIndexW(LPCWSTR szPath, INT nIndex, BOOL bSimulateDoc)
+INT WINAPI Shell_GetCachedImageIndexW(LPCWSTR szPath, INT nIndex, BOOL bSimulateDoc)
 {
 	WARN("(%s,%08x,%08x) semi-stub.\n",debugstr_w(szPath), nIndex, bSimulateDoc);
 
@@ -790,7 +789,7 @@ UINT WINAPI ExtractIconExA(LPCSTR lpszFile, INT nIconIndex, HICON * phiconLarge,
 {
     UINT ret = 0;
     INT len = MultiByteToWideChar(CP_ACP, 0, lpszFile, -1, NULL, 0);
-    LPWSTR lpwstrFile = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR));
+    LPWSTR lpwstrFile = heap_alloc( len * sizeof(WCHAR));
 
     TRACE("%s %i %p %p %i\n", lpszFile, nIconIndex, phiconLarge, phiconSmall, nIcons);
 
@@ -798,7 +797,7 @@ UINT WINAPI ExtractIconExA(LPCSTR lpszFile, INT nIconIndex, HICON * phiconLarge,
     {
         MultiByteToWideChar(CP_ACP, 0, lpszFile, -1, lpwstrFile, len);
         ret = ExtractIconExW(lpwstrFile, nIconIndex, phiconLarge, phiconSmall, nIcons);
-        HeapFree(GetProcessHeap(), 0, lpwstrFile);
+        heap_free(lpwstrFile);
     }
     return ret;
 }
@@ -818,7 +817,7 @@ HICON WINAPI ExtractAssociatedIconA(HINSTANCE hInst, LPSTR lpIconPath, LPWORD lp
      * lpIconPath itself is supposed to be large enough, so make sure lpIconPathW
      * is large enough too. Yes, I am puking too.
      */
-    LPWSTR lpIconPathW = HeapAlloc(GetProcessHeap(), 0, MAX_PATH * sizeof(WCHAR));
+    LPWSTR lpIconPathW = heap_alloc(MAX_PATH * sizeof(WCHAR));
 
     TRACE("%p %s %p\n", hInst, debugstr_a(lpIconPath), lpiIcon);
 
@@ -827,7 +826,7 @@ HICON WINAPI ExtractAssociatedIconA(HINSTANCE hInst, LPSTR lpIconPath, LPWORD lp
         MultiByteToWideChar(CP_ACP, 0, lpIconPath, -1, lpIconPathW, len);
         hIcon = ExtractAssociatedIconW(hInst, lpIconPathW, lpiIcon);
         WideCharToMultiByte(CP_ACP, 0, lpIconPathW, -1, lpIconPath, MAX_PATH , NULL, NULL);
-        HeapFree(GetProcessHeap(), 0, lpIconPathW);
+        heap_free(lpIconPathW);
     }
     return hIcon;
 }
@@ -864,7 +863,7 @@ HICON WINAPI ExtractAssociatedIconW(HINSTANCE hInst, LPWSTR lpIconPath, LPWORD l
       }
 
       if( hIcon == (HICON)1 )
-        *lpiIcon = 2;   /* MSDOS icon - we found .exe but no icons in it */
+        *lpiIcon = 2;   /* MS-DOS icon - we found .exe but no icons in it */
       else
         *lpiIcon = 6;   /* generic icon - found nothing */
 
@@ -896,13 +895,13 @@ HICON WINAPI ExtractAssociatedIconExA(HINSTANCE hInst, LPSTR lpIconPath, LPWORD
 {
   HICON ret;
   INT len = MultiByteToWideChar( CP_ACP, 0, lpIconPath, -1, NULL, 0 );
-  LPWSTR lpwstrFile = HeapAlloc( GetProcessHeap(), 0, len * sizeof(WCHAR) );
+  LPWSTR lpwstrFile = heap_alloc( len * sizeof(WCHAR) );
 
   TRACE("%p %s %p %p)\n", hInst, lpIconPath, lpiIconIdx, lpiIconId);
 
   MultiByteToWideChar( CP_ACP, 0, lpIconPath, -1, lpwstrFile, len );
   ret = ExtractAssociatedIconExW(hInst, lpwstrFile, lpiIconIdx, lpiIconId);
-  HeapFree(GetProcessHeap(), 0, lpwstrFile);
+  heap_free(lpwstrFile);
   return ret;
 }
 
@@ -943,13 +942,13 @@ HRESULT WINAPI SHDefExtractIconA(LPCSTR pszIconFile, int iIndex, UINT uFlags,
 {
   HRESULT ret;
   INT len = MultiByteToWideChar(CP_ACP, 0, pszIconFile, -1, NULL, 0);
-  LPWSTR lpwstrFile = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR));
+  LPWSTR lpwstrFile = heap_alloc(len * sizeof(WCHAR));
 
   TRACE("%s %d 0x%08x %p %p %d\n", pszIconFile, iIndex, uFlags, phiconLarge, phiconSmall, nIconSize);
 
   MultiByteToWideChar(CP_ACP, 0, pszIconFile, -1, lpwstrFile, len);
   ret = SHDefExtractIconW(lpwstrFile, iIndex, uFlags, phiconLarge, phiconSmall, nIconSize);
-  HeapFree(GetProcessHeap(), 0, lpwstrFile);
+  heap_free(lpwstrFile);
   return ret;
 }
 
@@ -995,8 +994,6 @@ INT WINAPI SHGetIconOverlayIndexW(LPCWSTR pszIconPath, INT iIconIndex)
  */
 HRESULT WINAPI SHGetStockIconInfo(SHSTOCKICONID id, UINT flags, SHSTOCKICONINFO *sii)
 {
-    static const WCHAR shell32dll[] = {'\\','s','h','e','l','l','3','2','.','d','l','l',0};
-
     FIXME("(%d, 0x%x, %p) semi-stub\n", id, flags, sii);
     if ((id < 0) || (id >= SIID_MAX_ICONS) || !sii || (sii->cbSize != sizeof(SHSTOCKICONINFO))) {
         return E_INVALIDARG;
@@ -1006,7 +1003,7 @@ HRESULT WINAPI SHGetStockIconInfo(SHSTOCKICONID id, UINT flags, SHSTOCKICONINFO
 
     /* no icons defined: use default */
     sii->iIcon = -IDI_SHELL_FILE;
-    lstrcatW(sii->szPath, shell32dll);
+    lstrcatW(sii->szPath, L"\\shell32.dll");
 
     if (flags)
         FIXME("flags 0x%x not implemented\n", flags);
@@ -1018,3 +1015,24 @@ HRESULT WINAPI SHGetStockIconInfo(SHSTOCKICONID id, UINT flags, SHSTOCKICONINFO
 
     return S_OK;
 }
+
+/*************************************************************************
+ *              SHGetImageList (SHELL32.727)
+ *
+ * Returns a copy of a shell image list.
+ *
+ * NOTES
+ *   Windows XP features 4 sizes of image list, and Vista 5. Wine currently
+ *   only supports the traditional small and large image lists, so requests
+ *   for the others will currently fail.
+ */
+HRESULT WINAPI SHGetImageList(int iImageList, REFIID riid, void **ppv)
+{
+    TRACE("(%d, %s, %p)\n", iImageList, debugstr_guid(riid), ppv);
+
+    if (iImageList < 0 || iImageList > SHIL_LAST)
+        return E_FAIL;
+
+    InitOnceExecuteOnce( &sic_init_once, SIC_Initialize, NULL, NULL );
+    return HIMAGELIST_QueryInterface(shell_imagelists[iImageList], riid, ppv);
+}
