@@ -20,9 +20,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
@@ -41,8 +38,8 @@
 #include "ole2.h"
 #include "shlguid.h"
 
+#include "wininet.h"
 #include "pidl.h"
-#include "undocshell.h"
 #include "shell32_main.h"
 #include "shresdef.h"
 #include "shlwapi.h"
@@ -154,7 +151,7 @@ static HRESULT WINAPI ISF_Desktop_fnParseDisplayName (IShellFolder2 * iface,
                 DWORD * pchEaten, LPITEMIDLIST * ppidl, DWORD * pdwAttributes)
 {
     IDesktopFolderImpl *This = impl_from_IShellFolder2(iface);
-    WCHAR szElement[MAX_PATH];
+    WCHAR c, szElement[MAX_PATH];
     LPCWSTR szNext = NULL;
     LPITEMIDLIST pidlTemp = NULL;
     PARSEDURLW urldata;
@@ -182,13 +179,16 @@ static HRESULT WINAPI ISF_Desktop_fnParseDisplayName (IShellFolder2 * iface,
         SHCLSIDFromStringW (szElement + 2, &clsid);
         pidlTemp = _ILCreateGuid (PT_GUID, &clsid);
     }
-    else if (PathGetDriveNumberW (lpszDisplayName) >= 0)
+    /* we can't use PathGetDriveNumberW because we can't have the \\?\ prefix */
+    else if ((c = towupper(lpszDisplayName[0])) >= 'A' && c <= 'Z' && lpszDisplayName[1] == ':')
     {
         /* it's a filesystem path with a drive. Let MyComputer/UnixDosFolder parse it */
-        if (UNIXFS_is_rooted_at_desktop()) 
-            pidlTemp = _ILCreateGuid(PT_GUID, &CLSID_UnixDosFolder);
-        else
-            pidlTemp = _ILCreateMyComputer ();
+        pidlTemp = _ILCreateMyComputer ();
+        szNext = lpszDisplayName;
+    }
+    else if (!wcsncmp( lpszDisplayName, L"\\\\?\\unix\\", 9 ))
+    {
+        pidlTemp = _ILCreateGuid(PT_GUID, &CLSID_UnixDosFolder);
         szNext = lpszDisplayName;
     }
     else if (PathIsUNCW(lpszDisplayName))
@@ -272,26 +272,18 @@ static HRESULT WINAPI ISF_Desktop_fnParseDisplayName (IShellFolder2 * iface,
 
     *ppidl = pidlTemp;
 
-    TRACE ("(%p)->(-- ret=0x%08x)\n", This, hr);
+    TRACE ("(%p)->(-- ret=0x%08lx)\n", This, hr);
 
     return hr;
 }
 
 static void add_shell_namespace_extensions(IEnumIDListImpl *list, HKEY root)
 {
-    static const WCHAR Desktop_NameSpaceW[] = { 'S','O','F','T','W','A','R','E','\\',
-        'M','i','c','r','o','s','o','f','t','\\','W','i','n','d','o','w','s','\\',
-        'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\',
-        'E','x','p','l','o','r','e','r','\\','D','e','s','k','t','o','p','\\',
-        'N','a','m','e','s','p','a','c','e','\0' };
-    static const WCHAR clsidfmtW[] = {'C','L','S','I','D','\\','%','s','\\',
-        'S','h','e','l','l','F','o','l','d','e','r',0};
-    static const WCHAR attributesW[] = {'A','t','t','r','i','b','u','t','e','s',0};
-    WCHAR guid[39], clsidkeyW[ARRAY_SIZE(clsidfmtW) + 39];
+    WCHAR guid[39], clsidkeyW[60];
     DWORD size, i = 0;
     HKEY hkey;
 
-    if (RegOpenKeyExW(root, Desktop_NameSpaceW, 0, KEY_READ, &hkey))
+    if (RegOpenKeyExW(root, L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Desktop\\Namespace", 0, KEY_READ, &hkey))
         return;
 
     size = ARRAY_SIZE(guid);
@@ -300,8 +292,8 @@ static void add_shell_namespace_extensions(IEnumIDListImpl *list, HKEY root)
         DWORD attributes, value_size = sizeof(attributes);
 
         /* Check if extension is configured as nonenumerable */
-        sprintfW(clsidkeyW, clsidfmtW, guid);
-        RegGetValueW(HKEY_CLASSES_ROOT, clsidkeyW, attributesW, RRF_RT_REG_DWORD | RRF_ZEROONFAILURE,
+        swprintf(clsidkeyW, ARRAY_SIZE(clsidkeyW), L"CLSID\\%s\\ShellFolder", guid);
+        RegGetValueW(HKEY_CLASSES_ROOT, clsidkeyW, L"Attributes", RRF_RT_REG_DWORD | RRF_ZEROONFAILURE,
             NULL, &attributes, &value_size);
 
         if (!(attributes & SFGAO_NONENUMERATED))
@@ -320,7 +312,7 @@ static BOOL CreateDesktopEnumList(IEnumIDListImpl *list, DWORD dwFlags)
     BOOL ret = TRUE;
     WCHAR szPath[MAX_PATH];
 
-    TRACE("(%p)->(flags=0x%08x)\n", list, dwFlags);
+    TRACE("(%p)->(flags=0x%08lx)\n", list, dwFlags);
 
     /* enumerate the root folders */
     if (dwFlags & SHCONTF_FOLDERS)
@@ -346,7 +338,7 @@ static HRESULT WINAPI ISF_Desktop_fnEnumObjects (IShellFolder2 * iface,
     IDesktopFolderImpl *This = impl_from_IShellFolder2(iface);
     IEnumIDListImpl *list;
 
-    TRACE ("(%p)->(HWND=%p flags=0x%08x pplist=%p)\n",
+    TRACE ("(%p)->(HWND=%p flags=0x%08lx pplist=%p)\n",
            This, hwndOwner, dwFlags, ppEnumIDList);
 
     if (!(list = IEnumIDList_Constructor()))
@@ -370,7 +362,7 @@ static HRESULT WINAPI ISF_Desktop_fnBindToObject (IShellFolder2 * iface,
     TRACE ("(%p)->(pidl=%p,%p,%s,%p)\n",
            This, pidl, pbcReserved, shdebugstr_guid (riid), ppvOut);
 
-    return SHELL32_BindToChild( This->pidlRoot, This->sPathTarget, pidl, riid, ppvOut );
+    return SHELL32_BindToChild( This->pidlRoot, &CLSID_ShellFSFolder, This->sPathTarget, pidl, riid, ppvOut );
 }
 
 /**************************************************************************
@@ -397,9 +389,9 @@ static HRESULT WINAPI ISF_Desktop_fnCompareIDs (IShellFolder2 *iface,
     IDesktopFolderImpl *This = impl_from_IShellFolder2(iface);
     HRESULT hr;
 
-    TRACE ("(%p)->(0x%08lx,pidl1=%p,pidl2=%p)\n", This, lParam, pidl1, pidl2);
+    TRACE ("(%p)->(0x%08Ix,pidl1=%p,pidl2=%p)\n", This, lParam, pidl1, pidl2);
     hr = SHELL32_CompareIDs(iface, lParam, pidl1, pidl2);
-    TRACE ("-- 0x%08x\n", hr);
+    TRACE ("-- 0x%08lx\n", hr);
     return hr;
 }
 
@@ -428,8 +420,7 @@ static HRESULT WINAPI ISF_Desktop_fnCreateViewObject (IShellFolder2 * iface,
     }
     else if (IsEqualIID (riid, &IID_IContextMenu))
     {
-        WARN ("IContextMenu not implemented\n");
-        hr = E_NOTIMPL;
+        hr = BackgroundMenu_Constructor((IShellFolder*)iface, TRUE, riid, ppvOut);
     }
     else if (IsEqualIID (riid, &IID_IShellView))
     {
@@ -459,7 +450,7 @@ static HRESULT WINAPI ISF_Desktop_fnGetAttributesOf (IShellFolder2 * iface,
         SFGAO_CANRENAME | SFGAO_CANDELETE | SFGAO_HASPROPSHEET |
         SFGAO_DROPTARGET | SFGAO_FILESYSANCESTOR | SFGAO_FOLDER | SFGAO_HASSUBFOLDER;
 
-    TRACE ("(%p)->(cidl=%d apidl=%p mask=%p (0x%08x))\n",
+    TRACE ("(%p)->(cidl=%d apidl=%p mask=%p (0x%08lx))\n",
            This, cidl, apidl, rgfInOut, rgfInOut ? *rgfInOut : 0);
 
     if (!rgfInOut)
@@ -489,7 +480,7 @@ static HRESULT WINAPI ISF_Desktop_fnGetAttributesOf (IShellFolder2 * iface,
     /* make sure SFGAO_VALIDATE is cleared, some apps depend on that */
     *rgfInOut &= ~SFGAO_VALIDATE;
 
-    TRACE ("-- result=0x%08x\n", *rgfInOut);
+    TRACE ("-- result=0x%08lx\n", *rgfInOut);
 
     return S_OK;
 }
@@ -570,7 +561,7 @@ static HRESULT WINAPI ISF_Desktop_fnGetUIObjectOf (IShellFolder2 * iface,
         hr = E_OUTOFMEMORY;
 
     *ppvOut = pObj;
-    TRACE ("(%p)->hr=0x%08x\n", This, hr);
+    TRACE ("(%p)->hr=0x%08lx\n", This, hr);
     return hr;
 }
 
@@ -587,7 +578,7 @@ static HRESULT WINAPI ISF_Desktop_fnGetDisplayNameOf (IShellFolder2 * iface,
     HRESULT hr = S_OK;
     LPWSTR pszPath;
 
-    TRACE ("(%p)->(pidl=%p,0x%08x,%p)\n", This, pidl, dwFlags, strRet);
+    TRACE ("(%p)->(pidl=%p,0x%08lx,%p)\n", This, pidl, dwFlags, strRet);
     pdump (pidl);
 
     if (!strRet)
@@ -601,7 +592,7 @@ static HRESULT WINAPI ISF_Desktop_fnGetDisplayNameOf (IShellFolder2 * iface,
     {
         if ((GET_SHGDN_RELATION (dwFlags) == SHGDN_NORMAL) &&
             (GET_SHGDN_FOR (dwFlags) & SHGDN_FORPARSING))
-            strcpyW(pszPath, This->sPathTarget);
+            lstrcpyW(pszPath, This->sPathTarget);
         else
             HCR_GetClassNameW(&CLSID_ShellDesktop, pszPath, MAX_PATH);
     }
@@ -629,21 +620,13 @@ static HRESULT WINAPI ISF_Desktop_fnGetDisplayNameOf (IShellFolder2 * iface,
                 else
                 {
                     /* get the "WantsFORPARSING" flag from the registry */
-                    static const WCHAR clsidW[] =
-                     { 'C','L','S','I','D','\\',0 };
-                    static const WCHAR shellfolderW[] =
-                     { '\\','s','h','e','l','l','f','o','l','d','e','r',0 };
-                    static const WCHAR wantsForParsingW[] =
-                     { 'W','a','n','t','s','F','o','r','P','a','r','s','i','n',
-                     'g',0 };
                     WCHAR szRegPath[100];
                     LONG r;
 
-                    lstrcpyW (szRegPath, clsidW);
+                    lstrcpyW (szRegPath, L"CLSID\\");
                     SHELL32_GUIDToStringW (clsid, &szRegPath[6]);
-                    lstrcatW (szRegPath, shellfolderW);
-                    r = SHGetValueW(HKEY_CLASSES_ROOT, szRegPath,
-                                    wantsForParsingW, NULL, NULL, NULL);
+                    lstrcatW (szRegPath, L"\\shellfolder");
+                    r = SHGetValueW(HKEY_CLASSES_ROOT, szRegPath, L"WantsForParsing", NULL, NULL, NULL);
                     if (r == ERROR_SUCCESS)
                         bWantsForParsing = TRUE;
                     else
@@ -721,7 +704,7 @@ static HRESULT WINAPI ISF_Desktop_fnGetDisplayNameOf (IShellFolder2 * iface,
     else
         CoTaskMemFree(pszPath);
 
-    TRACE ("-- (%p)->(%s,0x%08x)\n", This,
+    TRACE ("-- (%p)->(%s,0x%08lx)\n", This,
     strRet->uType == STRRET_CSTR ? strRet->u.cStr :
     debugstr_w(strRet->u.pOleStr), hr);
     return hr;
@@ -745,7 +728,7 @@ static HRESULT WINAPI ISF_Desktop_fnSetNameOf (IShellFolder2 * iface,
 {
     IDesktopFolderImpl *This = impl_from_IShellFolder2(iface);
 
-    FIXME ("(%p)->(%p,pidl=%p,%s,%u,%p) stub\n", This, hwndOwner, pidl,
+    FIXME ("(%p)->(%p,pidl=%p,%s,%lu,%p) stub\n", This, hwndOwner, pidl,
            debugstr_w (lpName), dwFlags, pPidlOut);
 
     return E_FAIL;
@@ -770,7 +753,7 @@ static HRESULT WINAPI ISF_Desktop_fnGetDefaultColumn(IShellFolder2 *iface, DWORD
 {
     IDesktopFolderImpl *This = impl_from_IShellFolder2(iface);
 
-    TRACE ("(%p)->(%#x, %p, %p)\n", This, reserved, sort, display);
+    TRACE ("(%p)->(%#lx, %p, %p)\n", This, reserved, sort, display);
 
     return E_NOTIMPL;
 }
@@ -803,8 +786,6 @@ static HRESULT WINAPI ISF_Desktop_fnGetDetailsOf (IShellFolder2 * iface,
 {
     IDesktopFolderImpl *This = impl_from_IShellFolder2(iface);
 
-    HRESULT hr = S_OK;
-
     TRACE ("(%p)->(%p %i %p)\n", This, pidl, iColumn, psd);
 
     if (!psd || iColumn >= ARRAY_SIZE(desktop_header))
@@ -813,29 +794,7 @@ static HRESULT WINAPI ISF_Desktop_fnGetDetailsOf (IShellFolder2 * iface,
     if (!pidl)
         return SHELL32_GetColumnDetails(desktop_header, iColumn, psd);
 
-    /* the data from the pidl */
-    psd->str.uType = STRRET_CSTR;
-    switch (iColumn)
-    {
-    case 0:        /* name */
-        hr = IShellFolder2_GetDisplayNameOf(iface, pidl,
-                   SHGDN_NORMAL | SHGDN_INFOLDER, &psd->str);
-        break;
-    case 1:        /* size */
-        _ILGetFileSize (pidl, psd->str.u.cStr, MAX_PATH);
-        break;
-    case 2:        /* type */
-        _ILGetFileType (pidl, psd->str.u.cStr, MAX_PATH);
-        break;
-    case 3:        /* date */
-        _ILGetFileDate (pidl, psd->str.u.cStr, MAX_PATH);
-        break;
-    case 4:        /* attributes */
-        _ILGetFileAttributes (pidl, psd->str.u.cStr, MAX_PATH);
-        break;
-    }
-
-    return hr;
+    return shellfolder_get_file_details( iface, pidl, desktop_header, iColumn, psd );
 }
 
 static HRESULT WINAPI ISF_Desktop_fnMapColumnToSCID(IShellFolder2 *iface, UINT column, SHCOLUMNID *scid)
@@ -957,7 +916,7 @@ HRESULT WINAPI ISF_Desktop_Constructor (
     {
         IDesktopFolderImpl *sf;
 
-        if (!SHGetSpecialFolderPathW( 0, szMyPath, CSIDL_DESKTOPDIRECTORY, TRUE ))
+        if (!SHGetSpecialFolderPathW( 0, szMyPath, CSIDL_DESKTOPDIRECTORY, FALSE ))
             return E_UNEXPECTED;
 
         sf = LocalAlloc( LMEM_ZEROINIT, sizeof (IDesktopFolderImpl) );
@@ -982,3 +941,211 @@ HRESULT WINAPI ISF_Desktop_Constructor (
 
     return IShellFolder2_QueryInterface( &cached_sf->IShellFolder2_iface, riid, ppv );
 }
+
+static HRESULT WINAPI active_desktop_QueryInterface(IActiveDesktop *iface, REFIID riid, void **obj)
+{
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(riid), obj);
+
+    *obj = NULL;
+
+    if (IsEqualIID(riid, &IID_IActiveDesktop)
+            || IsEqualIID(riid, &IID_IUnknown))
+    {
+        *obj = iface;
+    }
+
+    if (*obj)
+    {
+        IUnknown_AddRef((IUnknown *)*obj);
+        return S_OK;
+    }
+
+    WARN("Unsupported interface %s.\n", debugstr_guid(riid));
+    *obj = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI active_desktop_AddRef(IActiveDesktop *iface)
+{
+    return 2;
+}
+
+static ULONG WINAPI active_desktop_Release(IActiveDesktop *iface)
+{
+    return 1;
+}
+
+static HRESULT WINAPI active_desktop_ApplyChanges(IActiveDesktop *iface, DWORD flags)
+{
+    FIXME("%p, %#lx.\n", iface, flags);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_GetWallpaper(IActiveDesktop *iface, PWSTR wallpaper, UINT length, DWORD flags)
+{
+    FIXME("%p, %p, %u, %#lx.\n", iface, wallpaper, length, flags);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_SetWallpaper(IActiveDesktop *iface, PCWSTR wallpaper, DWORD reserved)
+{
+    FIXME("%p, %s, %#lx.\n", iface, debugstr_w(wallpaper), reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_GetWallpaperOptions(IActiveDesktop *iface, LPWALLPAPEROPT options, DWORD reserved)
+{
+    FIXME("%p, %p, %#lx.\n", iface, options, reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_SetWallpaperOptions(IActiveDesktop *iface, LPCWALLPAPEROPT options, DWORD reserved)
+{
+    FIXME("%p, %p, %#lx.\n", iface, options, reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_GetPattern(IActiveDesktop *iface, PWSTR pattern, UINT length, DWORD reserved)
+{
+    FIXME("%p, %p, %u, %#lx.\n", iface, pattern, length, reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_SetPattern(IActiveDesktop *iface, PCWSTR pattern, DWORD reserved)
+{
+    FIXME("%p, %s, %#lx.\n", iface, debugstr_w(pattern), reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_GetDesktopItemOptions(IActiveDesktop *iface, LPCOMPONENTSOPT options, DWORD reserved)
+{
+    FIXME("%p, %p, %#lx.\n", iface, options, reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_SetDesktopItemOptions(IActiveDesktop *iface, LPCCOMPONENTSOPT options, DWORD reserved)
+{
+    FIXME("%p, %p, %#lx.\n", iface, options, reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_AddDesktopItem(IActiveDesktop *iface, LPCCOMPONENT component, DWORD reserved)
+{
+    FIXME("%p, %p, %#lx.\n", iface, component, reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_AddDesktopItemWithUI(IActiveDesktop *iface, HWND hwnd, LPCOMPONENT component, DWORD reserved)
+{
+    FIXME("%p, %p, %p, %#lx.\n", iface, hwnd, component, reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_ModifyDesktopItem(IActiveDesktop *iface, LPCCOMPONENT component, DWORD flags)
+{
+    FIXME("%p, %p, %#lx.\n", iface, component, flags);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_RemoveDesktopItem(IActiveDesktop *iface, LPCCOMPONENT component, DWORD reserved)
+{
+    FIXME("%p, %p, %#lx.\n", iface, component, reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_GetDesktopItemCount(IActiveDesktop *iface, int *count, DWORD reserved)
+{
+    FIXME("%p, %p, %#lx.\n", iface, count, reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_GetDesktopItem(IActiveDesktop *iface, int index, LPCOMPONENT component, DWORD reserved)
+{
+    FIXME("%p, %d, %p, %#lx.\n", iface, index, component, reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_GetDesktopItemByID(IActiveDesktop *iface, ULONG_PTR id, LPCOMPONENT component, DWORD reserved)
+{
+    FIXME("%p, %Ix, %p, %#lx.\n", iface, id, component, reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_GenerateDesktopItemHtml(IActiveDesktop *iface, PCWSTR filename, LPCOMPONENT component, DWORD reserved)
+{
+    FIXME("%p, %s, %p, %#lx.\n", iface, debugstr_w(filename), component, reserved);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_AddUrl(IActiveDesktop *iface, HWND hwnd, PCWSTR source, LPCOMPONENT component, DWORD flags)
+{
+    FIXME("%p, %p, %s, %p, %#lx.\n", iface, hwnd, debugstr_w(source), component, flags);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI active_desktop_GetDesktopItemBySource(IActiveDesktop *iface, PCWSTR source, LPCOMPONENT component, DWORD reserved)
+{
+    FIXME("%p, %s, %p, %#lx.\n", iface, debugstr_w(source), component, reserved);
+
+    return E_NOTIMPL;
+}
+
+static const IActiveDesktopVtbl active_desktop_vtbl =
+{
+    active_desktop_QueryInterface,
+    active_desktop_AddRef,
+    active_desktop_Release,
+    active_desktop_ApplyChanges,
+    active_desktop_GetWallpaper,
+    active_desktop_SetWallpaper,
+    active_desktop_GetWallpaperOptions,
+    active_desktop_SetWallpaperOptions,
+    active_desktop_GetPattern,
+    active_desktop_SetPattern,
+    active_desktop_GetDesktopItemOptions,
+    active_desktop_SetDesktopItemOptions,
+    active_desktop_AddDesktopItem,
+    active_desktop_AddDesktopItemWithUI,
+    active_desktop_ModifyDesktopItem,
+    active_desktop_RemoveDesktopItem,
+    active_desktop_GetDesktopItemCount,
+    active_desktop_GetDesktopItem,
+    active_desktop_GetDesktopItemByID,
+    active_desktop_GenerateDesktopItemHtml,
+    active_desktop_AddUrl,
+    active_desktop_GetDesktopItemBySource,
+};
+
+HRESULT WINAPI ActiveDesktop_Constructor(IUnknown *outer, REFIID riid, void **obj)
+{
+    static IActiveDesktop object = { &active_desktop_vtbl };
+    HRESULT hr;
+
+    TRACE("%p, %s, %p.\n", outer, debugstr_guid(riid), obj);
+
+    if (outer)
+        return CLASS_E_NOAGGREGATION;
+
+    hr = IUnknown_QueryInterface(&object, riid, obj);
+    IUnknown_Release(&object);
+
+    return hr;
+}
