@@ -19,9 +19,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
@@ -38,7 +35,6 @@
 #include "wingdi.h"
 #include "pidl.h"
 #include "shlguid.h"
-#include "undocshell.h"
 #include "shell32_main.h"
 #include "shresdef.h"
 #include "shlwapi.h"
@@ -162,7 +158,7 @@ static ULONG WINAPI ISF_MyComputer_fnAddRef (IShellFolder2 * iface)
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
     ULONG refCount = InterlockedIncrement(&This->ref);
 
-    TRACE ("(%p)->(count=%u)\n", This, refCount - 1);
+    TRACE ("(%p)->(count=%lu)\n", This, refCount - 1);
 
     return refCount;
 }
@@ -172,7 +168,7 @@ static ULONG WINAPI ISF_MyComputer_fnRelease (IShellFolder2 * iface)
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
     ULONG refCount = InterlockedDecrement(&This->ref);
 
-    TRACE ("(%p)->(count=%u)\n", This, refCount + 1);
+    TRACE ("(%p)->(count=%lu)\n", This, refCount + 1);
 
     if (!refCount)
     {
@@ -193,7 +189,7 @@ static HRESULT WINAPI ISF_MyComputer_fnParseDisplayName (IShellFolder2 *iface,
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
     HRESULT hr = E_INVALIDARG;
     LPCWSTR szNext = NULL;
-    WCHAR szElement[MAX_PATH];
+    WCHAR c, szElement[MAX_PATH];
     LPITEMIDLIST pidlTemp = NULL;
     CLSID clsid;
 
@@ -213,13 +209,17 @@ static HRESULT WINAPI ISF_MyComputer_fnParseDisplayName (IShellFolder2 *iface,
         SHCLSIDFromStringW (szElement + 2, &clsid);
         pidlTemp = _ILCreateGuid (PT_GUID, &clsid);
     }
-    /* do we have an absolute path name ? */
-    else if (PathGetDriveNumberW (lpszDisplayName) >= 0 &&
-              lpszDisplayName[2] == (WCHAR) '\\')
+    /* we can't use PathGetDriveNumberW because we can't have the \\?\ prefix */
+    else if ((c = towupper(lpszDisplayName[0])) >= 'A' && c <= 'Z' &&
+             lpszDisplayName[1] == ':' &&
+             (lpszDisplayName[2] == '\\' || lpszDisplayName[2] == '\0'))
     {
-        szNext = GetNextElementW (lpszDisplayName, szElement, MAX_PATH);
-        /* make drive letter uppercase to enable PIDL comparison */
-        szElement[0] = toupper(szElement[0]);
+        /* drive letter has to be uppercase to enable PIDL comparison */
+        szElement[0] = c;
+        szElement[1] = ':';
+        szElement[2] = '\\';
+        szElement[3] = '\0';
+        szNext = &lpszDisplayName[2] + (lpszDisplayName[2] == '\\');
         pidlTemp = _ILCreateDrive (szElement);
     }
 
@@ -228,7 +228,7 @@ static HRESULT WINAPI ISF_MyComputer_fnParseDisplayName (IShellFolder2 *iface,
         hr = SHELL32_ParseNextElement (iface, hwndOwner, pbc, &pidlTemp,
                               (LPOLESTR) szNext, pchEaten, pdwAttributes);
     }
-    else
+    else if (pidlTemp)
     {
         if (pdwAttributes && *pdwAttributes)
             SHELL32_GetItemAttributes (&This->IShellFolder2_iface, pidlTemp, pdwAttributes);
@@ -237,7 +237,7 @@ static HRESULT WINAPI ISF_MyComputer_fnParseDisplayName (IShellFolder2 *iface,
 
     *ppidl = pidlTemp;
 
-    TRACE ("(%p)->(-- ret=0x%08x)\n", This, hr);
+    TRACE ("(%p)->(-- ret=0x%08lx)\n", This, hr);
 
     return hr;
 }
@@ -245,13 +245,6 @@ static HRESULT WINAPI ISF_MyComputer_fnParseDisplayName (IShellFolder2 *iface,
 /* retrieve a map of drives that should be displayed */
 static DWORD get_drive_map(void)
 {
-    static const WCHAR policiesW[] = {'S','o','f','t','w','a','r','e','\\',
-                                      'M','i','c','r','o','s','o','f','t','\\',
-                                      'W','i','n','d','o','w','s','\\',
-                                      'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\',
-                                      'P','o','l','i','c','i','e','s','\\',
-                                      'E','x','p','l','o','r','e','r',0};
-    static const WCHAR nodrivesW[] = {'N','o','D','r','i','v','e','s',0};
     static DWORD drive_mask;
     static BOOL init_done = FALSE;
 
@@ -260,17 +253,17 @@ static DWORD get_drive_map(void)
         DWORD type, size, data, mask = 0;
         HKEY hkey;
 
-        if (!RegOpenKeyW( HKEY_LOCAL_MACHINE, policiesW, &hkey ))
+        if (!RegOpenKeyW( HKEY_LOCAL_MACHINE, L"Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer", &hkey ))
         {
             size = sizeof(data);
-            if (!RegQueryValueExW( hkey, nodrivesW, NULL, &type, (LPBYTE)&data, &size ) && type == REG_DWORD)
+            if (!RegQueryValueExW( hkey, L"NoDrives", NULL, &type, (LPBYTE)&data, &size ) && type == REG_DWORD)
                 mask |= data;
             RegCloseKey( hkey );
         }
-        if (!RegOpenKeyW( HKEY_CURRENT_USER, policiesW, &hkey ))
+        if (!RegOpenKeyW( HKEY_CURRENT_USER, L"Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer", &hkey ))
         {
             size = sizeof(data);
-            if (!RegQueryValueExW( hkey, nodrivesW, NULL, &type, (LPBYTE)&data, &size ) && type == REG_DWORD)
+            if (!RegQueryValueExW( hkey, L"NoDrives", NULL, &type, (LPBYTE)&data, &size ) && type == REG_DWORD)
                 mask |= data;
             RegCloseKey( hkey );
         }
@@ -284,22 +277,16 @@ static DWORD get_drive_map(void)
 /**************************************************************************
  *  CreateMyCompEnumList()
  */
-static const WCHAR MyComputer_NameSpaceW[] = { 'S','O','F','T','W','A','R','E',
- '\\','M','i','c','r','o','s','o','f','t','\\','W','i','n','d','o','w','s','\\',
- 'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\','E','x','p','l',
- 'o','r','e','r','\\','M','y','C','o','m','p','u','t','e','r','\\','N','a','m',
- 'e','s','p','a','c','e','\0' };
-
 static BOOL CreateMyCompEnumList(IEnumIDListImpl *list, DWORD dwFlags)
 {
     BOOL ret = TRUE;
 
-    TRACE("(%p)->(flags=0x%08x)\n", list, dwFlags);
+    TRACE("(%p)->(flags=0x%08lx)\n", list, dwFlags);
 
     /* enumerate the folders */
     if (dwFlags & SHCONTF_FOLDERS)
     {
-        WCHAR wszDriveName[] = {'A', ':', '\\', '\0'};
+        WCHAR wszDriveName[] = L"A:\\";
         DWORD dwDrivemap = get_drive_map();
         HKEY hkey;
         UINT i;
@@ -315,7 +302,8 @@ static BOOL CreateMyCompEnumList(IEnumIDListImpl *list, DWORD dwFlags)
         TRACE("-- (%p)-> enumerate (mycomputer shell extensions)\n",list);
         for (i=0; i<2; i++) {
             if (ret && !RegOpenKeyExW(i == 0 ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER,
-                                      MyComputer_NameSpaceW, 0, KEY_READ, &hkey))
+                                      L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\Namespace",
+                                      0, KEY_READ, &hkey))
             {
                 WCHAR iid[50];
                 int i=0;
@@ -355,7 +343,7 @@ static HRESULT WINAPI ISF_MyComputer_fnEnumObjects (IShellFolder2 *iface,
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
     IEnumIDListImpl *list;
 
-    TRACE("(%p)->(HWND=%p flags=0x%08x pplist=%p)\n", This,
+    TRACE("(%p)->(HWND=%p flags=0x%08lx pplist=%p)\n", This,
           hwndOwner, dwFlags, ppEnumIDList);
 
     if (!(list = IEnumIDList_Constructor()))
@@ -379,7 +367,7 @@ static HRESULT WINAPI ISF_MyComputer_fnBindToObject (IShellFolder2 *iface,
     TRACE("(%p)->(pidl=%p,%p,%s,%p)\n", This,
           pidl, pbcReserved, shdebugstr_guid (riid), ppvOut);
 
-    return SHELL32_BindToChild (This->pidlRoot, NULL, pidl, riid, ppvOut);
+    return SHELL32_BindToChild (This->pidlRoot, &CLSID_ShellFSFolder, NULL, pidl, riid, ppvOut);
 }
 
 /**************************************************************************
@@ -407,9 +395,9 @@ static HRESULT WINAPI ISF_MyComputer_fnCompareIDs (IShellFolder2 *iface,
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
     HRESULT hr;
 
-    TRACE ("(%p)->(0x%08lx,pidl1=%p,pidl2=%p)\n", This, lParam, pidl1, pidl2);
+    TRACE ("(%p)->(0x%08Ix,pidl1=%p,pidl2=%p)\n", This, lParam, pidl1, pidl2);
     hr = SHELL32_CompareIDs(&This->IShellFolder2_iface, lParam, pidl1, pidl2);
-    TRACE ("-- 0x%08x\n", hr);
+    TRACE ("-- 0x%08lx\n", hr);
     return hr;
 }
 
@@ -438,8 +426,7 @@ static HRESULT WINAPI ISF_MyComputer_fnCreateViewObject (IShellFolder2 *iface,
     }
     else if (IsEqualIID (riid, &IID_IContextMenu))
     {
-        WARN ("IContextMenu not implemented\n");
-        hr = E_NOTIMPL;
+        hr = BackgroundMenu_Constructor((IShellFolder*)iface, FALSE, riid, ppvOut);
     }
     else if (IsEqualIID (riid, &IID_IShellView))
     {
@@ -463,7 +450,7 @@ static HRESULT WINAPI ISF_MyComputer_fnGetAttributesOf (IShellFolder2 * iface,
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
     HRESULT hr = S_OK;
 
-    TRACE ("(%p)->(cidl=%d apidl=%p mask=%p (0x%08x))\n",
+    TRACE ("(%p)->(cidl=%d apidl=%p mask=%p (0x%08lx))\n",
            This, cidl, apidl, rgfInOut, rgfInOut ? *rgfInOut : 0);
 
     if (!rgfInOut)
@@ -494,7 +481,7 @@ static HRESULT WINAPI ISF_MyComputer_fnGetAttributesOf (IShellFolder2 * iface,
     /* make sure SFGAO_VALIDATE is cleared, some apps depend on that */
     *rgfInOut &= ~SFGAO_VALIDATE;
 
-    TRACE ("-- result=0x%08x\n", *rgfInOut);
+    TRACE ("-- result=0x%08lx\n", *rgfInOut);
     return hr;
 }
 
@@ -571,7 +558,7 @@ static HRESULT WINAPI ISF_MyComputer_fnGetUIObjectOf (IShellFolder2 * iface,
         hr = E_OUTOFMEMORY;
 
     *ppvOut = pObj;
-    TRACE ("(%p)->hr=0x%08x\n", This, hr);
+    TRACE ("(%p)->hr=0x%08lx\n", This, hr);
     return hr;
 }
 
@@ -586,7 +573,7 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDisplayNameOf (IShellFolder2 *iface,
     LPWSTR pszPath;
     HRESULT hr = S_OK;
 
-    TRACE ("(%p)->(pidl=%p,0x%08x,%p)\n", This, pidl, dwFlags, strRet);
+    TRACE ("(%p)->(pidl=%p,0x%08lx,%p)\n", This, pidl, dwFlags, strRet);
     pdump (pidl);
 
     if (!strRet)
@@ -617,13 +604,6 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDisplayNameOf (IShellFolder2 *iface,
             {
                 if ((GET_SHGDN_FOR (dwFlags) & (SHGDN_FORPARSING | SHGDN_FORADDRESSBAR)) == SHGDN_FORPARSING)
                 {
-                    static const WCHAR clsidW[] =
-                     { 'C','L','S','I','D','\\',0 };
-                    static const WCHAR shellfolderW[] =
-                     { '\\','s','h','e','l','l','f','o','l','d','e','r',0 };
-                    static const WCHAR wantsForParsingW[] =
-                     { 'W','a','n','t','s','F','o','r','P','a','r','s','i','n',
-                     'g',0 };
                     BOOL bWantsForParsing = FALSE;
                     WCHAR szRegPath[100];
                     LONG r;
@@ -639,11 +619,10 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDisplayNameOf (IShellFolder2 *iface,
                      * Get the "WantsFORPARSING" flag from the registry
                      */
 
-                    lstrcpyW (szRegPath, clsidW);
+                    lstrcpyW (szRegPath, L"CLSID\\");
                     SHELL32_GUIDToStringW (clsid, &szRegPath[6]);
-                    lstrcatW (szRegPath, shellfolderW);
-                    r = SHGetValueW (HKEY_CLASSES_ROOT, szRegPath, 
-                                     wantsForParsingW, NULL, NULL, NULL);
+                    lstrcatW (szRegPath, L"\\shellfolder");
+                    r = SHGetValueW (HKEY_CLASSES_ROOT, szRegPath, L"WantsForParsing", NULL, NULL, NULL);
                     if (r == ERROR_SUCCESS)
                         bWantsForParsing = TRUE;
 
@@ -694,16 +673,14 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDisplayNameOf (IShellFolder2 *iface,
             /* long view "lw_name (C:)" */
             if (!(dwFlags & SHGDN_FORPARSING))
             {
-                static const WCHAR wszOpenBracket[] = {' ','(',0};
-                static const WCHAR wszCloseBracket[] = {')',0};
                 WCHAR wszDrive[32 /* label */ + 6 /* ' (C:)'\0 */] = {0};
 
                 GetVolumeInformationW (pszPath, wszDrive, ARRAY_SIZE(wszDrive) - 5, NULL, NULL,
                         NULL, NULL, 0);
-                strcatW (wszDrive, wszOpenBracket);
-                lstrcpynW (wszDrive + strlenW(wszDrive), pszPath, 3);
-                strcatW (wszDrive, wszCloseBracket);
-                strcpyW (pszPath, wszDrive);
+                lstrcatW (wszDrive, L" (");
+                lstrcpynW (wszDrive + lstrlenW(wszDrive), pszPath, 3);
+                lstrcatW (wszDrive, L")");
+                lstrcpyW (pszPath, wszDrive);
             }
         }
         else 
@@ -761,7 +738,7 @@ static HRESULT WINAPI ISF_MyComputer_fnSetNameOf (
                LPCOLESTR lpName, DWORD dwFlags, LPITEMIDLIST * pPidlOut)
 {
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
-    FIXME ("(%p)->(%p,pidl=%p,%s,%u,%p)\n", This,
+    FIXME ("(%p)->(%p,pidl=%p,%s,%lu,%p)\n", This,
            hwndOwner, pidl, debugstr_w (lpName), dwFlags, pPidlOut);
     return E_FAIL;
 }
@@ -785,7 +762,7 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDefaultColumn(IShellFolder2 *iface, DW
 {
     IMyComputerFolderImpl *This = impl_from_IShellFolder2(iface);
 
-    TRACE("(%p)->(%#x, %p, %p)\n", This, reserved, sort, display);
+    TRACE("(%p)->(%#lx, %p, %p)\n", This, reserved, sort, display);
 
     return E_NOTIMPL;
 }
@@ -835,13 +812,6 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDetailsOf (IShellFolder2 *iface,
 
     switch (iColumn)
     {
-        case 0:        /* name */
-            hr = IShellFolder2_GetDisplayNameOf (iface, pidl,
-                       SHGDN_NORMAL | SHGDN_INFOLDER, &psd->str);
-            break;
-        case 1:        /* type */
-            _ILGetFileType (pidl, psd->str.u.cStr, MAX_PATH);
-            break;
         case 2:        /* total size */
             if (_ILIsDrive (pidl))
             {
@@ -858,6 +828,9 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDetailsOf (IShellFolder2 *iface,
                 StrFormatByteSizeA (ulBytes.u.LowPart, psd->str.u.cStr, MAX_PATH);
             }
             break;
+
+        default:
+            return shellfolder_get_file_details( iface, pidl, mycomputer_header, iColumn, psd );
     }
 
     return hr;
@@ -917,7 +890,7 @@ static HRESULT WINAPI IMCFldr_PersistFolder2_QueryInterface (
 static ULONG WINAPI IMCFldr_PersistFolder2_AddRef (IPersistFolder2 * iface)
 {
     IMyComputerFolderImpl *This = impl_from_IPersistFolder2(iface);
-    TRACE ("(%p)->(count=%u)\n", This, This->ref);
+    TRACE ("(%p)->(count=%lu)\n", This, This->ref);
     return IShellFolder2_AddRef (&This->IShellFolder2_iface);
 }
 
@@ -927,7 +900,7 @@ static ULONG WINAPI IMCFldr_PersistFolder2_AddRef (IPersistFolder2 * iface)
 static ULONG WINAPI IMCFldr_PersistFolder2_Release (IPersistFolder2 * iface)
 {
     IMyComputerFolderImpl *This = impl_from_IPersistFolder2(iface);
-    TRACE ("(%p)->(count=%u)\n", This, This->ref);
+    TRACE ("(%p)->(count=%lu)\n", This, This->ref);
     return IShellFolder2_Release (&This->IShellFolder2_iface);
 }
 
