@@ -56,7 +56,7 @@ static void FillNumberFmt(NUMBERFMTW *fmt, LPWSTR decimal_buffer, int decimal_bu
   WCHAR *c;
 
   GetLocaleInfoW(LOCALE_USER_DEFAULT, LOCALE_ILZERO|LOCALE_RETURN_NUMBER, (LPWSTR)&fmt->LeadingZero, sizeof(fmt->LeadingZero)/sizeof(WCHAR));
-  GetLocaleInfoW(LOCALE_USER_DEFAULT, LOCALE_INEGNUMBER|LOCALE_RETURN_NUMBER, (LPWSTR)&fmt->LeadingZero, sizeof(fmt->NegativeOrder)/sizeof(WCHAR));
+  GetLocaleInfoW(LOCALE_USER_DEFAULT, LOCALE_INEGNUMBER|LOCALE_RETURN_NUMBER, (LPWSTR)&fmt->NegativeOrder, sizeof(fmt->NegativeOrder)/sizeof(WCHAR));
   fmt->NumDigits = 0;
   GetLocaleInfoW(LOCALE_USER_DEFAULT, LOCALE_SDECIMAL, decimal_buffer, decimal_bufwlen);
   GetLocaleInfoW(LOCALE_USER_DEFAULT, LOCALE_STHOUSAND, thousand_buffer, thousand_bufwlen);
@@ -229,7 +229,7 @@ HRESULT WINAPI StrRetToBufA (LPSTRRET src, const ITEMIDLIST *pidl, LPSTR dest, U
 	    break;
 
 	  case STRRET_OFFSET:
-	    lstrcpynA((LPSTR)dest, ((LPCSTR)&pidl->mkid)+src->u.uOffset, len);
+            lstrcpynA(dest, ((LPCSTR)&pidl->mkid)+src->u.uOffset, len);
 	    break;
 
 	  default:
@@ -370,7 +370,7 @@ HRESULT WINAPI StrRetToStrW(LPSTRRET lpStrRet, const ITEMIDLIST *pidl, LPWSTR *p
   return hRet;
 }
 
-/* Create an ASCII string copy using SysAllocString() */
+/* Makes a Unicode copy of an ANSI string using SysAllocString() */
 static HRESULT _SHStrDupAToBSTR(LPCSTR src, BSTR *pBstrOut)
 {
     *pBstrOut = NULL;
@@ -539,7 +539,7 @@ LPWSTR WINAPI StrNCatW(LPWSTR lpszStr, LPCWSTR lpszCat, INT cchMax)
 /*************************************************************************
  *      _SHStrDupAA	[INTERNAL]
  *
- * Duplicates a ASCII string to ASCII. The destination buffer is allocated.
+ * Duplicates a ANSI string to ANSI. The destination buffer is allocated.
  */
 static HRESULT _SHStrDupAA(LPCSTR src, LPSTR * dest)
 {
@@ -606,7 +606,7 @@ HRESULT WINAPI SHStrDupA(LPCSTR lpszStr, LPWSTR * lppszDest)
 /*************************************************************************
  *      _SHStrDupAW	[INTERNAL]
  *
- * Duplicates a UNICODE to a ASCII string. The destination buffer is allocated.
+ * Duplicates a UNICODE to a ANSI string. The destination buffer is allocated.
  */
 static HRESULT _SHStrDupAW(LPCWSTR src, LPSTR * dest)
 {
@@ -754,7 +754,7 @@ INT WINAPI StrFromTimeIntervalA(LPSTR lpszStr, UINT cchMax, DWORD dwMS,
 {
   INT iRet = 0;
 
-  TRACE("(%p,%d,%d,%d)\n", lpszStr, cchMax, dwMS, iDigits);
+  TRACE("(%p,%d,%ld,%d)\n", lpszStr, cchMax, dwMS, iDigits);
 
   if (lpszStr && cchMax)
   {
@@ -776,7 +776,7 @@ INT WINAPI StrFromTimeIntervalW(LPWSTR lpszStr, UINT cchMax, DWORD dwMS,
 {
   INT iRet = 0;
 
-  TRACE("(%p,%d,%d,%d)\n", lpszStr, cchMax, dwMS, iDigits);
+  TRACE("(%p,%d,%ld,%d)\n", lpszStr, cchMax, dwMS, iDigits);
 
   if (lpszStr && cchMax)
   {
@@ -840,6 +840,30 @@ typedef struct tagSHLWAPI_BYTEFORMATS
  *  There is no StrFormatByteSize64W function, it is called StrFormatByteSizeW().
  */
 LPWSTR WINAPI StrFormatByteSizeW(LONGLONG llBytes, LPWSTR lpszDest, UINT cchMax)
+{
+  HRESULT hr;
+
+  TRACE("(0x%s,%p,%d)\n", wine_dbgstr_longlong(llBytes), lpszDest, cchMax);
+
+  if (!lpszDest || !cchMax)
+    return lpszDest;
+
+  hr = StrFormatByteSizeEx(llBytes, SFBS_FLAGS_TRUNCATE_UNDISPLAYED_DECIMAL_DIGITS,
+                           lpszDest, cchMax);
+
+  if (FAILED(hr))
+    return NULL;
+
+  return lpszDest;
+}
+
+/*************************************************************************
+ * StrFormatByteSizeEx  [SHLWAPI.@]
+ *
+ */
+
+HRESULT WINAPI StrFormatByteSizeEx(LONGLONG llBytes, SFBS_FLAGS flags, LPWSTR lpszDest,
+                                   UINT cchMax)
 {
 #define KB ((ULONGLONG)1024)
 #define MB (KB*KB)
@@ -870,17 +894,17 @@ LPWSTR WINAPI StrFormatByteSizeW(LONGLONG llBytes, LPWSTR lpszDest, UINT cchMax)
   double dBytes;
   UINT i = 0;
 
-  TRACE("(0x%s,%p,%d)\n", wine_dbgstr_longlong(llBytes), lpszDest, cchMax);
+  TRACE("(0x%s,%d,%p,%d)\n", wine_dbgstr_longlong(llBytes), flags, lpszDest, cchMax);
 
-  if (!lpszDest || !cchMax)
-    return lpszDest;
+  if (!cchMax)
+    return E_INVALIDARG;
 
   if (llBytes < 1024)  /* 1K */
   {
     WCHAR wszBytesFormat[64];
     LoadStringW(shlwapi_hInstance, IDS_BYTES_FORMAT, wszBytesFormat, 64);
     swprintf(lpszDest, cchMax, wszBytesFormat, (int)llBytes);
-    return lpszDest;
+    return S_OK;
   }
 
   /* Note that if this loop completes without finding a match, i will be
@@ -903,13 +927,24 @@ LPWSTR WINAPI StrFormatByteSizeW(LONGLONG llBytes, LPWSTR lpszDest, UINT cchMax)
   else
     dBytes = (double)llBytes + 0.00001;
 
-  dBytes = floor(dBytes / bfFormats[i].dDivisor) / bfFormats[i].dNormaliser;
+  switch(flags)
+  {
+  case SFBS_FLAGS_ROUND_TO_NEAREST_DISPLAYED_DIGIT:
+      dBytes = round(dBytes / bfFormats[i].dDivisor) / bfFormats[i].dNormaliser;
+      break;
+  case SFBS_FLAGS_TRUNCATE_UNDISPLAYED_DECIMAL_DIGITS:
+      dBytes = floor(dBytes / bfFormats[i].dDivisor) / bfFormats[i].dNormaliser;
+      break;
+  default:
+      return E_INVALIDARG;
+  }
 
   if (!FormatDouble(dBytes, bfFormats[i].nDecimals, lpszDest, cchMax))
-    return NULL;
+    return E_FAIL;
+
   wszAdd[1] = bfFormats[i].wPrefix;
   StrCatBuffW(lpszDest, wszAdd, cchMax);
-  return lpszDest;
+  return S_OK;
 }
 
 /*************************************************************************
@@ -942,12 +977,12 @@ LPSTR WINAPI StrFormatByteSize64A(LONGLONG llBytes, LPSTR lpszDest, UINT cchMax)
  *  lpszDest.
  *
  * NOTES
- *  The Ascii and Unicode versions of this function accept a different
+ *  The ANSI and Unicode versions of this function accept a different
  *  integer type for dwBytes. See StrFormatByteSize64A().
  */
 LPSTR WINAPI StrFormatByteSizeA(DWORD dwBytes, LPSTR lpszDest, UINT cchMax)
 {
-  TRACE("(%d,%p,%d)\n", dwBytes, lpszDest, cchMax);
+  TRACE("(%ld,%p,%d)\n", dwBytes, lpszDest, cchMax);
 
   return StrFormatByteSize64A(dwBytes, lpszDest, cchMax);
 }
@@ -1020,11 +1055,11 @@ WCHAR WINAPI SHStripMneumonicW(LPCWSTR lpszStr)
 /*************************************************************************
  *      @	[SHLWAPI.216]
  *
- * Convert an Ascii string to Unicode.
+ * Convert an ANSI string to Unicode.
  *
  * PARAMS
  * dwCp     [I] Code page for the conversion
- * lpSrcStr [I] Source Ascii string to convert
+ * lpSrcStr [I] Source ANSI string to convert
  * lpDstStr [O] Destination for converted Unicode string
  * iLen     [I] Length of lpDstStr
  *
@@ -1036,17 +1071,17 @@ DWORD WINAPI SHAnsiToUnicodeCP(DWORD dwCp, LPCSTR lpSrcStr, LPWSTR lpDstStr, int
   DWORD dwRet;
 
   dwRet = MultiByteToWideChar(dwCp, 0, lpSrcStr, -1, lpDstStr, iLen);
-  TRACE("%s->%s,ret=%d\n", debugstr_a(lpSrcStr), debugstr_w(lpDstStr), dwRet);
+  TRACE("%s->%s,ret=%ld\n", debugstr_a(lpSrcStr), debugstr_w(lpDstStr), dwRet);
   return dwRet;
 }
 
 /*************************************************************************
  *      @	[SHLWAPI.215]
  *
- * Convert an Ascii string to Unicode.
+ * Convert an ANSI string to Unicode.
  *
  * PARAMS
- * lpSrcStr [I] Source Ascii string to convert
+ * lpSrcStr [I] Source ANSI string to convert
  * lpDstStr [O] Destination for converted Unicode string
  * iLen     [I] Length of lpDstStr
  *
@@ -1064,12 +1099,12 @@ DWORD WINAPI SHAnsiToUnicode(LPCSTR lpSrcStr, LPWSTR lpDstStr, int iLen)
 /*************************************************************************
  *      @	[SHLWAPI.218]
  *
- * Convert a Unicode string to Ascii.
+ * Convert a Unicode string to ANSI.
  *
  * PARAMS
  *  CodePage [I] Code page to use for the conversion
  *  lpSrcStr [I] Source Unicode string to convert
- *  lpDstStr [O] Destination for converted Ascii string
+ *  lpDstStr [O] Destination for converted ANSI string
  *  dstlen   [I] Length of buffer at lpDstStr
  *
  * RETURNS
@@ -1167,11 +1202,11 @@ DWORD WINAPI SHUnicodeToAnsiCP(UINT CodePage, LPCWSTR lpSrcStr, LPSTR lpDstStr,
 /*************************************************************************
  *      @	[SHLWAPI.217]
  *
- * Convert a Unicode string to Ascii.
+ * Convert a Unicode string to ANSI.
  *
  * PARAMS
  *  lpSrcStr [I] Source Unicode string to convert
- *  lpDstStr [O] Destination for converted Ascii string
+ *  lpDstStr [O] Destination for converted ANSI string
  *  iLen     [O] Length of lpDstStr in characters
  *
  * RETURNS
@@ -1188,15 +1223,15 @@ INT WINAPI SHUnicodeToAnsi(LPCWSTR lpSrcStr, LPSTR lpDstStr, INT iLen)
 /*************************************************************************
  *      @	[SHLWAPI.364]
  *
- * Determine if an Ascii string converts to Unicode and back identically.
+ * Determine if an ANSI string converts to Unicode and back identically.
  *
  * PARAMS
  *  lpSrcStr [I] Source Unicode string to convert
- *  lpDst    [O] Destination for resulting Ascii string
+ *  lpDst    [O] Destination for resulting ANSI string
  *  iLen     [I] Length of lpDst in characters
  *
  * RETURNS
- *  TRUE, since Ascii strings always convert identically.
+ *  TRUE, since ANSI strings always convert identically.
  */
 BOOL WINAPI DoesStringRoundTripA(LPCSTR lpSrcStr, LPSTR lpDst, INT iLen)
 {
@@ -1207,15 +1242,15 @@ BOOL WINAPI DoesStringRoundTripA(LPCSTR lpSrcStr, LPSTR lpDst, INT iLen)
 /*************************************************************************
  *      @	[SHLWAPI.365]
  *
- * Determine if a Unicode string converts to Ascii and back identically.
+ * Determine if a Unicode string converts to ANSI and back identically.
  *
  * PARAMS
  *  lpSrcStr [I] Source Unicode string to convert
- *  lpDst    [O] Destination for resulting Ascii string
+ *  lpDst    [O] Destination for resulting ANSI string
  *  iLen     [I] Length of lpDst in characters
  *
  * RETURNS
- *  TRUE, if lpSrcStr converts to Ascii and back identically,
+ *  TRUE, if lpSrcStr converts to ANSI and back identically,
  *  FALSE otherwise.
  */
 BOOL WINAPI DoesStringRoundTripW(LPCWSTR lpSrcStr, LPSTR lpDst, INT iLen)
