@@ -98,7 +98,7 @@ static ULONG WINAPI Builtin_Release(IDispatch *iface)
 
     if(!ref) {
         assert(!This->ctx);
-        heap_free(This);
+        free(This);
     }
 
     return ref;
@@ -249,7 +249,7 @@ static HRESULT WINAPI Builtin_Invoke(IDispatch *iface, DISPID id, REFIID riid, L
     if(argn <= ARRAY_SIZE(args_buf)) {
         args = args_buf;
     }else {
-        args = heap_alloc(argn * sizeof(*args));
+        args = malloc(argn * sizeof(*args));
         if(!args)
             return E_OUTOFMEMORY;
     }
@@ -263,7 +263,7 @@ static HRESULT WINAPI Builtin_Invoke(IDispatch *iface, DISPID id, REFIID riid, L
 
     hres = prop->proc(This, args, dp->cArgs, res);
     if(args != args_buf)
-        heap_free(args);
+        free(args);
     return hres;
 }
 
@@ -281,7 +281,7 @@ static HRESULT create_builtin_dispatch(script_ctx_t *ctx, const builtin_prop_t *
 {
     BuiltinDisp *disp;
 
-    if(!(disp = heap_alloc(sizeof(*disp))))
+    if(!(disp = malloc(sizeof(*disp))))
         return E_OUTOFMEMORY;
 
     disp->IDispatch_iface.lpVtbl = &BuiltinDispVtbl;
@@ -439,6 +439,20 @@ static HRESULT to_double(VARIANT *v, double *ret)
     return S_OK;
 }
 
+static HRESULT to_float(VARIANT *v, float *ret)
+{
+    VARIANT dst;
+    HRESULT hres;
+
+    V_VT(&dst) = VT_EMPTY;
+    hres = VariantChangeType(&dst, v, 0, VT_R4);
+    if(FAILED(hres))
+        return hres;
+
+    *ret = V_R4(&dst);
+    return S_OK;
+}
+
 static HRESULT to_string(VARIANT *v, BSTR *ret)
 {
     VARIANT dst;
@@ -597,7 +611,7 @@ static HRESULT show_msgbox(script_ctx_t *ctx, BSTR prompt, unsigned type, BSTR o
         if(orig_title && *orig_title) {
             WCHAR *ptr;
 
-            title = title_buf = heap_alloc(sizeof(L"VBScript") + (lstrlenW(orig_title)+2)*sizeof(WCHAR));
+            title = title_buf = malloc(sizeof(L"VBScript") + (lstrlenW(orig_title)+2)*sizeof(WCHAR));
             if(!title)
                 return E_OUTOFMEMORY;
 
@@ -623,7 +637,7 @@ static HRESULT show_msgbox(script_ctx_t *ctx, BSTR prompt, unsigned type, BSTR o
         }
     }
 
-    heap_free(title_buf);
+    free(title_buf);
     IActiveScriptSiteWindow_Release(acts_window);
     if(FAILED(hres)) {
         FIXME("failed: %08lx\n", hres);
@@ -1075,16 +1089,70 @@ static HRESULT Global_Sqr(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VA
         return return_double(res, sqrt(d));
 }
 
+static unsigned int get_next_rnd(int value)
+{
+    return (value * 0x43fd43fd + 0xc39ec3) & 0xffffff;
+}
+
 static HRESULT Global_Randomize(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
 {
-    FIXME("\n");
-    return E_NOTIMPL;
+    union
+    {
+        double d;
+        unsigned int i[2];
+    } dtoi;
+    unsigned int seed;
+    HRESULT hres;
+
+    assert(args_cnt == 0 || args_cnt == 1);
+    if (args_cnt == 1) {
+        hres = to_double(arg, &dtoi.d);
+        if (FAILED(hres))
+            return hres;
+    }
+    else
+        dtoi.d = GetTickCount() * 0.001;
+
+    seed = dtoi.i[1];
+    seed ^= (seed >> 16);
+    seed = ((seed & 0xffff) << 8) | (This->ctx->script_obj->rnd & 0xff);
+    This->ctx->script_obj->rnd = seed;
+
+    return res ? DISP_E_TYPEMISMATCH : S_OK;
 }
 
 static HRESULT Global_Rnd(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
 {
-    FIXME("\n");
-    return E_NOTIMPL;
+    static const float modulus = 16777216.0f;
+    unsigned int value;
+    HRESULT hres;
+    float f;
+
+    assert(args_cnt == 0 || args_cnt == 1);
+
+    value = This->ctx->script_obj->rnd;
+    if (args_cnt == 1)
+    {
+        hres = to_float(arg, &f);
+        if (FAILED(hres))
+            return hres;
+
+        if (f < 0.0f)
+        {
+            value = *(unsigned int *)&f;
+            This->ctx->script_obj->rnd = value = get_next_rnd(value + (value >> 24));
+        }
+        else if (f == 0.0f)
+            value = This->ctx->script_obj->rnd;
+        else
+            This->ctx->script_obj->rnd = value = get_next_rnd(value);
+    }
+    else
+    {
+        This->ctx->script_obj->rnd = value = get_next_rnd(value);
+    }
+
+    return return_float(res, (float)value / modulus);
 }
 
 static HRESULT Global_Timer(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
@@ -1102,7 +1170,7 @@ static HRESULT Global_LBound(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt,
 {
     SAFEARRAY *sa;
     HRESULT hres;
-    LONG ubound;
+    LONG lbound;
     int dim;
 
     assert(args_cnt == 1 || args_cnt == 2);
@@ -1129,11 +1197,11 @@ static HRESULT Global_LBound(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt,
         dim = 1;
     }
 
-    hres = SafeArrayGetLBound(sa, dim, &ubound);
+    hres = SafeArrayGetLBound(sa, dim, &lbound);
     if(FAILED(hres))
         return hres;
 
-    return return_int(res, ubound);
+    return return_int(res, lbound);
 }
 
 static HRESULT Global_UBound(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
@@ -1280,7 +1348,20 @@ static HRESULT Global_Right(BuiltinDisp *This, VARIANT *args, unsigned args_cnt,
 
     TRACE("(%s %s)\n", debugstr_variant(args), debugstr_variant(args+1));
 
-    if(V_VT(args+1) == VT_BSTR) {
+    if(V_VT(args+1) == VT_NULL)
+        return MAKE_VBSERROR(VBSE_ILLEGAL_NULL_USE);
+
+    hres = to_int(args+1, &len);
+    if(FAILED(hres))
+        return hres;
+
+    if(len < 0)
+        return MAKE_VBSERROR(VBSE_ILLEGAL_FUNC_CALL);
+
+    if(V_VT(args) == VT_NULL)
+        return return_null(res);
+
+    if(V_VT(args) == VT_BSTR) {
         str = V_BSTR(args);
     }else {
         hres = to_string(args, &conv_str);
@@ -1289,15 +1370,6 @@ static HRESULT Global_Right(BuiltinDisp *This, VARIANT *args, unsigned args_cnt,
         str = conv_str;
     }
 
-    hres = to_int(args+1, &len);
-    if(FAILED(hres))
-        return hres;
-
-    if(len < 0) {
-        FIXME("len = %d\n", len);
-        return E_FAIL;
-    }
-
     str_len = SysStringLen(str);
     if(len > str_len)
         len = str_len;
@@ -1319,35 +1391,49 @@ static HRESULT Global_RightB(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt,
 static HRESULT Global_Mid(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VARIANT *res)
 {
     int len = -1, start, str_len;
-    BSTR str;
+    BSTR str, conv_str = NULL;
     HRESULT hres;
 
     TRACE("(%s %s ...)\n", debugstr_variant(args), debugstr_variant(args+1));
 
     assert(args_cnt == 2 || args_cnt == 3);
 
-    if(V_VT(args) != VT_BSTR) {
-        FIXME("args[0] = %s\n", debugstr_variant(args));
-        return E_NOTIMPL;
-    }
+    if(V_VT(args) == VT_EMPTY)
+        return MAKE_VBSERROR(VBSE_ILLEGAL_FUNC_CALL);
 
-    str = V_BSTR(args);
+    if(V_VT(args+1) == VT_NULL || (args_cnt == 3 && V_VT(args+2) == VT_NULL))
+        return MAKE_VBSERROR(VBSE_ILLEGAL_NULL_USE);
+
+    if(V_VT(args+1) == VT_EMPTY)
+        return MAKE_VBSERROR(VBSE_ILLEGAL_FUNC_CALL);
 
     hres = to_int(args+1, &start);
     if(FAILED(hres))
         return hres;
 
+    if(start < 0)
+        return MAKE_VBSERROR(VBSE_ILLEGAL_FUNC_CALL);
+
     if(args_cnt == 3) {
+        if(V_VT(args+2) == VT_EMPTY)
+            return MAKE_VBSERROR(VBSE_ILLEGAL_FUNC_CALL);
+
         hres = to_int(args+2, &len);
         if(FAILED(hres))
             return hres;
 
-        if(len < 0) {
-            FIXME("len = %d\n", len);
-            return E_FAIL;
-        }
+        if(len < 0)
+            return MAKE_VBSERROR(VBSE_ILLEGAL_FUNC_CALL);
     }
 
+    if(V_VT(args) == VT_BSTR) {
+        str = V_BSTR(args);
+    }else {
+        hres = to_string(args, &conv_str);
+        if(FAILED(hres))
+            return hres;
+        str = conv_str;
+    }
 
     str_len = SysStringLen(str);
     start--;
@@ -1363,10 +1449,12 @@ static HRESULT Global_Mid(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, V
         V_VT(res) = VT_BSTR;
         V_BSTR(res) = SysAllocStringLen(str+start, len);
         if(!V_BSTR(res))
-            return E_OUTOFMEMORY;
+            hres = E_OUTOFMEMORY;
     }
 
-    return S_OK;
+    SysFreeString(conv_str);
+
+    return hres;
 }
 
 static HRESULT Global_MidB(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
@@ -1754,10 +1842,12 @@ static HRESULT Global_Asc(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VA
 
     if(!SysStringLen(str))
         hres = MAKE_VBSERROR(VBSE_ILLEGAL_FUNC_CALL);
+    else if (This->ctx->codepage == CP_UTF8)
+        hres = return_short(res, *str);
     else {
         unsigned char buf[2];
         short val = 0;
-        int n = WideCharToMultiByte(CP_ACP, 0, str, 1, (char*)buf, sizeof(buf), NULL, NULL);
+        int n = WideCharToMultiByte(This->ctx->codepage, 0, str, 1, (char*)buf, sizeof(buf), NULL, NULL);
         switch(n) {
         case 1:
             val = buf[0];
@@ -1777,9 +1867,6 @@ static HRESULT Global_Asc(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VA
     return hres;
 }
 
-/* The function supports only single-byte and double-byte character sets. It
- * ignores language specified by IActiveScriptSite::GetLCID. The argument needs
- * to be in range of short or unsigned short. */
 static HRESULT Global_Chr(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
 {
     int cp, c, len = 0;
@@ -1794,7 +1881,7 @@ static HRESULT Global_Chr(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VA
     if(FAILED(hres))
         return hres;
 
-    cp = GetACP();
+    cp = This->ctx->codepage;
     if(!GetCPInfo(cp, &cpi))
         cpi.MaxCharSize = 1;
 
@@ -1804,13 +1891,20 @@ static HRESULT Global_Chr(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VA
         return MAKE_VBSERROR(VBSE_ILLEGAL_FUNC_CALL);
     }
 
-    if(c>>8)
-        buf[len++] = c>>8;
-    if(!len || IsDBCSLeadByteEx(cp, buf[0]))
-        buf[len++] = c;
-    if(!MultiByteToWideChar(CP_ACP, 0, buf, len, &ch, 1)) {
-        WARN("invalid arg %d, cp %d\n", c, cp);
-        return E_FAIL;
+    if (cp == CP_UTF8)
+    {
+        ch = c;
+    }
+    else
+    {
+        if(c>>8)
+            buf[len++] = c>>8;
+        if(!len || IsDBCSLeadByteEx(cp, buf[0]))
+            buf[len++] = c;
+        if(!MultiByteToWideChar(cp, 0, buf, len, &ch, 1)) {
+            WARN("invalid arg %d, cp %d\n", c, cp);
+            return E_FAIL;
+        }
     }
 
     if(res) {
@@ -2362,6 +2456,10 @@ static HRESULT Global_DatePart(BuiltinDisp *This, VARIANT *arg, unsigned args_cn
 
 static HRESULT Global_TypeName(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
 {
+    ITypeInfo *typeinfo;
+    BSTR name = NULL;
+    HRESULT hres;
+
     TRACE("(%s)\n", debugstr_variant(arg));
 
     assert(args_cnt == 1);
@@ -2394,6 +2492,18 @@ static HRESULT Global_TypeName(BuiltinDisp *This, VARIANT *arg, unsigned args_cn
             return return_string(res, L"Empty");
         case VT_NULL:
             return return_string(res, L"Null");
+        case VT_DISPATCH:
+            if (SUCCEEDED(IDispatch_GetTypeInfo(V_DISPATCH(arg), 0, GetUserDefaultLCID(), &typeinfo)))
+            {
+                hres = ITypeInfo_GetDocumentation(typeinfo, MEMBERID_NIL, &name, NULL, NULL, NULL);
+                ITypeInfo_Release(typeinfo);
+
+                if (SUCCEEDED(hres) && name && *name)
+                    return return_bstr(res, name);
+
+                SysFreeString(name);
+            }
+            return return_string(res, L"Object");
         default:
             FIXME("arg %s not supported\n", debugstr_variant(arg));
             return E_NOTIMPL;
@@ -2462,12 +2572,12 @@ static HRESULT Global_Join(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, V
 
 static HRESULT Global_Split(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VARIANT *res)
 {
-    BSTR str, string, delimiter = NULL;
+    BSTR string, delimiter = NULL;
     int count, max, mode, len, start, end, ret, delimiterlen = 1;
-    int i,*indices = NULL, indices_max = 8;
+    int i, *indices = NULL, *new_indices, indices_max = 8;
     SAFEARRAYBOUND bounds;
     SAFEARRAY *sa = NULL;
-    VARIANT *data, var;
+    VARIANT *data;
     HRESULT hres = S_OK;
 
     TRACE("%s %u...\n", debugstr_variant(args), args_cnt);
@@ -2526,7 +2636,7 @@ static HRESULT Global_Split(BuiltinDisp *This, VARIANT *args, unsigned args_cnt,
     len = SysStringLen(string);
     count = 0;
 
-    indices = heap_alloc( indices_max * sizeof(int));
+    indices = malloc( indices_max * sizeof(int));
     if(!indices) {
         hres = E_OUTOFMEMORY;
         goto error;
@@ -2546,12 +2656,13 @@ static HRESULT Global_Split(BuiltinDisp *This, VARIANT *args, unsigned args_cnt,
         }
 
         if (count == indices_max) {
-            indices_max *= 2;
-            indices = heap_realloc( indices, indices_max * sizeof(int));
-            if(!indices) {
+            new_indices = realloc(indices, indices_max * 2 * sizeof(int));
+            if(!new_indices) {
                 hres = E_OUTOFMEMORY;
                 goto error;
             }
+            indices = new_indices;
+            indices_max *= 2;
         }
         indices[count++] = end;
 
@@ -2574,19 +2685,13 @@ static HRESULT Global_Split(BuiltinDisp *This, VARIANT *args, unsigned args_cnt,
 
     start = 0;
     for (i = 0; i < count; i++) {
-        str = SysAllocStringLen(string + start, indices[i] - start);
-        if (!str) {
+        V_VT(&data[i]) = VT_BSTR;
+        V_BSTR(&data[i]) = SysAllocStringLen(string + start, indices[i] - start);
+
+        if (!V_BSTR(&data[i])) {
             hres = E_OUTOFMEMORY;
             break;
         }
-        V_VT(&var) = VT_BSTR;
-        V_BSTR(&var) = str;
-
-        hres = VariantCopyInd(data+i, &var);
-        if(FAILED(hres)) {
-            SafeArrayUnaccessData(sa);
-            goto error;
-        }
         start = indices[i]+delimiterlen;
     }
     SafeArrayUnaccessData(sa);
@@ -2599,7 +2704,7 @@ error:
         SafeArrayDestroy(sa);
     }
 
-    heap_free(indices);
+    free(indices);
     if(V_VT(args) != VT_BSTR)
         SysFreeString(string);
     if(args_cnt > 1 && V_VT(args+1) != VT_BSTR)
@@ -2841,28 +2946,128 @@ static HRESULT Global_ScriptEngineBuildVersion(BuiltinDisp *This, VARIANT *arg,
     return return_int(res, VBSCRIPT_BUILD_VERSION);
 }
 
-static HRESULT Global_FormatNumber(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
+static HRESULT Global_FormatNumber(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VARIANT *res)
 {
-    FIXME("\n");
-    return E_NOTIMPL;
+    union
+    {
+        struct
+        {
+            int num_dig, inc_lead, use_parens, group;
+        } s;
+        int val[4];
+    } int_args = { .s.num_dig = -1, .s.inc_lead = -2, .s.use_parens = -2, .s.group = -2 };
+    HRESULT hres;
+    BSTR str;
+    int i;
+
+    TRACE("\n");
+
+    assert(1 <= args_cnt && args_cnt <= 5);
+
+    for (i = 1; i < args_cnt; ++i)
+    {
+        if (V_VT(args+i) == VT_ERROR) continue;
+        if (V_VT(args+i) == VT_NULL) return MAKE_VBSERROR(VBSE_ILLEGAL_NULL_USE);
+        if (FAILED(hres = to_int(args+i, &int_args.val[i-1]))) return hres;
+    }
+
+    hres = VarFormatNumber(args, int_args.s.num_dig, int_args.s.inc_lead, int_args.s.use_parens,
+        int_args.s.group, 0, &str);
+    if (FAILED(hres)) return hres;
+
+    return return_bstr(res, str);
 }
 
-static HRESULT Global_FormatCurrency(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
+static HRESULT Global_FormatCurrency(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VARIANT *res)
 {
-    FIXME("\n");
-    return E_NOTIMPL;
+    union
+    {
+        struct
+        {
+            int num_dig, inc_lead, use_parens, group;
+        } s;
+        int val[4];
+    } int_args = { .s.num_dig = -1, .s.inc_lead = -2, .s.use_parens = -2, .s.group = -2 };
+    HRESULT hres;
+    BSTR str;
+    int i;
+
+    TRACE("\n");
+
+    assert(1 <= args_cnt && args_cnt <= 5);
+
+    for (i = 1; i < args_cnt; ++i)
+    {
+        if (V_VT(args+i) == VT_ERROR) continue;
+        if (V_VT(args+i) == VT_NULL) return MAKE_VBSERROR(VBSE_ILLEGAL_NULL_USE);
+        if (FAILED(hres = to_int(args+i, &int_args.val[i-1]))) return hres;
+    }
+
+    hres = VarFormatCurrency(args, int_args.s.num_dig, int_args.s.inc_lead, int_args.s.use_parens,
+        int_args.s.group, 0, &str);
+    if (FAILED(hres)) return hres;
+
+    return return_bstr(res, str);
 }
 
-static HRESULT Global_FormatPercent(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
+static HRESULT Global_FormatPercent(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VARIANT *res)
 {
-    FIXME("\n");
-    return E_NOTIMPL;
+    union
+    {
+        struct
+        {
+            int num_dig, inc_lead, use_parens, group;
+        } s;
+        int val[4];
+    } int_args = { .s.num_dig = -1, .s.inc_lead = -2, .s.use_parens = -2, .s.group = -2 };
+    HRESULT hres;
+    BSTR str;
+    int i;
+
+    TRACE("\n");
+
+    assert(1 <= args_cnt && args_cnt <= 5);
+
+    for (i = 1; i < args_cnt; ++i)
+    {
+        if (V_VT(args+i) == VT_ERROR) continue;
+        if (V_VT(args+i) == VT_NULL) return MAKE_VBSERROR(VBSE_ILLEGAL_NULL_USE);
+        if (FAILED(hres = to_int(args+i, &int_args.val[i-1]))) return hres;
+    }
+
+    hres = VarFormatPercent(args, int_args.s.num_dig, int_args.s.inc_lead, int_args.s.use_parens,
+        int_args.s.group, 0, &str);
+    if (FAILED(hres)) return hres;
+
+    return return_bstr(res, str);
 }
 
-static HRESULT Global_FormatDateTime(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
+static HRESULT Global_FormatDateTime(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VARIANT *res)
 {
-    FIXME("\n");
-    return E_NOTIMPL;
+    int format = 0;
+    HRESULT hres;
+    BSTR str;
+
+    TRACE("\n");
+
+    assert(1 <= args_cnt && args_cnt <= 2);
+
+    if (V_VT(args) == VT_NULL)
+        return MAKE_VBSERROR(VBSE_TYPE_MISMATCH);
+
+    if (args_cnt == 2)
+    {
+        if (V_VT(args+1) == VT_NULL) return MAKE_VBSERROR(VBSE_ILLEGAL_NULL_USE);
+        if (V_VT(args+1) != VT_ERROR)
+        {
+            if (FAILED(hres = to_int(args+1, &format))) return hres;
+        }
+    }
+
+    hres = VarFormatDateTime(args, format, 0, &str);
+    if (FAILED(hres)) return hres;
+
+    return return_bstr(res, str);
 }
 
 static HRESULT Global_WeekdayName(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VARIANT *res)
@@ -3092,12 +3297,12 @@ static const builtin_prop_t global_props[] = {
     {L"MsgBox",                    Global_MsgBox, 0, 1, 5},
     {L"Now",                       Global_Now, 0, 0},
     {L"Oct",                       Global_Oct, 0, 1},
-    {L"Randomize",                 Global_Randomize, 0, 1},
+    {L"Randomize",                 Global_Randomize, 0, 0, 1},
     {L"Replace",                   Global_Replace, 0, 3, 6},
     {L"RGB",                       Global_RGB, 0, 3},
     {L"Right",                     Global_Right, 0, 2},
     {L"RightB",                    Global_RightB, 0, 2},
-    {L"Rnd",                       Global_Rnd, 0, 1},
+    {L"Rnd",                       Global_Rnd, 0, 0, 1},
     {L"Round",                     Global_Round, 0, 1, 2},
     {L"RTrim",                     Global_RTrim, 0, 1},
     {L"ScriptEngine",              Global_ScriptEngine, 0, 0},
@@ -3288,7 +3493,7 @@ static HRESULT Err_Raise(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VA
 {
     BSTR source = NULL, description = NULL, helpfile = NULL;
     int code,  helpcontext = 0;
-    HRESULT hres, error;
+    HRESULT hres;
 
     TRACE("%s %u...\n", debugstr_variant(args), args_cnt);
 
@@ -3310,20 +3515,14 @@ static HRESULT Err_Raise(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VA
     if(SUCCEEDED(hres)) {
         script_ctx_t *ctx = This->ctx;
 
-        error = (code & ~0xffff) ? map_hres(code) : MAKE_VBSERROR(code);
-
         if(source) {
             SysFreeString(ctx->ei.bstrSource);
             ctx->ei.bstrSource = source;
         }
-        if(!ctx->ei.bstrSource)
-            ctx->ei.bstrSource = get_vbscript_string(VBS_RUNTIME_ERROR);
         if(description) {
             SysFreeString(ctx->ei.bstrDescription);
             ctx->ei.bstrDescription = description;
         }
-        if(!ctx->ei.bstrDescription)
-            ctx->ei.bstrDescription = get_vbscript_error_string(error);
         if(helpfile) {
             SysFreeString(ctx->ei.bstrHelpFile);
             ctx->ei.bstrHelpFile = helpfile;
@@ -3331,7 +3530,9 @@ static HRESULT Err_Raise(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VA
         if(args_cnt >= 5)
             ctx->ei.dwHelpContext = helpcontext;
 
-        ctx->ei.scode = error;
+        ctx->ei.scode = (code & ~0xffff) ? code : MAKE_VBSERROR(code);
+        map_vbs_exception(&ctx->ei);
+
         hres = SCRIPT_E_RECORDED;
     }else {
         SysFreeString(source);
