@@ -99,22 +99,22 @@ static void test_WsCreateServiceProxy(void)
 
     hr = WsCreateServiceProxy( WS_CHANNEL_TYPE_REQUEST, WS_HTTP_CHANNEL_BINDING, NULL, NULL,
                                0, NULL, 0, NULL, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     proxy = NULL;
     hr = WsCreateServiceProxy( WS_CHANNEL_TYPE_REQUEST, WS_HTTP_CHANNEL_BINDING, NULL, NULL,
                                0, NULL, 0, &proxy, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
     ok( proxy != NULL, "proxy not set\n" );
 
     /* write-only property */
     value = 0xdeadbeef;
     hr = WsGetServiceProxyProperty( proxy, WS_PROXY_PROPERTY_CALL_TIMEOUT, &value, sizeof(value), NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     state = 0xdeadbeef;
     hr = WsGetServiceProxyProperty( proxy, WS_PROXY_PROPERTY_STATE, &state, sizeof(state), NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
     ok( state == WS_SERVICE_PROXY_STATE_CREATED, "got %u\n", state );
 
     WsFreeServiceProxy( proxy );
@@ -128,17 +128,17 @@ static void test_WsCreateServiceProxyFromTemplate(void)
 
     hr = WsCreateServiceProxyFromTemplate( WS_CHANNEL_TYPE_REQUEST, NULL, 0, WS_HTTP_BINDING_TEMPLATE_TYPE,
                                            NULL, 0, NULL, 0, NULL, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     hr = WsCreateServiceProxyFromTemplate( WS_CHANNEL_TYPE_REQUEST, NULL, 0, WS_HTTP_BINDING_TEMPLATE_TYPE,
                                            NULL, 0, NULL, 0, &proxy, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     memset( &policy, 0, sizeof(policy) );
     proxy = NULL;
     hr = WsCreateServiceProxyFromTemplate( WS_CHANNEL_TYPE_REQUEST, NULL, 0, WS_HTTP_BINDING_TEMPLATE_TYPE,
                                            NULL, 0, &policy, sizeof(policy), &proxy, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
     ok( proxy != NULL, "proxy not set\n" );
 
     WsFreeServiceProxy( proxy );
@@ -146,7 +146,6 @@ static void test_WsCreateServiceProxyFromTemplate(void)
 
 static void test_WsOpenServiceProxy(void)
 {
-    WCHAR url[] = {'h','t','t','p',':','/','/','l','o','c','a','l','h','o','s','t','/'};
     HRESULT hr;
     WS_SERVICE_PROXY *proxy;
     WS_SERVICE_PROXY_STATE state;
@@ -156,30 +155,30 @@ static void test_WsOpenServiceProxy(void)
     memset( &policy, 0, sizeof(policy) );
     hr = WsCreateServiceProxyFromTemplate( WS_CHANNEL_TYPE_REQUEST, NULL, 0, WS_HTTP_BINDING_TEMPLATE_TYPE,
                                            NULL, 0, &policy, sizeof(policy), &proxy, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     state = 0xdeadbeef;
     hr = WsGetServiceProxyProperty( proxy, WS_PROXY_PROPERTY_STATE, &state, sizeof(state), NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
     ok( state == WS_SERVICE_PROXY_STATE_CREATED, "got %u\n", state );
 
     memset( &addr, 0, sizeof(addr) );
-    addr.url.length = ARRAY_SIZE( url );
-    addr.url.chars  = url;
+    addr.url.length = ARRAY_SIZE( L"http://localhost/" ) - 1;
+    addr.url.chars  = (WCHAR *)L"http://localhost/";
     hr = WsOpenServiceProxy( proxy, &addr, NULL, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     state = 0xdeadbeef;
     hr = WsGetServiceProxyProperty( proxy, WS_PROXY_PROPERTY_STATE, &state, sizeof(state), NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
     ok( state == WS_SERVICE_PROXY_STATE_OPEN, "got %u\n", state );
 
     hr = WsCloseServiceProxy( proxy , NULL, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     state = 0xdeadbeef;
     hr = WsGetServiceProxyProperty( proxy, WS_PROXY_PROPERTY_STATE, &state, sizeof(state), NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
     ok( state == WS_SERVICE_PROXY_STATE_CLOSED, "got %u\n", state );
 
     WsFreeServiceProxy( proxy );
@@ -187,7 +186,6 @@ static void test_WsOpenServiceProxy(void)
 
 static void test_WsResetServiceProxy(void)
 {
-    WCHAR url[] = {'h','t','t','p',':','/','/','l','o','c','a','l','h','o','s','t','/'};
     HRESULT hr;
     WS_SERVICE_PROXY *proxy;
     WS_ENDPOINT_ADDRESS addr;
@@ -195,34 +193,34 @@ static void test_WsResetServiceProxy(void)
 
     hr = WsCreateServiceProxy( WS_CHANNEL_TYPE_REQUEST, WS_HTTP_CHANNEL_BINDING, NULL, NULL,
                                0, NULL, 0, &proxy, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     hr = WsResetServiceProxy( proxy, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     state = 0xdeadbeef;
     hr = WsGetServiceProxyProperty( proxy, WS_PROXY_PROPERTY_STATE, &state, sizeof(state), NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
     ok( state == WS_SERVICE_PROXY_STATE_CREATED, "got %u\n", state );
 
     memset( &addr, 0, sizeof(addr) );
-    addr.url.length = ARRAY_SIZE( url );
-    addr.url.chars  = url;
+    addr.url.length = ARRAY_SIZE( L"http://localhost/" ) - 1;
+    addr.url.chars  = (WCHAR *)L"http://localhost/";
     hr = WsOpenServiceProxy( proxy, &addr, NULL, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     hr = WsResetServiceProxy( proxy, NULL );
-    ok( hr == WS_E_INVALID_OPERATION, "got %08x\n", hr );
+    ok( hr == WS_E_INVALID_OPERATION, "got %#lx\n", hr );
 
     hr = WsCloseServiceProxy( proxy , NULL, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     hr = WsResetServiceProxy( proxy, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     state = 0xdeadbeef;
     hr = WsGetServiceProxyProperty( proxy, WS_PROXY_PROPERTY_STATE, &state, sizeof(state), NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
     ok( state == WS_SERVICE_PROXY_STATE_CREATED, "got %u\n", state );
 
     WsFreeServiceProxy( proxy );
@@ -230,8 +228,6 @@ static void test_WsResetServiceProxy(void)
 
 static HRESULT create_channel( int port, WS_CHANNEL **ret )
 {
-    static const WCHAR fmt[] =
-        {'h','t','t','p',':','/','/','1','2','7','.','0','.','0','.','1',':','%','u',0};
     WS_CHANNEL_PROPERTY prop[2];
     WS_ENVELOPE_VERSION env_version = WS_ENVELOPE_VERSION_SOAP_1_1;
     WS_ADDRESSING_VERSION addr_version = WS_ADDRESSING_VERSION_TRANSPORT;
@@ -253,7 +249,7 @@ static HRESULT create_channel( int port, WS_CHANNEL **ret )
     if (hr != S_OK) return hr;
 
     memset( &addr, 0, sizeof(addr) );
-    addr.url.length = wsprintfW( buf, fmt, port );
+    addr.url.length = wsprintfW( buf, L"http://127.0.0.1:%u", port );
     addr.url.chars  = buf;
     hr = WsOpenChannel( channel, &addr, NULL, NULL );
     if (hr == S_OK) *ret = channel;
@@ -282,27 +278,27 @@ static void test_WsSendMessage( int port, WS_XML_STRING *action )
     HRESULT hr;
 
     hr = create_channel( port, &channel );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     hr = WsCreateMessageForChannel( channel, NULL, 0, &msg, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     set_elem_desc( &body, &req, &ns, WS_INT32_TYPE, NULL );
     set_msg_desc( &desc, action, &body );
     hr = WsSendMessage( NULL, msg, &desc, WS_WRITE_REQUIRED_VALUE, &val, sizeof(val), NULL, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     hr = WsSendMessage( channel, NULL, &desc, WS_WRITE_REQUIRED_VALUE, &val, sizeof(val), NULL, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     hr = WsSendMessage( channel, msg, NULL, WS_WRITE_REQUIRED_VALUE, &val, sizeof(val), NULL, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     hr = WsSendMessage( channel, msg, &desc, WS_WRITE_REQUIRED_VALUE, &val, sizeof(val), NULL, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     hr = WsCloseChannel( channel, NULL, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     WsFreeChannel( channel );
     WsFreeMessage( msg );
@@ -320,42 +316,42 @@ static void test_WsReceiveMessage( int port )
     HRESULT hr;
 
     hr = create_channel( port, &channel );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     hr = WsCreateMessageForChannel( channel, NULL, 0, &msg, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     set_elem_desc( &body, &req, &ns, WS_INT32_TYPE, NULL );
     set_msg_desc( &desc_req, &req, &body );
     hr = WsSendMessage( channel, msg, &desc_req, WS_WRITE_REQUIRED_VALUE, &val, sizeof(val), NULL, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
     WsFreeMessage( msg );
 
     hr = WsCreateMessageForChannel( channel, NULL, 0, &msg, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     set_elem_desc( &body, &resp, &ns, WS_INT32_TYPE, NULL );
     set_msg_desc( &desc_resp, &resp, &body );
     desc[0] = &desc_resp;
     hr = WsReceiveMessage( NULL, msg, desc, 1, WS_RECEIVE_REQUIRED_MESSAGE, WS_READ_REQUIRED_VALUE,
                            NULL, &val, sizeof(val), NULL, NULL, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     hr = WsReceiveMessage( channel, NULL, desc, 1, WS_RECEIVE_REQUIRED_MESSAGE, WS_READ_REQUIRED_VALUE,
                            NULL, &val, sizeof(val), NULL, NULL, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     hr = WsReceiveMessage( channel, msg, NULL, 1, WS_RECEIVE_REQUIRED_MESSAGE, WS_READ_REQUIRED_VALUE,
                            NULL, &val, sizeof(val), NULL, NULL, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     hr = WsReceiveMessage( channel, msg, desc, 1, WS_RECEIVE_REQUIRED_MESSAGE, WS_READ_REQUIRED_VALUE,
                            NULL, &val, sizeof(val), NULL, NULL, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
     ok( val == -2, "got %d\n", val );
 
     hr = WsCloseChannel( channel, NULL, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     WsFreeChannel( channel );
     WsFreeMessage( msg );
@@ -383,8 +379,6 @@ static WS_HTTP_HEADER_MAPPING *response_header_mappings[] =
 
 static HRESULT create_proxy( int port, WS_SERVICE_PROXY **ret )
 {
-    static const WCHAR fmt[] =
-        {'h','t','t','p',':','/','/','1','2','7','.','0','.','0','.','1',':','%','u','/',0};
     WS_ENVELOPE_VERSION env_version;
     WS_ADDRESSING_VERSION addr_version;
     WS_HTTP_MESSAGE_MAPPING mapping;
@@ -421,7 +415,7 @@ static HRESULT create_proxy( int port, WS_SERVICE_PROXY **ret )
     if (hr != S_OK) return hr;
 
     memset( &addr, 0, sizeof(addr) );
-    addr.url.length = wsprintfW( url, fmt, port );
+    addr.url.length = wsprintfW( url, L"http://127.0.0.1:%u", port );
     addr.url.chars  = url;
     hr = WsOpenServiceProxy( proxy, &addr, NULL, NULL );
     if (hr == S_OK) *ret = proxy;
@@ -444,20 +438,20 @@ static void check_output_headers( WS_MESSAGE *msg )
     HRESULT hr;
 
     hr = WsCreateWriter( NULL, 0, &writer, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     hr = set_output( writer );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     hr = WsGetMessageProperty( msg, WS_MESSAGE_PROPERTY_HEADER_BUFFER, &buf, sizeof(buf), NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     hr = WsWriteXmlBuffer( writer, buf, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     memset( &bytes, 0, sizeof(bytes) );
     hr = WsGetWriterProperty( writer, WS_XML_WRITER_PROPERTY_BYTES, &bytes, sizeof(bytes), NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
     WsFreeWriter( writer );
 }
 
@@ -477,7 +471,7 @@ static HRESULT CALLBACK send_callback( WS_MESSAGE *msg, WS_HEAP *heap, void *sta
     HRESULT hr;
 
     hr = WsAddMappedHeader( msg, &header, WS_XML_STRING_TYPE, WS_WRITE_REQUIRED_VALUE, &value, sizeof(value), NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
     check_output_headers( msg );
     return S_OK;
 }
@@ -485,21 +479,19 @@ static HRESULT CALLBACK send_callback( WS_MESSAGE *msg, WS_HEAP *heap, void *sta
 static HRESULT CALLBACK recv_callback( WS_MESSAGE *msg, WS_HEAP *heap, void *state, WS_ERROR *error )
 {
     static const WS_XML_STRING header = {20, (BYTE *)"MappedResponseHeader"};
-    static const WCHAR valueW[] = {'v','a','l','u','e',0};
     WCHAR *str;
     HRESULT hr;
 
     check_output_headers( msg );
     hr = WsGetMappedHeader( msg, &header, WS_SINGLETON_HEADER, 0, WS_WSZ_TYPE, WS_READ_OPTIONAL_POINTER, heap,
                             &str, sizeof(str), NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
-    ok( !lstrcmpW(str, valueW), "wrong value %s\n", wine_dbgstr_w(str) );
+    ok( hr == S_OK, "got %#lx\n", hr );
+    ok( !wcscmp(str, L"value"), "wrong value %s\n", wine_dbgstr_w(str) );
     return S_OK;
 }
 
 static void test_WsCall( int port )
 {
-    static const WCHAR testW[] = {'t','e','s','t',0}, test2W[] = {'t','e','s','t','2',0};
     WS_XML_STRING str = {3, (BYTE *)"str"};
     WS_XML_STRING req = {3, (BYTE *)"req"};
     WS_XML_STRING resp = {4, (BYTE *)"resp"};
@@ -540,16 +532,16 @@ static void test_WsCall( int port )
     } out;
 
     hr = WsCreateHeap( 1 << 16, 0, NULL, 0, &heap, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     hr = WsCall( NULL, NULL, NULL, NULL, NULL, 0, NULL, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     hr = create_proxy( port, &proxy );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     hr = WsCall( proxy, NULL, NULL, NULL, NULL, 0, NULL, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     set_field_desc( &f, WS_ELEMENT_FIELD_MAPPING, &val, &ns, WS_INT32_TYPE, NULL, 0, 0, 0, NULL, NULL );
     set_field_desc( &f4, WS_REPEATING_ELEMENT_FIELD_MAPPING, NULL, NULL, WS_WSZ_TYPE, NULL,
@@ -581,11 +573,11 @@ static void test_WsCall( int port )
 
     set_op_desc( &op, &input_msg, &output_msg, 6, param );
     hr = WsCall( proxy, &op, NULL, NULL, NULL, 0, NULL, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     in.val   = 1;
-    str_array[0] = testW;
-    str_array[1] = test2W;
+    str_array[0] = L"test";
+    str_array[1] = L"test2";
     in.str   = str_array;
     in.count = 2;
 
@@ -619,14 +611,14 @@ static void test_WsCall( int port )
     prop[1].valueSize = sizeof(ctx_recv);
 
     hr = WsCall( proxy, &op, args, heap, prop, ARRAY_SIZE(prop), NULL, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
-    ok( !lstrcmpW( out.str, testW ), "wrong data\n" );
-    ok( out.count == 2, "got %u\n", out.count );
+    ok( hr == S_OK, "got %#lx\n", hr );
+    ok( !wcscmp( out.str, L"test" ), "wrong data\n" );
+    ok( out.count == 2, "got %lu\n", out.count );
     ok( out.val[0] == 1, "got %u\n", out.val[0] );
     ok( out.val[1] == 2, "got %u\n", out.val[1] );
 
     hr = WsCloseServiceProxy( proxy, NULL, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     WsFreeServiceProxy( proxy );
     WsFreeHeap( heap );
@@ -667,10 +659,10 @@ static void test_empty_response( int port )
     } in;
 
     hr = WsCreateHeap( 1 << 16, 0, NULL, 0, &heap, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     hr = create_proxy( port, &proxy );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     set_field_desc( &f, WS_ELEMENT_FIELD_MAPPING, &val, &ns, WS_INT32_TYPE, NULL, 0, 0, 0, NULL, NULL );
     fields[0] = &f;
@@ -689,20 +681,216 @@ static void test_empty_response( int port )
     in.val = 1;
     args[0] = &in.val;
     hr = WsCall( proxy, &op, args, heap, NULL, 0, NULL, NULL );
-    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
 
     set_elem_desc( &output_elem, &resp_elem, &ns, WS_STRUCT_TYPE, &output_struct );
     hr = WsCall( proxy, &op, args, heap, NULL, 0, NULL, NULL );
-    ok( hr == WS_E_INVALID_FORMAT, "got %08x\n", hr );
+    ok( hr == WS_E_INVALID_FORMAT, "got %#lx\n", hr );
 
     hr = WsCloseServiceProxy( proxy, NULL, NULL );
-    ok( hr == S_OK, "got %08x\n", hr );
+    ok( hr == S_OK, "got %#lx\n", hr );
 
     WsFreeServiceProxy( proxy );
     WsFreeHeap( heap );
 }
 
+static const char req_test4[] =
+    "<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\"><s:Body>"
+    "<req_test4 xmlns=\"ns\"><str>test4</str><val>1</val><val>2</val><val>3</val></req_test4>"
+    "</s:Body></s:Envelope>";
+
+static const char resp_test4[] =
+    "<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\"><s:Body>"
+    "<resp_test4 xmlns=\"ns\"><str>out4</str><val>4</val><val>5</val></resp_test4>"
+    "</s:Body></s:Envelope>";
+
+static void test_inout_params( int port )
+{
+    WS_XML_STRING str = {3, (BYTE *)"str"};
+    WS_XML_STRING req = {3, (BYTE *)"req"};
+    WS_XML_STRING resp = {4, (BYTE *)"resp"};
+    WS_XML_STRING req_elem = {9, (BYTE *)"req_test4"};
+    WS_XML_STRING resp_elem = {10, (BYTE *)"resp_test4"};
+    WS_XML_STRING req_action = {9, (BYTE *)"req_test4"};
+    WS_XML_STRING resp_action = {10, (BYTE *)"resp_test4"};
+    WS_XML_STRING val = {3, (BYTE *)"val"};
+    WS_XML_STRING ns = {2, (BYTE *)"ns"};
+    HRESULT hr;
+    WS_SERVICE_PROXY *proxy;
+    WS_OPERATION_DESCRIPTION op;
+    WS_MESSAGE_DESCRIPTION input_msg, output_msg;
+    WS_ELEMENT_DESCRIPTION input_elem, output_elem;
+    WS_STRUCT_DESCRIPTION input_struct, output_struct;
+    WS_FIELD_DESCRIPTION f, f2, *fields[2];
+    WS_PARAMETER_DESCRIPTION param[3];
+    const void *args[3];
+    WS_HEAP *heap;
+    INT32 val_array[] = {1, 2, 3};
+    WCHAR **str_ptr;
+    INT32 **val_ptr;
+    ULONG *count_ptr;
+    struct data
+    {
+        WCHAR *str;
+        ULONG  count;
+        INT32 *val;
+    } data;
+
+    hr = WsCreateHeap( 1 << 16, 0, NULL, 0, &heap, NULL );
+    ok( hr == S_OK, "got %#lx\n", hr );
+
+    hr = WsCall( NULL, NULL, NULL, NULL, NULL, 0, NULL, NULL );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
+
+    hr = create_proxy( port, &proxy );
+    ok( hr == S_OK, "got %#lx\n", hr );
+
+    hr = WsCall( proxy, NULL, NULL, NULL, NULL, 0, NULL, NULL );
+    ok( hr == E_INVALIDARG, "got %#lx\n", hr );
+
+    set_field_desc( &f, WS_ELEMENT_FIELD_MAPPING, &str, &ns, WS_WSZ_TYPE, NULL,
+                    FIELD_OFFSET(struct data, str), 0, 0, NULL, NULL );
+    set_field_desc( &f2, WS_REPEATING_ELEMENT_FIELD_MAPPING, NULL, NULL, WS_INT32_TYPE, NULL,
+                    FIELD_OFFSET(struct data, val), 0, FIELD_OFFSET(struct data, count), &val, &ns );
+
+    fields[0] = &f;
+    fields[1] = &f2;
+
+    set_struct_desc( &input_struct, sizeof(struct data), TYPE_ALIGNMENT(struct data), fields, 2, &req, &ns, 0 );
+    set_elem_desc( &input_elem, &req_elem, &ns, WS_STRUCT_TYPE, &input_struct );
+    set_msg_desc( &input_msg, &req_action, &input_elem );
+
+    set_struct_desc( &output_struct, sizeof(struct data), TYPE_ALIGNMENT(struct data), fields, 2, &resp, &ns, 0 );
+    set_elem_desc( &output_elem, &resp_elem, &ns, WS_STRUCT_TYPE, &output_struct );
+    set_msg_desc( &output_msg, &resp_action, &output_elem );
+
+    set_param_desc( &param[0], WS_PARAMETER_TYPE_NORMAL, 0, 0 );
+    set_param_desc( &param[1], WS_PARAMETER_TYPE_ARRAY, 1, 1 );
+    set_param_desc( &param[2], WS_PARAMETER_TYPE_ARRAY_COUNT, 1, 1 );
+
+    set_op_desc( &op, &input_msg, &output_msg, 3, param );
+
+    data.str   = (WCHAR *) L"test4";
+    data.count = ARRAY_SIZE(val_array);
+    data.val   = val_array;
+
+    str_ptr   = &data.str;
+    val_ptr   = &data.val;
+    count_ptr = &data.count;
+
+    args[0] = &str_ptr;
+    args[1] = &val_ptr;
+    args[2] = &count_ptr;
+
+    hr = WsCall( proxy, &op, args, heap, NULL, 0, NULL, NULL );
+    ok( hr == S_OK, "got %#lx\n", hr );
+    ok( !wcscmp( data.str, L"out4" ), "wrong str\n" );
+    ok( data.count == 2, "got %lu\n", data.count );
+    ok( data.val[0] == 4, "got %u\n", data.val[0] );
+    ok( data.val[1] == 5, "got %u\n", data.val[1] );
+
+    hr = WsCloseServiceProxy( proxy, NULL, NULL );
+    ok( hr == S_OK, "got %#lx\n", hr );
+
+    WsFreeServiceProxy( proxy );
+    WsFreeHeap( heap );
+}
+
+static const char req_test5[] =
+    "<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\"><s:Body>"
+    "<req_test5 xmlns=\"ns\"><val>1</val></req_test5>"
+    "</s:Body></s:Envelope>";
+
+static const char resp_test5[] =
+    "<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\"><s:Body>"
+    "<s:Fault><faultcode>s:Client</faultcode><faultstring>OLS Exception</faultstring><detail>"
+    "<ServerFault xmlns=\"http://schemas.microsoft.com/office/licensingservice/API/2012/01/ClientApi\" "
+    "xmlns:a=\"http://schemas.datacontract.org/2004/07/Microsoft.Office.LicensingService\" "
+    "xmlns:i=\"http://www.w3.org/2001/XMLSchema-instance\"><a:ErrorCode>1030</a:ErrorCode><a:Message/>"
+    "<a:Url>https://portal.office.com/Account/#installs</a:Url></ServerFault></detail>"
+    "</s:Fault></s:Body></s:Envelope>";
+
+static void test_fault_response( int port )
+{
+    WS_XML_STRING faultcode = {6, (BYTE *)"Client"};
+    WS_STRING faultstring = {13, (WCHAR *)L"OLS Exception"};
+    WS_XML_STRING req = {3, (BYTE *)"req"};
+    WS_XML_STRING resp = {4, (BYTE *)"resp"};
+    WS_XML_STRING req_action = {9, (BYTE *)"req_test5"};
+    WS_XML_STRING resp_action = {10, (BYTE *)"resp_test5"};
+    WS_XML_STRING req_elem = {9, (BYTE *)"req_test5"};
+    WS_XML_STRING resp_elem = {10, (BYTE *)"resp_test5"};
+    WS_XML_STRING ns = {2, (BYTE *)"ns"};
+    WS_XML_STRING ns2 = {0, (BYTE *)""};
+    WS_XML_STRING val = {3, (BYTE *)"val"};
+    HRESULT hr;
+    WS_SERVICE_PROXY *proxy;
+    WS_FIELD_DESCRIPTION f, *fields[1];
+    WS_STRUCT_DESCRIPTION input_struct, output_struct;
+    WS_ELEMENT_DESCRIPTION input_elem, output_elem;
+    WS_MESSAGE_DESCRIPTION input_msg, output_msg;
+    WS_PARAMETER_DESCRIPTION param[1];
+    WS_OPERATION_DESCRIPTION op;
+    const void *args[1];
+    WS_HEAP *heap;
+    struct input
+    {
+        INT32 val;
+    } in;
+    WS_ERROR *error;
+    WS_FAULT *fault;
+
+    hr = WsCreateHeap( 1 << 16, 0, NULL, 0, &heap, NULL );
+    ok( hr == S_OK, "got %#lx\n", hr );
+
+    hr = create_proxy( port, &proxy );
+    ok( hr == S_OK, "got %#lx\n", hr );
+
+    set_field_desc( &f, WS_ELEMENT_FIELD_MAPPING, &val, &ns, WS_INT32_TYPE, NULL, 0, 0, 0, NULL, NULL );
+    fields[0] = &f;
+
+    set_struct_desc( &input_struct, sizeof(struct input), TYPE_ALIGNMENT(struct input), fields, 1, &req, &ns, 0 );
+    set_elem_desc( &input_elem, &req_elem, &ns, WS_STRUCT_TYPE, &input_struct );
+    set_msg_desc( &input_msg, &req_action, &input_elem );
+
+    set_struct_desc( &output_struct, 0, 1, NULL, 0, &resp, &ns2, 0x6 );
+    set_elem_desc( &output_elem, &resp_elem, &ns, WS_STRUCT_TYPE, &output_struct );
+    set_msg_desc( &output_msg, &resp_action, &output_elem );
+
+    set_param_desc( param, WS_PARAMETER_TYPE_NORMAL, 0, 0xffff );
+    set_op_desc( &op, &input_msg, &output_msg, 1, param );
+
+    in.val = 1;
+    args[0] = &in.val;
+
+    hr = WsCreateError( NULL, 0, &error );
+    ok( hr == S_OK, "got %#lx\n", hr );
+
+    hr = WsCall( proxy, &op, args, heap, NULL, 0, NULL, error );
+    ok( hr == WS_E_ENDPOINT_FAULT_RECEIVED, "got %#lx\n", hr );
+
+    hr = WsGetFaultErrorProperty( error, WS_FAULT_ERROR_PROPERTY_FAULT, &fault, sizeof(fault) );
+    ok( hr == S_OK, "got %#lx\n", hr );
+    ok( fault != NULL, "fault not set\n" );
+    ok( fault->code->value.localName.length == faultcode.length, "got %lu\n", fault->code->value.localName.length );
+    ok( !memcmp( fault->code->value.localName.bytes, faultcode.bytes, faultcode.length ), "wrong fault code\n" );
+    ok( !fault->code->subCode, "subcode is not NULL\n" );
+    ok( fault->reasonCount == 1, "got %lu\n", fault->reasonCount );
+    ok( fault->reasons[0].text.length == faultstring.length, "got %lu\n", fault->reasons[0].text.length );
+    ok( !memcmp( fault->reasons[0].text.chars, faultstring.chars, faultstring.length * sizeof(WCHAR) ),
+        "wrong fault string\n" );
+    ok( fault->detail != NULL, "fault detail not set\n" );
+
+    hr = WsCloseServiceProxy( proxy, NULL, NULL );
+    ok( hr == S_OK, "got %#lx\n", hr );
+
+    WsFreeError( error );
+    WsFreeServiceProxy( proxy );
+    WsFreeHeap( heap );
+}
+
 static const char status_200[] = "HTTP/1.1 200 OK\r\n";
+static const char status_500[] = "HTTP/1.1 500 Internal Server Error\r\n";
 
 static const struct
 {
@@ -718,6 +906,8 @@ tests[] =
     { "req_test1", req_test1, sizeof(req_test1)-1, status_200, resp_test1, sizeof(resp_test1)-1 },
     { "req_test2", req_test2, sizeof(req_test2)-1, status_200, resp_test2, sizeof(resp_test2)-1 },
     { "req_test3", req_test3, sizeof(req_test3)-1, status_200, resp_test3, sizeof(resp_test3)-1 },
+    { "req_test4", req_test4, sizeof(req_test4)-1, status_200, resp_test4, sizeof(resp_test4)-1 },
+    { "req_test5", req_test5, sizeof(req_test5)-1, status_500, resp_test5, sizeof(resp_test5)-1 },
 };
 
 static void send_response( int c, const char *status, const char *data, unsigned int len )
@@ -837,10 +1027,10 @@ START_TEST(proxy)
     info.port  = 7533;
     info.event = CreateEventW( NULL, 0, 0, NULL );
     thread = CreateThread( NULL, 0, server_proc, &info, 0, NULL );
-    ok( thread != NULL, "failed to create server thread %u\n", GetLastError() );
+    ok( thread != NULL, "failed to create server thread %lu\n", GetLastError() );
 
     ret = WaitForSingleObject( info.event, 3000 );
-    ok(ret == WAIT_OBJECT_0, "failed to start test server %u\n", GetLastError());
+    ok(ret == WAIT_OBJECT_0, "failed to start test server %lu\n", GetLastError());
     if (ret != WAIT_OBJECT_0)
     {
         CloseHandle(thread);
@@ -851,6 +1041,8 @@ START_TEST(proxy)
     test_WsReceiveMessage( info.port );
     test_WsCall( info.port );
     test_empty_response( info.port );
+    test_inout_params( info.port );
+    test_fault_response( info.port );
 
     test_WsSendMessage( info.port, &quit );
     WaitForSingleObject( thread, 3000 );
