@@ -324,6 +324,12 @@ enum AsyncContentLoadedState {
     AsyncContentLoadedState_Completed = 0x02,
 };
 
+enum NormalizeState {
+    NormalizeState_None   = 0x00,
+    NormalizeState_View   = 0x01,
+    NormalizeState_Custom = 0x02,
+};
+
 struct UiaEventArgs {
     enum EventArgsType Type;
     int EventId;
@@ -379,7 +385,15 @@ struct UiaWindowClosedEventArgs {
     int cRuntimeIdLen;
 };
 
+struct UiaFindParams {
+    int MaxDepth;
+    BOOL FindFirst;
+    BOOL ExcludeRoot;
+    struct UiaCondition *pFindCondition;
+};
+
 typedef SAFEARRAY * WINAPI UiaProviderCallback(HWND hwnd,enum ProviderType providerType);
+typedef void WINAPI UiaEventCallback(struct UiaEventArgs *pArgs,SAFEARRAY *pRequestedData,BSTR pTreeStructure);
 
 HRESULT WINAPI UiaGetReservedMixedAttributeValue(IUnknown **value);
 HRESULT WINAPI UiaGetReservedNotSupportedValue(IUnknown **value);
@@ -397,6 +411,19 @@ HRESULT WINAPI UiaNodeFromProvider(IRawElementProviderSimple *elprov, HUIANODE *
 BOOL WINAPI UiaNodeRelease(HUIANODE huianode);
 HRESULT WINAPI UiaGetRuntimeId(HUIANODE huianode, SAFEARRAY **runtime_id);
 HRESULT WINAPI UiaHUiaNodeFromVariant(VARIANT *in_val, HUIANODE *huianode);
+HRESULT WINAPI UiaNodeFromHandle(HWND hwnd, HUIANODE *huianode);
+HRESULT WINAPI UiaDisconnectProvider(IRawElementProviderSimple *elprov);
+HRESULT WINAPI UiaGetUpdatedCache(HUIANODE huianode, struct UiaCacheRequest *cache_req, enum NormalizeState normalize_state,
+        struct UiaCondition *normalize_cond, SAFEARRAY **out_req, BSTR *tree_struct);
+HRESULT WINAPI UiaNavigate(HUIANODE huianode, enum NavigateDirection dir, struct UiaCondition *nav_condition,
+        struct UiaCacheRequest *cache_req, SAFEARRAY **out_req, BSTR *tree_struct);
+HRESULT WINAPI UiaFind(HUIANODE huianode, struct UiaFindParams *find_params, struct UiaCacheRequest *cache_req, SAFEARRAY **out_req,
+        SAFEARRAY **out_offsets, SAFEARRAY **out_tree_structs);
+HRESULT WINAPI UiaAddEvent(HUIANODE huianode, EVENTID event_id, UiaEventCallback *callback, enum TreeScope scope,
+        PROPERTYID *prop_ids, int prop_ids_count, struct UiaCacheRequest *cache_req, HUIAEVENT *huiaevent);
+HRESULT WINAPI UiaRemoveEvent(HUIAEVENT huiaevent);
+HRESULT WINAPI UiaEventAddWindow(HUIAEVENT huiaevent, HWND hwnd);
+HRESULT WINAPI UiaEventRemoveWindow(HUIAEVENT huiaevent, HWND hwnd);
 
 #ifdef __cplusplus
 }
