@@ -70,6 +70,9 @@ typedef struct
     LPITEMIDLIST pidl;
     IImageList *icon_list;
     DWORD advise_cookie;
+
+    IShellWindows *sw;
+    LONG sw_cookie;
 } explorer_info;
 
 enum
@@ -77,6 +80,13 @@ enum
     BACK_BUTTON,FORWARD_BUTTON,UP_BUTTON
 };
 
+static void variant_from_pidl(VARIANT *var, const ITEMIDLIST *pidl)
+{
+    V_VT(var) = VT_ARRAY | VT_UI1;
+    V_ARRAY(var) = SafeArrayCreateVector(VT_UI1, 0, ILGetSize(pidl));
+    memcpy(V_ARRAY(var)->pvData, pidl, ILGetSize(pidl));
+}
+
 typedef struct
 {
     IExplorerBrowserEvents IExplorerBrowserEvents_iface;
@@ -265,6 +275,15 @@ static HRESULT WINAPI IExplorerBrowserEventsImpl_fnOnNavigationComplete(IExplore
     STRRET strret;
     WCHAR *name;
 
+    if (This->info->sw)
+    {
+        VARIANT var;
+
+        variant_from_pidl(&var, pidl);
+        IShellWindows_OnNavigate(This->info->sw, This->info->sw_cookie, &var);
+        VariantClear(&var);
+    }
+
     ILFree(This->info->pidl);
     This->info->pidl = ILClone(pidl);
     update_path_box(This->info);
@@ -325,7 +344,35 @@ static IExplorerBrowserEvents *make_explorer_events(explorer_info *info)
     return &ret->IExplorerBrowserEvents_iface;
 }
 
-static void make_explorer_window(IShellFolder* startFolder)
+static IShellFolder *get_starting_shell_folder(WCHAR *path)
+{
+    IShellFolder* desktop,*folder;
+    LPITEMIDLIST root_pidl;
+    HRESULT hres;
+
+    SHGetDesktopFolder(&desktop);
+
+    if (!path)
+        return desktop;
+
+    hres = IShellFolder_ParseDisplayName(desktop, NULL, NULL, path, NULL, &root_pidl, NULL);
+    if(FAILED(hres))
+    {
+        return desktop;
+    }
+    hres = IShellFolder_BindToObject(desktop,root_pidl,NULL,
+                                     &IID_IShellFolder,
+                                     (void**)&folder);
+    ILFree(root_pidl);
+    if(FAILED(hres))
+    {
+        return desktop;
+    }
+    IShellFolder_Release(desktop);
+    return folder;
+}
+
+static void make_explorer_window(parameters_struct *params)
 {
     RECT rect;
     HWND rebar,nav_toolbar;
@@ -339,8 +386,52 @@ static void make_explorer_window(IShellFolder* startFolder)
     TBBUTTON nav_buttons[3];
     int hist_offset,view_offset;
     REBARBANDINFOW band_info;
+    VARIANT var, empty_var;
+    IShellFolder *folder;
+    IDispatch *dispatch;
+    WCHAR *path = NULL;
+    IShellWindows *sw;
+    ITEMIDLIST *pidl;
     UINT dpix, dpiy;
+    DWORD size;
+    LONG hwnd;
     HDC hdc;
+    MSG msg;
+
+    CoCreateInstance(&CLSID_ShellWindows, NULL, CLSCTX_LOCAL_SERVER,
+            &IID_IShellWindows, (void **)&sw);
+
+    if (params->root[0])
+    {
+        size = GetFullPathNameW(params->root, 0, NULL, NULL);
+        path = malloc( size * sizeof(WCHAR) );
+        GetFullPathNameW(params->root, size, path, NULL);
+    }
+
+    if (sw && path)
+    {
+        if (!(pidl = ILCreateFromPathW(path)))
+        {
+            ERR("Failed to create PIDL for %s.\n", debugstr_w(path));
+            IShellWindows_Release(sw);
+            free(path);
+            return;
+        }
+
+        variant_from_pidl(&var, pidl);
+        V_VT(&empty_var) = VT_EMPTY;
+        hres = IShellWindows_FindWindowSW(sw, &var, &empty_var, SWC_EXPLORER, &hwnd, 0, &dispatch);
+        VariantClear(&var);
+        ILFree(pidl);
+        if (hres == S_OK)
+        {
+            TRACE("Found window %#lx already browsing path %s.\n", hwnd, debugstr_w(path));
+            SetForegroundWindow((HWND)(LONG_PTR)hwnd);
+            IShellWindows_Release(sw);
+            free(path);
+            return;
+        }
+    }
 
     memset(nav_buttons,0,sizeof(nav_buttons));
 
@@ -360,6 +451,8 @@ static void make_explorer_window(IShellFolder* startFolder)
     if(!info)
     {
         WINE_ERR("Could not allocate an explorer_info struct\n");
+        IShellWindows_Release(sw);
+        free(path);
         return;
     }
     hres = CoCreateInstance(&CLSID_ExplorerBrowser,NULL,CLSCTX_INPROC_SERVER,
@@ -368,6 +461,8 @@ static void make_explorer_window(IShellFolder* startFolder)
     {
         WINE_ERR("Could not obtain an instance of IExplorerBrowser\n");
         HeapFree(GetProcessHeap(),0,info);
+        IShellWindows_Release(sw);
+        free(path);
         return;
     }
     info->rebar_height=0;
@@ -376,6 +471,12 @@ static void make_explorer_window(IShellFolder* startFolder)
                         CW_USEDEFAULT,CW_USEDEFAULT,default_width,
                         default_height,NULL,NULL,explorer_hInstance,NULL);
 
+    if (sw)
+    {
+        IShellWindows_Register(sw, NULL, (LONG_PTR)info->main_window, SWC_EXPLORER, &info->sw_cookie);
+        info->sw = sw;
+    }
+
     fs.ViewMode = FVM_DETAILS;
     fs.fFlags = FWF_AUTOARRANGE;
 
@@ -439,11 +540,21 @@ static void make_explorer_window(IShellFolder* startFolder)
     SendMessageW(rebar,RB_INSERTBANDW,-1,(LPARAM)&band_info);
     events = make_explorer_events(info);
     IExplorerBrowser_Advise(info->browser,events,&info->advise_cookie);
-    IExplorerBrowser_BrowseToObject(info->browser,(IUnknown*)startFolder,
-                                    SBSP_ABSOLUTE);
+
+    folder = get_starting_shell_folder(path);
+    IExplorerBrowser_BrowseToObject(info->browser, (IUnknown *)folder, SBSP_ABSOLUTE);
+    IShellFolder_Release(folder);
+    free(path);
+
     ShowWindow(info->main_window,SW_SHOWDEFAULT);
     UpdateWindow(info->main_window);
     IExplorerBrowserEvents_Release(events);
+
+    while (GetMessageW(&msg, NULL, 0, 0))
+    {
+        TranslateMessage(&msg);
+        DispatchMessageW(&msg);
+    }
 }
 
 static void update_window_size(explorer_info *info, int height, int width)
@@ -556,18 +667,61 @@ static LRESULT explorer_on_notify(explorer_info* info,NMHDR* notification)
     return 0;
 }
 
+static BOOL handle_copydata(const explorer_info *info, const COPYDATASTRUCT *cds)
+{
+    static const unsigned int magic = 0xe32ee32e;
+    unsigned int i, flags, count;
+    const ITEMIDLIST *child;
+    unsigned char *ptr;
+    IShellView *sv;
+    SVSIF sv_flags;
+
+    TRACE("\n");
+
+    /* For SHOpenFolderAndSelectItems() */
+    if (cds->dwData != magic)
+        return FALSE;
+
+    ptr = cds->lpData;
+    memcpy(&count, ptr, sizeof(count));
+    ptr += sizeof(count);
+    memcpy(&flags, ptr, sizeof(flags));
+    ptr += sizeof(flags);
+
+    sv_flags = flags & OFASI_EDIT ? SVSI_EDIT : SVSI_SELECT;
+
+    IExplorerBrowser_GetCurrentView(info->browser, &IID_IShellView, (void **)&sv);
+    for (i = 0; i < count; ++i)
+    {
+        child = (const ITEMIDLIST *)ptr;
+        if (i == 0)
+            IShellView_SelectItem(sv, child, sv_flags | SVSI_ENSUREVISIBLE | SVSI_FOCUSED | SVSI_DESELECTOTHERS);
+        else
+            IShellView_SelectItem(sv, child, sv_flags);
+        ptr += ILGetSize(child);
+    }
+    IShellView_Release(sv);
+    return TRUE;
+}
+
 static LRESULT CALLBACK explorer_wnd_proc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
     explorer_info *info
         = (explorer_info*)GetWindowLongPtrW(hwnd,EXPLORER_INFO_INDEX);
     IExplorerBrowser *browser = NULL;
 
-    WINE_TRACE("(hwnd=%p,uMsg=%u,wParam=%lx,lParam=%lx)\n",hwnd,uMsg,wParam,lParam);
+    WINE_TRACE("(hwnd=%p,uMsg=%u,wParam=%Ix,lParam=%Ix)\n",hwnd,uMsg,wParam,lParam);
     if(info)
         browser = info->browser;
     switch(uMsg)
     {
     case WM_DESTROY:
+        if(info->sw)
+        {
+            IShellWindows_Revoke(info->sw, info->sw_cookie);
+            IShellWindows_Release(info->sw);
+        }
+
         IExplorerBrowser_Unadvise(browser,info->advise_cookie);
         IExplorerBrowser_Destroy(browser);
         IExplorerBrowser_Release(browser);
@@ -601,6 +755,8 @@ static LRESULT CALLBACK explorer_wnd_proc(HWND hwnd, UINT uMsg, WPARAM wParam, L
     case WM_SIZE:
         update_window_size(info,HIWORD(lParam),LOWORD(lParam));
         break;
+    case WM_COPYDATA:
+        return handle_copydata(info, (const COPYDATASTRUCT *)lParam);
     default:
         return DefWindowProcW(hwnd,uMsg,wParam,lParam);
     }
@@ -625,47 +781,6 @@ static void register_explorer_window_class(void)
     RegisterClassExW(&window_class);
 }
 
-static IShellFolder* get_starting_shell_folder(parameters_struct* params)
-{
-    IShellFolder* desktop,*folder;
-    LPITEMIDLIST root_pidl;
-    WCHAR *fullpath = NULL;
-    HRESULT hres;
-    DWORD size;
-
-    SHGetDesktopFolder(&desktop);
-    if (!params->root[0])
-    {
-        return desktop;
-    }
-
-    size = GetFullPathNameW(params->root, 0, fullpath, NULL);
-    if (!size)
-        return desktop;
-    fullpath = heap_alloc(size * sizeof(WCHAR));
-    GetFullPathNameW(params->root, size, fullpath, NULL);
-
-    hres = IShellFolder_ParseDisplayName(desktop,NULL,NULL,
-                                         fullpath,NULL,
-                                         &root_pidl,NULL);
-    heap_free(fullpath);
-
-    if(FAILED(hres))
-    {
-        return desktop;
-    }
-    hres = IShellFolder_BindToObject(desktop,root_pidl,NULL,
-                                     &IID_IShellFolder,
-                                     (void**)&folder);
-    ILFree(root_pidl);
-    if(FAILED(hres))
-    {
-        return desktop;
-    }
-    IShellFolder_Release(desktop);
-    return folder;
-}
-
 static WCHAR *copy_path_string(WCHAR *target, WCHAR *source)
 {
     INT i = 0;
@@ -793,8 +908,6 @@ int WINAPI wWinMain(HINSTANCE hinstance,
 
     parameters_struct   parameters;
     HRESULT hres;
-    MSG msg;
-    IShellFolder *folder;
     INITCOMMONCONTROLSEX init_info;
 
     memset(&parameters,0,sizeof(parameters));
@@ -817,13 +930,6 @@ int WINAPI wWinMain(HINSTANCE hinstance,
         ExitProcess(EXIT_FAILURE);
     }
     register_explorer_window_class();
-    folder = get_starting_shell_folder(&parameters);
-    make_explorer_window(folder);
-    IShellFolder_Release(folder);
-    while(GetMessageW( &msg, NULL, 0, 0 ) != 0)
-    {
-	    TranslateMessage(&msg);
-	    DispatchMessageW(&msg);
-    }
+    make_explorer_window(&parameters);
     return 0;
 }
