@@ -22,6 +22,7 @@
 #define __MAIN_H__
 
 #include <stdio.h>
+#include <stdlib.h>
 #include "resource.h"
 
 #define STATUS_WINDOW   2001
@@ -95,10 +96,7 @@ extern HMENU     hPopupMenus;
 extern HFONT     hFont;
 extern enum OPTION_FLAGS Options;
 
-extern WCHAR szTitle[];
-extern const WCHAR szFrameClass[];
 extern const WCHAR szChildClass[];
-extern const WCHAR szHexEditClass[];
 extern WCHAR g_pszDefaultValueName[];
 
 extern DWORD g_columnToSort;
@@ -150,7 +148,7 @@ int CALLBACK CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
 BOOL RefreshListView(HWND hwndLV, HKEY hKeyRoot, LPCWSTR keyPath, LPCWSTR highlightValue);
 HWND StartValueRename(HWND hwndLV);
 LPWSTR GetItemText(HWND hwndLV, UINT item);
-LPCWSTR GetValueName(HWND hwndLV);
+WCHAR *GetValueName(HWND hwndLV);
 BOOL ListWndNotifyProc(HWND hWnd, WPARAM wParam, LPARAM lParam, BOOL *Result);
 BOOL IsDefaultValue(HWND hwndLV, int i);
 
@@ -159,8 +157,6 @@ void WINAPIV output_message(unsigned int id, ...);
 void WINAPIV error_exit(unsigned int id, ...);
 
 /* regproc.c */
-void *heap_xalloc(size_t size);
-void *heap_xrealloc(void *buf, size_t size);
 char *GetMultiByteString(const WCHAR *strW);
 BOOL import_registry_file(FILE *reg_file);
 void delete_registry_key(WCHAR *reg_key_name);
