@@ -22,16 +22,12 @@
  */
 
 #include <errno.h>
-#include <stdlib.h>
 #include <fcntl.h>
 #include <io.h>
 #include <windows.h>
 #include <commctrl.h>
-#include <wine/debug.h>
-#include <wine/heap.h>
-#include "main.h"
 
-WINE_DEFAULT_DEBUG_CHANNEL(regedit);
+#include "main.h"
 
 #define REG_VAL_BUF_SIZE        4096
 
@@ -40,30 +36,6 @@ static HKEY reg_class_keys[] = {
             HKEY_CURRENT_CONFIG, HKEY_CURRENT_USER, HKEY_DYN_DATA
         };
 
-void *heap_xalloc(size_t size)
-{
-    void *buf = heap_alloc(size);
-    if (!buf)
-    {
-        ERR("Out of memory!\n");
-        exit(1);
-    }
-    return buf;
-}
-
-void *heap_xrealloc(void *buf, size_t size)
-{
-    void *new_buf = heap_realloc(buf, size);
-
-    if (!new_buf)
-    {
-        ERR("Out of memory!\n");
-        exit(1);
-    }
-
-    return new_buf;
-}
-
 /******************************************************************************
  * Allocates memory and converts input from multibyte to wide chars
  * Returned string must be freed by the caller
@@ -75,7 +47,7 @@ static WCHAR* GetWideString(const char* strA)
         WCHAR* strW;
         int len = MultiByteToWideChar(CP_ACP, 0, strA, -1, NULL, 0);
 
-        strW = heap_xalloc(len * sizeof(WCHAR));
+        strW = malloc(len * sizeof(WCHAR));
         MultiByteToWideChar(CP_ACP, 0, strA, -1, strW, len);
         return strW;
     }
@@ -93,7 +65,7 @@ static WCHAR* GetWideStringN(const char* strA, int chars, DWORD *len)
         WCHAR* strW;
         *len = MultiByteToWideChar(CP_ACP, 0, strA, chars, NULL, 0);
 
-        strW = heap_xalloc(*len * sizeof(WCHAR));
+        strW = malloc(*len * sizeof(WCHAR));
         MultiByteToWideChar(CP_ACP, 0, strA, chars, strW, *len);
         return strW;
     }
@@ -112,7 +84,7 @@ char* GetMultiByteString(const WCHAR* strW)
         char* strA;
         int len = WideCharToMultiByte(CP_ACP, 0, strW, -1, NULL, 0, NULL, NULL);
 
-        strA = heap_xalloc(len);
+        strA = malloc(len);
         WideCharToMultiByte(CP_ACP, 0, strW, -1, strA, len, NULL, NULL);
         return strA;
     }
@@ -130,7 +102,7 @@ static char* GetMultiByteStringN(const WCHAR* strW, int chars, DWORD* len)
         char* strA;
         *len = WideCharToMultiByte(CP_ACP, 0, strW, chars, NULL, 0, NULL, NULL);
 
-        strA = heap_xalloc(*len);
+        strA = malloc(*len);
         WideCharToMultiByte(CP_ACP, 0, strW, chars, strA, *len, NULL, NULL);
         return strA;
     }
@@ -277,7 +249,7 @@ static BOOL convert_hex_csv_to_hex(struct parser *parser, WCHAR **str)
 
     /* The worst case is 1 digit + 1 comma per byte */
     size = ((lstrlenW(*str) + 1) / 2) + parser->data_size;
-    parser->data = heap_xrealloc(parser->data, size);
+    parser->data = realloc(parser->data, size);
 
     s = *str;
     d = (BYTE *)parser->data + parser->data_size;
@@ -331,18 +303,13 @@ static BOOL parse_data_type(struct parser *parser, WCHAR **line)
 {
     struct data_type { const WCHAR *tag; int len; int type; int parse_type; };
 
-    static const WCHAR quote[] = {'"'};
-    static const WCHAR hex[] = {'h','e','x',':'};
-    static const WCHAR dword[] = {'d','w','o','r','d',':'};
-    static const WCHAR hexp[] = {'h','e','x','('};
-
     static const struct data_type data_types[] = {
-    /*    tag    len  type         parse type    */
-        { quote,  1,  REG_SZ,      REG_SZ },
-        { hex,    4,  REG_BINARY,  REG_BINARY },
-        { dword,  6,  REG_DWORD,   REG_DWORD },
-        { hexp,   4,  -1,          REG_BINARY }, /* REG_NONE, REG_EXPAND_SZ, REG_MULTI_SZ */
-        { NULL,   0,  0,           0 }
+    /*    tag       len  type         parse type    */
+        { L"\"",     1,  REG_SZ,      REG_SZ },
+        { L"hex:",   4,  REG_BINARY,  REG_BINARY },
+        { L"dword:", 6,  REG_DWORD,   REG_DWORD },
+        { L"hex(",   4,  -1,          REG_BINARY }, /* REG_NONE, REG_EXPAND_SZ, REG_MULTI_SZ */
+        { NULL,      0,  0,           0 }
     };
 
     const struct data_type *ptr;
@@ -402,8 +369,7 @@ static BOOL REGPROC_unescape_string(WCHAR *str, WCHAR **unparsed)
                 str[val_idx] = '\r';
                 break;
             case '0':
-                str[val_idx] = '\0';
-                break;
+                return FALSE;
             case '\\':
             case '"':
                 str[val_idx] = str[str_idx];
@@ -453,7 +419,7 @@ static void close_key(struct parser *parser)
 {
     if (parser->hkey)
     {
-        heap_free(parser->key_name);
+        free(parser->key_name);
         parser->key_name = NULL;
 
         RegCloseKey(parser->hkey);
@@ -482,7 +448,7 @@ static LONG open_key(struct parser *parser, WCHAR *path)
 
     if (res == ERROR_SUCCESS)
     {
-        parser->key_name = heap_xalloc((lstrlenW(path) + 1) * sizeof(WCHAR));
+        parser->key_name = malloc((lstrlenW(path) + 1) * sizeof(WCHAR));
         lstrcpyW(parser->key_name, path);
     }
     else
@@ -494,7 +460,7 @@ static LONG open_key(struct parser *parser, WCHAR *path)
 static void free_parser_data(struct parser *parser)
 {
     if (parser->parse_type == REG_DWORD || parser->parse_type == REG_BINARY)
-        heap_free(parser->data);
+        free(parser->data);
 
     parser->data = NULL;
     parser->data_size = 0;
@@ -528,7 +494,7 @@ static void prepare_hex_string_data(struct parser *parser)
 
             parser->data = GetWideStringN(parser->data, parser->data_size, &parser->data_size);
             parser->data_size *= sizeof(WCHAR);
-            heap_free(data);
+            free(data);
         }
     }
 }
@@ -543,21 +509,17 @@ enum reg_versions {
 
 static enum reg_versions parse_file_header(const WCHAR *s)
 {
-    static const WCHAR header_31[] = {'R','E','G','E','D','I','T',0};
-    static const WCHAR header_40[] = {'R','E','G','E','D','I','T','4',0};
-    static const WCHAR header_50[] = {'W','i','n','d','o','w','s',' ',
-                                      'R','e','g','i','s','t','r','y',' ','E','d','i','t','o','r',' ',
-                                      'V','e','r','s','i','o','n',' ','5','.','0','0',0};
+    static const WCHAR header_31[] = L"REGEDIT";
 
     while (*s == ' ' || *s == '\t') s++;
 
     if (!lstrcmpW(s, header_31))
         return REG_VERSION_31;
 
-    if (!lstrcmpW(s, header_40))
+    if (!lstrcmpW(s, L"REGEDIT4"))
         return REG_VERSION_40;
 
-    if (!lstrcmpW(s, header_50))
+    if (!lstrcmpW(s, L"Windows Registry Editor Version 5.00"))
         return REG_VERSION_50;
 
     /* The Windows version accepts registry file headers beginning with "REGEDIT" and ending
@@ -581,12 +543,12 @@ static WCHAR *header_state(struct parser *parser, WCHAR *pos)
 
     if (!parser->is_unicode)
     {
-        header = heap_xalloc((lstrlenW(line) + 3) * sizeof(WCHAR));
+        header = malloc((lstrlenW(line) + 3) * sizeof(WCHAR));
         header[0] = parser->two_wchars[0];
         header[1] = parser->two_wchars[1];
         lstrcpyW(header + 2, line);
         parser->reg_version = parse_file_header(header);
-        heap_free(header);
+        free(header);
     }
     else parser->reg_version = parse_file_header(line);
 
@@ -611,13 +573,13 @@ static WCHAR *header_state(struct parser *parser, WCHAR *pos)
 static WCHAR *parse_win31_line_state(struct parser *parser, WCHAR *pos)
 {
     WCHAR *line, *value;
-    static WCHAR hkcr[] = {'H','K','E','Y','_','C','L','A','S','S','E','S','_','R','O','O','T'};
+    static WCHAR hkcr[] = L"HKEY_CLASSES_ROOT";
     unsigned int key_end = 0;
 
     if (!(line = get_line(parser->file)))
         return NULL;
 
-    if (wcsncmp(line, hkcr, ARRAY_SIZE(hkcr)))
+    if (wcsncmp(line, hkcr, lstrlenW(hkcr)))
         return line;
 
     /* get key name */
@@ -718,7 +680,7 @@ static WCHAR *delete_key_state(struct parser *parser, WCHAR *pos)
 /* handler for parser DEFAULT_VALUE_NAME state */
 static WCHAR *default_value_name_state(struct parser *parser, WCHAR *pos)
 {
-    heap_free(parser->value_name);
+    free(parser->value_name);
     parser->value_name = NULL;
 
     set_state(parser, DATA_START);
@@ -730,14 +692,14 @@ static WCHAR *quoted_value_name_state(struct parser *parser, WCHAR *pos)
 {
     WCHAR *val_name = pos, *p;
 
-    heap_free(parser->value_name);
+    free(parser->value_name);
     parser->value_name = NULL;
 
     if (!REGPROC_unescape_string(val_name, &p))
         goto invalid;
 
     /* copy the value name in case we need to parse multiple lines and the buffer is overwritten */
-    parser->value_name = heap_xalloc((lstrlenW(val_name) + 1) * sizeof(WCHAR));
+    parser->value_name = malloc((lstrlenW(val_name) + 1) * sizeof(WCHAR));
     lstrcpyW(parser->value_name, val_name);
 
     set_state(parser, DATA_START);
@@ -848,7 +810,7 @@ static WCHAR *dword_data_state(struct parser *parser, WCHAR *pos)
 {
     WCHAR *line = pos;
 
-    parser->data = heap_xalloc(sizeof(DWORD));
+    parser->data = malloc(sizeof(DWORD));
 
     if (!convert_hex_to_dword(line, parser->data))
         goto invalid;
@@ -968,14 +930,14 @@ static WCHAR *get_lineA(FILE *fp)
     static char *buf, *next;
     char *line;
 
-    heap_free(lineW);
+    free(lineW);
 
     if (!fp) goto cleanup;
 
     if (!size)
     {
         size = REG_VAL_BUF_SIZE;
-        buf = heap_xalloc(size);
+        buf = malloc(size);
         *buf = 0;
         next = buf;
     }
@@ -992,7 +954,7 @@ static WCHAR *get_lineA(FILE *fp)
             if (size - len < 3)
             {
                 size *= 2;
-                buf = heap_xrealloc(buf, size);
+                buf = realloc(buf, size);
             }
             if (!(count = fread(buf + len, 1, size - len - 1, fp)))
             {
@@ -1014,7 +976,7 @@ static WCHAR *get_lineA(FILE *fp)
 
 cleanup:
     lineW = NULL;
-    if (size) heap_free(buf);
+    if (size) free(buf);
     size = 0;
     return NULL;
 }
@@ -1030,7 +992,7 @@ static WCHAR *get_lineW(FILE *fp)
     if (!size)
     {
         size = REG_VAL_BUF_SIZE;
-        buf = heap_xalloc(size * sizeof(WCHAR));
+        buf = malloc(size * sizeof(WCHAR));
         *buf = 0;
         next = buf;
     }
@@ -1038,8 +1000,7 @@ static WCHAR *get_lineW(FILE *fp)
 
     while (next)
     {
-        static const WCHAR line_endings[] = {'\r','\n',0};
-        WCHAR *p = wcspbrk(line, line_endings);
+        WCHAR *p = wcspbrk(line, L"\r\n");
         if (!p)
         {
             size_t len, count;
@@ -1048,7 +1009,7 @@ static WCHAR *get_lineW(FILE *fp)
             if (size - len < 3)
             {
                 size *= 2;
-                buf = heap_xrealloc(buf, size * sizeof(WCHAR));
+                buf = realloc(buf, size * sizeof(WCHAR));
             }
             if (!(count = fread(buf + len, sizeof(WCHAR), size - len - 1, fp)))
             {
@@ -1067,7 +1028,7 @@ static WCHAR *get_lineW(FILE *fp)
     }
 
 cleanup:
-    if (size) heap_free(buf);
+    if (size) free(buf);
     size = 0;
     return NULL;
 }
@@ -1110,7 +1071,7 @@ BOOL import_registry_file(FILE *reg_file)
     if (parser.reg_version == REG_VERSION_FUZZY || parser.reg_version == REG_VERSION_INVALID)
         return parser.reg_version == REG_VERSION_FUZZY;
 
-    heap_free(parser.value_name);
+    free(parser.value_name);
     close_key(&parser);
 
     return TRUE;
@@ -1151,7 +1112,7 @@ static void REGPROC_write_line(FILE *fp, const WCHAR *str, BOOL unicode)
     {
         char *strA = GetMultiByteString(str);
         fputs(strA, fp);
-        heap_free(strA);
+        free(strA);
     }
 }
 
@@ -1170,7 +1131,7 @@ static WCHAR *REGPROC_escape_string(WCHAR *str, size_t str_len, size_t *line_len
             escape_count++;
     }
 
-    buf = heap_xalloc((str_len + escape_count + 1) * sizeof(WCHAR));
+    buf = malloc((str_len + escape_count + 1) * sizeof(WCHAR));
 
     for (i = 0, pos = 0; i < str_len; i++, pos++)
     {
@@ -1208,18 +1169,17 @@ static WCHAR *REGPROC_escape_string(WCHAR *str, size_t str_len, size_t *line_len
 
 static size_t export_value_name(FILE *fp, WCHAR *name, size_t len, BOOL unicode)
 {
-    static const WCHAR quoted_fmt[] = {'"','%','s','"','=',0};
-    static const WCHAR default_name[] = {'@','=',0};
+    static const WCHAR default_name[] = L"@=";
     size_t line_len;
 
     if (name && *name)
     {
         WCHAR *str = REGPROC_escape_string(name, len, &line_len);
-        WCHAR *buf = heap_xalloc((line_len + 4) * sizeof(WCHAR));
-        line_len = swprintf(buf, line_len + 4, quoted_fmt, str);
+        WCHAR *buf = malloc((line_len + 4) * sizeof(WCHAR));
+        line_len = swprintf(buf, line_len + 4, L"\"%s\"=", str);
         REGPROC_write_line(fp, buf, unicode);
-        heap_free(buf);
-        heap_free(str);
+        free(buf);
+        free(str);
     }
     else
     {
@@ -1234,28 +1194,24 @@ static void export_string_data(WCHAR **buf, WCHAR *data, size_t size)
 {
     size_t len = 0, line_len;
     WCHAR *str;
-    static const WCHAR fmt[] = {'"','%','s','"',0};
 
     if (size)
         len = size / sizeof(WCHAR) - 1;
     str = REGPROC_escape_string(data, len, &line_len);
-    *buf = heap_xalloc((line_len + 3) * sizeof(WCHAR));
-    swprintf(*buf, line_len + 3, fmt, str);
-    heap_free(str);
+    *buf = malloc((line_len + 3) * sizeof(WCHAR));
+    swprintf(*buf, line_len + 3, L"\"%s\"", str);
+    free(str);
 }
 
 static void export_dword_data(WCHAR **buf, DWORD *data)
 {
-    static const WCHAR fmt[] = {'d','w','o','r','d',':','%','0','8','x',0};
-
-    *buf = heap_xalloc(15 * sizeof(WCHAR));
-    swprintf(*buf, 15, fmt, *data);
+    *buf = malloc(15 * sizeof(WCHAR));
+    swprintf(*buf, 15, L"dword:%08x", *data);
 }
 
 static size_t export_hex_data_type(FILE *fp, DWORD type, BOOL unicode)
 {
-    static const WCHAR hex[] = {'h','e','x',':',0};
-    static const WCHAR hexp_fmt[] = {'h','e','x','(','%','x',')',':',0};
+    static const WCHAR hex[] = L"hex:";
     size_t line_len;
 
     if (type == REG_BINARY)
@@ -1265,10 +1221,10 @@ static size_t export_hex_data_type(FILE *fp, DWORD type, BOOL unicode)
     }
     else
     {
-        WCHAR *buf = heap_xalloc(15 * sizeof(WCHAR));
-        line_len = swprintf(buf, 15, hexp_fmt, type);
+        WCHAR *buf = malloc(15 * sizeof(WCHAR));
+        line_len = swprintf(buf, 15, L"hex(%x):", type);
         REGPROC_write_line(fp, buf, unicode);
-        heap_free(buf);
+        free(buf);
     }
 
     return line_len;
@@ -1279,8 +1235,6 @@ static size_t export_hex_data_type(FILE *fp, DWORD type, BOOL unicode)
 static void export_hex_data(FILE *fp, WCHAR **buf, DWORD type, DWORD line_len,
                             void *data, DWORD size, BOOL unicode)
 {
-    static const WCHAR fmt[] = {'%','0','2','x',0};
-    static const WCHAR hex_concat[] = {'\\','\r','\n',' ',' ',0};
     size_t num_commas, i, pos;
 
     line_len += export_hex_data_type(fp, type, unicode);
@@ -1291,11 +1245,11 @@ static void export_hex_data(FILE *fp, WCHAR **buf, DWORD type, DWORD line_len,
         data = GetMultiByteStringN(data, size / sizeof(WCHAR), &size);
 
     num_commas = size - 1;
-    *buf = heap_xalloc(size * 3 * sizeof(WCHAR));
+    *buf = malloc(size * 3 * sizeof(WCHAR));
 
     for (i = 0, pos = 0; i < size; i++)
     {
-        pos += swprintf(*buf + pos, 3, fmt, ((BYTE *)data)[i]);
+        pos += swprintf(*buf + pos, 3, L"%02x", ((BYTE *)data)[i]);
         if (i == num_commas) break;
         (*buf)[pos++] = ',';
         (*buf)[pos] = 0;
@@ -1304,7 +1258,7 @@ static void export_hex_data(FILE *fp, WCHAR **buf, DWORD type, DWORD line_len,
         if (line_len >= MAX_HEX_CHARS)
         {
             REGPROC_write_line(fp, *buf, unicode);
-            REGPROC_write_line(fp, hex_concat, unicode);
+            REGPROC_write_line(fp, L"\\\r\n  ", unicode);
             line_len = 2;
             pos = 0;
         }
@@ -1313,9 +1267,7 @@ static void export_hex_data(FILE *fp, WCHAR **buf, DWORD type, DWORD line_len,
 
 static void export_newline(FILE *fp, BOOL unicode)
 {
-    static const WCHAR newline[] = {'\r','\n',0};
-
-    REGPROC_write_line(fp, newline, unicode);
+    REGPROC_write_line(fp, L"\r\n", unicode);
 }
 
 static void export_data(FILE *fp, WCHAR *value_name, DWORD value_len, DWORD type,
@@ -1348,7 +1300,7 @@ static void export_data(FILE *fp, WCHAR *value_name, DWORD value_len, DWORD type
     if (size || type == REG_SZ)
     {
         REGPROC_write_line(fp, buf, unicode);
-        heap_free(buf);
+        free(buf);
     }
 
     export_newline(fp, unicode);
@@ -1357,28 +1309,26 @@ static void export_data(FILE *fp, WCHAR *value_name, DWORD value_len, DWORD type
 static WCHAR *build_subkey_path(WCHAR *path, DWORD path_len, WCHAR *subkey_name, DWORD subkey_len)
 {
     WCHAR *subkey_path;
-    static const WCHAR fmt[] = {'%','s','\\','%','s',0};
 
-    subkey_path = heap_xalloc((path_len + subkey_len + 2) * sizeof(WCHAR));
-    swprintf(subkey_path, path_len + subkey_len + 2, fmt, path, subkey_name);
+    subkey_path = malloc((path_len + subkey_len + 2) * sizeof(WCHAR));
+    swprintf(subkey_path, path_len + subkey_len + 2, L"%s\\%s", path, subkey_name);
 
     return subkey_path;
 }
 
 static void export_key_name(FILE *fp, WCHAR *name, BOOL unicode)
 {
-    static const WCHAR fmt[] = {'\r','\n','[','%','s',']','\r','\n',0};
     WCHAR *buf;
 
-    buf = heap_xalloc((lstrlenW(name) + 7) * sizeof(WCHAR));
-    swprintf(buf, lstrlenW(name) + 7, fmt, name);
+    buf = malloc((lstrlenW(name) + 7) * sizeof(WCHAR));
+    swprintf(buf, lstrlenW(name) + 7, L"\r\n[%s]\r\n", name);
     REGPROC_write_line(fp, buf, unicode);
-    heap_free(buf);
+    free(buf);
 }
 
 #define MAX_SUBKEY_LEN   257
 
-static int export_registry_data(FILE *fp, HKEY key, WCHAR *path, BOOL unicode)
+static void export_registry_data(FILE *fp, HKEY key, WCHAR *path, BOOL unicode)
 {
     LONG rc;
     DWORD max_value_len = 256, value_len;
@@ -1391,8 +1341,8 @@ static int export_registry_data(FILE *fp, HKEY key, WCHAR *path, BOOL unicode)
 
     export_key_name(fp, path, unicode);
 
-    value_name = heap_xalloc(max_value_len * sizeof(WCHAR));
-    data = heap_xalloc(max_data_bytes);
+    value_name = malloc(max_value_len * sizeof(WCHAR));
+    data = malloc(max_data_bytes);
 
     i = 0;
     for (;;)
@@ -1410,21 +1360,21 @@ static int export_registry_data(FILE *fp, HKEY key, WCHAR *path, BOOL unicode)
             if (data_size > max_data_bytes)
             {
                 max_data_bytes = data_size;
-                data = heap_xrealloc(data, max_data_bytes);
+                data = realloc(data, max_data_bytes);
             }
             else
             {
                 max_value_len *= 2;
-                value_name = heap_xrealloc(value_name, max_value_len * sizeof(WCHAR));
+                value_name = realloc(value_name, max_value_len * sizeof(WCHAR));
             }
         }
         else break;
     }
 
-    heap_free(data);
-    heap_free(value_name);
+    free(data);
+    free(value_name);
 
-    subkey_name = heap_xalloc(MAX_SUBKEY_LEN * sizeof(WCHAR));
+    subkey_name = malloc(MAX_SUBKEY_LEN * sizeof(WCHAR));
 
     path_len = lstrlenW(path);
 
@@ -1441,35 +1391,30 @@ static int export_registry_data(FILE *fp, HKEY key, WCHAR *path, BOOL unicode)
                 export_registry_data(fp, subkey, subkey_path, unicode);
                 RegCloseKey(subkey);
             }
-            heap_free(subkey_path);
+            free(subkey_path);
             i++;
         }
         else break;
     }
 
-    heap_free(subkey_name);
-    return 0;
+    free(subkey_name);
 }
 
 static FILE *REGPROC_open_export_file(WCHAR *file_name, BOOL unicode)
 {
     FILE *file;
-    static const WCHAR hyphen[] = {'-',0};
 
-    if (!lstrcmpW(file_name, hyphen))
+    if (!lstrcmpW(file_name, L"-"))
     {
         file = stdout;
         _setmode(_fileno(file), _O_BINARY);
     }
     else
     {
-        static const WCHAR wb_mode[] = {'w','b',0};
-
-        file = _wfopen(file_name, wb_mode);
+        file = _wfopen(file_name, L"wb");
         if (!file)
         {
-            static const WCHAR regedit[] = {'r','e','g','e','d','i','t',0};
-            _wperror(regedit);
+            _wperror(L"regedit");
             error_exit(STRING_CANNOT_OPEN_FILE, file_name);
         }
     }
@@ -1477,12 +1422,10 @@ static FILE *REGPROC_open_export_file(WCHAR *file_name, BOOL unicode)
     if (unicode)
     {
         static const BYTE bom[] = {0xff,0xfe};
-        static const WCHAR header[] = {'W','i','n','d','o','w','s',' ',
-                                       'R','e','g','i','s','t','r','y',' ','E','d','i','t','o','r',' ',
-                                       'V','e','r','s','i','o','n',' ','5','.','0','0','\r','\n'};
+        static const WCHAR header[] = L"Windows Registry Editor Version 5.00\r\n";
 
         fwrite(bom, sizeof(BYTE), ARRAY_SIZE(bom), file);
-        fwrite(header, sizeof(WCHAR), ARRAY_SIZE(header), file);
+        fwrite(header, sizeof(WCHAR), lstrlenW(header), file);
     }
     else
         fputs("REGEDIT4\r\n", file);
@@ -1506,7 +1449,6 @@ static BOOL export_key(WCHAR *file_name, WCHAR *path, BOOL unicode)
     HKEY key_class, key;
     WCHAR *subkey;
     FILE *fp;
-    BOOL ret;
 
     if (!(key_class = parse_key_name(path, &subkey)))
     {
@@ -1519,12 +1461,12 @@ static BOOL export_key(WCHAR *file_name, WCHAR *path, BOOL unicode)
         return FALSE;
 
     fp = REGPROC_open_export_file(file_name, unicode);
-    ret = export_registry_data(fp, key, path, unicode);
+    export_registry_data(fp, key, path, unicode);
     export_newline(fp, unicode);
     fclose(fp);
 
     RegCloseKey(key);
-    return ret;
+    return TRUE;
 }
 
 static BOOL export_all(WCHAR *file_name, WCHAR *path, BOOL unicode)
@@ -1544,12 +1486,12 @@ static BOOL export_all(WCHAR *file_name, WCHAR *path, BOOL unicode)
             return FALSE;
         }
 
-        class_name = heap_xalloc((lstrlenW(reg_class_namesW[i]) + 1) * sizeof(WCHAR));
+        class_name = malloc((lstrlenW(reg_class_namesW[i]) + 1) * sizeof(WCHAR));
         lstrcpyW(class_name, reg_class_namesW[i]);
 
         export_registry_data(fp, classes[i], class_name, unicode);
 
-        heap_free(class_name);
+        free(class_name);
         RegCloseKey(key);
     }
 
