@@ -35,8 +35,6 @@
 #include "services.h"
 #include "svcctl.h"
 
-extern HANDLE CDECL __wine_make_process_system(void);
-
 WINE_DEFAULT_DEBUG_CHANNEL(service);
 
 static const GENERIC_MAPPING g_scm_generic =
@@ -104,8 +102,10 @@ static void sc_notify_release(struct sc_notify_handle *notify)
     if (r == 0)
     {
         CloseHandle(notify->event);
-        HeapFree(GetProcessHeap(), 0, notify->params_list);
-        HeapFree(GetProcessHeap(), 0, notify);
+        if (notify->params_list)
+            free(notify->params_list->NotifyParamsArray[0].params);
+        free(notify->params_list);
+        free(notify);
     }
 }
 
@@ -199,22 +199,22 @@ static void free_service_strings(struct service_entry *old, struct service_entry
     QUERY_SERVICE_CONFIGW *new_cfg = &new->config;
 
     if (old_cfg->lpBinaryPathName != new_cfg->lpBinaryPathName)
-        HeapFree(GetProcessHeap(), 0, old_cfg->lpBinaryPathName);
+        free(old_cfg->lpBinaryPathName);
 
     if (old_cfg->lpLoadOrderGroup != new_cfg->lpLoadOrderGroup)
-        HeapFree(GetProcessHeap(), 0, old_cfg->lpLoadOrderGroup);
+        free(old_cfg->lpLoadOrderGroup);
 
     if (old_cfg->lpServiceStartName != new_cfg->lpServiceStartName)
-        HeapFree(GetProcessHeap(), 0, old_cfg->lpServiceStartName);
+        free(old_cfg->lpServiceStartName);
 
     if (old_cfg->lpDisplayName != new_cfg->lpDisplayName)
-        HeapFree(GetProcessHeap(), 0, old_cfg->lpDisplayName);
+        free(old_cfg->lpDisplayName);
 
     if (old->dependOnServices != new->dependOnServices)
-        HeapFree(GetProcessHeap(), 0, old->dependOnServices);
+        free(old->dependOnServices);
 
     if (old->dependOnGroups != new->dependOnGroups)
-        HeapFree(GetProcessHeap(), 0, old->dependOnGroups);
+        free(old->dependOnGroups);
 }
 
 /* Check if the given handle is of the required type and allows the requested access. */
@@ -224,13 +224,13 @@ static DWORD validate_context_handle(SC_RPC_HANDLE handle, DWORD type, DWORD nee
 
     if (type != SC_HTYPE_DONT_CARE && hdr->type != type)
     {
-        WINE_ERR("Handle is of an invalid type (%d, %d)\n", hdr->type, type);
+        WINE_ERR("Handle is of an invalid type (%d, %ld)\n", hdr->type, type);
         return ERROR_INVALID_HANDLE;
     }
 
     if ((needed_access & hdr->access) != needed_access)
     {
-        WINE_ERR("Access denied - handle created with access %x, needed %x\n", hdr->access, needed_access);
+        WINE_ERR("Access denied - handle created with access %lx, needed %lx\n", hdr->access, needed_access);
         return ERROR_ACCESS_DENIED;
     }
 
@@ -273,7 +273,7 @@ DWORD __cdecl svcctl_OpenSCManagerW(
 {
     struct sc_manager_handle *manager;
 
-    WINE_TRACE("(%s, %s, %x)\n", wine_dbgstr_w(MachineName), wine_dbgstr_w(DatabaseName), dwAccessMask);
+    WINE_TRACE("(%s, %s, %lx)\n", wine_dbgstr_w(MachineName), wine_dbgstr_w(DatabaseName), dwAccessMask);
 
     if (DatabaseName != NULL && DatabaseName[0])
     {
@@ -283,7 +283,7 @@ DWORD __cdecl svcctl_OpenSCManagerW(
             return ERROR_INVALID_NAME;
     }
 
-    if (!(manager = HeapAlloc(GetProcessHeap(), 0, sizeof(*manager))))
+    if (!(manager = malloc(sizeof(*manager))))
         return ERROR_NOT_ENOUGH_SERVER_MEMORY;
 
     manager->hdr.type = SC_HTYPE_MANAGER;
@@ -306,7 +306,7 @@ static void SC_RPC_HANDLE_destroy(SC_RPC_HANDLE handle)
         case SC_HTYPE_MANAGER:
         {
             struct sc_manager_handle *manager = (struct sc_manager_handle *)hdr;
-            HeapFree(GetProcessHeap(), 0, manager);
+            free(manager);
             break;
         }
         case SC_HTYPE_SERVICE:
@@ -321,7 +321,7 @@ static void SC_RPC_HANDLE_destroy(SC_RPC_HANDLE handle)
             }
             service_unlock(service->service_entry);
             release_service(service->service_entry);
-            HeapFree(GetProcessHeap(), 0, service);
+            free(service);
             break;
         }
         default:
@@ -340,7 +340,7 @@ DWORD __cdecl svcctl_GetServiceDisplayNameW(
     struct service_entry *entry;
     DWORD err;
 
-    WINE_TRACE("(%s, %d)\n", wine_dbgstr_w(lpServiceName), *cchBufSize);
+    WINE_TRACE("(%s, %ld)\n", wine_dbgstr_w(lpServiceName), *cchBufSize);
 
     if ((err = validate_scm_handle(hSCManager, 0, &manager)) != ERROR_SUCCESS)
         return err;
@@ -384,7 +384,7 @@ DWORD __cdecl svcctl_GetServiceKeyNameW(
     struct sc_manager_handle *manager;
     DWORD err;
 
-    WINE_TRACE("(%s, %d)\n", wine_dbgstr_w(lpServiceDisplayName), *cchBufSize);
+    WINE_TRACE("(%s, %ld)\n", wine_dbgstr_w(lpServiceDisplayName), *cchBufSize);
 
     if ((err = validate_scm_handle(hSCManager, 0, &manager)) != ERROR_SUCCESS)
         return err;
@@ -420,7 +420,7 @@ static DWORD create_handle_for_service(struct service_entry *entry, DWORD dwDesi
 {
     struct sc_service_handle *service;
 
-    if (!(service = HeapAlloc(GetProcessHeap(), 0, sizeof(*service))))
+    if (!(service = malloc(sizeof(*service))))
     {
         release_service(entry);
         return ERROR_NOT_ENOUGH_SERVER_MEMORY;
@@ -454,7 +454,7 @@ DWORD __cdecl svcctl_OpenServiceW(
     struct service_entry *entry;
     DWORD err;
 
-    WINE_TRACE("(%s, 0x%x)\n", wine_dbgstr_w(lpServiceName), dwDesiredAccess);
+    WINE_TRACE("(%s, 0x%lx)\n", wine_dbgstr_w(lpServiceName), dwDesiredAccess);
 
     if ((err = validate_scm_handle(hSCManager, 0, &manager)) != ERROR_SUCCESS)
         return err;
@@ -496,7 +496,7 @@ static DWORD parse_dependencies(const WCHAR *dependencies, struct service_entry
     if (!len_services) entry->dependOnServices = NULL;
     else
     {
-        services = HeapAlloc(GetProcessHeap(), 0, (len_services + 1) * sizeof(WCHAR));
+        services = malloc((len_services + 1) * sizeof(WCHAR));
         if (!services)
             return ERROR_OUTOFMEMORY;
 
@@ -518,10 +518,10 @@ static DWORD parse_dependencies(const WCHAR *dependencies, struct service_entry
     if (!len_groups) entry->dependOnGroups = NULL;
     else
     {
-        groups = HeapAlloc(GetProcessHeap(), 0, (len_groups + 1) * sizeof(WCHAR));
+        groups = malloc((len_groups + 1) * sizeof(WCHAR));
         if (!groups)
         {
-            HeapFree(GetProcessHeap(), 0, services);
+            free(services);
             return ERROR_OUTOFMEMORY;
         }
         s = groups;
@@ -566,7 +566,7 @@ static DWORD create_serviceW(
     struct sc_manager_handle *manager;
     DWORD err;
 
-    WINE_TRACE("(%s, %s, 0x%x, %s)\n", wine_dbgstr_w(lpServiceName), wine_dbgstr_w(lpDisplayName), dwDesiredAccess, wine_dbgstr_w(lpBinaryPathName));
+    WINE_TRACE("(%s, %s, 0x%lx, %s)\n", wine_dbgstr_w(lpServiceName), wine_dbgstr_w(lpDisplayName), dwDesiredAccess, wine_dbgstr_w(lpBinaryPathName));
 
     if ((err = validate_scm_handle(hSCManager, SC_MANAGER_CREATE_SERVICE, &manager)) != ERROR_SUCCESS)
         return err;
@@ -593,10 +593,10 @@ static DWORD create_serviceW(
     entry->config.dwServiceType = entry->status.dwServiceType = dwServiceType;
     entry->config.dwStartType = dwStartType;
     entry->config.dwErrorControl = dwErrorControl;
-    entry->config.lpBinaryPathName = strdupW(lpBinaryPathName);
-    entry->config.lpLoadOrderGroup = strdupW(lpLoadOrderGroup);
-    entry->config.lpServiceStartName = strdupW(lpServiceStartName);
-    entry->config.lpDisplayName = strdupW(lpDisplayName);
+    entry->config.lpBinaryPathName = wcsdup(lpBinaryPathName);
+    entry->config.lpLoadOrderGroup = wcsdup(lpLoadOrderGroup);
+    entry->config.lpServiceStartName = wcsdup(lpServiceStartName);
+    entry->config.lpDisplayName = wcsdup(lpDisplayName);
 
     if (lpdwTagId)      /* TODO: In most situations a non-NULL TagId will generate an ERROR_INVALID_PARAMETER. */
         entry->config.dwTagId = *lpdwTagId;
@@ -659,7 +659,7 @@ DWORD __cdecl svcctl_CreateServiceW(
     DWORD dwPasswordSize,
     SC_RPC_HANDLE *phService)
 {
-    WINE_TRACE("(%s, %s, 0x%x, %s)\n", wine_dbgstr_w(lpServiceName), wine_dbgstr_w(lpDisplayName), dwDesiredAccess, wine_dbgstr_w(lpBinaryPathName));
+    WINE_TRACE("(%s, %s, 0x%lx, %s)\n", wine_dbgstr_w(lpServiceName), wine_dbgstr_w(lpDisplayName), dwDesiredAccess, wine_dbgstr_w(lpBinaryPathName));
     return create_serviceW(hSCManager, lpServiceName, lpDisplayName, dwDesiredAccess, dwServiceType, dwStartType,
         dwErrorControl, lpBinaryPathName, lpLoadOrderGroup, lpdwTagId, lpDependencies, dwDependenciesSize, lpServiceStartName,
         lpPassword, dwPasswordSize, phService, FALSE);
@@ -704,12 +704,12 @@ DWORD __cdecl svcctl_QueryServiceConfigW(
     config->dwServiceType = service->service_entry->config.dwServiceType;
     config->dwStartType = service->service_entry->config.dwStartType;
     config->dwErrorControl = service->service_entry->config.dwErrorControl;
-    config->lpBinaryPathName = strdupW(service->service_entry->config.lpBinaryPathName);
-    config->lpLoadOrderGroup = strdupW(service->service_entry->config.lpLoadOrderGroup);
+    config->lpBinaryPathName = wcsdup(service->service_entry->config.lpBinaryPathName);
+    config->lpLoadOrderGroup = wcsdup(service->service_entry->config.lpLoadOrderGroup);
     config->dwTagId = service->service_entry->config.dwTagId;
     config->lpDependencies = NULL; /* TODO */
-    config->lpServiceStartName = strdupW(service->service_entry->config.lpServiceStartName);
-    config->lpDisplayName = strdupW(service->service_entry->config.lpDisplayName);
+    config->lpServiceStartName = wcsdup(service->service_entry->config.lpServiceStartName);
+    config->lpDisplayName = wcsdup(service->service_entry->config.lpDisplayName);
     service_unlock(service->service_entry);
 
     return ERROR_SUCCESS;
@@ -804,16 +804,16 @@ DWORD __cdecl svcctl_ChangeServiceConfigW(
 
     /* configuration OK. The strings needs to be duplicated */
     if (lpBinaryPathName != NULL)
-        new_entry.config.lpBinaryPathName = strdupW(lpBinaryPathName);
+        new_entry.config.lpBinaryPathName = wcsdup(lpBinaryPathName);
 
     if (lpLoadOrderGroup != NULL)
-        new_entry.config.lpLoadOrderGroup = strdupW(lpLoadOrderGroup);
+        new_entry.config.lpLoadOrderGroup = wcsdup(lpLoadOrderGroup);
 
     if (lpServiceStartName != NULL)
-        new_entry.config.lpServiceStartName = strdupW(lpServiceStartName);
+        new_entry.config.lpServiceStartName = wcsdup(lpServiceStartName);
 
     if (lpDisplayName != NULL)
-        new_entry.config.lpDisplayName = strdupW(lpDisplayName);
+        new_entry.config.lpDisplayName = wcsdup(lpDisplayName);
 
     /* try to save to Registry, commit or rollback depending on success */
     err = save_service_config(&new_entry);
@@ -843,12 +843,14 @@ static void fill_notify(struct sc_notify_handle *notify, struct service_entry *s
     SC_RPC_NOTIFY_PARAMS_LIST *list;
     SERVICE_NOTIFY_STATUS_CHANGE_PARAMS_2 *cparams;
 
-    list = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
-            sizeof(SC_RPC_NOTIFY_PARAMS_LIST) + sizeof(SERVICE_NOTIFY_STATUS_CHANGE_PARAMS_2));
+    list = calloc(1, sizeof(SC_RPC_NOTIFY_PARAMS_LIST));
     if (!list)
         return;
-
-    cparams = (SERVICE_NOTIFY_STATUS_CHANGE_PARAMS_2 *)(list + 1);
+    if (!(cparams = calloc(1, sizeof(SERVICE_NOTIFY_STATUS_CHANGE_PARAMS_2))))
+    {
+        free(list);
+        return;
+    }
 
     cparams->dwNotifyMask = notify->notify_mask;
     fill_status_process(&cparams->ServiceStatus, service);
@@ -859,7 +861,7 @@ static void fill_notify(struct sc_notify_handle *notify, struct service_entry *s
     list->cElements = 1;
 
     list->NotifyParamsArray[0].dwInfoLevel = 2;
-    list->NotifyParamsArray[0].u.params = cparams;
+    list->NotifyParamsArray[0].params = cparams;
 
     InterlockedExchangePointer((void**)&notify->params_list, list);
 
@@ -934,39 +936,39 @@ DWORD __cdecl svcctl_ChangeServiceConfig2W( SC_RPC_HANDLE hService, SC_RPC_CONFI
         {
             WCHAR *descr = NULL;
 
-            if (!config.u.descr->lpDescription)
+            if (!config.descr->lpDescription)
                 break;
 
-            if (config.u.descr->lpDescription[0])
+            if (config.descr->lpDescription[0])
             {
-                if (!(descr = strdupW( config.u.descr->lpDescription )))
+                if (!(descr = wcsdup( config.descr->lpDescription )))
                     return ERROR_NOT_ENOUGH_MEMORY;
             }
 
             WINE_TRACE( "changing service %p descr to %s\n", service, wine_dbgstr_w(descr) );
             service_lock( service->service_entry );
-            HeapFree( GetProcessHeap(), 0, service->service_entry->description );
+            free( service->service_entry->description );
             service->service_entry->description = descr;
             save_service_config( service->service_entry );
             service_unlock( service->service_entry );
         }
         break;
     case SERVICE_CONFIG_FAILURE_ACTIONS:
-        WINE_FIXME( "SERVICE_CONFIG_FAILURE_ACTIONS not implemented: period %u msg %s cmd %s\n",
-                    config.u.actions->dwResetPeriod,
-                    wine_dbgstr_w(config.u.actions->lpRebootMsg),
-                    wine_dbgstr_w(config.u.actions->lpCommand) );
+        WINE_FIXME( "SERVICE_CONFIG_FAILURE_ACTIONS not implemented: period %lu msg %s cmd %s\n",
+                    config.actions->dwResetPeriod,
+                    wine_dbgstr_w(config.actions->lpRebootMsg),
+                    wine_dbgstr_w(config.actions->lpCommand) );
         break;
     case SERVICE_CONFIG_PRESHUTDOWN_INFO:
-        WINE_TRACE( "changing service %p preshutdown timeout to %d\n",
-                service, config.u.preshutdown->dwPreshutdownTimeout );
+        WINE_TRACE( "changing service %p preshutdown timeout to %ld\n",
+                service, config.preshutdown->dwPreshutdownTimeout );
         service_lock( service->service_entry );
-        service->service_entry->preshutdown_timeout = config.u.preshutdown->dwPreshutdownTimeout;
+        service->service_entry->preshutdown_timeout = config.preshutdown->dwPreshutdownTimeout;
         save_service_config( service->service_entry );
         service_unlock( service->service_entry );
         break;
     default:
-        WINE_FIXME("level %u not implemented\n", config.dwInfoLevel);
+        WINE_FIXME("level %lu not implemented\n", config.dwInfoLevel);
         err = ERROR_INVALID_LEVEL;
         break;
     }
@@ -981,7 +983,7 @@ DWORD __cdecl svcctl_QueryServiceConfig2W( SC_RPC_HANDLE hService, DWORD level,
 
     memset(buffer, 0, size);
 
-    if ((err = validate_service_handle(hService, SERVICE_QUERY_STATUS, &service)) != 0)
+    if ((err = validate_service_handle(hService, SERVICE_QUERY_CONFIG, &service)) != 0)
         return err;
 
     switch (level)
@@ -1027,7 +1029,7 @@ DWORD __cdecl svcctl_QueryServiceConfig2W( SC_RPC_HANDLE hService, DWORD level,
         break;
 
     default:
-        WINE_FIXME("level %u not implemented\n", level);
+        WINE_FIXME("level %lu not implemented\n", level);
         err = ERROR_INVALID_LEVEL;
         break;
     }
@@ -1170,7 +1172,7 @@ static BOOL process_send_command(struct process_entry *process, const void *data
     if (!r || count != sizeof *result)
     {
         WINE_ERR("service protocol error - failed to read pipe "
-            "r = %d  count = %d!\n", r, count);
+            "r = %d  count = %ld!\n", r, count);
         *result = (!r ? GetLastError() : ERROR_READ_FAULT);
         return FALSE;
     }
@@ -1199,7 +1201,7 @@ BOOL process_send_control(struct process_entry *process, BOOL shared_process, co
     /* calculate how much space we need to send the startup info */
     len = (lstrlenW(name) + 1) * sizeof(WCHAR) + data_size;
 
-    ssi = HeapAlloc(GetProcessHeap(),0,FIELD_OFFSET(service_start_info, data[len]));
+    ssi = malloc(FIELD_OFFSET(service_start_info, data[len]));
     ssi->magic = SERVICE_PROTOCOL_MAGIC;
     ssi->control = control;
     ssi->total_size = FIELD_OFFSET(service_start_info, data[len]);
@@ -1208,7 +1210,7 @@ BOOL process_send_control(struct process_entry *process, BOOL shared_process, co
     if (data_size) memcpy(&ssi->data[ssi->name_size * sizeof(WCHAR)], data, data_size);
 
     r = process_send_command(process, ssi, ssi->total_size, result);
-    HeapFree( GetProcessHeap(), 0, ssi );
+    free(ssi);
     return r;
 }
 
@@ -1220,7 +1222,7 @@ DWORD __cdecl svcctl_StartServiceW(
     struct sc_service_handle *service;
     DWORD err;
 
-    WINE_TRACE("(%p, %d, %p)\n", hService, dwNumServiceArgs, lpServiceArgVectors);
+    WINE_TRACE("(%p, %ld, %p)\n", hService, dwNumServiceArgs, lpServiceArgVectors);
 
     if ((err = validate_service_handle(hService, SERVICE_START, &service)) != 0)
         return err;
@@ -1248,7 +1250,7 @@ DWORD __cdecl svcctl_ControlService(
     BOOL shared_process;
     DWORD result;
 
-    WINE_TRACE("(%p, %d, %p)\n", hService, dwControl, lpServiceStatus);
+    WINE_TRACE("(%p, %ld, %p)\n", hService, dwControl, lpServiceStatus);
 
     switch (dwControl)
     {
@@ -1365,55 +1367,23 @@ DWORD __cdecl svcctl_CloseServiceHandle(
     return ERROR_SUCCESS;
 }
 
-static void SC_RPC_LOCK_destroy(SC_RPC_LOCK hLock)
-{
-    struct sc_lock *lock = hLock;
-    scmdatabase_unlock_startup(lock->db);
-    HeapFree(GetProcessHeap(), 0, lock);
-}
-
 void __RPC_USER SC_RPC_LOCK_rundown(SC_RPC_LOCK hLock)
 {
-    SC_RPC_LOCK_destroy(hLock);
 }
 
-DWORD __cdecl svcctl_LockServiceDatabase(
-    SC_RPC_HANDLE hSCManager,
-    SC_RPC_LOCK *phLock)
+DWORD __cdecl svcctl_LockServiceDatabase(SC_RPC_HANDLE manager, SC_RPC_LOCK *lock)
 {
-    struct sc_manager_handle *manager;
-    struct sc_lock *lock;
-    DWORD err;
-
-    WINE_TRACE("(%p, %p)\n", hSCManager, phLock);
-
-    if ((err = validate_scm_handle(hSCManager, SC_MANAGER_LOCK, &manager)) != ERROR_SUCCESS)
-        return err;
-
-    if (!scmdatabase_lock_startup(manager->db, 0))
-        return ERROR_SERVICE_DATABASE_LOCKED;
-
-    lock = HeapAlloc(GetProcessHeap(), 0, sizeof(struct sc_lock));
-    if (!lock)
-    {
-        scmdatabase_unlock_startup(manager->db);
-        return ERROR_NOT_ENOUGH_SERVER_MEMORY;
-    }
-
-    lock->db = manager->db;
-    *phLock = lock;
+    TRACE("(%p, %p)\n", manager, lock);
 
+    *lock = (SC_RPC_LOCK)0xdeadbeef;
     return ERROR_SUCCESS;
 }
 
-DWORD __cdecl svcctl_UnlockServiceDatabase(
-    SC_RPC_LOCK *phLock)
+DWORD __cdecl svcctl_UnlockServiceDatabase(SC_RPC_LOCK *lock)
 {
-    WINE_TRACE("(&%p)\n", *phLock);
-
-    SC_RPC_LOCK_destroy(*phLock);
-    *phLock = NULL;
+    TRACE("(&%p)\n", *lock);
 
+    *lock = NULL;
     return ERROR_SUCCESS;
 }
 
@@ -1433,7 +1403,7 @@ static BOOL map_state(DWORD state, DWORD mask)
         if (SERVICE_INACTIVE & mask) return TRUE;
         break;
     default:
-        WINE_ERR("unknown state %u\n", state);
+        WINE_ERR("unknown state %lu\n", state);
         break;
     }
     return FALSE;
@@ -1454,7 +1424,7 @@ DWORD __cdecl svcctl_EnumServicesStatusW(
     struct service_entry *service;
     struct enum_service_status *s;
 
-    WINE_TRACE("(%p, 0x%x, 0x%x, %p, %u, %p, %p, %p)\n", hmngr, type, state, buffer, size, needed, returned, resume);
+    WINE_TRACE("(%p, 0x%lx, 0x%lx, %p, %lu, %p, %p, %p)\n", hmngr, type, state, buffer, size, needed, returned, resume);
 
     if (!type || !state)
         return ERROR_INVALID_PARAMETER;
@@ -1570,7 +1540,7 @@ DWORD __cdecl svcctl_EnumServicesStatusExW(
     struct service_entry *service;
     struct enum_service_status_process *s;
 
-    WINE_TRACE("(%p, 0x%x, 0x%x, %p, %u, %p, %p, %s)\n", hmngr, type, state, buffer, size,
+    WINE_TRACE("(%p, 0x%lx, 0x%lx, %p, %lu, %p, %p, %s)\n", hmngr, type, state, buffer, size,
                needed, returned, wine_dbgstr_w(group));
 
     if (resume_handle)
@@ -1688,7 +1658,7 @@ DWORD __cdecl svcctl_CreateServiceWOW64W(
     DWORD password_size,
     SC_RPC_HANDLE *service)
 {
-    WINE_TRACE("(%s, %s, 0x%x, %s)\n", wine_dbgstr_w(servicename), wine_dbgstr_w(displayname), accessmask, wine_dbgstr_w(imagepath));
+    WINE_TRACE("(%s, %s, 0x%lx, %s)\n", wine_dbgstr_w(servicename), wine_dbgstr_w(displayname), accessmask, wine_dbgstr_w(imagepath));
     return create_serviceW(scmanager, servicename, displayname, accessmask, service_type, start_type, error_control, imagepath,
         loadordergroup, tagid, dependencies, depend_size, start_name, password, password_size, service, TRUE);
 }
@@ -1713,8 +1683,8 @@ DWORD __cdecl svcctl_NotifyServiceStatusChange(
     struct sc_notify_handle *notify;
     struct sc_handle *hdr = handle;
 
-    WINE_TRACE("(%p, NotifyMask: 0x%x, %p, %p, %p, %p)\n", handle,
-            params.u.params->dwNotifyMask, clientprocessguid, scmprocessguid,
+    WINE_TRACE("(%p, NotifyMask: 0x%lx, %p, %p, %p, %p)\n", handle,
+            params.params->dwNotifyMask, clientprocessguid, scmprocessguid,
             createremotequeue, hNotify);
 
     switch (hdr->type)
@@ -1739,7 +1709,7 @@ DWORD __cdecl svcctl_NotifyServiceStatusChange(
         return ERROR_CALL_NOT_IMPLEMENTED;
     }
 
-    notify = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*notify));
+    notify = calloc(1, sizeof(*notify));
     if (!notify)
         return ERROR_NOT_ENOUGH_SERVER_MEMORY;
 
@@ -1748,7 +1718,7 @@ DWORD __cdecl svcctl_NotifyServiceStatusChange(
 
     notify->event = CreateEventW(NULL, TRUE, FALSE, NULL);
 
-    notify->notify_mask = params.u.params->dwNotifyMask;
+    notify->notify_mask = params.params->dwNotifyMask;
 
     service_lock(service->service_entry);
 
@@ -2145,29 +2115,30 @@ DWORD RPC_Init(void)
 
     if (!(cleanup_group = CreateThreadpoolCleanupGroup()))
     {
-        WINE_ERR("CreateThreadpoolCleanupGroup failed with error %u\n", GetLastError());
+        WINE_ERR("CreateThreadpoolCleanupGroup failed with error %lu\n", GetLastError());
         return GetLastError();
     }
 
     if ((err = RpcServerUseProtseqEpW(transport, 0, endpoint, NULL)) != ERROR_SUCCESS)
     {
-        WINE_ERR("RpcServerUseProtseq failed with error %u\n", err);
+        WINE_ERR("RpcServerUseProtseq failed with error %lu\n", err);
         return err;
     }
 
     if ((err = RpcServerRegisterIf(svcctl_v2_0_s_ifspec, 0, 0)) != ERROR_SUCCESS)
     {
-        WINE_ERR("RpcServerRegisterIf failed with error %u\n", err);
+        WINE_ERR("RpcServerRegisterIf failed with error %lu\n", err);
         return err;
     }
 
     if ((err = RpcServerListen(1, RPC_C_LISTEN_MAX_CALLS_DEFAULT, TRUE)) != ERROR_SUCCESS)
     {
-        WINE_ERR("RpcServerListen failed with error %u\n", err);
+        WINE_ERR("RpcServerListen failed with error %lu\n", err);
         return err;
     }
 
-    exit_event = __wine_make_process_system();
+    NtSetInformationProcess( GetCurrentProcess(), ProcessWineMakeProcessSystem,
+                             &exit_event, sizeof(HANDLE *) );
     return ERROR_SUCCESS;
 }
 
@@ -2193,10 +2164,10 @@ void __RPC_USER SC_NOTIFY_RPC_HANDLE_rundown(SC_NOTIFY_RPC_HANDLE handle)
 
 void  __RPC_FAR * __RPC_USER MIDL_user_allocate(SIZE_T len)
 {
-    return HeapAlloc(GetProcessHeap(), 0, len);
+    return malloc(len);
 }
 
 void __RPC_USER MIDL_user_free(void __RPC_FAR * ptr)
 {
-    HeapFree(GetProcessHeap(), 0, ptr);
+    free(ptr);
 }
