@@ -20,7 +20,6 @@
  */
 
 #define WIN32_LEAN_AND_MEAN
-#define _WIN32_IE 0x0400
 
 #include <stdarg.h>
 #include <stdlib.h>
@@ -240,8 +239,7 @@ static void set_caption(LPCWSTR wszNewFileName)
     else
         wszNewFileName = file_basename((LPWSTR)wszNewFileName);
 
-    wszCaption = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
-                lstrlenW(wszNewFileName)*sizeof(WCHAR)+sizeof(wszSeparator)+sizeof(wszAppTitle));
+    wszCaption = calloc(1, lstrlenW(wszNewFileName)*sizeof(WCHAR)+sizeof(wszSeparator)+sizeof(wszAppTitle));
 
     if(!wszCaption)
         return;
@@ -254,7 +252,7 @@ static void set_caption(LPCWSTR wszNewFileName)
 
     SetWindowTextW(hMainWnd, wszCaption);
 
-    HeapFree(GetProcessHeap(), 0, wszCaption);
+    free(wszCaption);
 }
 
 static BOOL validate_endptr(LPCWSTR endptr, UNIT *punit)
@@ -537,36 +535,6 @@ static void on_fontlist_modified(LPWSTR wszNewFaceName)
         set_font(wszNewFaceName);
 }
 
-static void add_font(LPCWSTR fontName, DWORD fontType, HWND hListWnd, const NEWTEXTMETRICEXW *ntmc)
-{
-    COMBOBOXEXITEMW cbItem;
-    WCHAR buffer[MAX_PATH];
-    int fontHeight = 0;
-
-    cbItem.mask = CBEIF_TEXT;
-    cbItem.pszText = buffer;
-    cbItem.cchTextMax = MAX_STRING_LEN;
-    cbItem.iItem = 0;
-
-    while(SendMessageW(hListWnd, CBEM_GETITEMW, 0, (LPARAM)&cbItem))
-    {
-        if(lstrcmpiW(cbItem.pszText, fontName) <= 0)
-            cbItem.iItem++;
-        else
-            break;
-    }
-    cbItem.pszText = HeapAlloc( GetProcessHeap(), 0, (lstrlenW(fontName) + 1)*sizeof(WCHAR) );
-    lstrcpyW( cbItem.pszText, fontName );
-
-    cbItem.mask |= CBEIF_LPARAM;
-    if(fontType & RASTER_FONTTYPE)
-        fontHeight = ntmc->ntmTm.tmHeight - ntmc->ntmTm.tmInternalLeading;
-
-    cbItem.lParam = MAKELONG(fontType,fontHeight);
-    SendMessageW(hListWnd, CBEM_INSERTITEMW, 0, (LPARAM)&cbItem);
-    HeapFree( GetProcessHeap(), 0, cbItem.pszText );
-}
-
 static void dialog_choose_font(void)
 {
     CHOOSEFONTW cf;
@@ -615,36 +583,132 @@ static void dialog_choose_font(void)
     }
 }
 
+struct font_desc
+{
+    WCHAR *name;
+    LPARAM lParam;
+};
+
+struct font_array
+{
+    struct font_desc *fonts;
+    size_t count;
+    size_t capacity;
+};
+
+static BOOL array_reserve(void **elements, size_t *capacity, size_t count, size_t size)
+{
+    size_t new_capacity, max_capacity;
+    void *new_elements;
+
+    if (count <= *capacity)
+        return TRUE;
+
+    max_capacity = ~(SIZE_T)0 / size;
+    if (count > max_capacity)
+        return FALSE;
+
+    new_capacity = max(4, *capacity);
+    while (new_capacity < count && new_capacity <= max_capacity / 2)
+        new_capacity *= 2;
+    if (new_capacity < count)
+        new_capacity = max_capacity;
+
+    new_elements = *elements ? realloc(*elements, new_capacity * size) :
+            malloc(new_capacity * size);
+    if (!new_elements)
+        return FALSE;
+
+    *elements = new_elements;
+    *capacity = new_capacity;
+
+    return TRUE;
+}
+
+static void add_font(struct font_array *fonts, LPCWSTR fontName, DWORD fontType, const NEWTEXTMETRICEXW *ntmc)
+{
+    int fontHeight = 0;
+    size_t idx;
+
+    if (!array_reserve((void **)&fonts->fonts, &fonts->capacity, fonts->count + 1, sizeof(*fonts->fonts)))
+        return;
+
+    if (fontType & RASTER_FONTTYPE)
+        fontHeight = ntmc->ntmTm.tmHeight - ntmc->ntmTm.tmInternalLeading;
+
+    idx = fonts->count;
+    fonts->fonts[idx].name = malloc((lstrlenW(fontName) + 1)*sizeof(WCHAR) );
+    lstrcpyW( fonts->fonts[idx].name, fontName );
+    fonts->fonts[idx].lParam = MAKELONG(fontType, fontHeight);
+
+    fonts->count++;
+}
 
 static int CALLBACK enum_font_proc(const LOGFONTW *lpelfe, const TEXTMETRICW *lpntme,
                             DWORD FontType, LPARAM lParam)
 {
-    HWND hListWnd = (HWND) lParam;
+    struct font_array *fonts = (void *)lParam;
 
     if (lpelfe->lfFaceName[0] == '@') return 1;  /* ignore vertical fonts */
 
-    if(SendMessageW(hListWnd, CB_FINDSTRINGEXACT, -1, (LPARAM)lpelfe->lfFaceName) == CB_ERR)
-    {
-
-        add_font(lpelfe->lfFaceName, FontType, hListWnd, (const NEWTEXTMETRICEXW*)lpntme);
-    }
+    add_font(fonts, lpelfe->lfFaceName, FontType, (const NEWTEXTMETRICEXW *)lpntme);
 
     return 1;
 }
 
+static int __cdecl fonts_desc_compare(const void *a, const void *b)
+{
+    const struct font_desc *left = a, *right = b;
+    return lstrcmpiW(left->name, right->name);
+}
+
 static void populate_font_list(HWND hListWnd)
 {
+    struct font_array font_array = { 0 };
     HDC hdc = GetDC(hMainWnd);
     LOGFONTW fontinfo;
     HWND hListEditWnd = (HWND)SendMessageW(hListWnd, CBEM_GETEDITCONTROL, 0, 0);
     CHARFORMAT2W fmt;
+    size_t i, j;
 
     fontinfo.lfCharSet = DEFAULT_CHARSET;
     *fontinfo.lfFaceName = '\0';
     fontinfo.lfPitchAndFamily = 0;
 
-    EnumFontFamiliesExW(hdc, &fontinfo, enum_font_proc,
-                        (LPARAM)hListWnd, 0);
+    /* Collect font names, sort, remove duplicates. */
+    EnumFontFamiliesExW(hdc, &fontinfo, enum_font_proc, (LPARAM)&font_array, 0);
+
+    qsort(font_array.fonts, font_array.count, sizeof(*font_array.fonts), fonts_desc_compare);
+
+    for (i = 1, j = 0; i < font_array.count; ++i)
+    {
+        if (!lstrcmpiW(font_array.fonts[i].name, font_array.fonts[j].name))
+        {
+            free(font_array.fonts[i].name);
+            font_array.fonts[i].name = NULL;
+        }
+        else if (++j != i)
+        {
+            font_array.fonts[j] = font_array.fonts[i];
+            font_array.fonts[i].name = NULL;
+        }
+    }
+    font_array.count = j + 1;
+
+    for (i = 0; i < font_array.count; ++i)
+    {
+        COMBOBOXEXITEMW cbitem = { 0 };
+
+        cbitem.mask = CBEIF_TEXT | CBEIF_LPARAM;
+        cbitem.pszText = font_array.fonts[i].name;
+        cbitem.iItem = -1;
+        cbitem.lParam = font_array.fonts[i].lParam;
+
+        SendMessageW(hListWnd, CBEM_INSERTITEMW, 0, (LPARAM)&cbitem);
+
+        free(font_array.fonts[i].name);
+    }
+    free(font_array.fonts);
 
     ZeroMemory(&fmt, sizeof(fmt));
     fmt.cbSize = sizeof(fmt);
@@ -950,8 +1014,7 @@ static BOOL prompt_save_changes(void)
         else
             displayFileName = file_basename(wszFileName);
 
-        text = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
-                         (lstrlenW(displayFileName)+lstrlenW(wszSaveChanges))*sizeof(WCHAR));
+        text = calloc(lstrlenW(displayFileName)+lstrlenW(wszSaveChanges), sizeof(WCHAR));
 
         if(!text)
             return FALSE;
@@ -960,7 +1023,7 @@ static BOOL prompt_save_changes(void)
 
         ret = MessageBoxW(hMainWnd, text, wszAppTitle, MB_YESNOCANCEL | MB_ICONEXCLAMATION);
 
-        HeapFree(GetProcessHeap(), 0, text);
+        free(text);
 
         switch(ret)
         {
@@ -1816,7 +1879,7 @@ static LRESULT OnCreate( HWND hWnd )
     HFONT font;
     HDC hdc;
     SIZE name_sz, size_sz;
-    int height;
+    int height, dpi;
     static const WCHAR wszRichEditDll[] = {'R','I','C','H','E','D','2','0','.','D','L','L','\0'};
     static const WCHAR wszRichEditText[] = {'R','i','c','h','E','d','i','t',' ','t','e','x','t','\0'};
     static const WCHAR font_text[] = {'T','i','m','e','s',' ','N','e','w',' ','R','o','m','a','n',0}; /* a long font name */
@@ -1840,8 +1903,12 @@ static LRESULT OnCreate( HWND hWnd )
       NULL, 0,
       24, 24, 16, 16, sizeof(TBBUTTON));
 
+    hdc = GetDC(hWnd);
+    dpi = GetDeviceCaps(hdc, LOGPIXELSY);
+    ReleaseDC(hWnd, hdc);
+
     ab.hInst = HINST_COMMCTRL;
-    ab.nID = IDB_STD_SMALL_COLOR;
+    ab.nID = dpi >= 120 ? IDB_STD_LARGE_COLOR : IDB_STD_SMALL_COLOR;
     nStdBitmaps = SendMessageW(hToolBarWnd, TB_ADDBITMAP, 0, (LPARAM)&ab);
 
     AddButton(hToolBarWnd, nStdBitmaps+STD_FILENEW, ID_FILE_NEW);
@@ -1870,6 +1937,8 @@ static LRESULT OnCreate( HWND hWnd )
     GetWindowRect(hFontListWnd, &rect);
     height = max(height, rect.bottom - rect.top);
 
+    SendMessageW(hToolBarWnd, TB_SETBUTTONSIZE, 0, MAKELPARAM(height, height));
+
     rbb.cbSize = REBARBANDINFOW_V6_SIZE;
     rbb.fMask = RBBIM_SIZE | RBBIM_CHILDSIZE | RBBIM_CHILD | RBBIM_STYLE | RBBIM_ID;
     rbb.fStyle = RBBS_CHILDEDGE | RBBS_BREAK | RBBS_NOGRIPPER;
@@ -1910,6 +1979,7 @@ static LRESULT OnCreate( HWND hWnd )
          IDC_FORMATBAR, 8, hInstance, IDB_FORMATBAR, NULL, 0, 16, 16, 16, 16, sizeof(TBBUTTON));
 
     SendMessageW(hFormatBarWnd, TB_SETEXTENDEDSTYLE, 0, TBSTYLE_EX_DRAWDDARROWS);
+    SendMessageW(hFormatBarWnd, TB_SETBUTTONSIZE, 0, MAKELPARAM(height, height));
 
     AddButton(hFormatBarWnd, 0, ID_FORMAT_BOLD);
     AddButton(hFormatBarWnd, 1, ID_FORMAT_ITALIC);
@@ -1954,7 +2024,7 @@ static LRESULT OnCreate( HWND hWnd )
 
     if (!hEditorWnd)
     {
-        fprintf(stderr, "Error code %u\n", GetLastError());
+        fprintf(stderr, "Error code %lu\n", GetLastError());
         return -1;
     }
     assert(hEditorWnd);
@@ -2095,7 +2165,7 @@ static LRESULT OnNotify( HWND hWnd, LPARAM lParam)
 
             update_font_list();
 
-            sprintf( buf,"selection = %d..%d, line count=%ld",
+            sprintf( buf,"selection = %ld..%ld, line count=%Id",
                      pSC->chrg.cpMin, pSC->chrg.cpMax,
                      SendMessageW(hwndEditor, EM_GETLINECOUNT, 0, 0));
             SetWindowTextA(GetDlgItem(hWnd, IDC_STATUSBAR), buf);
@@ -2332,20 +2402,20 @@ static LRESULT OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam)
     case ID_EDIT_GETTEXT:
         {
         int nLen = GetWindowTextLengthW(hwndEditor);
-        LPWSTR data = HeapAlloc( GetProcessHeap(), 0, (nLen+1)*sizeof(WCHAR) );
+        LPWSTR data = malloc((nLen+1)*sizeof(WCHAR) );
         TEXTRANGEW tr;
 
         GetWindowTextW(hwndEditor, data, nLen+1);
         MessageBoxW(NULL, data, wszAppTitle, MB_OK);
 
-        HeapFree( GetProcessHeap(), 0, data);
-        data = HeapAlloc(GetProcessHeap(), 0, (nLen+1)*sizeof(WCHAR));
+        free(data);
+        data = malloc((nLen+1)*sizeof(WCHAR));
         tr.chrg.cpMin = 0;
         tr.chrg.cpMax = nLen;
         tr.lpstrText = data;
         SendMessageW(hwndEditor, EM_GETTEXTRANGE, 0, (LPARAM)&tr);
         MessageBoxW(NULL, data, wszAppTitle, MB_OK);
-        HeapFree( GetProcessHeap(), 0, data );
+        free(data);
 
         /* SendMessage(hwndEditor, EM_SETSEL, 0, -1); */
         return 0;
@@ -2380,12 +2450,12 @@ static LRESULT OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam)
         WCHAR *data = NULL;
 
         SendMessageW(hwndEditor, EM_EXGETSEL, 0, (LPARAM)&range);
-        data = HeapAlloc(GetProcessHeap(), 0, sizeof(*data) * (range.cpMax-range.cpMin+1));
+        data = malloc(sizeof(*data) * (range.cpMax-range.cpMin+1));
         SendMessageW(hwndEditor, EM_GETSELTEXT, 0, (LPARAM)data);
-        sprintf(buf, "Start = %d, End = %d", range.cpMin, range.cpMax);
+        sprintf(buf, "Start = %ld, End = %ld", range.cpMin, range.cpMax);
         MessageBoxA(hWnd, buf, "Editor", MB_OK);
         MessageBoxW(hWnd, data, wszAppTitle, MB_OK);
-        HeapFree( GetProcessHeap(), 0, data);
+        free(data);
         /* SendMessage(hwndEditor, EM_SETSEL, 0, -1); */
         return 0;
         }
