@@ -23,7 +23,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -495,27 +494,6 @@ static int relay_type_compare( const void *e1, const void *e2 )
 }
 
 
-/*******************************************************************
- *         sort_func_list
- *
- * Sort a list of functions, removing duplicates.
- */
-static int sort_func_list( ORDDEF **list, int count,
-                           int (*compare)(const void *, const void *) )
-{
-    int i, j;
-
-    if (!count) return 0;
-    qsort( list, count, sizeof(*list), compare );
-
-    for (i = j = 0; i < count; i++)
-    {
-        if (compare( &list[j], &list[i] )) list[++j] = list[i];
-    }
-    return j + 1;
-}
-
-
 /*******************************************************************
  *         output_module16
  *
@@ -825,7 +803,7 @@ void output_spec16_file( DLLSPEC *spec16 )
     output_stubs( spec16 );
     output_exports( spec32 );
     output_imports( spec16 );
-    if (is_undefined( "__wine_call_from_16" )) output_asm_relays16();
+    if (!strcmp( spec16->dll_name, "kernel" )) output_asm_relays16();
     if (needs_get_pc_thunk) output_get_pc_thunk();
     if (spec16->main_module)
     {
@@ -851,7 +829,8 @@ void output_fake_module16( DLLSPEC *spec )
     const unsigned int lfanew = (0x40 + sizeof(fakedll_signature) + 15) & ~15;
     const unsigned int segtab = lfanew + 0x40;
 
-    unsigned int i, rsrctab, restab, namelen, modtab, imptab, enttab, cbenttab, codeseg, dataseg, rsrcdata;
+    unsigned int i, rsrctab, restab, namelen, modtab, imptab, enttab, cbenttab, codeseg, dataseg, rsrcdata, rsrc_size = 0;
+    void *rsrc_ptr = NULL;
 
     init_output_buffer();
 
@@ -865,6 +844,10 @@ void output_fake_module16( DLLSPEC *spec )
         restab += output_buffer_pos;
         free( output_buffer );
         init_output_buffer();
+        output_bin_res16_data( spec );
+        rsrc_ptr = output_buffer;
+        rsrc_size = output_buffer_pos;
+        init_output_buffer();
     }
 
     namelen  = strlen( spec->dll_name );
@@ -896,7 +879,7 @@ void output_fake_module16( DLLSPEC *spec )
     put_dword( 0 );
     put_word( 0 );            /* e_oemid */
     put_word( 0 );            /* e_oeminfo */
-    put_dword( 0 );           /* e_res2 */
+    put_dword( rsrcdata + rsrc_size ); /* e_res2 */
     put_dword( 0 );
     put_dword( 0 );
     put_dword( 0 );
@@ -975,7 +958,5 @@ void output_fake_module16( DLLSPEC *spec )
     put_data( data_segment, sizeof(data_segment) );
 
     /* resource data */
-    output_bin_res16_data( spec );
-
-    flush_output_buffer();
+    put_data( rsrc_ptr, rsrc_size );
 }
